jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayoutArea");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.model.HierarchyMapPair");
jQuery.sap.require("sas.stress.pfd.PfdStyles");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");
jQuery.sap.require("sas.stress.wkbench.StressGoTemplates");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.ProcessDiagram");

sap.ui.controller("sas.stress.config.view.Configuration", {

    rb: sap.ui.getCore().getLibraryResourceBundle("sas.stress.wkbench"),

    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("Configuration", this._init, this);
    },

    _init: function (event) {

        // this NavCheck should handle everything for configuration top views, no need to create one for each sub view
        //this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true);  // NavChecker should not destroy this view

        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_CONFIG_ROOT, this.updateViews, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_HIERARCHIES, this.viewHierarchies, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_HIERARCHY_MAPPING, this.editHierarchyMapping, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_WORKSHEET_TEMPLATES, this.viewWorksheetTemplates, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_WORKSHEET_TEMPLATE, this.createWorksheetTemplate, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODELS, this.viewModels, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_DETAILS, this.viewModelDetails, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_GROUPS, this.viewModelGroups, this);
        //sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_MODEL_GROUP, this.createModelGroupView, this);
    },

    onAfterRendering: function () {
        var view = this.getView();
        //Putting in timeout seems to improve performance
        setTimeout(function () {
            view.createTabViews();
        }, 0);
    },

    createWorksheetTemplate: function (channel, event, data) {
        var selectedCategoryKey = data.selectedCategoryKey;
        var template = data.template;
        var view = this.getView();
        var createWorksheetArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            content: sas.stress.wkbench.ViewManager.loadCreateWorksheetTemplateEditor({
                    template: template,
                    selectedCategoryKey: selectedCategoryKey
                }
            )
        });
        view.worksheetLayout.setCenter(createWorksheetArea);
    },

    viewWorksheetTemplates: function (channel, event, data) {
        var view = this.getView();
        if (view.worksheetLayout.getCenter() !== view.worksheetArea) {
            // in case the sub view is not destroyed
            view.worksheetLayout.getCenter().destroy();
            view.worksheetLayout.setCenter(view.worksheetArea);
        }
    },

    viewHierarchies: function (channel, event, data) {
        var view = this.getView();
        if (view.hierarchyLayout.getCenter() !== view.hierarchyArea) {
            // in case the sub view is not destroyed
            view.hierarchyLayout.getCenter().destroy();
            view.hierarchyLayout.setCenter(view.hierarchyArea);
        }
    },

    editHierarchyMapping: function (channel, event, data) {
        var hierarchyKey = data.hierarchyKey;
        var view = this.getView();
        var hierarchyArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            content: sas.stress.wkbench.ViewManager.loadHierarchyMappingEditor({ hierarchyKey: hierarchyKey })
        });
        view.hierarchyLayout.setCenter(hierarchyArea);
    },

    viewModels: function (channel, event, data) {
        var view = this.getView();
        if (view.modelLayout.getCenter() !== view.modelArea) {
            // in case the sub view is not destroyed
            view.modelLayout.getCenter().destroy();
            view.modelLayout.setCenter(view.modelArea);
        }
    },

    viewModelDetails: function (channel, event, data) {
        var model = data.model;
        var view = this.getView();
        var modelArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            content: sas.stress.wkbench.ViewManager.loadModelsDetails({ modelId: model.id })
        });
        view.modelLayout.setCenter(modelArea);
    },

    viewModelGroups: function (channel, event, data) {
        var view = this.getView();
        if (view.modelGroupLayout.getCenter() !== view.modelGroupArea) {
            // in case the sub view is not destroyed
            view.modelGroupLayout.getCenter().destroy();
            view.modelGroupLayout.setCenter(view.modelGroupArea);
        }
    },

    createModelGroupView: function (channel, event, data) {
        var template = data.template;
        var view = this.getView();
        var createModelGroupArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            content: sas.stress.wkbench.ViewManager.loadCreateModelGroupEditor({ template: template })
        });
        view.modelGroupLayout.setCenter(createModelGroupArea);
    },

    onExit: function () {
        // Remove any subscriptions to ensure we don't get duplicates the next time the controller is initialized
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_CONFIG_ROOT, this.updateViews, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_HIERARCHIES, this.viewHierarchies, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_HIERARCHY_MAPPING, this.editHierarchyMapping, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_WORKSHEET_TEMPLATES, this.viewWorksheetTemplates, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_WORKSHEET_TEMPLATE, this.createWorksheetTemplate, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODELS, this.viewModels, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_DETAILS, this.viewModelDetails, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_GROUPS, this.viewModelGroups, this);
      //  sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_MODEL_GROUP, this.createModelGroupView, this);
        this.router.detachExactRouteMatched("Configuration", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.config.view.Configuration", null);
        //this.navChecker.stop();
    },

    updateViews: function (event) {

        var view = this.getView();
        var that = this;

        if (view.hierarchyLayout.getCenter() &&
            view.hierarchyLayout.getCenter() !== view.hierarchyArea) {
            view.hierarchyLayout.getCenter().destroy();
            view.hierarchyLayout.setCenter(view.hierarchyArea);
        }
        if (view.worksheetLayout.getCenter() &&
            view.worksheetLayout.getCenter() !== view.worksheetArea) {
            view.worksheetLayout.getCenter().destroy();
            view.worksheetLayout.setCenter(view.worksheetArea);
        }
        //if (view.modelLayout.getCenter() &&
        //    view.modelLayout.getCenter() !== view.modelArea) {
        //    view.modelLayout.getCenter().destroy();
        //    view.modelLayout.setCenter(view.modelArea);
        //}
        if (view.mapGraphLayout.getCenter() &&
            view.mapGraphLayout.getCenter() !== view.mapGraphArea) {
            view.mapGraphLayout.getCenter().destroy();
            view.mapGraphLayout.setCenter(view.mapGraphArea);
        }
       // if (view.modelGroupLayout.getCenter() &&
       //     view.modelGroupLayout.getCenter() !== view.modelGroupArea) {
       //     view.modelGroupLayout.getCenter().destroy();
       //     view.modelGroupLayout.setCenter(view.modelGroupArea);
       // }
        if (typeof event === 'object' &&
            event.getParameter("tab") &&
            event.getParameter("tab").getId() === "mapGraphLayout") {
            this._getHierarchyMapPairs().done(function (hierarchyMapPairs) {
                // We build the graph every time this tab is selected because user
                // could have made changes in the previous tab that affects the graph.
                that._buildHierarchyMapPairsGraph(hierarchyMapPairs);
            });
        }
    },

    _getHierarchyMapPairs: function () {
        var deferred = $.Deferred();
        sas.stress.config.model.HierarchyMapPair.getHierarchyMapPairs().done(function (hierarchyMapPairs) {
            deferred.resolve(hierarchyMapPairs);
        }).fail(function () {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            deferred.reject();
        });
        return deferred;
    },

    _buildHierarchyMapPairsGraph: function (hierarchyMapPairs) {
        var controller = this;
        var nodeDataArray = [];
        var pfdStyles = sas.stress.pfd.PfdStyles.init();
        var view = this.getView();
        _.each(sas.stress.config.model.Hierarchy.getCachedHierarchies(), function (hierarchy) {
            nodeDataArray.push(
                {
                    key: hierarchy.getHierarchyKey(),
                    text: hierarchy.getName(),
                    fill: pfdStyles.hierarchySimpleNodeFillColor,
                    stroke: pfdStyles.hierarchySimpleNodeStrokeColor,
                    color: pfdStyles.hierarchySimpleNodeTextColor
                }
            );
        });
        var linkDataArray = [];
        _.each(hierarchyMapPairs, function (hierarchyMapPair) {
            linkDataArray.push({ from: hierarchyMapPair.getFromHierKey(), to: hierarchyMapPair.getToHierKey() });
        });
        var templates = new sas.stress.wkbench.StressGoTemplates();
        var hierarchyMapGraphPanelHTML = new sas.stress.wkbench.widget.ProcessDiagram({
            content: '<div style="width:100%; min-height:250px; height:100%;"></div>',
            preferDOM: false,
            options: {
                allowDelete: false,
                allowMove: false,
                allowLink: false,
                allowRelink: false,
                allowVerticalScroll: true,
                allowHorizontalScroll: true,
                hasHorizontalScrollbar: true,
                hasVerticalScrollbar: true,
                allowUndo: false,
                contentAlignment: go.Spot.Center,
                "toolManager.mouseWheelBehavior": go.ToolManager.WheelZoom,
                padding: 15
            },
            layout: templates.LAYOUT(),
            nodeTemplates: {
                "": templates.HIERARCHY_NODE()
            },
            linkTemplates: {
                "": templates.LINK()
            },
            nodes: nodeDataArray,
            links: linkDataArray,
            objectDoubleClicked: $.proxy(controller.diagramObjectDoubleClickedHandler, controller)
        });

        var zoomInButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMIN,
            tooltip: "{i18N>home.diagram.zoomInButton.txt}",
            press: function () {
                var commandHandler = hierarchyMapGraphPanelHTML.getDiagram().commandHandler;
                if (commandHandler.canIncreaseZoom()) {
                    commandHandler.increaseZoom();
                }
            }
        });

        var zoomOutButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMOUT,
            tooltip: "{i18N>home.diagram.zoomOutButton.txt}",
            press: function () {
                var commandHandler = hierarchyMapGraphPanelHTML.getDiagram().commandHandler;
                if (commandHandler.canDecreaseZoom()) {
                    commandHandler.decreaseZoom();
                }
            }
        });

        var zoomFitButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMTOFIT,
            tooltip: "{i18N>home.diagram.zoomFitButton.txt}",
            press: function () {
                var commandHandler = hierarchyMapGraphPanelHTML.getDiagram().commandHandler;
                commandHandler.zoomToFit();
            }
        });

        var zoomToolbar = new sas.hc.m.Toolbar({
            content: [
                new sas.hc.m.ToolbarSpacer(),
                zoomInButton,
                zoomOutButton,
                zoomFitButton
            ]
        }).addStyleClass("hierarchyDiagramToolbar");

        view.mapGraphToolbar.removeAllContent();
        view.mapGraphToolbar.addContent(zoomToolbar);
        view.mapGraphArea.removeAllContent();
        view.mapGraphArea.addContent(hierarchyMapGraphPanelHTML);
    },

    diagramObjectDoubleClickedHandler: function (event) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        if (capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.UPDATE)) {
            var view = this.getView();
            var hierarchyKey = event.getParameter("graphObject").key;

            if (hierarchyKey) {
                var hierarchyArea = new sas.hc.ui.commons.layout.BorderLayoutArea({ content: sas.stress.wkbench.ViewManager.loadHierarchyMappingEditor({ hierarchyKey: hierarchyKey }) });
                view.mapGraphLayout.setCenter(hierarchyArea);
            }
        }
    }

});