jQuery.sap.require("sap.ui.core.routing.Router");
jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.MessageBox");
jQuery.sap.require("sas.hc.m.ResponsivePopover");
jQuery.sap.require("sas.hc.ui.commons.layout.MatrixLayout");
jQuery.sap.require("sas.hc.ui.commons.layout.VerticalLayout");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.config.model.HierarchyCategory");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.model.HierarchyLineItem");
jQuery.sap.require("sas.stress.config.model.HierarchyMapPair");
jQuery.sap.require("sas.stress.pfd.PfdStyles");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressGoTemplates");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.AccessControlsDialog");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.wkbench.widget.ProcessDiagram");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require({ modName: "sas.stress.wkbench.view.CategoryDialog", type: "fragment" });

sap.ui.controller("sas.stress.config.view.ProjectConfiguration", {

    onInit: function (event) {
        //NavChecker in Configuration.controller should handle everything for the sub views.
        //this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true); // new sas.risk.common.utils.NavChecker(sas.config.getAppId(), rb.getText("project.dirty.question.txt"), this.router);
        this.refreshList();
        new sas.risk.common.utils.LayoutService(this.getView().hierarchiesTreeTable, "HierarchiesTreeTable");
    },
    onExit: function () {
        //this.navChecker.stop();
    },

    refreshList: function() {
        var controller = this;
        var view = this.getView();
        view.hierarchiesTreeTable.setBusyIndicatorDelay(0);
        view.hierarchiesTreeTable.setBusy(true);
        //Get Categories.
        var _getHierarchyCategories = controller._getHierarchyCategories();
        //Load hierarchies
        var _getHierarchies = controller._getHierarchies();

        $.when(_getHierarchies, _getHierarchyCategories).done(function () {
            //Create hierarchy called Entity in the Organization category if the Organization category exists.
            // removed Entity since 2.3
            //controller._createHierarchyEntity();

            //Set hierarchiesCount for each category.
            controller._setHierarchiesCountInCategoriesList();
            controller._selectPreferredCategory();
            view.hierarchiesTreeTable.setBusy(false);

        });
        //Load Hierarchy Map pairs. These can be used by the Edit and Delete Hierarchy dialogs.
        controller._getHierarchyMapPairs();
    },

    _setHierarchiesCountInCategoriesList: function () {
        var view = this.getView();
        var categoriesListModelData = this._getCategoriesInModel();
        //reset hierarchiesCount.
        _.each(categoriesListModelData, function (category) {
            category.hierarchiesCount = 0;
        });
        var hierarchies = view.hierarchiesTreeTable.getModel().getData().children;
        //set hierarchiesCount.
        categoriesListModelData[0].hierarchiesCount = hierarchies.length; //Count for category "All".
        _.each(hierarchies, function (hierarchy) {
            var category = _.find(categoriesListModelData, function (category) {
                return category.getCategoryKey() === hierarchy.categoryKey;
            });
            if (category !== undefined) {
                category.hierarchiesCount++;
            }
        });
        view.categoryFilter.getModel().refresh();
    },

    categoriesListSelectionChange: function () {
        this._toggleCategoriesListToolbarButtons();
        this._applyCategoryFilter();
        this._savePreferredCategory();
    },

    _toggleCategoriesListToolbarButtons: function () {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var selectedCategory = view.categoryFilter.getSelectedItem();
        var readOnly;
        var categoryKey;
        if (selectedCategory !== null) {
            categoryKey = this._getSelectedCategoryKey();
            var hierarchyCategory = _.find(this._getCategoriesInModel(), function (data) {
                return data.categoryKey === categoryKey;
            });
            if (hierarchyCategory !== undefined) {
                readOnly = hierarchyCategory.readOnly;
            }
        }
        view.byId("categoriesList.editButton").setEnabled(readOnly === false && categoryKey !== 0 &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.UPDATE)); //Enable if category is not read-only and not "All".
        view.byId("categoriesList.deleteButton").setEnabled(readOnly === false && categoryKey !== 0 &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.DELETE)); //Enable if category is not read-only and not "All".
        this._toggleCreateHierarchyButton();
    },

    _toggleCreateHierarchyButton: function () {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        //A hierarchy cannot be created without a valid category.
        //Check if at least 1 category exists which is not "All" and not read-only.
        var hierarchyCategory = _.find(this._getCategoriesInModel(), function (category) {
            return category.readOnly === false && category.categoryKey !== 0;
        });
        //Get the read-only status of the selected category.
        var selectedCategory = view.categoryFilter.getSelectedItem();
        var readOnly;
        if (selectedCategory !== null) {
            var categoryKey = this._getSelectedCategoryKey();
            var selectedHierarchyCategory = _.find(this._getCategoriesInModel(), function (data) {
                return data.categoryKey === categoryKey;
            });
            if (selectedHierarchyCategory !== undefined) {
                readOnly = selectedHierarchyCategory.readOnly;
            }
        }
        view.byId("hierarchiesTreeTable.createButton").setEnabled(hierarchyCategory !== undefined && readOnly === false &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.CREATE));
    },

    _getSelectedCategoryKey: function () {
        var view = this.getView();
        var categoryKey = null;
        var selectedCategory = view.categoryFilter.getSelectedCategory();
        if (!!selectedCategory) {
            var category = _.find(this._getCategoriesInModel(), function (data) {
                return data.name === selectedCategory;
            });
            if (category !== undefined) {
                categoryKey = category.categoryKey;
            }
        }
        return categoryKey;
    },

    /* Filters rows in the hierarchiesTreeTable based on the selected category. */
    _applyCategoryFilter: function () {
        var view = this.getView();
        var selectedCategory = view.categoryFilter.getSelectedCategory();
        if (selectedCategory !== null) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            if (selectedCategory === rb.getText("categories.all.txt")) {
                view.hierarchiesTreeTable.getBinding("rows").filter([]);
            } else {
                //Filter rows in the hierarchiesTreeTable.
                var categoryFilter = new sap.ui.model.Filter("categoryKey", sap.ui.model.FilterOperator.EQ, this._getSelectedCategoryKey());
                view.hierarchiesTreeTable.getBinding("rows").filter([categoryFilter]);
            }
        }
    },

    // iconTabBarSelect: function (event) {
    //     var controller = this;
    //     if (event.getParameter("item").getId() === controller.getView().byId("hierarchyMapIconTabFilter").getId()) {
    //         //We get the pairs every time this tab is selected because user could have made changes in the previous tab that affects the graph.
    //         controller._getHierarchyMapPairs().done(function (hierarchyMapPairs) {
    //             //We build the graph every time this tab is selected because user could have made changes in the previous tab that affects the graph.
    //             controller._buildHierarchyMapPairsGraph(hierarchyMapPairs);
    //         });
    //     }
    // },

    /* Displays graph in the Hierarchy Map Pairs Graph panel. */
    //_buildHierarchyMapPairsGraph: function (hierarchyMapPairs) {
    //    var controller = this;
    //    var nodeDataArray = [];
    //    var pfdStyles = sas.stress.pfd.PfdStyles.init();
    //
    //    _.each(sas.stress.config.model.Hierarchy.getCachedHierarchies(), function (hierarchy) {
    //        nodeDataArray.push(
    //            {
    //                key: hierarchy.getHierarchyKey(),
    //                text: hierarchy.getName(),
    //                fill: pfdStyles.hierarchySimpleNodeFillColor,
    //                stroke: pfdStyles.hierarchySimpleNodeStrokeColor,
    //                color: pfdStyles.hierarchySimpleNodeTextColor
    //            }
    //        );
    //    });
    //    var linkDataArray = [];
    //    _.each(hierarchyMapPairs, function (hierarchyMapPair) {
    //        linkDataArray.push({ from: hierarchyMapPair.getFromHierKey(), to: hierarchyMapPair.getToHierKey() });
    //    });
    //    var templates = new sas.stress.wkbench.StressGoTemplates();
    //    var hierarchyMapGraphPanelHTML = new sas.stress.wkbench.widget.ProcessDiagram({
    //        content: '<div style="width:100%; min-height:300px; height:100%;"></div>',
    //        preferDOM: false,
    //        options: {
    //            allowDelete: false,
    //            allowMove: false,
    //            allowLink: false,
    //            allowRelink: false,
    //            allowVerticalScroll: true,
    //            allowHorizontalScroll: true,
    //            hasHorizontalScrollbar: false,
    //            hasVerticalScrollbar: false,
    //            allowUndo: false,
    //            contentAlignment: go.Spot.Center,
    //            "toolManager.mouseWheelBehavior": go.ToolManager.WheelZoom,
    //            padding: 15
    //        },
    //        layout: templates.LAYOUT(),
    //        nodeTemplates: {
    //            "": templates.HIERARCHY_NODE()
    //        },
    //        linkTemplates: {
    //            "": templates.LINK()
    //        },
    //        nodes: nodeDataArray,
    //        links: linkDataArray,
    //        objectDoubleClicked: $.proxy(controller.diagramObjectDoubleClickedHandler, controller)
    //    });
    //    var hierarchyMapGraphPanel = controller.getView().byId("hierarchyMapGraphPanel");
    //    hierarchyMapGraphPanel.removeAllContent();
    //    hierarchyMapGraphPanel.addContent(hierarchyMapGraphPanelHTML);
    //},
    //
    //diagramObjectDoubleClickedHandler: function (event) {
    //    var hierarchyKey = event.getParameter("graphObject").key;
    //    if (hierarchyKey) {
    //        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_DETAILS, { hierarchyKey: hierarchyKey });
    //    }
    //},
    //
    _getHierarchyCategories: function (event) {
        var deferred = $.Deferred();
        var view = this.getView();
        var controller = this;
        var displayCategories = [];
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.stress.config.model.HierarchyCategory.listHierarchyCategories().done(function (hierarchyCategories) {
            hierarchyCategories.unshift(new sas.stress.config.model.HierarchyCategory(0, rb.getText("categories.all.txt"), 0, false)); //Add category "All" to the top of the list.
            controller._sortHierarchyCategories(hierarchyCategories);
            // Filter ORG and Scenario
            _.each(hierarchyCategories, function (hierarchyCategory) {
                if (hierarchyCategory.categoryCode !== "ORG") {
                    displayCategories.push(hierarchyCategory);
                }
            });
            var jsonModel = new sas.stress.wkbench.model.RiskJSONModel({ categories: displayCategories });
            if (view.categoryFilter) {
                view.categoryFilter.setModel(jsonModel);
            } else {
                view.setModel(jsonModel);
            }
            deferred.resolve(hierarchyCategories);
        }).fail(function (jqxhr) {
            if (jqxhr && jqxhr.responseJSON) {
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            } else {
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            }
        });
        return deferred;
    },

    _sortHierarchyCategories: function (hierarchyCategories) {
        //Sort by Name ascending. Keep "All" at the top.
        var sortedHierarchyCategories =
            hierarchyCategories.slice(1).sort(function (a, b) {
                var aName = a.getName().toLowerCase();
                var bName = b.getName().toLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            });
        hierarchyCategories.length = 1;
        hierarchyCategories.push.apply(hierarchyCategories, sortedHierarchyCategories);
    },

    _sortHierarchies: function (hierarchies) {
        //Sort by Name ascending
        hierarchies.sort(function (a, b) {
            var aName = a.name.toLowerCase();
            var bName = b.name.toLowerCase();
            return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
        });
    },

    /* Gets the list of hierarchies and displays this list in the Hierarchies table. */
    _getHierarchies: function () {
        var deferred = $.Deferred();
        var controller = this;
        var view = this.getView();
        sas.stress.config.model.Hierarchy.getHierarchyDetails().done(function (hierarchies) {
            var hierarchiesRootNode = { children: [] };
            _.each(hierarchies, function (hierarchy) {
                if (hierarchy.code !== "SCENARIO") {
                    var hierarchyRootNode = {
                        hierarchyKey: hierarchy.getHierarchyKey(),
                        categoryKey: hierarchy.getCategoryKey(),
                        name: hierarchy.getName(),
                        version: hierarchy.getVersion(),
                        description: hierarchy.getDescription(),
                        updatedTimestamp: hierarchy.getUpdatedTimestamp(),
                        updatedByUser: hierarchy.getUpdatedByUser(),
                        topLevelNames: hierarchy.getLevelNames(),
                        children: [],
                        readOnly: false
                    };
                    hierarchiesRootNode.children.push(hierarchyRootNode);
                    _.each(hierarchy.getLineItems(), function (lineItem) {
                        controller._addToTreeNode(hierarchyRootNode, lineItem);
                    });
                }
            });
            controller._sortHierarchies(hierarchiesRootNode.children);
            var jsonModel = new sas.stress.wkbench.model.RiskJSONModel(hierarchiesRootNode);
            view.hierarchiesTreeTable.setModel(jsonModel);
            view.hierarchiesTreeTable.bindRows("/children");
            deferred.resolve(hierarchies);
        }).fail(function () {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            deferred.reject();
        });
        return deferred;
    },

    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToTreeNode: function (rootNode, lineItem) {
        var currentNode = rootNode;
        $.each(lineItem.levelNames, function (index, levelName) {
            //check if currentNode.children contains node with levelName[i].
            var node = _.find(currentNode.children, function (childNode) {
                return childNode.name === levelName;
            });
            //If not, add node with levelName[i].
            if (node === undefined) {
                node = {
                    name: levelName,
                    hierarchyKey: lineItem.getHierarchyKey(),
                    categoryKey: currentNode.categoryKey,
                    children: [],
                    readOnly: true
                };
                //set liKey and contra on the leaf node.
                if (index === lineItem.getLevelNames().length - 1) {
                    node.liKey = lineItem.getLiKey();
                    node.contra = lineItem.getContra();
                    node.aggregation = lineItem.getAggregation();
                    node.format = lineItem.getFormat();
                    node.formula = lineItem.getFormula();
                    node.level = (rootNode.topLevelNames && lineItem.levelNum > 0 && lineItem.levelNum <= rootNode.topLevelNames.length) ? rootNode.topLevelNames[(lineItem.levelNum - 1)] : "";
                    node.workGroupNames = lineItem.getWorkGroupNames();
                    node.workGroupString = lineItem.getWorkGroupNames().join(", ");
                    //if (workGroupString && workGroupString.length > 0) {
                    //    node.workGroupString = [workGroupString, workGroupString, workGroupString].join(", ");
                    //}
                }
                currentNode.children.push(node);
            }
            //set currentNode as the node with levelName[i] for levelName[i+1].
            currentNode = node;
        });
    },

    _updateCategoryKeyInTree: function (rootNode) {
        var controller = this;
        $.each(rootNode.children, function (index, childNode) {
            childNode.categoryKey = rootNode.categoryKey;
            if (childNode.children.length > 0) {
                controller._updateCategoryKeyInTree(childNode);
            }
        });
    },

    hierarchiesTreeTableRowSelectionChange: function () {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        var rootNode;
        var readOnly = true;
        var hasWorkGroups = false;
        if (selectedIndex >= 0) {
            var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
            var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
            readOnly = selectedNode.readOnly;
            //hasWorkGroups =  selectedNode.workGroupNames && selectedNode.workGroupNames.length > 0;
            //Check if selected node is the root node of the hierarchy.
            rootNode = _.find(view.hierarchiesTreeTable.getModel().getData().children, function (data) {
                return selectedNode === data;
            });
        }
        //If hierarchy root is selected, enable Edit and Delete buttons, else disable these buttons.
        view.byId("hierarchiesTreeTable.deleteButton").setEnabled(rootNode !== undefined && !readOnly &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.DELETE));
        view.byId("hierarchiesTreeTable.editButton").setEnabled(rootNode !== undefined && !readOnly &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.UPDATE));
        view.byId("hierarchiesTreeTable.accessControlsButton").setEnabled(rootNode !== undefined && !readOnly &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.READ));
        view.byId("hierarchiesTreeTable.mapButton").setEnabled(rootNode !== undefined && !readOnly &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.CONFIGURE));
        view.byId("hierarchiesTreeTable.collapseButton").setEnabled(rootNode !== undefined &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.READ));
        view.byId("hierarchiesTreeTable.expandButton").setEnabled(rootNode !== undefined &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.READ));
    },

    /* Gets the list of hierarchy map pairs. */
    _getHierarchyMapPairs: function () {
        var deferred = $.Deferred();
        sas.stress.config.model.HierarchyMapPair.getHierarchyMapPairs().done(function (hierarchyMapPairs) {
            deferred.resolve(hierarchyMapPairs);
        }).fail(function () {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            deferred.reject();
        });
        return deferred;
    },

    /* Called when the user presses the Create button in the Hierarchies tree table. Displays the Add Hierarchy dialog. */
    hierarchiesTreeTableCreateButtonPress: function (evt) {
        var existingDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var addHierarchyDialog = sap.ui.jsfragment("sas.stress.config.view.EditHierarchyDialog", this);
        var categories = _.filter(this._getCategoriesInModel(), function (category) {
            //Skip "All" and "Organization".
            return category.categoryKey !== 0 && category.readOnly === false;
        });
        ////In the Category Select field, select the category user is presently in.
        // var selectedCategory = view.categoryFilter.getSelectedItem();
        //Get HierarchyCategory obj for the selected category.
        var category = sas.stress.config.model.HierarchyCategory.getHierarchyCategory(this._getSelectedCategoryKey());
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var newHierarchyJSON = {
            title: rb.getText("addHierarchyDialog.window.title"),
            name: sas.stress.config.model.Hierarchy.getNewHierarchyName(),
            description: "",
            categoryKey: category && category.getCategoryKey(),
            categories: categories
        };
        var model = new sas.stress.wkbench.model.RiskJSONModel(newHierarchyJSON);
        addHierarchyDialog.setModel(model);
        addHierarchyDialog.open();
        //if (category != undefined) {
        //    sap.ui.getCore().byId("EditHierarchyDialog.categorySelect").setSelectedKey(category.getCategoryKey());
        //}
    },


    /* Called when the user presses the Save button in the Add Hierarchy dialog. Makes a mid-tier call to create a new hierarchy.
     * Refreshes the Hierarchies tree table after the hierarchy is created. */
    addHierarchyDialogSaveButtonPress: function (evt) {
        var controller = this;
        var view = this.getView();
        var addHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        addHierarchyDialog.setBusy(true);
        var hierarchyName = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyNameInput").getValue();
        var hierarchyVersion = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyVersionInput").getValue();
        var hierarchyDescription = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyDescriptionInput").getValue();
        var selectedCategory = sap.ui.getCore().byId("EditHierarchyDialog.categorySelect").getSelectedItem();
        var hierarchyKey = 0; //New Hierarchy
        var hierarchy = new sas.stress.config.model.Hierarchy(hierarchyKey, selectedCategory.getKey(),
            selectedCategory.getText(), hierarchyName.trim(), hierarchyVersion.trim(), hierarchyDescription);
        sas.stress.config.model.Hierarchy.addHierarchy(hierarchy).done(function (hierarchy) {
            //var hierarchyRootNode = {
            //    hierarchyKey: hierarchy.getHierarchyKey(),
            //    categoryKey: hierarchy.getCategoryKey(),
            //    name: hierarchy.getName(),
            //    version: hierarchy.getVersion(),
            //    description: hierarchy.getDescription(),
            //    updatedTimestamp: hierarchy.getUpdatedTimestamp(),
            //    updatedByUser: hierarchy.getUpdatedByUser(),
            //    topLevelNames: hierarchy.getLevelNames(),
            //    children: [],
            //    readOnly: false
            //};
            //_.each(hierarchy.getLineItems(), function (lineItem) {
            //    controller._addToTreeNode(hierarchyRootNode, lineItem);
            //});
            //var hierarchiesRootNode = view.hierarchiesTreeTable.getModel().getData();
            //hierarchiesRootNode.children.push(hierarchyRootNode);
            //controller._sortHierarchies(hierarchiesRootNode.children); //Sort by Name ascending
            //view.hierarchiesTreeTable.getModel().refresh();
            //controller._applyCategoryFilter();
            //controller._setHierarchiesCountInCategoriesList();


            //Upload file if one is selected.
            var lineItemsFileUploader = sap.ui.getCore().byId("EditHierarchyDialog.lineItemsFileUploader");
            if (lineItemsFileUploader.getValue()) { //File selected?
                lineItemsFileUploader.setUploadUrl("rest/config/hierarchies/" + hierarchy.getHierarchyKey() + "/items");
                lineItemsFileUploader.upload();
            } else {
                addHierarchyDialog.setBusy(false);
                addHierarchyDialog.close();
                // refresh the hierarchy list in case the name or category is modified by another version
                controller.refreshList();
            }
        }).fail(function (jqxhr) {
            addHierarchyDialog.setBusy(false);
            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
        });
    },

    /* Called when the user changes any field in the Edit Hierarchy dialog. Validates the name by
     * checking if it is empty or duplicate of an existing hierarchy.
     * If validation fails, displays an error message around the Name and version field and disables the Save button. */
    handleHierarchyDialogFieldChange: function (event) {
        var isValid = this.validateNameVersionFields();
        //Toggle Save button based on the result of validation.
        sap.ui.getCore().byId("EditHierarchyDialog.saveHierarchyButton").setEnabled(isValid);
    },

    validateNameVersionFields: function () {
        var view = this.getView();
        var editHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        var hierarchyKey = editHierarchyDialog.getModel().getData().hierarchyKey;
        var hierarchyBeingEdited = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyKey);

        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;
        var nameInput = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyNameInput");
        var newName = nameInput.getValue().trim();
        var versionInput = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyVersionInput");
        var newVersion = versionInput.getValue().trim();
        //Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            //Check if hierarchy with this name/version already exists. Hierarchy name/version can be = name of the hierarchy being edited.
            var hierarchy = _.find(view.hierarchiesTreeTable.getModel().getData().children, function (hierarchy) {
                var found = (hierarchy.name && hierarchy.name.toLowerCase() === newName.toLowerCase())
                    && (hierarchy.version.toLowerCase() === newVersion.toLowerCase());
                return (found && (hierarchy.hierarchyKey !== hierarchyKey));
            });
            if (hierarchy) {
                isValid = false;
                nameInput.setValueStateText(rb.getText("error.nameAndVersionInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
                versionInput.setValueStateText(rb.getText("error.nameAndVersionInUse.txt"));
                versionInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                versionInput.setValueStateText("");
                versionInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        return isValid;
    },

    /* Called when the user changes Name field in the Edit Hierarchy dialog. Check to see if changes to the name would
     * affect (bulk update) any other hierarchy with the same name.  Display a warning message if it does. */
    handleHierarchyNameChange: function (event) {
        var isValid = this.validateNameVersionFields();
        if (!isValid) {
            return;  // no need to continue
        }
        var view = this.getView();
        var editHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        var newHierarchyName = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyNameInput").getValue().trim();
        var hierarchyKey = editHierarchyDialog.getModel().getData().hierarchyKey;
        var nameToBeChanged = null;
        if (hierarchyKey) {   // edit
            var hierarchyBeingEdited = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyKey);
            if (hierarchyBeingEdited) {
                var sameNameHierarchies = this.findHierarchiesWithName(hierarchyBeingEdited.name);
                if (sameNameHierarchies && sameNameHierarchies.length > 1 &&    // only if name is changed
                    hierarchyBeingEdited.name !== newHierarchyName) {
                    nameToBeChanged = hierarchyBeingEdited.name;
                }
            }
        } else {   // new
            var sameNameHierarchies = this.findHierarchiesWithName(newHierarchyName);
            if (sameNameHierarchies && sameNameHierarchies.length > 0 &&
                sameNameHierarchies[0].name !== newHierarchyName) {   // only possible when case is different
                nameToBeChanged = sameNameHierarchies[0].name;
            }
        }
        if (nameToBeChanged) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.warning(rb.getText("editHierarchyDialog.renameAllVersions.confirmation.msg.fmt.txt",
                [nameToBeChanged, newHierarchyName]),
                rb.getText("editHierarchyDialog.window.title"),
                [sas.hc.m.MessageBox.Action.OK], null, function (oAction) {
                    if (oAction === sas.hc.m.MessageBox.Action.OK) {
                        // do nothing
                    }
                }
            );
        }
    },

    /* Called when the user changes Category field in the Edit Hierarchy dialog. */
    handleHierarchyCategoryChange: function (event) {
        //Hotfix defect: S1312535. No longer to bulk update
        //Check to see if changes to the category would affect (bulk update) any other hierarchy with the same name.
        // Display a warning message if it does.
        //var view = this.getView();
        //var editHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        //var newHierarchyCategory = sap.ui.getCore().byId("EditHierarchyDialog.categorySelect").getSelectedItem().getText();
        //var newHierarchyName = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyNameInput").getValue().trim();
        //var hierarchyKey = editHierarchyDialog.getModel().getData().hierarchyKey;
        //var nameToBeChanged = null;
        //if (hierarchyKey) {   // edit
        //    var hierarchyBeingEdited = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyKey);
        //    if (hierarchyBeingEdited) {
        //        var sameNameHierarchies = this.findHierarchiesWithName(hierarchyBeingEdited.name);
        //        if (sameNameHierarchies && sameNameHierarchies.length > 1 &&    // only if category is changed
        //            hierarchyBeingEdited.categoryName !== newHierarchyCategory) {
        //            nameToBeChanged = hierarchyBeingEdited.name;
        //        }
        //    }
        //} else {   // new
        //    var sameNameHierarchies = this.findHierarchiesWithName(newHierarchyName);
        //    if (sameNameHierarchies && sameNameHierarchies.length > 0 &&
        //        sameNameHierarchies[0].categoryName !== newHierarchyCategory) {  // different category selected
        //        nameToBeChanged = sameNameHierarchies[0].name;
        //    }
        //}
        //if (nameToBeChanged) {
        //    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        //    sas.hc.m.MessageBox.warning(rb.getText("editHierarchyDialog.updateAllCategories.confirmation.msg.fmt.txt",
        //        [nameToBeChanged, newHierarchyCategory]),
        //        rb.getText("editHierarchyDialog.window.title"),
        //        [sas.hc.m.MessageBox.Action.OK], null, function (oAction) {
        //            if (oAction === sas.hc.m.MessageBox.Action.OK) {
        //                // do nothing
        //            }
        //        }
        //    );
        //}
        //S1301029: enable Save
        this.handleHierarchyDialogFieldChange(event);
    },

    findHierarchiesWithName: function (name) {
        var hierarchiesRootNode = this.getView().hierarchiesTreeTable.getModel().getData();
        return _.filter(hierarchiesRootNode.children, function (hierarchyNode) {
            return hierarchyNode.name.toLowerCase() === name.toLowerCase();
        });
    },

    handleHierarchyDialogSaveButtonPress: function (event) {
        if (this._isEditHierarchy()) {
            this.editHierarchyDialogSaveButtonPress(event);
        } else {
            this.addHierarchyDialogSaveButtonPress(event);
        }
    },

    /* Called when the user presses the Delete button in the Hierarchies Tree table.
     * Checks if there are any Hierarchy Map pairs containing this hierarchy.
     * Prompts with a confirmation to delete the selected hierarchy and Hierarchy Map pairs if any exist.
     * Makes a mid-tier call to delete the selected hierarchy.
     * Refreshes the Hierarchies tree table after the hierarchy is deleted. */
    hierarchiesTreeTableDeleteButtonPress: function (evt) {
        var view = this.getView();
        var controller = this;
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex < 0) {
            return;
        }
        var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
        var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
        var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(selectedNode.hierarchyKey);

        //Check if there are hierarchy map pairs involving this hierarchy
        var hierarchyMapPairs = _.filter(sas.stress.config.model.HierarchyMapPair.getCachedHierarchyMapPairs(), function (hierarchyMapPair) {
            return hierarchyMapPair.getFromHierKey() === hierarchy.getHierarchyKey() || hierarchyMapPair.getToHierKey() === hierarchy.getHierarchyKey();
        });
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var message = rb.getText("hierarchiesTable.delete.confirmation.msg.fmt.txt", [hierarchy.getNameVersion()]);
        if (hierarchyMapPairs.length > 0) {
            message += "\n" + rb.getText("hierarchiesTable.hierarchyMapPairsWillBeDeleted.msg.fmt.txt", [hierarchyMapPairs.length]);
        }
        sas.hc.m.MessageBox.warning(message, rb.getText("hierarchiesTable.delete.window.title"),
                                    [sas.hc.m.MessageBox.Action.DELETE, sas.hc.m.MessageBox.Action.CANCEL], null, function (oAction) {
                if (oAction === sas.hc.m.MessageBox.Action.DELETE) {
                    sas.stress.config.model.Hierarchy.deleteHierarchy(hierarchy.hierarchyKey).done(function () {
                        var hierarchiesTreeTableModel = view.hierarchiesTreeTable.getModel();
                        $.each(hierarchiesTreeTableModel.getData().children, function (index, hierarchyRootNode) {
                            if (hierarchyRootNode.hierarchyKey === hierarchy.hierarchyKey) {
                                hierarchiesTreeTableModel.getData().children.splice(index, 1);
                                return false; //break for loop
                            }
                        });
                        hierarchiesTreeTableModel.refresh();
                        view.hierarchiesTreeTable.clearSelection();
                        controller._applyCategoryFilter();
                        controller._setHierarchiesCountInCategoriesList();
                        if (hierarchyMapPairs.length > 0) {
                            //Refresh Hierarchy Map pairs.
                            controller._getHierarchyMapPairs();
                        }
                        // Need to update the worksheet template table whenever a hierarchy is deleted
                        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.HIERARCHY_DELETED, {});
                        // need to clear the userprefs for the project filter.. something's have changed
                        // fixes for S1272838
                        var projects = sas.stress.wkbench.model.Project._projects;  // TODO: we should not use the variable need to have a method
                        _.each(projects, function (project, projectId) {
                            project.clearFilters(); // fixes for S1268615... clear the project filters
                        });
                    }).fail(function (jqxhr) {
                        sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                    });
                }
            }
        );
    },

    /* Called when the user presses the Edit button in the Hierarchies table. Displays the Edit Hierarchy dialog. */
    hierarchiesTreeTableEditButtonPress: function (evt) {
        var view = this.getView();
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex < 0) {
            return;
        }
        var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
        var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
        var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(selectedNode.hierarchyKey);
        var existingDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var editHierarchyDialog = sap.ui.jsfragment("sas.stress.config.view.EditHierarchyDialog", this);
        var categories = _.filter(this._getCategoriesInModel(), function (category) {
            //Skip "All" and "Organization".
            return category.categoryKey !== 0 && category.readOnly === false;
        });
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var hierarchyJSON = {
            title: rb.getText("editHierarchyDialog.window.title"),
            hierarchyKey: hierarchy.getHierarchyKey(),
            name: hierarchy.getName(),
            version: hierarchy.getVersion(),
            description: hierarchy.getDescription(),
            categoryKey: hierarchy.getCategoryKey(),
            categories: categories
        };
        var model = new sas.stress.wkbench.model.RiskJSONModel(hierarchyJSON);
        editHierarchyDialog.setModel(model);
        editHierarchyDialog.open();
        //S1289122: disable save initially
        sap.ui.getCore().byId("EditHierarchyDialog.saveHierarchyButton").setEnabled(false);
    },

    /* Called from the Edit Hierarchy dialog when the user presses the Save button.
     * Prompts to confirm deleting any hierarchy map pairs containing this hierarchy.
     * Makes a mid-tier call to update the hierarchy.
     * Refreshes the Hierarchies tree table after the update is successful. */
    editHierarchyDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var editHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        var hierarchyKey = editHierarchyDialog.getModel().getData().hierarchyKey;
        var hierarchyName = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyNameInput").getValue();
        var hierarchyVersion = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyVersionInput").getValue();
        var hierarchyDescription = sap.ui.getCore().byId("EditHierarchyDialog.hierarchyDescriptionInput").getValue();
        var selectedCategory = sap.ui.getCore().byId("EditHierarchyDialog.categorySelect").getSelectedItem();
        var hierarchy = new sas.stress.config.model.Hierarchy(hierarchyKey, selectedCategory.getKey(), selectedCategory.getText(),
            hierarchyName.trim(), hierarchyVersion.trim(), hierarchyDescription);
        //Check if the change includes a file upload containing line items as this will affect hierarchy mappings.
        var fileUploader = sap.ui.getCore().byId("EditHierarchyDialog.lineItemsFileUploader");
        if (fileUploader.getValue()) { //File selected?
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageBox.warning(rb.getText("editHierarchyDialog.updateAndLoseData.confirmation.msg.txt"),
                                        rb.getText("editHierarchyDialog.window.title"),
                                        [sas.hc.m.MessageBox.Action.YES, sas.hc.m.MessageBox.Action.NO], null, function (oAction) {
                    if (oAction === sas.hc.m.MessageBox.Action.YES) {
                        controller._updateHierarchy(hierarchy);

                        // fixes for S1272838... need to clear the userprefs for the project filter.. only do it is lineitems being uploaded
                        // TODO: we should not use the variable need to have a method.. problem is getProjects() will fetch.. there is no cache
                        var projects = sas.stress.wkbench.model.Project._projects;
                        _.each(projects, function (project, projectId) {
                            project.clearFilters(); // fixes for S1268615... clear the project filters
                        });
                    }
                }
            );
        } else {
            controller._updateHierarchy(hierarchy);
        }
    },

    _updateHierarchy: function (hierarchy) {
        var view = this.getView();
        var controller = this;
        var editHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        editHierarchyDialog.setBusy(true);
        sas.stress.config.model.Hierarchy.updateHierarchy(hierarchy).done(function (updatedHierarchy) {
            //Update the hierarchy in the model.
            //var hierarchiesRootNode = view.hierarchiesTreeTable.getModel().getData();
            //var hierarchyRootNode = _.find(hierarchiesRootNode.children, function (hierarchyNode) {
            //    return hierarchyNode.hierarchyKey === updatedHierarchy.getHierarchyKey();
            //});
            //if (hierarchyRootNode) {
            //    hierarchyRootNode.categoryKey = updatedHierarchy.getCategoryKey();
            //    hierarchyRootNode.name = updatedHierarchy.getName();
            //    hierarchyRootNode.version = updatedHierarchy.getVersion();
            //    hierarchyRootNode.description = updatedHierarchy.getDescription();
            //    hierarchyRootNode.updatedTimestamp = updatedHierarchy.getUpdatedTimestamp();
            //    hierarchyRootNode.updatedByUser = updatedHierarchy.getUpdatedByUser();
            //    hierarchyRootNode.topLevelNames = updatedHierarchy.getLevelNames();   //fixes for S1261325
            //    controller._updateCategoryKeyInTree(hierarchyRootNode); //category could have changed, update categoryKey in all child nodes of this hierarchy.
            //    controller._sortHierarchies(hierarchiesRootNode.children); //Sort by Name ascending
            //    view.hierarchiesTreeTable.getModel().refresh();
            //    controller._applyCategoryFilter();
            //    controller._setHierarchiesCountInCategoriesList();
            //    controller.hierarchiesTreeTableRowSelectionChange();
            //}

            //Upload file if one is selected.
            var lineItemsFileUploader = sap.ui.getCore().byId("EditHierarchyDialog.lineItemsFileUploader");
            if (lineItemsFileUploader.getValue()) { //File selected?
                lineItemsFileUploader.setUploadUrl("rest/config/hierarchies/" + updatedHierarchy.getHierarchyKey() + "/items");
                lineItemsFileUploader.upload();
            } else {
                editHierarchyDialog.setBusy(false);
                editHierarchyDialog.close();
                // refresh the hierarchy list in case the name or category is modified by another version
                controller.refreshList();
            }
            // Need to update the worksheet template table whenever a hierarchy is updated
            // Using the same eventbusconstant as HIERARCHY_DELETED as in both cases we have to refresh the sheet template
            sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.HIERARCHY_DELETED, {});
        }).fail(function (jqxhr) {
            editHierarchyDialog.setBusy(false);
            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);

        });
    },

    editHierarchyLineItemsFileUploadComplete: function (event) {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var editHierarchyDialog = sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
        var response = event.getParameter("response");
        if (response) {
            var responseJSON = JSON.parse(response);
            if (responseJSON.hasOwnProperty("errorCode")) {
                //display error msg.
                sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON,
                                                                  rb.getText("editHierarchyDialog.loadLineItems.window.title"));
                editHierarchyDialog.setBusy(false);
                editHierarchyDialog.close();
            } else {
                var lineItems = [];
                _.each(responseJSON.lineItems, function (lineItem) {    // fixes for S1261325
                    var hierarchyItem = new sas.stress.config.model.HierarchyLineItem(lineItem.liKey, lineItem.hierarchyKey,
                        lineItem.liOrder, lineItem.contra, lineItem.aggregation, lineItem.levelNames, lineItem.memberName,
                        lineItem.format, lineItem.formula, lineItem.levelNum, lineItem.workgroupNames);
                    lineItems.push(hierarchyItem);
                });
                if (lineItems.length > 0) {
                    var hierarchyKey = lineItems[0].getHierarchyKey();
                    //Update hierarchy in cache.
                    var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyKey);
                    if (hierarchy) {
                        hierarchy.setLineItems(lineItems);
                        hierarchy.levelNames = responseJSON.hierarchy.levelNames; //fixes for S1261325
                    }

                    //Update hierarchy in tree.
                    var hierarchyRootNode = _.find(view.hierarchiesTreeTable.getModel().getData().children,
                                                   function (hierarchyNode) {
                                                       return hierarchyNode.hierarchyKey === hierarchyKey;
                                                   }
                    );
                    if (hierarchyRootNode) {
                        hierarchyRootNode.children = []; //delete existing line items.
                        hierarchyRootNode.topLevelNames = responseJSON.hierarchy.levelNames; //fixes for S1261325
                        _.each(hierarchy.getLineItems(), function (lineItem) {
                            controller._addToTreeNode(hierarchyRootNode, lineItem);
                        });
                        view.hierarchiesTreeTable.getModel().refresh();
                        controller._applyCategoryFilter();
                        controller.hierarchiesTreeTableRowSelectionChange();
                    }
                }
            }
        }
        editHierarchyDialog.setBusy(false);
        editHierarchyDialog.close();
        // refresh the hierarchy list in case the name or category is modified by another version
        controller.refreshList();
    },

    editHierarchyFileUploadTypeMissmatch: function (event) {
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = "*." + value;
        });
        var sSupportedFileTypes = aFileTypes.join(", ");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.hc.m.MessageBox.error(rb.getText("fileUploadTypeMismatch.msg.fmt.txt",
                                             [event.getParameter("fileType"), sSupportedFileTypes]), this._getHierarchyDialogTitle());
    },

    editHierarchyFileUploadFileSizeExceed: function (event) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.hc.m.MessageBox.error(rb.getText("fileUploadFileSizeExceed.msg.fmt.txt",
                                             [event.getSource().getMaximumFileSize()]), this._getHierarchyDialogTitle());
    },

    hierarchiesTreeTableMapButtonPress: function (event) {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex < 0) {
            return;
        }
        var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
        var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_HIERARCHY_MAPPING, {
            hierarchyKey: selectedNode.hierarchyKey,
            name: selectedNode.name
        });
    },

    /* Called when the user presses the access controls button in the Hierarchies table. Displays the Access Controls dialog. */
    hierarchyAccessControlsButtonPress: function (evt) {
        var view = this.getView();
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex < 0) {
            return;
        }
        var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
        var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
        var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(selectedNode.hierarchyKey);

        var accessControlsDialog = new sas.stress.wkbench.widget.AccessControlsDialog({});
        var view = this.getView();
        view.hierarchiesTreeTable.setBusy(true);
        sas.stress.config.StressAPI.getHierarchyAccessControls(selectedNode.hierarchyKey, selectedNode.liKey).done(function (accessControls) {
            view.hierarchiesTreeTable.setBusy(false);
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            var hierarchyName = selectedNode.name; //rb.getText("hierarchy.name.fmt", [selectedNode.name]);
            accessControlsDialog.setup(hierarchyName, accessControls, "hierarchy");
        }).fail(function (jqxhr) {
            view.hierarchiesTreeTable.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            //accessControlsDialog.setup(selectedNode.name, {});
        });

    },

    hierarchiesTreeTableCollapseButtonPress: function (event) {
        var view = this.getView();

        //view.setBusyIndicatorDelay(1000);
        //view.setBusy(true);

        //setTimeout(function () {
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex < 0) {
            return;
        }
        var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
        var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
        //Collapse all
        for (var i = view.hierarchiesTreeTable.getBinding("rows").getLength() - 1; i >= 0; i--) {
            var nodePath = view.hierarchiesTreeTable.getContextByIndex(i).getPath();
            var node = view.hierarchiesTreeTable.getModel().getProperty(nodePath);
            if (node.hierarchyKey === selectedNode.hierarchyKey && node.children.length > 0) {
                view.hierarchiesTreeTable.collapse(i);
            }
        }
        //     view.setBusy(false);
        // }, 0);
    },

    hierarchiesTreeTableExpandButtonPress: function (event) {
        var view = this.getView();
        // view.setBusyIndicatorDelay(1000);
        // view.setBusy(true);

        //setTimeout(function () {
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex < 0) {
            return;
        }
        var selectedNodePath = view.hierarchiesTreeTable.getContextByIndex(selectedIndex).getPath();
        var selectedNode = view.hierarchiesTreeTable.getModel().getProperty(selectedNodePath);
        //Expand all nodes with children
        for (var i = 0; i < view.hierarchiesTreeTable.getBinding("rows").getLength(); i++) {
            var nodePath = view.hierarchiesTreeTable.getContextByIndex(i).getPath();
            var node = view.hierarchiesTreeTable.getModel().getProperty(nodePath);
            if (node.hierarchyKey === selectedNode.hierarchyKey && node.children.length > 0) {
                view.hierarchiesTreeTable.expand(i);
            }
        }
        //    view.setBusy(false);
        //}, 0);
    },

    hierarchiesTreeTableWorkGroupsButtonPress: function (event) {
        var button = event.getSource();
        var customData = _.find(button.getCustomData(), function (customData) {
            return customData.getKey() === "workGroupNames";
        });
        var workGroupNames = customData? customData.getValue() : [];

        if (workGroupNames && workGroupNames.length > 0) {
            var workgroups = [];
            _.each(workGroupNames, function (name) {
                workgroups.push(new sas.hc.m.Text({ text: name, tooltip: name, wrapping: false }));
            });
        }
        var layout = new sas.hc.ui.layout.VerticalLayout({
            width:"100%",
            content: workgroups
        });

        var popover = new sas.hc.m.ResponsivePopover({
            placement: sap.m.PlacementType.Bottom,
            modal: false,
            showHeader: false,
            content: layout
        }).addStyleClass("hierarchyInfoPopover");
        popover.openBy(button);
        //this.accessControlsDialog = new sas.stress.wkbench.widget.AccessControlsDialog({});
        //this.accessControlsDialog.setup("", {workgroups:workgroups});
    },

    hierarchiesTreeTableInfoButtonPress: function (event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var button = event.getSource();
        var customData = _.find(button.getCustomData(), function (customData) {
            return customData.getKey() === "hierarchyKey";
        });
        var hierarchyKey = customData.getValue();
        var hierarchy = _.find(view.hierarchiesTreeTable.getModel().getData().children, function (hierarchy) {
            return hierarchy.hierarchyKey === hierarchyKey;
        });
        if (hierarchy === undefined) {
            return;
        }

        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();

        var dataLayout = new sas.hc.ui.commons.layout.MatrixLayout({
            layoutFixed: false
        });

        dataLayout.createRow(
            new sas.hc.m.Label({
                text: rb.getText("hierarchiesTreeTable.dateModified.label.txt"),
                design: sap.ui.commons.LabelDesign.Bold
            }),
            new sas.hc.m.Label({
                text: dateTimeFormat.format(new Date(hierarchy.updatedTimestamp))
            })
        );
        dataLayout.createRow(
            new sas.hc.m.Label({
                text: rb.getText("hierarchiesTreeTable.modifiedBy.label.txt"),
                design: sap.ui.commons.LabelDesign.Bold
            }),
            new sas.hc.m.Label({
                text: hierarchy.updatedByUser
            })
        );
        dataLayout.createRow(
            new sas.hc.m.Label({
                text: rb.getText("hierarchiesTreeTable.description.label.txt"),
                design: sap.ui.commons.LabelDesign.Bold
            }),
            new sas.hc.m.Label({
                text: hierarchy.description
            })
        );

        var popover = new sas.hc.m.ResponsivePopover({
            placement: sap.m.PlacementType.Bottom,
            modal: false,
            showHeader: false,
            content: [dataLayout]
        }).addStyleClass("hierarchyInfoPopover");
        popover.openBy(button);
    },

    categoriesListCreateButtonPress: function (event) {
        var view = this.getView();
        var existingDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var addCategoryDialog = sap.ui.jsfragment("sas.stress.wkbench.view.CategoryDialog", this);
        var newCategoryJSON = {
            title: rb.getText("addCategoryDialog.window.title"),
            name: sas.stress.config.model.HierarchyCategory.getNewCategoryName()
        };
        var jsonModel = new sas.stress.wkbench.model.RiskJSONModel(newCategoryJSON);
        addCategoryDialog.setModel(jsonModel);
        addCategoryDialog.open();
    },

    addCategoryDialogSaveButtonPress: function (event) {
        var controller = this;
        var view = this.getView();
        var addCategoryDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        addCategoryDialog.setBusy(true);
        var categoryName = view.byId("CategoryDialog.categoryNameInput").getValue();
        var categoryKey = 0; //New Category
        var newHierarchyCategory = new sas.stress.config.model.HierarchyCategory(categoryKey, categoryName.trim(), 0, false);
        sas.stress.config.model.HierarchyCategory.addHierarchyCategory(newHierarchyCategory).done(function (hierarchyCategory) {
            var categoriesModelData = controller._getCategoriesInModel();
            categoriesModelData.push(hierarchyCategory);
            controller._sortHierarchyCategories(categoriesModelData);
            view.categoryFilter.getModel().refresh();
            //Select the new category
            var categoriesListItems = controller._getCategoriesListItems();
            view.categoryFilter.setSelectedCategory(hierarchyCategory.getName(), true);
            //view.categoryFilter.setSelectedItem(_.find(categoriesListItems, function (category) {
            //    return category.getTitle() === hierarchyCategory.getName();
            //}), true);
            //controller.categoriesListSelectionChange();
            addCategoryDialog.setBusy(false);
            addCategoryDialog.close();
        }).fail(function (jqxhr) {
            addCategoryDialog.setBusy(false);
            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
        });
    },

    categoriesListEditButtonPress: function (event) {
        var view = this.getView();
        var selectedCategory = view.categoryFilter.getSelectedItem();
        if (selectedCategory === null) {
            return;
        }
        var hierarchyCategory = sas.stress.config.model.HierarchyCategory.getHierarchyCategory(this._getSelectedCategoryKey());
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var existingDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var editCategoryDialog = sap.ui.jsfragment("sas.stress.wkbench.view.CategoryDialog", this);
        var hierarchyCategoryJSON = {
            title: rb.getText("editCategoryDialog.window.title"),
            categoryKey: hierarchyCategory.getCategoryKey(),
            name: hierarchyCategory.getName()
        };
        var jsonModel = new sas.stress.wkbench.model.RiskJSONModel(hierarchyCategoryJSON);
        editCategoryDialog.setModel(jsonModel);
        editCategoryDialog.open();
    },

    editCategoryDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var editCategoryDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        var categoryKey = editCategoryDialog.getModel().getData().categoryKey;
        var categoryName = view.byId("CategoryDialog.categoryNameInput").getValue();
        var hierarchyCategory = new sas.stress.config.model.HierarchyCategory(categoryKey, categoryName.trim(), 0, false);
        editCategoryDialog.setBusy(true);
        sas.stress.config.model.HierarchyCategory.updateHierarchyCategory(hierarchyCategory).done(function (updatedHierarchyCategory) {
            //Update the hierarchy Category in the model.
            var hierarchyCategory = _.find(controller._getCategoriesInModel(), function (category) {
                return category.getCategoryKey() === updatedHierarchyCategory.getCategoryKey();
            });
            if (hierarchyCategory) {
                hierarchyCategory.name = updatedHierarchyCategory.getName();
                controller._sortHierarchyCategories(controller._getCategoriesInModel());
                view.categoryFilter.getModel().refresh();
                //Select the new category
                view.categoryFilter.setSelectedCategory(hierarchyCategory.getName(), true);
                //view.categoryFilter.setSelectedItem(_.find(controller._getCategoriesListItems(), function (category) {
                //    return category.getTitle() === hierarchyCategory.getName();
                //}));
            }
            editCategoryDialog.setBusy(false);
            editCategoryDialog.close();
        }).fail(function (jqxhr) {
            editCategoryDialog.setBusy(false);
            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
        });
    },

    /* Called when the user enters a value in the Name field in the Edit Category dialog. Validates the name by
     * checking if it is empty or duplicate of an existing category.
     * If validation fails, displays an error message around the Name field and disables the Save button. */
    handleCategoryNameInputChange: function (event) {
        var view = this.getView();
        var editCategoryDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        var categoryKey = editCategoryDialog.getModel().getData().categoryKey;
        var categoryBeingEdited = sas.stress.config.model.HierarchyCategory.getHierarchyCategory(categoryKey);
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;
        var nameInput = event.getSource();
        var newName = nameInput.getValue().trim();
        //Check if name is empty.
        if (newName === null || newName === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            //Check if hierarchy category with this name already exists. Hierarchy Category name can be = name of the hierarchy being edited.
            var hierarchyCategory = _.find(this._getCategoriesInModel(), function (hierarchyCategory) {
                var found = (hierarchyCategory.getName() !== null && hierarchyCategory.getName().toLowerCase() === newName.toLowerCase());
                return found && hierarchyCategory.categoryKey !== categoryKey;
            });
            if (hierarchyCategory) {
                isValid = false;
                nameInput.setValueStateText(rb.getText("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        //Toggle Save button based on the result of validation.
        view.byId("CategoryDialog.saveCategoryButton").setEnabled(isValid);
    },

    handleCategoryDialogSaveButtonPress: function (event) {
        var isEdit = this._isEditCategory();
        if (isEdit) {
            this.editCategoryDialogSaveButtonPress(event);
        } else {
            this.addCategoryDialogSaveButtonPress(event);
        }
    },

    categoriesListDeleteButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var selectedCategory = view.categoryFilter.getSelectedCategory();
        if (!selectedCategory) {
            return;
        }
        var selectedCategoryKey = controller._getSelectedCategoryKey();
        //Check if there are hierarchies involving this category
        var hierarchies = _.filter(view.hierarchiesTreeTable.getModel().getData().children, function (hierarchy) {
            return hierarchy.categoryKey === selectedCategoryKey;
        });
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var message = rb.getText("categoriesList.delete.category.confirmation.msg.fmt.txt", [selectedCategory]);
        if (hierarchies.length > 0) {
            message += "\n" + rb.getText("categoriesList.hierarchiesWillBeDeleted.msg.fmt.txt", [hierarchies.length]);
        }
        sas.hc.m.MessageBox.warning(message, rb.getText("categoriesList.delete.category.window.title"),
                                    [sas.hc.m.MessageBox.Action.DELETE, sas.hc.m.MessageBox.Action.CANCEL], null, function (oAction) {
                if (oAction === sas.hc.m.MessageBox.Action.DELETE) {
                    sas.stress.config.model.HierarchyCategory.deleteHierarchyCategory(selectedCategoryKey)
                       .done(function () {
                           var categoriesListModel = view.categoryFilter.getModel();
                           $.each(categoriesListModel.getData().categories, function (index, category) {
                               if (category.getCategoryKey() === selectedCategoryKey) {
                                   categoriesListModel.getData().categories.splice(index, 1);
                                   return false; //break for loop
                               }
                           });
                           categoriesListModel.refresh();
                           var categoriesListItems = controller._getCategoriesListItems();
                           if (categoriesListItems.length > 0) {
                               view.categoryFilter.setSelectedItem(categoriesListItems[0], true);
                               //controller.categoriesListSelectionChange();
                           }
                           //Refresh list of hierarchies and pairs in cache if hierarchies were also deleted and set hierarchies count in the categories list.
                           if (hierarchies.length > 0) {
                               controller._getHierarchies().done(function () {
                                   //Create hierarchy called Entity in the Organization category if the Organization category exists.
                                   //controller._createHierarchyEntity();
                                   controller._setHierarchiesCountInCategoriesList();
                               });
                               controller._getHierarchyMapPairs();
                           }

                           // Need to update the worksheet template table whenever a hierarchy is updated
                           // Using the same eventbusconstant as HIERARCHY_DELETED as in both cases we have to refresh the sheet template
                           sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.HIERARCHY_DELETED, {});

                       }).fail(function (jqxhr) {
                        sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                    });
                }
            }
        );
    },

    _savePreferredCategory: function (category) {
        if (category === undefined) {
            var view = this.getView();
            category = view.categoryFilter.getSelectedCategory();
        }
        if (category) {
            sas.stress.wkbench.UserPrefs.save("sas.stress.config.hierarchyCategory", category);
        } else {
            sas.stress.wkbench.UserPrefs.delete("sas.stress.config.hierarchyCategory", category);
        }
    },

    _selectPreferredCategory: function () {
        var view = this.getView();
        var preferredCategory = sas.stress.wkbench.UserPrefs.get("sas.stress.config.hierarchyCategory");
        view.categoryFilter.setSelectedCategory(preferredCategory, true);
    },

    _getCategoriesListItems: function () {
        var view = this.getView();
        return view.categoryFilter.getListItems();
    },

    _getCategoriesInModel: function () {
        var view = this.getView();
        return view.categoryFilter.getModel().getData().categories;
    },

    _getCategoryDialog: function () {
        return this.getView().byId("sas.stress.wkbench.view.CategoryDialog");
    },

    _isEditCategory: function () {
        var dialog = this._getCategoryDialog();
        var categoryKey = dialog ? dialog.getModel().getData().categoryKey : null;
        return (!!categoryKey);
    },

    _getHierarchyDialog: function () {
        return sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
    },

    _isEditHierarchy: function () {
        var hierarchyDialog = this._getHierarchyDialog();
        var hierarchyKey = hierarchyDialog ? hierarchyDialog.getModel().getData().hierarchyKey : null;
        return (!!hierarchyKey);
    },

    _getHierarchyDialogTitle: function () {
        var hierarchyDialog = this._getHierarchyDialog();
        return hierarchyDialog ? hierarchyDialog.getModel().getData().title : "";
    }
});