jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.Input");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.ui.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.ui.table.TreeTable");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.m.slider.HorizontalSlider");

sap.ui.jsfragment("sas.stress.config.view.RefineHierarchiesDialog", {
    createContent: function (controller) {
        //var view = this.getView();
        // var memberList = this.getModel().getData().selectedMemberList;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var horizontalSlider = new sas.hc.m.slider.HorizontalSlider({
            id: this.createId("refineHierarchiesDialog.horizontalSlider"),
            width: "100%",
            sliderLabel: rb.getText("refineHierarchyDetailsDialog.horizontalSlider.title"),
            min: 0,
            thumbRanges: 1,
            smallStepWidth: 1,
            showValueLabel: false,
            showStepLabels: true,
            showInput: false,
            enabled: true,
            visible: (controller.globalAxis !== controller.HIDDEN),
            formatCallback: generateLabel,
            liveChange: function (e) {
                var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
                if ( capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE)) {
                    updateTreeTable(this.getValues()[0], this.getValues()[1]);
                    controller.checkAndEnableRefineSaveButton();
                }
            }

            //   $.proxy(controller.refineHierarchiesSliderChange, controller)
        });

        horizontalSlider.bindProperty("labels", "/levelNames");
        horizontalSlider.bindProperty("max", "/totalLevelNames");
        //horizontalSlider.setValues([horizontalSlider.getMin(),horizontalSlider.getMax()]);
        horizontalSlider.setValues([controller.globalSelectedNode.startLevel - 1, controller.globalSelectedNode.endLevel - 1]);
        horizontalSlider.bindProperty("totalUnits", "/totalLevelNames");

        //horizontalSlider.hasPopin=function(){return false;};
        //horizontalSlider.getAllowReflow=function(){return false;};

        function updateTreeTable(min, max) {
            // Update the tree table based on the selection on the horizontal slider
            var table = sap.ui.getCore().byId("refineHierarchiesDialog.sourceHierarchiesTable");
            var model = table.getModel();

            var treeData = model.getData();

            var items = [];
            traverseTree(treeData, min + 1, max + 1, items);

            model.setProperty("/children", []);
            model.setProperty("/children/0", items[0]);
        }

        function traverseTree(treeData, min, max, items) {
            if (treeData.levelNum !== 'undefined' &&
                treeData.levelNum !== '-99' &&
                ((treeData.levelNum < min) || (treeData.levelNum > max))) {
                treeData.enabled = false;
                treeData.checked = false;
                checkSelectedMemberList(treeData);
            }

            if (treeData.levelNum !== 'undefined' &&
                treeData.levelNum !== '-99' &&
                ((treeData.levelNum >= min) && (treeData.levelNum <= max))) {
                treeData.enabled = true;
                checkSelectedMemberList(treeData);
            }
            if (treeData.children.length > 0) {
                _.each(treeData.children, function (data) {
                    if (items.length === 0) {
                        items.push(data);
                    }
                    traverseTree(data, min, max, items);
                });
            }
        }

        function generateLabel(value) {
            //sap.ui.getCore().byId("refineHierarchiesDialog.saveButton").setEnabled(true);
            var labelArr = horizontalSlider.getLabels();
            if (!labelArr || value >= labelArr.length) {
                return value;
            }
            return (labelArr[value]);
        }

        function setChildState(obj, state) {
            //Build list of checked members
            checkSelectedMemberList(obj);
            //obj.checked = state;
            obj.checked = (state && obj.enabled);


            getChildren(obj).forEach(function (x) {
                //x.checked = state;
                x.checked = (state && x.enabled);
                setChildState(x, state);
            });

        }

        function checkSelectedMemberList(obj) {
            if (obj.checked && obj.enabled) {
                if (obj.name !== undefined) {
                    var item = controller.globalSelectedMemberList.indexOf(obj.liKey);
                    if (item === -1 && (obj.liKey !== undefined)) {
                        controller.globalSelectedMemberList.push(obj.liKey);
                    }
                }
            } else {
                if (obj.name !== undefined) {
                    removeItem(controller.globalSelectedMemberList, obj.liKey);
                }
            }
        }

        function removeItem(arr) {
            var what, a = arguments, L = a.length, ax;
            while (L > 1 && arr.length) {
                what = a[--L];
                while ((ax = arr.indexOf(what)) !== -1) {
                    arr.splice(ax, 1);
                }
            }
            return arr;
        }

        function validateChild(model, path) {
            var cur = model.getProperty(path);
            setChildState(cur, cur.checked);
        }

        function getChildren(obj) {
            var children = [];
            Object.getOwnPropertyNames(obj).forEach(function (x) {
                //DD Code added
                //if (x == "children" && obj.children.length > 0) {
                //    children.push(obj[x]);
                //}
                if (typeof obj[x] === 'object') {
                    children.push(obj[x]);
                }

            });
            return children;
        }

        function validateParent(model, path) {
            if (path === '/children') {
                return;
            }
            var obj = model.getProperty(path);
            obj.checked = (true === obj.enabled);

            //var children = getChildren(obj);
            //
            //var selectedCount = children.filter(function(x) {
            //    return x.checked == true;
            //}).length;
            //
            //if (selectedCount === children.length) {
            //    obj.checked = true;
            //} else {
            //    var unselectedCount = children.filter(function(x) {
            //        return x.checked == false;
            //    }).length;
            //
            //    if (unselectedCount) {obj.checked = false; }
            //}
            checkSelectedMemberList(obj);

            model.setProperty(path, obj);
            path = path.substring(0, path.lastIndexOf('/'));
            if (path !== '/children/0') {
                validateParent(model, path);
            }
        }

        // var hierarchyDesc = new sas.hc.m.Label({text: rb.getText("refineHierarchyDetailsDialog.column.name.header.title")});

        var table = new sas.hc.ui.table.TreeTable({
            //visibleRowCount: 10,
            id: this.createId("refineHierarchiesDialog.sourceHierarchiesTable"),
            //width: "90%",
            width: "40rem",
            toolbar: _createRefineTableToolbar(controller),
            noData: rb.getText("refineHierarchyDetailsDialog.noDataText"),
            //visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            columns: [
                new sas.hc.ui.table.Column({
                    label: new sas.hc.m.Label({ text: rb.getText("refineHierarchyDetailsDialog.column.name.header.title") }),
                    template: new sas.hc.m.CheckBox({
                        text: '{name}',
                        //selectionState: '{checked}',
                        selected: '{checked}',
                        enabled: '{enabled}',
                        select: function (e) {
                            var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
                            if ( capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE)) {
                                controller.checkAndEnableRefineSaveButton();
                                var cxt = e.getSource().getBindingContext();
                                var path = cxt.getPath();
                                validateChild(this.getModel(), path);

                                path = path.substring(0, path.lastIndexOf('/'));
                                validateParent(this.getModel(), path);
                            }
                        }
                    })
                }),
                new sas.hc.ui.table.Column({
                    label: new sas.hc.m.Label({ text: rb.getText("refineHierarchyDetailsDialog.column.level.header.title") }),
                    template: new sas.hc.m.Text({
                        text: '{levelName}',
                        tooltip: '{levelName}',
                        //enabled: '{enabled}',
                        wrapping: false
                    }),
                    width: "10rem"
                })
            ],
            //selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionMode: sap.ui.table.SelectionBehavior.Single,
            //selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly
            selectionBehavior: sap.ui.table.SelectionBehavior.Row
            //expandAll : true,
        });

        table.bindRows("/children");

        // HF 2 changes--
        var showSubtotalBox = new sas.hc.m.CheckBox({
            id: this.createId("refineHierarchiesDialog.show_subtotals"),
            text: rb.getText("sheetTemplate.axis.include.subtotals.txt"),
            select: $.proxy(controller._refineDialogSubtotalSelect, controller),
            selected: { // if the value is YES_SUBTOTALS, the box is selected by default
                path:"/subtotalCode",
                formatter:function(subtotalCode){
                    return subtotalCode==="YES_SUBTOTALS";
                }
            },
            visible:{   // only visible if refining on xAxis
                path: "/axis",
                formatter:function(axis){
                    return "xAxis"===axis;
                }
            }
        }).addStyleClass("stressEditProjectRequireCommentsCheckBox");
        // HF 2 changes---

        function _createRefineTableToolbar(controller) {
            return new sas.hc.m.Toolbar({
                content: [
                    // this.searchField,
                    new sas.hc.m.ToolbarSpacer(),
                    new sas.hc.m.Button({
                        id: "refineTreeTableExpandButton",
                        icon: sas.icons.HC.SHOWALL,
                        tooltip: rb.getText("sheetTemplate.hierarchyTreeTable.expandCollapseButton.tip"),
                        press: $.proxy(controller.refineTreeTableExpandButtonPress, controller)
                    }),
                    new sas.hc.m.Button({
                        id: "refineTreeTableCollapseButton",
                        icon: sas.icons.HC.HIDEALL,
                        tooltip: rb.getText("sheetTemplate.hierarchyTreeTable.collapseButton.tip"),
                        press: $.proxy(controller.refineTreeTableCollapseButtonPress, controller)
                    })
                ]
            });
        }

        function _createRefineTimePeriodToolbar(controller) {
            var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
            return new sas.hc.ui.layout.HorizontalLayout({
                content: [
                    new sas.hc.m.Input({
                        id: "refineDialog.historyPeriodInput",
                        label: rb.getText("addTimeOptionsDialog.historyPeriod.control.label"),
                        labelPosition: sas.hc.m.InputLabelPosition.Top,
                        value: "{/numHistoryPeriods}",
                        type: sap.m.InputType.Text,
                        // required: true,
                        disableValidation: false,
                        enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE),
                        maxLength: 32,
                        // margin: '5 10 0 5',
                        liveChange: $.proxy(controller.refineHistoryPeriodInputLiveChange, controller),
                        visible: (controller.globalSelectedNode.code === controller.TIME_HIERARCHY_CODE)
                        //"aria-required":true
                    }),
                    new sas.hc.m.ToolbarSpacer(),
                    new sas.hc.m.Input({
                        id: "refineDialog.futurePeriodInput",
                        label: rb.getText("addTimeOptionsDialog.futurePeriod.control.label"),
                        labelPosition: sas.hc.m.InputLabelPosition.Top,
                        value: "{/numFuturePeriods}",
                        type: sap.m.InputType.Text,
                        //required: true,
                        disableValidation: false,
                        enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE),
                        maxLength: 32,
                        //margin: '5 0 0 10',
                        liveChange: $.proxy(controller.refineFuturePeriodInputLiveChange, controller),
                        visible: (controller.globalSelectedNode.code === controller.TIME_HIERARCHY_CODE)
                        //"aria-required":true
                    }).addStyleClass("stressRefineWorkCategoriesFutureTimePeriod")
                ]
            }).addStyleClass("stressRefineWorksheetCategories");
        }

        //function _createSearchField(controller){
        //    return new sas.hc.m.SearchField({
        //        id: "refineHierarchiesDialog.searchField",
        //        placeholder:rb.getText("sheetTemplate.refine.search.placeholder.txt"),
        //        liveChange:$.proxy(controller.refineListLiveChange, controller),
        //        visible: true,  // temporary, till we get it to work
        //        search:$.proxy(controller.refineListLiveChange, controller)
        //    });
        //}

        // HF2 changes---
        var verticalLayout;
        if (controller.globalSelectedNode.code === controller.TIME_HIERARCHY_CODE) {
            verticalLayout=new sas.hc.ui.layout.VerticalLayout({
                content: [horizontalSlider, //hierarchyDesc,
                    table, _createRefineTimePeriodToolbar(controller),
                    showSubtotalBox] //, selectedMemberTable]
            });
        }else{
            verticalLayout=new sas.hc.ui.layout.VerticalLayout({
                content: [
                    horizontalSlider,
                    table,
                    showSubtotalBox
                ]
            });
        }
        // HF2 changes---

        var dialog = new sas.hc.m.Dialog({
            id: this.createId("sas.stress.config.view.RefineHierarchiesDialog"),
            //modal: true,
            title: rb.getText("refineHierarchyDetailsDialog.window.title"),
            //initialFocus: searchField,

            content: verticalLayout,
            beginButton: new sas.hc.m.Button({
                id: this.createId("refineHierarchiesDialog.saveButton"),
                text: rb.getText("button.save.action.label"),
                press: $.proxy(controller.refineHierarchiesDialogSaveButtonPress, controller),
                enabled: false //will be enabled when a hierarchy is selected in the table.
            }),
            endButton: new sas.hc.m.Button({
                text: rb.getText("button.cancel.action.label"),
                press: function () {
                    dialog.close();
                }
            }),
            afterClose: function () {
                dialog.destroy();
            }
        });

        return dialog;
    }
});
