jQuery.sap.require("sap.ui.core.Orientation");
jQuery.sap.require("sap.ui.core.format.DateFormat");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.FlexItemData");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.ui.layout.Splitter");
jQuery.sap.require("sas.hc.ui.layout.SplitterLayoutData");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require({ modName: "sas.stress.config.view.SheetTemplateManager", type: "controller" });
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.stress.wkbench.widget.CategoryFilter");
jQuery.sap.require("sas.stress.wkbench.widget.KeyedListItem");
jQuery.sap.require("sas.stress.wkbench.widget.LifecycleTable"); // SOPHIE TODO: why even use this?
jQuery.sap.require("sas.stress.wkbench.widget.StressList");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");

sap.ui.jsview("sas.stress.config.view.SheetTemplateManager", {

    getControllerName: function () {
        return "sas.stress.config.view.SheetTemplateManager";
    },

    createContent: function (controller) {
        this.setHeight("100%");
        this.capabilityManager=sas.stress.wkbench.CapabilityManager.getInstance();

        var categoriesPanel = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding(
            {
                content: this._createCategoryFilter(controller)
            }
        ).addStyleClass("riskFacetPanel");

        this.sheetTemplateTable = this.createSheetTemplatesTable(controller);

        var sheetPanel = sas.risk.common.widgets.WidgetFactory.buildPanel(
            {
                rightPadding: 0,
                content: this.sheetTemplateTable
            }
        );

        categoriesPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
            size: "225px",
            minSize: 150
        }));

        return new sas.hc.ui.layout.Splitter({
            contentAreas: [
                sheetPanel,
                categoriesPanel
            ],
            orientation: sap.ui.core.Orientation.Horizontal,
            width: "100%",
            height: "100%"
        });

    },

    _createCategoryFilter: function (controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        this.categoryFilter = new sas.stress.wkbench.widget.CategoryFilter({
            title: "{i18N>categories.header.title}",
            actionButtons: [
                new sas.hc.m.Button({
                    icon: sas.icons.HC.NEWSTARBURST,
                    tooltip: "{i18N>sheetTemplates.categories.newButton.tip}",
                    enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.CREATE),
                    press: $.proxy(controller.pressedCreateSheetCategory, controller)
                }),
                new sas.hc.m.Button({
                    icon: sas.icons.HC.EDIT,
                    tooltip: "{i18N>sheetTemplates.categories.renameButton.tip}",
                    enabled: {
                        path: "/selectedSheetCategory",
                        formatter: function (selectedCategory) {
                            return selectedCategory && selectedCategory !== rb.getText("categories.all.txt") &&
                                capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE);
                        }
                    },
                    press: $.proxy(controller.pressedEditSheetCategory, controller)
                }),
                new sas.hc.m.Button({
                    icon: sas.icons.HC.DELETE,
                    tooltip: "{i18N>sheetTemplates.categories.deleteButton.tip}",
                    enabled: {
                        path: "/selectedSheetCategory",
                        formatter: function (selectedCategory) {
                            return selectedCategory && selectedCategory !== rb.getText("categories.all.txt") &&
                                capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.DELETE);
                        }
                    },
                    press: $.proxy(controller.pressedDeleteSheetCategory, controller)
                })
            ],
            itemsPath: "/categories",
            itemTemplate: new sas.stress.wkbench.widget.KeyedListItem({
                title: "{label}",
                counter: "{count}",
                key: "{id}"
            }),
            selectedCategory: "{/selectedSheetCategory}",
            selectionChange: $.proxy(controller.pressedSelectSheetCategory, controller)
        });

        return this.categoryFilter;
    },

    _createSheetTableToolbar: function (controller) {
        this.searchField = this._createSearchField(controller);
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        return new sas.hc.m.Toolbar({
            content: [
                this.searchField,
                new sas.hc.m.ToolbarSpacer(),
                this.createNewSheetButton = new sas.hc.m.Button({
                    icon: sas.icons.HC.NEWSTARBURST,
                    tooltip: "{i18N>sheetTemplates.templates.newButton.tip}",
                    enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.CREATE),
                    press: $.proxy(controller.pressedCreateSheetTemplate, controller)
                }),
                this.editSheetButton = new sas.hc.m.Button({
                    icon: sas.icons.HC.EDIT,
                    tooltip: "{i18N>sheetTemplates.templates.editButton.tip}",
                    enabled: false,
                    //enabled: {
                    //    path: "/selectedSheetTemplates",
                    //    formatter: function (selectedItems) {
                    //        return selectedItems.length === 1 &&
                    //            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.READ);
                    //    }
                    //},
                    press: $.proxy(controller.pressedEditSheetTemplate, controller)
                }),
                this.permissionsSheetButton = new sas.hc.m.Button({
                    icon: sas.icons.HC.PERMISSIONSINSPECTOR,
                    tooltip: "{i18N>sheetTemplates.workGroupButton.tip}",
                    enabled: false,
                    //enabled: {
                    //    path: "/selectedSheetTemplates",
                    //    formatter: function (selectedItems) {
                    //        return selectedItems.length === 1 &&
                    //            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE);
                    //    }
                    //},
                    press: $.proxy(controller.sheetTemplateAccessControlsButtonPress, controller)
                }),
                this.refreshSheetButton = new sas.hc.m.Button({
                    icon: sas.icons.HC.REFRESH,
                    tooltip: "{i18N>button.refresh.action.label}",
                    enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.READ),
                    press: $.proxy(controller.refresh, controller)
                }),
                this.deleteSheetButton = new sas.hc.m.Button({
                    icon: sas.icons.HC.DELETE,
                    tooltip: "{i18N>sheetTemplates.templates.deleteButton.tip}",
                    enabled: false,
                    //enabled: {
                    //    path: "/selectedSheetTemplates",
                    //    formatter: function (selectedItems) {
                    //        return selectedItems.length > 0 &&
                    //            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.DELETE);
                    //    }
                    //},
                    press: $.proxy(controller.pressedDeleteSheetTemplate, controller)
                })
            ]
        });

    },

    _createSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            placeholder: "{i18N>sheetTemplates.categories.search.placeholder.txt}",
            liveChange: $.proxy(controller.filterSheetTemplates, controller),
            search: $.proxy(controller.filterSheetTemplates, controller)
        });
    },

    createSheetTemplatesTable: function (controller) {
        this.table = new sas.stress.wkbench.widget.LifecycleTable({
            width: "100%",
            editable: false,
            showTableOptions: false,
            visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionBehavior: sap.ui.table.SelectionBehavior.Row,
            selectionMode: sap.ui.table.SelectionMode.MultiToggle,
            rowSelectionChange: $.proxy(controller.onRowSelectionChange, controller),
            selectedIndices: "{/selectedSheetTemplates}",
            toolbar: this._createSheetTableToolbar(controller)
        });
        var catIdView = new sas.hc.m.Text({ wrapping: false, tooltip: "{catId}" }).bindProperty("text", "catId");
        this.table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("catId"),
            template: catIdView,
            sortProperty: "catId",
            visible: false
        }));

        var nameView = new sas.hc.m.Text({ wrapping: false, tooltip: "{name}" }).bindProperty("text", "name");
        this.table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("name"),
            label: new sas.hc.m.Label({ text: "{i18N>sheetTemplates.templates.column.name.header.title}" }),
            template: nameView,
            sortProperty: "name",
            width: "15%"
        }));

        var versionView = new sas.hc.m.Text({ wrapping: false, tooltip: "{version}" }).bindProperty("text", "version");
        this.table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("version"),
            label: new sas.hc.m.Label({ text: "{i18N>sheetTemplates.templates.column.version.header.title}" }),
            template: versionView,
            sortProperty: "version",
            width: "15%"
        }));

        var descView = new sas.hc.m.Text({ wrapping: false, tooltip: "{description}" }).bindProperty("text", "description");
        this.table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("description"),
            label: new sas.hc.m.Label({ text: "{i18N>sheetTemplates.templates.column.description.header.title}" }),
            template: descView,
            sortProperty: "description",
            width: "15%"
        }));

        var lmuView = new sas.hc.m.Text({ wrapping: false, tooltip: "{lastModifiedUser}" }).bindProperty("text", "lastModifiedUser");
        this.table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("lastModifiedUser"),
            label: new sas.hc.m.Label({ text: "{i18N>sheetTemplates.templates.column.lastModifiedUser.header.title}" }),
            template: lmuView,
            sortProperty: "lastModifiedUser",
            width: "10%"
        }));

        this._addTimeColumn("lastModifiedTimestamp", this.table);

        this._addTxtColumn("history");
        this._addTxtColumn("future");
        this.table.bindRows("/templates");

        //S1256573: Default sort by Name/Version asc.
        var bindingInfo = this.table.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [new sap.ui.model.Sorter("name", false), new sap.ui.model.Sorter("version", false)];
            bindingInfo.sorter = sorters;
        }

        return this.table;
    },

    _addTxtColumn: function (columnName) {
        //var view = new sas.hc.m.Text({ wrapping: false, tooltip: "{columnName}" }).bindProperty("text", columnName);
        this.table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId(columnName),
            label: new sas.hc.m.Label({ text: "{i18N>sheetTemplates.templates.column." + columnName + ".header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false, tooltip: "{columnName}" }).bindProperty("text", columnName),
            sortProperty: columnName,
            width: "12%"
        }));
    },
    _addTimeColumn: function (columnName, table) {
        var view = new sas.hc.m.Text({
            wrapping: false,
            tooltip: {
                path: columnName,
                formatter: function (dttm) {
                    var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                    return dateTimeFormat.format(new Date(dttm));
                }
            },
            text: {
                path: columnName,
                formatter: function (dttm) {
                    var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                    return dateTimeFormat.format(new Date(dttm));
                }
            }
        });
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId(columnName),
            label: new sas.hc.m.Label({ text: "{i18N>sheetTemplates.templates.column." + columnName + ".header.title}" }),
            template: view,
            sortProperty: columnName ,
            width: "15%"
        }));
    }
});
