jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.data.DataAPI");

sap.ui.controller("sas.stress.data.view.LoadDataEditor", {
    onBeforeRendering: function () {
        var view = this.getView();
        var controller = this;
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.data.messagebundle" });
        view.setModel(i18N, "i18N");
        var model = new sas.stress.wkbench.model.RiskJSONModel();
        view.setModel(model);

        // True if we are importing project-specific data, false otherwise. If we are importing project-specific data,
        // then we need to display the list of available projects to the user.
        this.projectMode = view.getViewData().projectMode;

        //Get list of files on server.
        this._getFilesOnServer();

        if (true === this.projectMode) {
            // JSZ: view will set visible based on projectMode
            //view.projectSelectLabel.setProperty("visible", true);
            //view.projectSelect.setProperty("visible", true);

            //Get projects
            var getProjects = this._getProjects();
            $.when(getProjects).done(function (projects) {
                view.getModel().setProperty("/projects", projects);

                if (projects.length > 0) {
                    view.projectSelect.setSelectedIndex(0);
                }
                // load the landing area data for the project selected in the select.
                controller._getLandingAreaFiles();
            });
        }
    },

    _getFilesOnServer: function () {
        var view = this.getView();
        sas.stress.data.DataAPI.listServerFiles().done(function (files) {
            _.each(files,function(file){
                // update the name to be the typelessSurrogatePath
                file.actualName=file.name;
                file.name=file.typelessSurrogatePath;
            });
            var sortedFiles =
                files.sort(function (a, b) {
                    var aName = a.name.toLowerCase();
                    var bName = b.name.toLowerCase();
                    return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
                });
            if (sortedFiles.length === 1 ) {
                sortedFiles[0].selected = true;
            }
            view.getModel().setProperty("/files", sortedFiles);
        }).fail(function (jqxhr) {
            //display error msg.
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
            if (jqxhr && jqxhr.responseJSON) {
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            } else {
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            }
        });
    },

    _getLandingAreaFiles: function () {
        var controller = this;
        var view = this.getView();
        // Ensure the project select exists (it is not visible in global mode)
        if (view.projectSelect && view.projectSelect.getSelectedKey()) {
            var projectKey = view.projectSelect.getSelectedKey();
            $.when(sas.stress.data.DataAPI.listLandingAreaFiles(projectKey)).done(function (landingAreaFiles) {
                _.each(landingAreaFiles,function(file){
                    // update the name to be the typelessSurrogatePath
                    file.actualName=file.name;
                    file.name=file.typelessSurrogatePath;
                });

                var fileSort = function (a, b) {
                    var aName = a.name.toLowerCase();
                    var bName = b.name.toLowerCase();
                    return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
                };

                var sortedLandingAreaFiles = landingAreaFiles.sort(fileSort);
                if (sortedLandingAreaFiles.length === 1 ) {
                    sortedLandingAreaFiles[0].selected = true;
                }
                view.getModel().setProperty("/landingAreaFiles", sortedLandingAreaFiles);
                controller._toggleOKButton();
            }).fail(function (jqxhr) {
                //display error msg.
                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                if (jqxhr && jqxhr.responseJSON) {
                    sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                } else {
                    sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                }
            });
        } else {
            view.getModel().setProperty("/landingAreaFiles", []);
        }
    },

    _getProjects: function (event) {
        var deferred = $.Deferred();
        var view = this.getView();
        view.setBusy(true);
        sas.stress.wkbench.model.Project.getProjects().done(function (projects) {
            var sortedProjects =
                projects.sort(function (a, b) {
                    var aName = a.getName().toLowerCase();
                    var bName = b.getName().toLowerCase();
                    return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
                });
            deferred.resolve(sortedProjects);
            view.setBusy(false);
        }).fail(function (jqxhr) {
            view.setBusy(false);
            //display error msg.
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
            if (jqxhr && jqxhr.responseJSON) {
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            } else {
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            }
        });
        return deferred;
    },

    cancelButtonPress: function () {
        var view = this.getView();
        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "CANCEL" });
    },

    okButtonPress: function () {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");

        // Are we loading project-specific data?
        if (true === this.projectMode) {
            var projectKey = view.projectSelect.getSelectedKey();
            // Was a file selected for upload?
            if (view.radioButtonGroup.getSelectedIndex() === 0) {
                if (view.globalDataFileUploader.getValue()) { //File selected?
                    this.busyDialog = new sas.hc.m.BusyDialog({
                        title: rb.getText("importDialog.busyDialog.title")
                    });
                    this.busyDialog.open();
                    view.globalDataFileUploader.setUploadUrl("rest/projects/" + projectKey + "/projectData");
                    view.globalDataFileUploader.upload();
                }
            } else if (view.radioButtonGroup.getSelectedIndex() === 1) {

                var selectedIndex = view.globalDataList.getSelectedIndex();
                var selectedItem = view.globalDataList.getContextByIndex(selectedIndex);
                var fileId = "";
                var obj = selectedItem.getObject();
                if (obj !== undefined) {
                    fileId = obj.id || obj.getProperty("key");
                }
                if (fileId !== "") {
                    sas.stress.data.DataAPI.uploadProjectDataFromServer(projectKey, fileId).done(function (response) {
                        view.fireEvent(view.getId() + "_DATA_LOADED", { "view": view, "action": "FUTURE_DATA_LOADED" });
                        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
                    }).fail(function (jqxhr) {
                        //display error msg.
                        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                        if (jqxhr && jqxhr.responseJSON) {
                            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                        } else {
                            sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                        }
                    });
                }
            } else if (view.radioButtonGroup.getSelectedIndex() === 2) {

                var selectedIndex = view.landingAreaDataList.getSelectedIndex();
                var selectedItem = view.landingAreaDataList.getContextByIndex(selectedIndex);
                var fileId = "";
                var obj = selectedItem.getObject();
                if (obj !== undefined) {
                    fileId = obj.id || obj.getProperty("key");
                }
                if (fileId !== "") {
                    view.getParent().setBusy(true);
                    sas.stress.data.DataAPI.uploadProjectDataFromLandingArea(projectKey, fileId).done(function (response) {
                        view.getParent().setBusy(false);
                        view.fireEvent(view.getId() + "_DATA_LOADED", { "view": view, "action": "FUTURE_DATA_LOADED" });
                        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
                    }).fail(function (jqxhr) {
                        view.getParent().setBusy(false);
                        //display error msg.
                        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                        if (jqxhr && jqxhr.responseJSON) {
                            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                        } else {
                            sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                        }
                    });
                }
            }
        } else {
            // Was a file selected for upload?
            if (view.radioButtonGroup.getSelectedIndex() === 0) {
                if (view.globalDataFileUploader.getValue()) { //File selected?
                    this.busyDialog = new sas.hc.m.BusyDialog({
                        title: rb.getText("importDialog.busyDialog.title")
                    });
                    this.busyDialog.open();
                    view.globalDataFileUploader.setUploadUrl("rest/config/globalData");
                    view.globalDataFileUploader.upload();
                }
            } else if (view.radioButtonGroup.getSelectedIndex() === 1) {
                var selectedIndex = view.globalDataList.getSelectedIndex();
                var selectedItem = view.globalDataList.getContextByIndex(selectedIndex);
                var fileId = "";
                var obj = selectedItem.getObject();
                if (obj !== undefined) {
                    fileId = obj.id || obj.getProperty("key");
                }
                if (fileId !== "") {
                    view.getParent().setBusy(true);
                    //view.setBusy(true);
                    sas.stress.data.DataAPI.uploadGlobalDataFromServer(fileId).done(function (response) {
                        view.getParent().setBusy(false);
                        view.fireEvent(view.getId() + "_DATA_LOADED", {
                            "view": view,
                            "action": "HISTORICAL_DATA_LOADED"
                        });
                        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
                    }).fail(function (jqxhr) {
                        view.getParent().setBusy(false);
                        //display error msg.
                        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                        if (jqxhr && jqxhr.responseJSON) {
                            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                        } else {
                            sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                        }
                    });
                }
            }
        }
    },

    locationRadioButtonSelect: function (event) {
        var view = this.getView();
        var selectedIndex = event.getParameter("selectedIndex");
        var showLocal = false;
        var showServer = false;
        var showLandingArea = false;
        if (0 === selectedIndex) {
            showLocal = true;
        } else if (1 === selectedIndex) {
            showServer = true;
        } else if (2 === selectedIndex) {
            showLandingArea = true;
        }
        view.globalDataFileUploader.setVisible(showLocal);
        //view.searchField.setVisible(showServer);
        view.globalDataList.setVisible(showServer);
        //view.landingAreaSearchField.setVisible(showLandingArea);
        view.landingAreaDataList.setVisible(showLandingArea);
        this._toggleOKButton();
    },

    globalDataFileUploadComplete: function (event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var response = event.getParameter("response");
        if (this.busyDialog) {
            this.busyDialog.close();
        }
        if (response) {
            var responseJSON = JSON.parse(response);
            if (responseJSON.hasOwnProperty("errorCode")) {
                //display error msg.
                sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, rb.getText("loadDataEditor.loadFutureData.window.title"));
            }
        } else {
            view.fireEvent(view.getId() + "_DATA_LOADED", { "view": view, "action": "HISTORICAL_DATA_LOADED" });
            view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
        }
    },

    fileUploadTypeMissmatch: function (event) {
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = "*." + value;
        });
        var sSupportedFileTypes = aFileTypes.join(", ");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        sas.hc.m.MessageBox.error(rb.getText("fileUploadTypeMismatch.msg.fmt.txt", [event.getParameter("fileType"), sSupportedFileTypes]), rb.getText("loadDataEditor.window.title"));
    },

    fileUploadFileSizeExceed: function (event) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        sas.hc.m.MessageBox.error(rb.getText("fileUploadFileSizeExceed.msg.fmt.txt", [event.getSource().getMaximumFileSize()]), rb.getText("loadDataEditor.window.title"));
    },

    //filterServerFiles: function (event) {
    //    var value = event.getParameter("query") || event.getParameter("newValue") || "";
    //    var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
    //    var binding = this.getView().globalDataList.getBinding("items");
    //    binding.filter(nameFilter);
    //},
    //
    //filterLandingAreaFiles: function (event) {
    //    var value = event.getParameter("query") || event.getParameter("newValue") || "";
    //    var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
    //    var binding = this.getView().landingAreaDataList.getBinding("items");
    //    binding.filter(nameFilter);
    //},
    //
    landingAreaDataListChange: function (event) {
        this._toggleOKButton();
    },

    globalDataListChange: function (event) {
        this._toggleOKButton();
    },

    globalDataFileUploaderChange: function (event) {
        this._toggleOKButton();
    },

    _toggleOKButton: function (fileSelected) {
        //Enable the OK button if a local or server file is selected
        var view = this.getView();
        var fileSelected;
        if (view.radioButtonGroup.getSelectedIndex() === 0) {
            fileSelected = (view.globalDataFileUploader.getValue() !== "");
            // S1243360: Workaround for Chrome-specific issue inherited from SAP. When user hits "Cancel" after having
            // already made a file selection the file is cleared but the UI still shows the filename; we need to clear
            // the filename displayed in the UI so it is apparent to the user that no file is selected anymore.
            if (!fileSelected && view.globalDataFileUploader.getDomRef("fu_input-inner")) {
                view.globalDataFileUploader.getDomRef("fu_input-inner").value = "";
            }
        } else if (view.radioButtonGroup.getSelectedIndex() === 1) {
            var listItems = view.globalDataList.getListItems();
            fileSelected = (view.globalDataList.getSelectedIndex() !== -1 || listItems.length === 1);
        } else if (view.radioButtonGroup.getSelectedIndex() === 2) {
            var listItems = view.getModel().getProperty("/landingAreaFiles");
            fileSelected = (view.landingAreaDataList.getSelectedIndex() !== -1 || listItems.length === 1 );
        }
        view.okButton.setEnabled(fileSelected);
    }

});
