jQuery.sap.require("sap.ui.table.SelectionMode");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.RadioButtonGroup");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");
jQuery.sap.require({ modName: "sas.stress.data.view.LoadDataEditor", type: "controller" });

sap.ui.jsview("sas.stress.data.view.LoadDataEditor", {

    getControllerName: function () {
        return "sas.stress.data.view.LoadDataEditor";
    },
    createContent: function (controller) {
        this.addStyleClass("loadDataEditorView");

        this.radioButtonGroup = new sas.hc.m.RadioButtonGroup({
            selectedIndex: 0,
            columns: 3,
            width: "100%",
            select: $.proxy(controller.locationRadioButtonSelect, controller),
            buttons: [
                new sas.hc.m.RadioButton({
                    text: "{i18N>location.local.option.txt}",
                    groupName: "groupA",
                    width: "100%"
                }),
                new sas.hc.m.RadioButton({
                    text: "{i18N>location.server.option.txt}",
                    groupName: "groupA",
                    width: "100%"
                }),
                new sas.hc.m.RadioButton({
                    text: "{i18N>location.landingArea.option.txt}",
                    groupName: "groupA",
                    width: "100%",
                    enabled: {
                        path: "/projects",
                        formatter: function (projects) {
                            if (projects && projects.length) {
                                return projects.length > 0;
                            }
                            return false;
                        }
                    }
                })
            ]
        });

        this.globalDataFileUploader = new sas.hc.ui.unified.FileUploader({
            width: "100%",
            name: "file",
            uploadUrl: "",
            uploadOnChange: false,
            sendXHR: true, //using this to get the response in a better format.
            fileType: "csv,xls,xlsx",
            placeholder: "{i18N>loadDataEditor.fileUploader.placeholder.txt}",
            maximumFileSize: 100, //100MB
            change: $.proxy(controller.globalDataFileUploaderChange, controller),
            uploadComplete: $.proxy(controller.globalDataFileUploadComplete, controller),
            typeMissmatch: $.proxy(controller.fileUploadTypeMissmatch, controller),
            fileSizeExceed: $.proxy(controller.fileUploadFileSizeExceed, controller)
        });

        this.globalDataList = this.createDataSelectList("/files", "timestamp");
        this.globalDataList.attachSelectionChange(controller.globalDataListChange, controller);

        this.landingAreaDataList = this.createDataSelectList("/landingAreaFiles", "timestamp");
        this.landingAreaDataList.attachSelectionChange(controller.landingAreaDataListChange, controller);

        this.projectSelect = new sas.hc.m.Select({
            items: {
                path: "/projects",
                template: new sas.hc.ui.core.Item({ key: "{id}", text: "{name}" })
            },
            change: $.proxy(controller._getLandingAreaFiles, controller)
        });

        var fileSelectLabel;
        var projectMode = this.getViewData().projectMode;
        if (true === projectMode) {
            fileSelectLabel = new sas.hc.m.Label({
                text: "{i18N>loadProjectDataEditor.loadFutureData.control.label}",
                required: true
            });
        } else {
            fileSelectLabel = new sas.hc.m.Label({
                text: "{i18N>loadDataEditor.loadHistoricalData.control.label}",
                required: true
            });
        }

        this.okButton = new sas.hc.m.Button({
            text: "{i18N>button.ok.action.label}",
            press: $.proxy(controller.okButtonPress, controller),
            enabled: false
        });
        this.cancelButton = new sas.hc.m.Button({
            text: "{i18N>button.cancel.action.label}",
            press: $.proxy(controller.cancelButtonPress, controller)
        });

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                //columnsL: 1,
                //labelSpanS: 12,
                //labelSpanM: 3,
                //labelSpanL: 3
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: "{i18N>loadDataEditor.project.control.label}",
                                required: true
                            }),
                            fields: [this.projectSelect],
                            visible: (true === projectMode)
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: "{i18N>loadDataEditor.chooseLocation.control.label}",
                                required: true
                            }),
                            fields: [this.radioButtonGroup]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: fileSelectLabel,
                            fields: [
                                this.globalDataFileUploader,
                                //this.searchField,
                                this.globalDataList,
                                //this.landingAreaSearchField,
                                this.landingAreaDataList
                            ]
                        })
                    ]
                })
            ]
        });
        form.addStyleClass("stressLargeForm");
        return form;
    },

    createDataSelectList: function (itemsPath, timePath) {
        var singleSelectTable = new sas.risk.common.widgets.SelectTable({
            listItems: "{" + itemsPath + "}",
            listName: "{i18N>loadDataEditor.fileList.name.txt}",
            selectionMode: sap.ui.table.SelectionMode.Single,
            //selectionBehavior: sap.ui.table.SelectionBehavior.RowSelector,
            visibleRowCount: 9,
            visible: false,
            width:"100%"
        });

        singleSelectTable.addColumn(new sas.hc.ui.table.Column({
            template: new sas.hc.m.Text({
                text: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                tooltip: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                wrapping: false
                //maxLines: 1
            }),
            width: "35%"
        }));
        singleSelectTable.bindRows(itemsPath);
        return singleSelectTable;
    },

    attachClosing: function (callBack, listener) {
        this.attachEvent(this.getId() + "_CLOSING", callBack, listener);
    },

    attachDataLoaded: function (callBack, listener) {
        this.attachEvent(this.getId() + "_DATA_LOADED", callBack, listener);
    }
});