jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.stress.data.DataAPI");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");

sap.ui.controller("sas.stress.data.view.LoadProjectDataEditor", {
    onBeforeRendering: function () {
        var view = this.getView();
        var controller = this;
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.data.messagebundle" });
        view.setModel(i18N, "i18N");
        var model = new sas.stress.wkbench.model.RiskJSONModel();
        view.setModel(model);

        //Get list of files on server.
        this._getFilesOnServer();
    },

    _getFilesOnServer: function () {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var view = this.getView();
        var projectKey = view.getViewData().projectId;
        var controller = this;
        $.when(sas.stress.data.DataAPI.listServerFiles(), sas.stress.data.DataAPI.listLandingAreaFiles(projectKey)).done(function (serverFiles,
                                                                                                                                   landingAreaFiles) {
            _.each(serverFiles,function(file){
                // update the name to be the typelessSurrogatePath
                file.actualName=file.name;
                file.name=file.typelessSurrogatePath;
            });
            _.each(landingAreaFiles,function(file){
                // update the name to be the typelessSurrogatePath
                file.actualName=file.name;
                file.name=file.typelessSurrogatePath;
            });
            var fileSort = function (a, b) {
                var aName = a.name.toLowerCase();
                var bName = b.name.toLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            };

            var sortedServerFiles = serverFiles.sort(fileSort);
            var sortedLandingAreaFiles = landingAreaFiles.sort(fileSort);

            if (sortedServerFiles.length === 1 ) {
                sortedServerFiles[0].selected = true;            }
            if (sortedLandingAreaFiles.length === 1 ) {
                sortedLandingAreaFiles[0].selected = true;
            }
            view.getModel().setProperty("/projectFiles", sortedServerFiles);
            view.getModel().setProperty("/landingAreaFiles", sortedLandingAreaFiles);

            // handle process orchestration commands, if they exist
            if (sap.ui.getCore().getModel("ProjectDataParams")) {
                controller.handleProcessOrchestration();
            }
        }).fail(function (jqxhr) {
            //display error msg.
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
            if (jqxhr && jqxhr.responseJSON) {
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            } else {
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            }
        });
    },

    handleProcessOrchestration: function () {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var view = this.getView();
        var projectDataParams = sap.ui.getCore().getModel("ProjectDataParams").getData();
        if (projectDataParams) {
            if (projectDataParams.location) {
                if (projectDataParams.location.toLocaleLowerCase() === "local") {
                    view.radioButtonGroup.setSelectedIndex(0);
                    view.radioButtonGroup.fireSelect({selectedIndex:0});
                    // We can't assist them with the actual file, so this is as far as we go here.
                } else if (projectDataParams.location.toLocaleLowerCase() === "server") {
                    view.radioButtonGroup.setSelectedIndex(1);
                    view.radioButtonGroup.fireSelect({selectedIndex:1});
                    if (projectDataParams.file) {
                        var foundFile = false;
                        var listItems = view.projectDataList.getListItems();
                        for (var i = 0; i < listItems.length; ++i) {
                            if (projectDataParams.file.toLocaleLowerCase() === listItems[i].name.toLocaleLowerCase()) {
                                listItems[i].selected = true;
                                // view.projectDataList.setSelectedItem(listItems[i]);
                                view.projectDataList.fireSelectionChange({  selectedItems: [listItems[i]]
                                    //listItem: listItems[i],
                                    //listItems: [listItems[i]],
                                    //selected: true
                                });//Fire
                                view.projectDataList.setFirstVisibleRow(i);
                                // Scroll to the selected index, needs to be in a timeout...
                                //setTimeout(function () {
                                //    view.projectDataList.$().find('tbody').find('tr:nth-child(' + i + ')').focus();
                                //}, 100);
                                foundFile = true;
                                break;
                            }
                        }
                        if (!foundFile) {
                            // Specified file does not exist
                            sas.hc.m.MessageBox.error(rb.getText("loadProjectDataEditor.processOrchestration.invalidServerFile.fmt.txt", [projectDataParams.file]), rb.getText("error.window.title"));
                        }
                    }
                } else if (projectDataParams.location.toLocaleLowerCase() === "landingarea") {
                    view.radioButtonGroup.setSelectedIndex(2);
                    view.radioButtonGroup.fireSelect({selectedIndex:2});
                    if (projectDataParams.file) {
                        var foundFile = false;
                        var listItems = view.landingAreaDataList.getListItems();
                        for (var i = 0; i < listItems.length; ++i) {
                            if (projectDataParams.file.toLocaleLowerCase() === listItems[i].name.toLocaleLowerCase()) {
                                listItems[i].selected = true;
                                //view.landingAreaDataList.setSelectedItem(listItems[i]);
                                view.landingAreaDataList.fireSelectionChange({ selectedItems: [listItems[i]]
                                    //listItem: listItems[i],
                                    //listItems: [listItems[i]],
                                    //selected: true
                                });//Fire
                                view.landingAreaDataList.setFirstVisibleRow(i);
                                // Scroll to the selected index, needs to be in a timeout...
                                //setTimeout(function () {
                                //    view.landingAreaDataList.$().find('ul').find('li:nth-child(' + i + ')').focus();
                                //}, 100);
                                foundFile = true;
                                break;
                            }
                        }
                        if (!foundFile) {
                            // Specified file does not exist
                            sas.hc.m.MessageBox.error(rb.getText("loadProjectDataEditor.processOrchestration.invalidLandingAreaFile.fmt.txt", [projectDataParams.file]), rb.getText("error.window.title"));
                        }
                    }
                } else {
                    // Invalid file location type was specified
                    sas.hc.m.MessageBox.error(rb.getText("loadProjectDataEditor.processOrchestration.invalidLocation.fmt.txt", [projectDataParams.location]), rb.getText("error.window.title"));
                }
            }
        }
    },

    cancelButtonPress: function () {
        var view = this.getView();
        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "CANCEL" });
        sap.ui.getCore().setModel(undefined, "ProjectDataParams");
    },

    okButtonPress: function () {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var projectKey = view.getViewData().projectId;
        //Load project data
        if (view.radioButtonGroup.getSelectedIndex() === 0) {
            if (view.projectDataFileUploader.getValue()) { //File selected?
                this.busyDialog = new sas.hc.m.BusyDialog({
                    title: rb.getText("importDialog.busyDialog.title")
                });
                this.busyDialog.open();
                view.projectDataFileUploader.setUploadUrl("rest/projects/" + projectKey + "/projectData");
                view.projectDataFileUploader.upload();
            }
        } else if (view.radioButtonGroup.getSelectedIndex() === 1) {
            var selectedIndex = view.projectDataList.getSelectedIndex();
            var selectedItem = view.projectDataList.getContextByIndex(selectedIndex);
            var fileId = "";
            var obj = selectedItem.getObject();
            if (obj !== undefined) {
                fileId = obj.id || obj.getProperty("key");
            }
            if (fileId !== "") {
                view.setBusy(true);
                sas.stress.data.DataAPI.uploadProjectDataFromServer(projectKey, fileId).done(function (response) {
                    view.setBusy(false);
                    view.fireEvent(view.getId() + "_DATA_LOADED", { "view": view, "action": "FUTURE_DATA_LOADED" });
                    view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
                }).fail(function (jqxhr) {
                    view.setBusy(false);
                    //display error msg.
                    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                    if (jqxhr && jqxhr.responseJSON) {
                        sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                    } else {
                        sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                    }
                });
            }
        } else if (view.radioButtonGroup.getSelectedIndex() === 2) {

            var selectedIndex = view.landingAreaDataList.getSelectedIndex();
            var selectedItem = view.landingAreaDataList.getContextByIndex(selectedIndex);
            var fileId = "";
            var obj = selectedItem.getObject();
            if (obj !== undefined) {
                fileId = obj.id || obj.getProperty("key");
            }
            if (fileId !== "") {
                view.getParent().setBusy(true);
                sas.stress.data.DataAPI.uploadProjectDataFromLandingArea(projectKey, fileId).done(function (response) {
                    view.getParent().setBusy(false);
                    view.fireEvent(view.getId() + "_DATA_LOADED", { "view": view, "action": "FUTURE_DATA_LOADED" });
                    view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
                }).fail(function (jqxhr) {
                    view.getParent().setBusy(false);
                    //display error msg.
                    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                    if (jqxhr && jqxhr.responseJSON) {
                        sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                    } else {
                        sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
                    }
                });
            }
        }
        sap.ui.getCore().setModel(undefined, "ProjectDataParams");
    },

    locationRadioButtonSelect: function (event) {
        var view = this.getView();
        var selectedIndex = event.getParameter("selectedIndex");
        var showLocal = false;
        var showServer = false;
        var showLandingArea = false;
        if (0 === selectedIndex) {
            showLocal = true;
        } else if (1 === selectedIndex) {
            showServer = true;
        } else if (2 === selectedIndex) {
            showLandingArea = true;
        }
        view.fileUploaderElement.setVisible(showLocal);
        //view.searchField.setVisible(showServer);
        view.projectDataElement.setVisible(showServer);
        //view.landingAreaSearchField.setVisible(showLandingArea);
        view.landingAreaDataElement.setVisible(showLandingArea);
        this._toggleOKButton();
    },

    projectDataFileUploadComplete: function (event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var response = event.getParameter("response");
        if (this.busyDialog) {
            this.busyDialog.close();
        }
        if (response) {
            var responseJSON = JSON.parse(response);
            if (responseJSON.hasOwnProperty("errorCode")) {
                //display error msg.
                sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, rb.getText("loadProjectDataEditor.window.title"));
            }
        } else {
            view.fireEvent(view.getId() + "_DATA_LOADED", { "view": view, "action": "FUTURE_DATA_LOADED" });
            view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
        }
    },

    fileUploadTypeMissmatch: function (event) {
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = "*." + value;
        });
        var sSupportedFileTypes = aFileTypes.join(", ");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        sas.hc.m.MessageBox.error(rb.getText("fileUploadTypeMismatch.msg.fmt.txt", [event.getParameter("fileType"), sSupportedFileTypes]), rb.getText("loadProjectDataEditor.window.title"));
    },

    fileUploadFileSizeExceed: function (event) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        sas.hc.m.MessageBox.error(rb.getText("fileUploadFileSizeExceed.msg.fmt.txt", [event.getSource().getMaximumFileSize()]), rb.getText("loadProjectDataEditor.window.title"));
    },

    //filterServerFiles: function (event) {
    //    var value = event.getParameter("query") || event.getParameter("newValue") || "";
    //    var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
    //    var binding = this.getView().projectDataList.getBinding("items");
    //    binding.filter(nameFilter);
    //},
    //
    //filterLandingAreaFiles: function (event) {
    //    var value = event.getParameter("query") || event.getParameter("newValue") || "";
    //    var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
    //    var binding = this.getView().landingAreaDataList.getBinding("items");
    //    binding.filter(nameFilter);
    //},
    //
    projectDataListChange: function (event) {
        this._toggleOKButton();
    },

    landingAreaDataListChange: function (event) {
        this._toggleOKButton();
    },

    projectDataFileUploaderChange: function (event) {
        this._toggleOKButton();
    },

    _toggleOKButton: function () {
        //Enable the OK button if a project file is selected
        var view = this.getView();
        var fileSelected;
        if (view.radioButtonGroup.getSelectedIndex() === 0) {
            fileSelected = (view.projectDataFileUploader.getValue() !== "");
            // S1243360: Workaround for Chrome-specific issue inherited from SAP. When user hits "Cancel" after having
            // already made a file selection the file is cleared but the UI still shows the filename; we need to clear
            // the filename displayed in the UI so it is apparent to the user that no file is selected anymore.
            if (!fileSelected && view.projectDataFileUploader.getDomRef("fu_input-inner")) {
                view.projectDataFileUploader.getDomRef("fu_input-inner").value = "";
            }
        } else if (view.radioButtonGroup.getSelectedIndex() === 1) {
            var listItems = view.projectDataList.getListItems();
            fileSelected = (view.projectDataList.getSelectedIndex() !== -1 || listItems.length === 1);
        } else if (view.radioButtonGroup.getSelectedIndex() === 2) {
            var listItems = view.landingAreaDataList.getListItems();
            fileSelected = (view.landingAreaDataList.getSelectedIndex() !== -1 || listItems.length === 1 );
        }
        view.okButton.setEnabled(fileSelected);
    }

});
