jQuery.sap.require("sap.m.ListMode");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.RadioButtonGroup");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");
jQuery.sap.require({ modName: "sas.stress.data.view.LoadProjectDataEditor", type: "controller" });

sap.ui.jsview("sas.stress.data.view.LoadProjectDataEditor", {

    getControllerName: function () {
        return "sas.stress.data.view.LoadProjectDataEditor";
    },
    createContent: function (controller) {
        this.addStyleClass("loadProjectDataEditorView");

        this.radioButtonGroup = new sas.hc.m.RadioButtonGroup({
            selectedIndex: 0,
            columns: 3,
            width: "100%",
            select: $.proxy(controller.locationRadioButtonSelect, controller),
            buttons: [
                new sas.hc.m.RadioButton({
                    text: "{i18N>location.local.option.txt}",
                    groupName: "groupA",
                    width: "100%"
                }),
                new sas.hc.m.RadioButton({
                    text: "{i18N>location.server.option.txt}",
                    groupName: "groupA",
                    width: "100%"
                }),
                new sas.hc.m.RadioButton({
                    text: "{i18N>location.landingArea.option.txt}",
                    groupName: "groupA",
                    width: "100%"
                })
            ]
        });

        this.projectDataFileUploader = new sas.hc.ui.unified.FileUploader({
            width: "100%",
            name: "file",
            uploadUrl: "",
            uploadOnChange: false,
            sendXHR: true, //using this to get the response in a better format.
            fileType: "csv,xls,xlsx",
            placeholder: "{i18N>loadProjectDataEditor.fileUploader.placeholder.txt}",
            maximumFileSize: 100, //100MB
            change: $.proxy(controller.projectDataFileUploaderChange, controller),
            uploadComplete: $.proxy(controller.projectDataFileUploadComplete, controller),
            typeMissmatch: $.proxy(controller.fileUploadTypeMissmatch, controller),
            fileSizeExceed: $.proxy(controller.fileUploadFileSizeExceed, controller)
        });

        this.fileUploaderElement = this.createDataFormElement(this.projectDataFileUploader, true);

        this.projectDataList = this.createDataSelectList("/projectFiles", "timestamp");
        this.projectDataList.attachSelectionChange(controller.projectDataListChange, controller);
        this.projectDataElement = this.createDataFormElement(this.projectDataList, false);

        this.landingAreaDataList = this.createDataSelectList("/landingAreaFiles", "timestamp");
        this.landingAreaDataList.attachSelectionChange(controller.landingAreaDataListChange, controller);
        this.landingAreaDataElement = this.createDataFormElement(this.landingAreaDataList, false);

        this.okButton = new sas.hc.m.Button({
            text: "{i18N>button.ok.action.label}",
            press: $.proxy(controller.okButtonPress, controller),
            enabled: false
        });
        this.cancelButton = new sas.hc.m.Button({
            text: "{i18N>button.cancel.action.label}",
            press: $.proxy(controller.cancelButtonPress, controller)
        });

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: "{i18N>loadProjectDataEditor.chooseLocation.control.label}",
                                required: true
                            }),
                            fields: [this.radioButtonGroup]
                        }),
                        this.fileUploaderElement,
                        this.projectDataElement,
                        this.landingAreaDataElement
                    ]
                })
            ]
        });
        form.addStyleClass("stressXLargeForm");
        return form;
    },
    createDataFormElement: function (field, vis) {
        return new sas.hc.ui.layout.form.FormElement({
            label: new sas.hc.m.Label({
                text: "{i18N>loadProjectDataEditor.loadFutureData.control.label}",
                required: true
            }),
            fields: [field],
            visible: vis
        });
    },

    createDataSelectList: function (itemsPath, timePath) {
        //var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var singleSelectTable = new sas.risk.common.widgets.SelectTable({
            listItems: "{" + itemsPath + "}",
            listName: "{i18N>loadProjectDataEditor.fileList.name.txt}",
            selectionMode: sap.ui.table.SelectionMode.Single,
            //selectionBehavior: sap.ui.table.SelectionBehavior.RowSelector,
            visibleRowCount: 9
            //visible: false
        });

        singleSelectTable.addColumn(new sas.hc.ui.table.Column({
            template: new sas.hc.m.Text({
                text: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                tooltip: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                wrapping: false
                //maxLines: 1
            }),
            width: "35%"
        }));
        singleSelectTable.bindRows(itemsPath);
        return singleSelectTable;
    },

    attachClosing: function (callBack, listener) {
        this.attachEvent(this.getId() + "_CLOSING", callBack, listener);
    },

    attachDataLoaded: function (callBack, listener) {
        this.attachEvent(this.getId() + "_DATA_LOADED", callBack, listener);
    }
});