jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.AccessControlsDialog");
jQuery.sap.require("sas.stress.wkbench.widget.ZeroStateLayout");
jQuery.sap.require("sas.stress.models.ModelGroupRunAPI");
jQuery.sap.require("sas.stress.models.ModelGroupRunUtils");
jQuery.sap.require("sas.risk.common.utils.LayoutService");

sap.ui.controller("sas.stress.models.view.execution.ModelGroupRunsManager", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("ModelGroupRunsManager", this._init, this);
        this.modelGroupCache = {};
        this.scenarioCache = {};
        this.entityCache = {};
        this._pollIntervalIDs = [];
        this.loadResults = [];
        this.CONSTANTS = sas.stress.models.ModelGroupRunConstants;
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());

        new sas.risk.common.utils.LayoutService(this.getView().runsTable, "ModelGroupRunsManagerTable");
    },

    refreshModel: function (projectId, project) {
        var controller = this;
        var view = controller.getView();

        controller.projectId = projectId;
        $.when(sas.stress.models.ModelGroupRunAPI.getModelGroupRuns(projectId),
               sas.stress.models.ModelGroupRunAPI.getScenariosForRuns(projectId),
               sas.stress.models.ModelGroupRunAPI.getEntityLineItemsForProject(projectId),
               sas.stress.models.ModelGroupRunAPI.getModelGroupsForRuns(projectId)).done(
            function (modelGroupRuns, scenarios, entities, modelGroups) {
                controller.scenarioCache = {};
                controller.entityCache = {};
                controller.modelGroupCache = {};

                // To maintain any sort already applied on the table, if model exists, set data on the model.
                // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
                var model = view.getModel();
                if (!(model instanceof sas.stress.wkbench.model.RiskJSONModel)) {  // Lives in StressFixes
                    model = new sas.stress.wkbench.model.RiskJSONModel();
                    view.setModel(model);
                    //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                    //var sorters = [new sap.ui.model.Sorter("name", false)];
                    //view.getTable().getBinding("rows").sort(sorters);
                }

                // Warning: This code is order specific
                // sas.stress.models.ModelGroupRunConstants.scenariosCache = scenarios;
                // sas.stress.models.ModelGroupRunConstants.entitiesCache = entities;
                // sas.stress.models.ModelGroupRunConstants.modelGroupsCache = modelGroups;
                model.setProperty(controller.CONSTANTS.PATH_SCENARIOS, scenarios);
                model.setProperty(controller.CONSTANTS.PATH_ENTITIES, entities);
                model.setProperty(controller.CONSTANTS.PATH_MODEL_GROUPS, modelGroups);
                controller.updateModelGroupRunsWithAssociatedNames(controller, modelGroupRuns);
                model.setProperty(controller.CONSTANTS.PATH_MODEL_GROUP_RUNS, modelGroupRuns);
                if (controller.project) {
                    model.setProperty("/projectName", controller.project.getName());
                    model.setProperty("/currentIterationNo", controller.project.getCurrentIterationNo());
                    model.setProperty("/asOfDate", controller.project.getAsOfDate());
                    model.setProperty("/projectIsLocked", controller.project.isLocked());
                }

                view.setBusy(false);
                view.fireSelectionEvent();
                controller._initController();

            }
        ).fail(function (jqxhr) {
            controller._clearInterval();
            view.setBusy(false);
            controller.onServerFailure(jqxhr);
        });
    },

    updateModelGroupRunsWithAssociatedNames: function (controller, modelGroupRuns) {
        // For sorting on scenarios, entities, or model groups we have to add a sort property for each
        // to our list of runs, so here goes.
        _.each(modelGroupRuns, function (run) {
            var scenario = controller.getScenario(run.scenarioKey);
            if (scenario) {
                run.scenarioName = scenario.name;
            }
            var entity = controller.getEntity(run.entityLIKey);
            if (entity) {
                run.entityName = entity.memberName;
            }
            var modelGroup = controller.getModelGroup(run.modelGroupKey);
            if (modelGroup) {
                run.modelGroupName = modelGroup.name;
                run.modelGroupVersion = modelGroup.version;
            }
        });

    },
    getModelGroup: function (key) {
        if (!key) {
            return null;
        }
        var modelGroup = this.modelGroupCache[key];
        if (!modelGroup) {
            var modelGroups = this.getView().getModel().getProperty("/modelGroupsForRuns");
            if (modelGroups) {
                modelGroup = _.find(modelGroups, function (modelGroup) {
                    return modelGroup.key === key;
                });
                if (modelGroup) {
                    this.modelGroupCache[key] = modelGroup;
                }
            }
        }
        return modelGroup;
    },

    getScenario: function (key) {
        if (!key) {
            return null;
        }
        var scenario = this.scenarioCache[key];
        if (!scenario) {
            var scenarios = this.getView().getModel().getProperty("/scenariosForRuns");
            if (scenarios) {
                for (var i = 0; i < scenarios.length; i++) {
                    if (scenarios[i].scenarioKey === key) {
                        scenario = scenarios[i];
                        this.scenarioCache[key] = scenario;
                        break;
                    }
                }
            }
        }
        return scenario;
    },

    getEntity: function (key) {
        if (!key) {
            return null;
        }
        var normalizedKey = key.toUpperCase();
        var entity = this.entityCache[normalizedKey];
        if (!entity) {
            var entities = this.getView().getModel().getProperty("/entitiesForRuns");
            if (entities) {
                for (var i = 0; i < entities.length; i++) {
                    if (entities[i].liKey.toUpperCase() === normalizedKey) {
                        entity = entities[i];
                        this.entityCache[normalizedKey] = entity;
                        break;
                    }
                }
            }
        }
        return entity;
    },

    //_handleAjaxFailure: function (controller, view, jqXHR) {
    //    //controller._updateBreadCrumb();
    //    view.setTableBusy(false);
    //    if (jqXHR) {
    //        var responseJSON = (jqXHR.responseJSON) ? jqXHR.responseJSON : JSON.parse(jqXHR.responseText);
    //        sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
    //    }
    //},

    _init: function (event) {
        var controller = this;
        var view = this.getView();

        /* these are the time out IDs for multiple model runs and checking */
        controller._pollTimeOutIDs = [];
        /* the following are the variables needed for selected scenarios and entities for model run */
        controller.aScenarioIsSelected = false;
        controller.anEntityIsSelected = false;
        controller.scenariolist = [];
        controller.entitylist = [];

        view.setBusy(true);
        /*
         * this is a map of the visible columns on the table
         * key = column name from json, value=(visible)true/false
         */
        // Not sure how this is supposed to work so pulled out. MCA
//        controller._mdlVisibleColumns = {};

        controller.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.models.messagebundle" });
        view.setModel(controller.i18N, "i18N");

        var projectId = event.getParameter("arguments").projectId;

        controller.projectId = projectId;
        // S1273684: don't use the cached project since it might be stale or deleted by others
        //controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
        //if (!controller.project) { // If we don't, load it into memory
        $.when(sas.stress.wkbench.model.Project.loadProject(projectId, true)).done(function (proj) {
            controller.project = proj;
            //Load data in the table
            controller.refreshModel(controller.projectId);
            //ctrl._initcontroller(view, executing);
        }).fail(function (jqxhr) {
            controller.onServerFailure(jqxhr);
            view.setBusy(false);
            sas.stress.wkbench.ViewManager.loadProjectManager();

        });
        //} else { //
        //    // update breadcrumb.. do it here because there could be a failure and the bread crumb will nenver showup
        //    controller.refreshModel(controller.projectId);
        //    //controller._updateBreadCrumb();
        //}

    },
    onExit: function () {
        this._clearInterval();
        this.router.detachExactRouteMatched("ModelGroupRunsManager", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.models.view.execution.ModelGroupRunsManager", null);
        this.navChecker.stop();
    },

    _areRunsRunning: function () {
        var model = this.getView().getModel();
        if (!model) return false;
        var data = model.getProperty("/modelGroupRuns");
        for (var i = 0; i < data.length; i++) {
            if (data[i].status.toUpperCase() === 'RUNNING' || data[i].status.toUpperCase() === 'READYTORUN') {
                return true;
            }
        }
        return false;
    },

    _initController: function () {
        this.poll(true);
        //this._updateBreadCrumb();
    },

    poll: function (atLeastOnce) {
        var controller = this;
        if (controller._areRunsRunning() || atLeastOnce) {
            controller._pollIntervalIDs.push(setInterval(function () {
                sas.stress.models.ModelGroupRunAPI.getModelGroupRuns(controller.projectId).done(function (modelGroupRuns) {
                    console.log("ModelGroupRuns: Polling...");
                    var model = controller.getView().getModel();
                    controller.updateModelGroupRunsWithAssociatedNames(controller, modelGroupRuns);
                    model.setProperty("/modelGroupRuns", modelGroupRuns);
                    if (!controller._areRunsRunning()) {
                        controller._clearInterval();
                    }
                }).fail(function (jqxhr) {
                    controller._clearInterval();
                    controller.onServerFailure(jqxhr);
                }).always(function () {
                    controller.getView().setBusy(false);
                });
            }, 2500));
        } else {
            controller._clearInterval();
        }
    },

    _clearInterval: function () {
        // clear the interval ID
        _.each(this._pollIntervalIDs, function (intervalId) {
            clearInterval(intervalId);
        });
        this._pollIntervalIDs = [];
    },

    onRunSelected: function (e) {
        var rb = this.i18N.getResourceBundle();
        var selectedRuns = this.getView().getSelectedRuns();
        var controller = this;

        _.each(selectedRuns, function (run) {
            run.status = "READYTORUN";
        });
        this.getView().getTable().getModel().updateBindings(false);

        var runIds = _.map(selectedRuns, function (run) {
            return run.id;
        });
        sas.stress.models.ModelGroupRunAPI.rerunRuns(runIds).done(
            function (results) {
                controller.poll(true);
            }
        ).fail(
            function (jqxhr) {
                controller.poll(true);
                controller.onServerFailure(jqxhr);
            }
        );
    },
    //onAccessControlsButtonClick: function (event) {
    //    var selectedRuns = this.getView().getSelectedRuns();
    //    if (selectedRuns && selectedRuns.length === 1) {
    //        var run = selectedRuns[0];
    //        if (run) {
    //            var accessControlsDialog = new sas.stress.wkbench.widget.AccessControlsDialog({});
    //            var view = this.getView();
    //            view.getTable().setBusy(true);
    //            sas.stress.models.ModelGroupRunAPI.getModelGroupRunAccessControls(run.id).done(function (accessControls) {
    //                view.getTable().setBusy(false);
    //                accessControlsDialog.setup(run.name, accessControls);
    //            }).fail(function (jqxhr) {
    //                view.getTable().setBusy(false);
    //                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    //            });
    //        }
    //    }
    //},
    openLoadDialog: function (data, controller) {
        var rb = this.i18N.getResourceBundle();
        var dialogModel = new sas.stress.wkbench.model.RiskJSONModel();
        // fix type of value

        // All of this is totally lame but I needed to get it done in the most expediant way.
        // TODO: Should either use binding all the way or not!!
        var valueString = data.sampleDataPointLoaded.value;
        data.valueAsFloat = Number(valueString);
        var formatter = sap.ui.core.format.NumberFormat.getFloatInstance({ maxFractionDigits: 2 });
        data.valueForDisplay = formatter.format(data.sampleDataPointLoaded.value);
        controller.partiallyLoaded = data.partialDataLoad;
        controller.numSuccessfullyLoaded = data.dataValuesCountLoaded;
        var startDate = data.sampleDataPointLoaded.intervalStartDate;
        var endDate = data.sampleDataPointLoaded.intervalEndDate;
        var interval = data.sampleDataPointLoaded.interval;
        controller.intervalDisplayString = rb.getText("results.dialog.dateinterval.value.fmt", [
            sas.stress.models.ModelGroupRunUtils.dateFormatter(startDate),
            sas.stress.models.ModelGroupRunUtils.dateFormatter(endDate),
            sas.stress.models.ModelGroupRunUtils.intervalFormatter(interval)
        ]);

        dialogModel.setData(data);
        var existingDialog = sap.ui.getCore().byId("LoadResultsDialog");
        if (existingDialog) {
            existingDialog.open();
            existingDialog.setModel(dialogModel);
        } else {
            var dialog = sap.ui.jsfragment("sas.stress.models.view.execution.LoadResultsDialog", controller);
            dialog.setModel(dialogModel);
            dialog.setBindingContext("/");
            dialog.open();
        }

    },

    onExecuteLoad: function (resultsFile) {
        var rb = this.i18N.getResourceBundle();
        var controller = this;
        this.loadResults = [];
        this.getView().setBusy(true);

        // Test code so we don't have to actually have a SAS backend
        // var fakeData = {
        //     partialDataLoad: true,
        //     dataValuesCountLoaded: 50,
        //     sampleDataPointLoaded: {
        //         value:"45.234",
        //         intervalStartDate: "2015-06-04",
        //         intervalEndDate: "2015-08-15",
        //         interval: "Month"
        //     }
        //
        // };
        // controller.openLoadDialog(fakeData,controller);
        // var loadResultsLogName = rb.getText('modelGroupRun.outputfile.log.loadresults.txt');
        // var nonloadableDataSetName = rb.getText('modelGroupRun.outputfile.sas.nonloadabledata.txt');
        //
        // sas.stress.models.ModelGroupRunUtils.showErrorDialog(rb.getText("modelGroupRuns.load.nothingloaded.error.fmt",[loadResultsLogName,nonloadableDataSetName]), rb.getText("modelGroupRuns.load.nothingloaded.error.title"));
//        return;
        sas.stress.models.ModelGroupRunAPI.loadDataFromRun(resultsFile).done(
            function (data) {
                controller.getView().setBusy(false);
                var numLoaded = data.dataValuesCountLoaded;
                if (numLoaded > 0) {
                    controller.openLoadDialog(data, controller);
                } else {
                    var loadResultsLogName = rb.getText('modelGroupRun.outputfile.log.loadresults.txt');
                    var nonloadableDataSetName = rb.getText('modelGroupRun.outputfile.sas.nonloadabledata.txt');
                    sas.stress.models.ModelGroupRunUtils.showErrorDialog(rb.getText("modelGroupRuns.load.nothingloaded.error.fmt", [loadResultsLogName, nonloadableDataSetName]), rb.getText("modelGroupRuns.load.nothingloaded.error.title"));
                }

                controller.poll(true);
            }
        ).fail(
            function (jqxhr) {
                controller.getView().setBusy(false);
                controller.poll(true);
                controller.onServerFailure(jqxhr);
            }
        );
    },

    onDeleteSelected: function () {
        var rb = this.i18N.getResourceBundle();
        var controller = this;
        sas.stress.models.ModelGroupRunUtils.showConfirmationDialog(
            rb.getText("modelGroupRuns.confirm.delete.title"),
            rb.getText("modelGroupRuns.confirm.delete.txt"),
            null,
            $.proxy(controller._handleDeleteRunInstance, controller)
        );
    },

    navigateToCreateView: function () {
        sas.stress.wkbench.ViewManager.loadModelGroupRunsCreate(sas.stress.wkbench.model.Project.getCurrentProject());
    },

    /**
     * Handler for the delete run instance button. Deletes the currently selected
     * run instance.
     */
    _handleDeleteRunInstance: function () {
        var rb = this.i18N.getResourceBundle();
        var view = this.getView();
        var controller = this;

        view.setBusy(true);
        var selectedRuns = view.getSelectedRuns();
        var runIds = _.map(selectedRuns, function (run) {
            return run.id;
        });
        sas.stress.models.ModelGroupRunAPI.deleteRuns(runIds).done(
            function () {
                // remove the instance from the table model
                var runsModel = view.getTable().getModel();
                view.getTable().setSelectedIndex(-1);
                var runs = runsModel.getData();

                _.each(runIds, function (id) {
                    // find the index in the model/runs array
                    var deletedIndex = sas.stress.models.ModelGroupRunUtils.findIndex([id], function (run) {
                        return run.id === id;
                    });
                    if (deletedIndex !== -1) {
                        runs.splice(deletedIndex, 1);
                    }
                });

                controller.poll(true);
            }
        ).fail(
            function (restError) {
                sas.stress.models.ModelGroupRunUtils.showErrorDialog(rb.getText("modelGroupRuns.error.delete.txt"), rb.getText("modelGroupRuns.error.delete.title"));
            }
        );
    },
    //_poll: function (executing) {
    //    var controller = this;
    //    var view = controller.getView();
    //
    //    if (executing > 0) {
    //        controller._pollTimeOutIDs.push(setTimeout(function () {
    //            controller._loadModels().done(function (executing) {
    //                controller._poll(executing);
    //            }).fail(controller.onServerFailure);
    //        }, 2500));
    //    } else {
    //        controller._clearTimeout();
    //        //sas.hc.m.MessageBox.information("Models execution complete!!", "executed");
    //        sas.hc.m.MessageToast.show(controller.i18N.getResourceBundle().getText("models.execution.complete.message.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
    //
    //        // clear the selections... this will at least give users an indication that execution is complete
    //        view.table.setSelectedIndex(-1);
    //        // the buttons are en-abled
    //        var toggle = view.getModel("toggle");
    //        toggle.setProperty("/toggleButtons", false);
    //    }
    //},
    onRefreshButton: function (e) {
        //var that = this;
        //this._loadModels().done(function (executing) {
        //    if (executing > 0) {
        //        that.poll(true);
        //    }
        //}).fail(function (jqXHR) {
        //    var responseJSON = (jqXHR.responseJSON) ? jqXHR.responseJSON : JSON.parse(jqXHR.responseText);
        //    sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName, that.fncallback);
        //});
    },

    /*
     * common method used between loading the view initially and for execution statuses during RUN, REFRESH and navigate out-in
     */
    _setOrUpdateViewModel: function (json, executing) {
        var view = this.getView();

        // To maintain any sort already applied on the table, if model exists, set data on the model.
        // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
        sas.stress.models.Model._json = json;
        if (view.getModel() instanceof sas.stress.wkbench.model.RiskJSONModel) {
            view.getModel().setData(json);
        } else {
            var model = new sas.stress.wkbench.model.RiskJSONModel(json);
            view.setModel(model);
        }
        return executing;
    },

    _clearTimeout: function () {
        // clear the timeout ID
        _.each(this._pollTimeOutIDs, function (timeoutid) {
            clearTimeout(timeoutid);
        });
        this._pollTimeOutIDs = [];
    },

    /*
     * we will loop through the visible columns map to create filters for each
     */
    filterModelGroupRuns: function () {
        var search = this.getView().getSearchField().getValue().trim() || "";
        var table = this.getView().getTable();
        // var filters = [];
        // _.each(this._mdlVisibleColumns, function (value, key) {
        //     if (value === true) {
        //         filters.push(new sap.ui.model.Filter(key, sap.ui.model.FilterOperator.Contains, search));
        //     }
        // });

        var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search);
        var scenarioFilter = new sap.ui.model.Filter("scenarioName", sap.ui.model.FilterOperator.Contains, search);
        var entityFilter = new sap.ui.model.Filter("entityName", sap.ui.model.FilterOperator.Contains, search);
        var descriptionFitler = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, search);
        var modelGroupFilter = new sap.ui.model.Filter("modelGroupName", sap.ui.model.FilterOperator.Contains, search);
        var userFilter = new sap.ui.model.Filter("runByUserName", sap.ui.model.FilterOperator.Contains, search);
//        var statusFilter = new sap.ui.model.Filter("status", sap.ui.model.FilterOperator.Contains, search);
        var allFilters = new sap.ui.model.Filter([nameFilter, scenarioFilter, entityFilter, descriptionFitler, modelGroupFilter, userFilter]);
        var binding = table.getBinding("rows");
        if (binding) {
            binding.filter(allFilters);
        }
    },

    //onBreadCrumbClick: function (e) {
    //    var item = e.getParameter("item");
    //    switch (item.getProperty("key")) {
    //        case "ProjectManager":
    //            sas.stress.wkbench.ViewManager.loadProjectManager();
    //            break;
    //        default:
    //            sas.stress.wkbench.ViewManager.loadHomePage(this.project);
    //            break;
    //    }
    //},
    onRowSelectionChange: function (oEvent) {
        var controller = this;
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var selectedIndices = oEvent.getSource().getSelectedIndices();
        switch (selectedIndices.length) {
            case 0:
                view.getRunButton().setEnabled(false);
                view.getDeleteButton().setEnabled(false);
                break;
            case 1:
                view.getRunButton().setEnabled(
                    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.RUN) &&
                    (controller.project && !controller.project.isLocked()));
                view.getDeleteButton().setEnabled(
                    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.DELETE) &&
                    (controller.project && !controller.project.isLocked()));
                break;
            default:
                // For now we only support running one at a time, not optimal user experience, so leaving
                // this code in.
                // var allRunnable = true;
                // _.each(selectedIndices, function (selectedIndex) {
                //     var context = oEvent.getSource().getContextByIndex(selectedIndex);
                //     var obj = context.getObject();
                //     if (obj.actionControls !== undefined && obj.actionControls.runnable !== undefined) {
                //         allRunnable = allRunnable && obj.actionControls.runnable;
                //     }
                // });
                view.getRunButton().setEnabled(false);
                view.getDeleteButton().setEnabled(
                    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.DELETE) &&
                    (controller.project && !controller.project.isLocked()));
                break;
        }
    },
    onCellClick: function (event) {
        //var view = this.getView();
        //var ctx = event.getSource().getContextByIndex(event.getParameter("rowIndex"));
        //var name = ctx.getObject().name;
        //
        //var themodel = _.find(view.getModel().getData()["models"], function (model) {
        //    if (model.name === name) {
        //        return model;
        //    }
        //});
        //
        //var cell = event.getParameter("cellControl");
        //if (cell.getId().indexOf("run-log") === 0) {
        //    if (themodel.runStatus === "Completed" || themodel.runStatus === "Error") {
        //        this._showLogDialog(themodel);
        //    } else {
        //        this.navigateToModelDetailsPage(themodel);
        //    }
        //} else {
        //    this.navigateToModelDetailsPage(themodel);
        //}
    },
    // onColumnSelect: function (event) {
    //     var view = this.getView();
    //
    //     var column = event.getParameter("column");
    //     if (column.getId() === "table-log") {
    //
    //     }
    // },

    //okPressed: function () {
        //var controller = this;
        //var view = controller.getView();
        //var model = view.getModel();
        //var table = view.table;
        //var modelKeyList = [];
        //var entitylist = [];
        //var scenariolist = [];
        //var selectedList = [];
        //
        //var dialog = sap.ui.getCore().byId("RunModelForEntitiesScenarios");
        //var enlist = dialog.getEntitiesList();
        //var sclist = dialog.getScenariosList();
        //
        //// we will need to call server to start execute and when it returns update the view table..
        //if (table.getSelectedIndices().length) {
        //    _.each(table.getSelectedIndices(), function (index) {
        //        var ctx = table.getContextByIndex(index);
        //        modelKeyList.push(ctx.getObject().id);
        //    });
        //} else {
        //    _.each(model.getData()["models"], function (data) {
        //        if (data.category === controller.selectedCategoryItem.getText()) {
        //            modelKeyList.push(data.id);
        //        }
        //    });
        //}
        //// add the models, entities and scenarios to the list
        //selectedList.push(modelKeyList);
        //selectedList.push(controller.entitylist);
        //selectedList.push(controller.scenariolist);
        //
        //dialog.close();
        //// the buttons are dis-abled
        //var toggle = view.getModel("toggle");
        //toggle.setProperty("/toggleButtons", false);
        //
        //sas.stress.wkbench.StressAPI.runModels(controller.project.getId(), selectedList).done(function (modelsJSON) {
        //    // use the server status to update the view (the execution status)...
        //    // Execution status moves from = scheduled, executing, complete/error
        //    var executing = 0;
        //    sas.stress.models.Model._jsonifymodels(modelsJSON, controller.i18N.getResourceBundle(), executing).done(function (json, executing) {
        //        //deferred.resolve(that._setOrUpdateViewModel(json, executing));
        //        controller._poll(/*controller*/sas.stress.models.ModelsCommon._setOrUpdateViewModel(controller, json, executing));
        //    }).fail(function (e) {
        //        sas.hc.MessageBox.error(controller.i18N.getResourceBundle().getText("Problem"), e);
        //    });
        //}).fail(function (jqxhr) {
        //    var responseJSON = (jqxhr.responseJSON) ? jqxhr.responseJSON : JSON.parse(jqxhr.responseText);
        //    sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
        //});
   // },

    // fncallback: function () {
    //     return;
    // },
    // poll should be started by _initController()
    //onAfterRendering: function () {
    //    this.poll(true);
    //},
    /**
     * event comes after all the other arguments
     */
    //_listSelectionChange: function (allChkBox, e) {
    //    var ctrl = this;
    //    var existingDialog = sap.ui.getCore().byId("RunModelForEntitiesScenarios");
    //    if (existingDialog) {
    //        var list = e.getSource();
    //        if (list.getId().indexOf("scenarios-list") === 0) {
    //            ctrl.aScenarioIsSelected = (list.getSelectedItems().length > 0);
    //            // list of scenarios
    //            ctrl.scenariolist = [];
    //            _.each(list.getSelectedContexts(), function (ctx) {
    //                ctrl.scenariolist.push(ctx.getProperty("scenarioKey"));	// need to send in the scenarioKey not the ID S1191834
    //            });
    //        } else if (list.getId().indexOf("entities-list") === 0) {
    //            ctrl.anEntityIsSelected = (list.getSelectedItems().length > 0);
    //            // list of entities
    //            ctrl.entitylist = [];
    //            _.each(list.getSelectedContexts(), function (ctx) {
    //                ctrl.entitylist.push(ctx.getProperty("id"));
    //            });
    //        }
    //        // are we good to go? required scenario and entity selected?
    //        existingDialog.getBeginButton().setEnabled(ctrl.aScenarioIsSelected && ctrl.anEntityIsSelected);
    //
    //        // check the allBOX.. if it's not selected
    //        if (!allChkBox.getSelected()) {
    //            if (list.getSelectedItems().length === list.getItems().length) {
    //                allChkBox.setSelected(true);
    //            } else {
    //                allChkBox.setSelected(false);
    //            }
    //        }
    //    }
    //}
    onServerFailure: function (jqxhr) {
        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    }

});