jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.hc.ui.commons.CascadeSelect");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.models.Model");
jQuery.sap.require("sas.stress.models.model.ModelGroup");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.wkbench.widget.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.stress.config.model.HierarchyCategory");

sap.ui.controller("sas.stress.models.view.groups.CreateModelGroup",{

    onInit : function(event) {
        var view = this.getView();

        this.capabilityManager=sas.stress.wkbench.CapabilityManager.getInstance();
        this.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.models.messagebundle" });
        view.setModel(this.i18N, "i18N");
        this.router = sap.ui.core.routing.Router.getRouter("router");
        var viewId=event.getParameter("id");
        var viewKey=viewId.substring(0,viewId.indexOf("View"));
        this.router.attachExactRouteMatched("ModelGroupDetails", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, view);

        new sas.risk.common.utils.LayoutService(this.getView().bizVariablesTable, "ModelGroupDetailsBizAssumptionsTable");
        new sas.risk.common.utils.LayoutService(this.getView().variablesTable, "ModelGroupDetailsVariablesTable");
    },

    _init: function(event) {
        var modelGroupId = Number(event.getParameter("arguments").modelGroupId);
        this.refresh(modelGroupId);
    },

    refresh: function (modelGroupId) {
        var view = this.getView();
        var that = this;

        this.json = {};
        this.modelGroup = undefined;
        this._saveAsVisible = false;

        this.json = {
            listEnabled: true,
            modelIntervals: []
        };
        if (view.getModel() instanceof sap.ui.model.json.JSONModel) {
            var modelExists = true;
        } else {
            var modelCreateModel = new sas.stress.wkbench.model.RiskJSONModel(this.json);
            view.setModel(modelCreateModel);
        }
        var model = view.getModel();
        model.setProperty("/modelGroupId",modelGroupId);
        model.setProperty("/modelGroupName","");
        // the various properties of the model
        model.setProperty("/modelGroup",null);
        //model.setProperty("/modelIntervals",[]);
        model.setProperty("/models",[]);
        model.setProperty("/bizAssumptions",[]);
        model.setProperty("/outputvars",[]);
        model.setProperty("/modelsexist",false);

        var intervals=sas.stress.models.Model._getDateIntervals();
        var modelIntervals=[];
        if (_.size(intervals)) {
            _.each(intervals, function (count, interval) {
                switch(interval){
                    case "YEAR":
                        that.json.modelIntervals.push({
                            key: interval,
                            name: that.i18N.getResourceBundle().getText("modelgroup.periodtype.year.txt"),
                            count: count
                        });
                        break;
                    case "SEMIYEAR":
                        that.json.modelIntervals.push({
                            key: interval,
                            name: that.i18N.getResourceBundle().getText("modelgroup.periodtype.semiyear.txt"),
                            count: count
                        });
                        break;
                    case "QUARTER":
                        that.json.modelIntervals.push({
                            key: interval,
                            name: that.i18N.getResourceBundle().getText("modelgroup.periodtype.quarter.txt"),
                            count: count
                        });
                        break;
                    case "MONTH":
                        that.json.modelIntervals.push({
                            key: interval,
                            name: that.i18N.getResourceBundle().getText("modelgroup.periodtype.month.txt"),
                            count: count
                        });
                        break;
                    case "WEEK":
                        that.json.modelIntervals.push({
                            key: interval,
                            name: that.i18N.getResourceBundle().getText("modelgroup.periodtype.week.txt"),
                            count: count
                        });
                        break;
                    default:
                        break;
                }

            });
        }else{
            that.json.modelIntervals.push({
                key:"YEAR",
                name:that.i18N.getResourceBundle().getText("modelgroup.periodtype.year.txt"),
                count:0
            });
            that.json.modelIntervals.push({
                key:"SEMIYEAR",
                name:that.i18N.getResourceBundle().getText("modelgroup.periodtype.semiyear.txt"),
                count:0
            });
            that.json.modelIntervals.push({
                key:"QUARTER",
                name:that.i18N.getResourceBundle().getText("modelgroup.periodtype.quarter.txt"),
                count:0
            });
            that.json.modelIntervals.push({
                key:"MONTH",
                name:that.i18N.getResourceBundle().getText("modelgroup.periodtype.month.txt"),
                count:0
            });
            that.json.modelIntervals.push({
                key:"WEEK",
                name:that.i18N.getResourceBundle().getText("modelgroup.periodtype.week.txt"),
                count:0
            });
        }
        model.setProperty("/modelIntervals",this.json.modelIntervals);
        // requires a call to the server to load the categories and hierarchies.. fixes for defect S1291026
        $.when(sas.stress.config.model.HierarchyCategory.listHierarchyCategories(), sas.stress.config.model.Hierarchy.getHierarchyDetails()).done(function(categories,hierarchies){
            // if it's an edit, the template is provided
            if (modelGroupId !== 0){
                var modelGroup = sas.stress.models.model.ModelGroup.getModelGroup(modelGroupId);
                that.template=modelGroup;
                that._makeMGViewModel(modelGroup,hierarchies);
                var interval=_.find(that.json.modelIntervals,function(interval,idx){
                    // S1349000 and S1354222.. SRINI
                    return interval.key===modelGroup.modelInterval;
                });
                if (interval){
                    that._autoSelectFirstModelGroupInterval(interval.key.toString());
                } else {
                    that._autoSelectFirstModelGroupInterval("YEAR");
                }
                model.setProperty("/modelGroupName",that.modelGroup.name);
                that.navChecker.setDirty(false);
                that._toggleSaveCreateModelGroupButton(false);
                that._toggleSaveAsModelGroupButton(true,true);
            } else {
                that.template=undefined;
                that._autoSelectFirstModelGroupInterval("YEAR");
                that.modelGroup=sas.stress.models.model.ModelGroup.createModelGroup();
                // create, the button is ON.. Per Don/Dan
                that._toggleSaveCreateModelGroupButton(true);
                that._toggleSaveAsModelGroupButton(false);

                // we are dirty.. why because the save button is enabled
                that._setDirty(true);
                that._setupBizAssumptionsModel();
                that._setupOutputVarsModel(hierarchies);
            }
            model.setProperty("/modelGroup", that.modelGroup);
            model.setProperty("/models", that.modelGroup.getModels());
            model.setProperty("/modelGroup/displayForEdit", modelGroupId !== 0);

            // store these for later
            model.setProperty("/hierarchies", hierarchies);
        }).fail(function(jqxhr){
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },
    onSave:function(){
        var view=this.getView();
        view.setBusy(true);
        view.setBusyIndicatorDelay(0);

        var model=view.getModel();
        var controller = this;
        var mg=model.getProperty("/modelGroup");
        var isCreate=(mg.id==="-1");

        // var template=view.getViewData().template
        sas.stress.models.model.ModelGroup.commitChanges(model,view.getSelectedInterval(),view.udlCodeEditor.getText(),isCreate)
            .done(function(group){
                controller._setDirty(false);
                controller._makeMGViewModel(group,model.getProperty("/hierarchies"));   // use the old hierarchies... now some could have been deleted however we are not going to reload
                // we need to set the property on the view's model to use the new group
                model.setProperty("/modelGroup",group);
                model.setProperty("/modelGroupId",group.id);
                model.setProperty("/modelGroupName",group.name);
                var rb=controller.i18N.getResourceBundle();
                if (isCreate) {
                    sap.m.MessageToast.show(rb.getText("model.group.create.success.msg.fmt",group.name), {at: sap.ui.core.Popup.Dock.CenterTop});
                }else{
                    sap.m.MessageToast.show(rb.getText("model.group.save.success.msg.fmt",group.name), {at: sap.ui.core.Popup.Dock.CenterTop});
                }
                view.setBusy(false);
                //sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_GROUPS, {});
            }).fail(function (jqxhr) {
                view.setBusy(false);
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            });
    },
    onSaveAs:function(){
        var view=this.getView();
        view.setBusy(true);
        view.setBusyIndicatorDelay(0);
        var model=view.getModel();
        var controller = this;

        var existingDialog = sap.ui.getCore().byId("sas.stress.models.view.groups.ModelGroupSaveAsDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.models.view.groups.ModelGroupSaveAsDialog", this);
        var newCategoryJSON = {name: sas.stress.models.model.ModelGroup.getNewModelGroupName(),existingname:model.getData().modelGroup.name};
        var jsonModel = new sas.stress.wkbench.model.RiskJSONModel(newCategoryJSON);
        dialog.setModel(jsonModel);
        dialog.open();
    },
    _makeMGViewModel:function(template,hierarchies){
        var view=this.getView();
        if (template){
            this.modelGroup=sas.stress.models.model.ModelGroup.tempModelGroupFromExisting(template);
        }
        // fixes for S1255041
        // if the modelgroup has been created, the template is undefined. however because the save was successful, we need to update this
        this.template=template;
        view.udlCodeEditor.setText(this.template.getUDLGlueCode());
        this.json.deleteEnabled=true;

        // re-create the bizassumptions and output variables model
        this._setupBizAssumptionsModel();
        this._setupOutputVarsModel(hierarchies);

        // assume we are good
        this._toggleSaveCreateModelGroupButton(false);
        // save as is also enabled and visible
        this._saveAsVisible=true;
        this._toggleSaveAsModelGroupButton(true);
    },
    saveAsModelGroupButtonPress:function(){
        var controller = this;
        var view = this.getView();
        var model=view.getModel();
        var rb=controller.i18N.getResourceBundle();

        var saveAsDialog = sap.ui.getCore().byId("sas.stress.models.view.groups.ModelGroupSaveAsDialog");
        saveAsDialog.setBusy(true);
        var modelGroupSaveAsName = sap.ui.getCore().byId("ModelGroupSaveAsDialog.newNameInput").getValue();
        var modelGroupSaveAsVersion = sap.ui.getCore().byId("ModelGroupSaveAsDialog.newVersionInput").getValue();
        var existinggroup=model.getProperty("/modelGroup");
        var newGroup=sas.stress.models.model.ModelGroup.makeACopy(existinggroup,modelGroupSaveAsName,modelGroupSaveAsVersion,model.getProperty("/bizAssumptions"),model.getProperty("/outputvars"));
        model.setProperty("/modelGroup",newGroup);
        sas.stress.models.model.ModelGroup.commitChanges(model,view.getSelectedInterval(),view.udlCodeEditor.getText(),true)
            .done(function(group){
                saveAsDialog.setBusy(false);
                saveAsDialog.close();
                sap.m.MessageToast.show(rb.getText("model.group.save.success.msg.fmt",group.name), {at: sap.ui.core.Popup.Dock.CenterTop});
                controller._makeMGViewModel(group,model.getProperty("/hierarchies"));   // use the old hierarchies... now some could have been deleted however we are not going to reload
                model.setProperty("/modelGroup",group); // fixes for
                model.setProperty("/modelGroupId",group.id);
                model.setProperty("/modelGroupName",group.name);
                controller._setDirty(false);
                view.setBusy(false);
                //sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_GROUPS, {});
            }).fail(function (jqxhr) {
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                view.setBusy(false);
                saveAsDialog.setBusy(false);
            });
    },
    createOrEditModelGroupNameVersionLiveChange:function(e){
        var nameInput = sap.ui.getCore().byId("createModelGroupNameInput");
        var newName = nameInput.getValue().trim();
        var versionInput = sap.ui.getCore().byId("createModelGroupVersionInput");
        var newVersion = versionInput.getValue().trim();

        var isValid = true;
        if (newName == null || newName.trim() === ""){
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            nameInput.setPlaceholder(this.i18N.getProperty("modelgroup.name.placeholder.txt"));
            isValid=false;
        } else {
            //Check if Biz Assumption with this name already exists.
            /*if (this.modelGroup&&newName===this.modelGroup.name){
                isValid=false;
                this.navChecker.setDirty(false);
                return;
            }else */{
                var exists = sas.stress.models.model.ModelGroup.getModelGroupByNameAndVersion(newName, newVersion);
                if (exists) {
                    if (exists.id===this.modelGroup.id){    // fixes for S1254608 and now S1298567 in 2.3 after version introduced
                        //same Name is not save.
                        nameInput.setValueStateText("");
                        nameInput.setValueState(sap.ui.core.ValueState.None);
                        isValid=false;
                    }else {
                        //isValid = false;
                        nameInput.setValueStateText(this.i18N.getProperty("modelgroup.nameAndVersion.inUse.txt"));
                        nameInput.setValueState(sap.ui.core.ValueState.Error);
                        versionInput.setValueStateText(this.i18N.getProperty("modelgroup.nameAndVersion.inUse.txt"));
                        versionInput.setValueState(sap.ui.core.ValueState.Error);
                        isValid = false;
                    }
                } else {
                    //Name is good.
                    nameInput.setValueStateText("");
                    nameInput.setValueState(sap.ui.core.ValueState.None);
                    versionInput.setValueStateText("");
                    versionInput.setValueState(sap.ui.core.ValueState.None);
                }
            }
        }
        //Toggle Save button based on the result of validation.
        this._setDirty(isValid);

        return isValid;
    },
    createOrEditModelGroupDescLiveChange:function(e){
        var newDesc= e.getParameter("newValue");
        var dirty=true;
        if (this.modelGroup&&this.modelGroup.description!==newDesc) {
            dirty=true;
        }
        this.navChecker.setDirty(dirty);
        this._toggleSaveCreateModelGroupButton(dirty);
        this._toggleSaveAsModelGroupButton(dirty);
    },
    /*
     * We should make sure there is no model group that has the same name because this is a Save As
     * @param e {Event} event
     * @returns {boolean} isValid
     */
    saveAsModelGroupNameLiveChange:function(e){
        //var newName= e.getParameter("newValue");
        //var nameInput=e.getSource();

        var nameInput = sap.ui.getCore().byId("ModelGroupSaveAsDialog.newNameInput");
        var newName = nameInput.getValue().trim();
        var versionInput = sap.ui.getCore().byId("ModelGroupSaveAsDialog.newVersionInput");
        var newVersion = versionInput.getValue().trim();

        var isValid = true;
        if (newName == null || newName.trim() === ""){
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            nameInput.setPlaceholder(this.i18N.getProperty("modelgroup.name.placeholder.txt"));
            isValid=false;
        } else {
            var exists = sas.stress.models.model.ModelGroup.getModelGroupByNameAndVersion(newName, newVersion);
            if (exists) {
                //same Name is not save as.
                nameInput.setValueStateText(this.i18N.getProperty("modelgroup.name.inUse.different.version.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
                if (newVersion.length>0) {
                    versionInput.setValueStateText(this.i18N.getProperty("modelgroup.nameAndVersion.inUse.txt"));
                }else{
                    versionInput.setValueStateText(this.i18N.getProperty("modelgroup.name.inUse.different.version.txt"));
                }
                versionInput.setValueState(sap.ui.core.ValueState.Error);
                isValid=false;
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                versionInput.setValueStateText("");
                versionInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        return isValid;
    },
    _autoSelectFirstModelGroupInterval:function(key){
        var view=this.getView();
        var item = view.modelIntervalsList.getItemByKey(key);
        if (item) {
            view.modelIntervalsList.setSelectedItemId(item.getId());
            view.modelIntervalsList.setSelectedItemId(item.getId());
            view.modelIntervalsList.fireChange({selectedItem: item});//for some odd reason i had to manually fire it.
        }
    },
    modelgroupModelIntervalChange:function(e){
        var view=this.getView();
        var that=this;
        var model=view.getModel();
        var selecteditem= e.getParameter("selectedItem");
        var selectedInterval=selecteditem.getKey();
        var modelsByInterval=sas.stress.models.Model.getModelsByInterval(selectedInterval);
        model.setProperty("/modelsexist",modelsByInterval.length>0);
        var nodata;
        if (modelsByInterval.length===0){
            var text= "";
            var _rb=that.i18N.getResourceBundle();
            switch (selectedInterval){
                case "YEAR":
                    text=_rb.getText("modelgroup.periodtype.year.txt");
                    break;
                case "SEMIYEAR":
                    text=_rb.getText("modelgroup.periodtype.semiyear.txt");
                    break;
                case "QUARTER":
                    text=_rb.getText("modelgroup.periodtype.quarter.txt");
                    break;
                case "MONTH":
                    text=_rb.getText("modelgroup.periodtype.month.txt");
                    break;
                case "WEEK":
                    text=_rb.getText("modelgroup.periodtype.week.txt");
                    break;
            }
            text=_rb.getText("modelgroup.no.data.no.models.fmt", text);
            nodata=view.modelsTable.getNoData();
            view.modelsTable.setNoData(undefined);
            if(nodata){
                nodata.destroy();
                nodata=undefined;
            }

            view.modelsTable.setNoData(new sas.stress.wkbench.widget.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                title: "{i18N>modelgroup.no.models.defined.txt}",
                text: text,
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>model.groups.add.models.button.txt}",
                        tooltip: "{i18N>model.groups.add.models.button.tooltip}",
                        enabled: modelsByInterval.length>0,
                        press: $.proxy(that._showAddModelsDialog, that)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        }else{
            nodata=view.modelsTable.getNoData();
            view.modelsTable.setNoData(undefined);
            if(nodata){
                nodata.destroy();
                nodata=undefined;
            }
            view.modelsTable.setNoData(new sas.stress.wkbench.widget.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                title: "{i18N>modelgroup.no.models.defined.txt}",
                text: "{i18N>model.groups.add.models.button.tooltip}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>model.groups.add.models.button.txt}",
                        tooltip: "{i18N>model.groups.add.models.button.tooltip}",
                        enabled: modelsByInterval.length>0,
                        press: $.proxy(that._showAddModelsDialog, that)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        }

        this._setDirty(true);   // fixes for S1252750
    },
    onExit: function (e) {
        this.router.detachExactRouteMatched("ModelGroupDetails", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.models.view.groups.CreateModelGroup", null);
        this.navChecker.stop();
    },
    _showModelDetailsDialog:function(e) {
        var that=this;
        var view=this.getView();
        var ctx;
        if (view.modelsTable.getSelectedIndices().length===1) {
            ctx=view.modelsTable.getContextByIndex(view.modelsTable.getSelectedIndices()[0]);
        }else{
            ctx=view.modelsTable.getContextByIndex(e.getParameter("rowIndex"));

        }
        if (!ctx){
            return;
        }
        var existingmodels = view.getModel().getData().models;
        var selectedModel=_.find(existingmodels,function(model){
            return model.id===ctx.getObject().id;
        });

        var _rb=this.i18N.getResourceBundle();
        var modelDeteailsDialogView=sas.stress.wkbench.ViewManager.loadModelDetailsForGroupDialogView(selectedModel.id,_rb);
        var rightButton = new sas.hc.m.Button({
            text:_rb.getText("model.edit.close.button.txt"),
            press: function () {
                detailsDlg.close();
            }
        });

        var detailsDlg = new sas.hc.m.Dialog({
            title: _rb.getText("model.edit.models.dialog.header.title")/*"{i18N>model.add.models.dialog.header.title}"*/,
            content: modelDeteailsDialogView,
            rightButton: rightButton,
            afterClose: function () {
                detailsDlg.destroy();
            },
            contentHeight: "90%",
            contentWidth: "90%",
            initialFocus: rightButton
        });
        detailsDlg.open();
    },

    _showAddModelsDialog: function (e) {
        var that = this;
        var view = this.getView();
        var existingmodels = view.getModel().getData().models;
        var selectedModelIds = [];
        _.each(existingmodels, function (model) {
            selectedModelIds.push(model.name/*Id()*/);
        });
        var _rb=this.i18N.getResourceBundle();
        var addModelsDlgView=sas.stress.wkbench.ViewManager.loadAddModelsForGroupDialogView(this,this.getView().getSelectedInterval(),selectedModelIds,_rb);
        var addModelsDlg = new sas.hc.m.Dialog({
            title: _rb.getText("model.add.models.dialog.header.title"),
            content: addModelsDlgView,
            leftButton:addModelsDlgView.beginbutton,
            rightButton: addModelsDlgView.endbutton,
            afterClose: function () {
                this.destroy();
            },
            contentHeight:"50%",
            contentWidth:"80%",
            initialFocus: addModelsDlgView.endbutton
        });
        addModelsDlgView.attachClosing(function(e){
            addModelsDlg.close();
            if (e.getParameter("action")==="OK"){   //fixes for S1255041
                // on Add we need to set the view to dirty and enable the buttons
                that._setDirty(true);
            }
        },addModelsDlg);

        addModelsDlg.open();
    },

    _onRemoveModels: function (/*e*/) {
        var that = this;
        var view = this.getView();
        var model = view.getModel();
        var models = model.getProperty("/models");
        var existingidx = -1;
        var selectedIndices = view.modelsTable.getSelectedIndices();
        // fixes for S1266356.. need to loop in the reverse direction because we are splicing the models array
        for (var index=selectedIndices.length-1;index>=0;--index){  // loop through one by one
            var selectedIndex=selectedIndices[index];   // fixes for S1267998
            var selectedModel = view.modelsTable.getContextByIndex(selectedIndex).getObject();
            _.find(models, function (model, index) {
                if (model.modelId === selectedModel.modelId) {
                    existingidx = index;
                    return true;
                }
            });
            if (existingidx >= 0) {
                // splice.. remove the item
                models.splice(existingidx, 1);
                /*var search=that._searchForModelNameInUDLCode(selectedModel);
                if (search.length>0){
                    var udlCodeEditor=view.udlCodeEditor;
                    var modelnamesearchstring=that._runModelCode(selectedModel.name);
                    // remove the model from the UDL code
                    var result=udlCodeEditor.replace(modelnamesearchstring,"",{caseIgnored:true,wrapped:true,all:true});
                }*/
            }
            //view.getModel().setData(modeljson);
        }
        model.setProperty("/models",models);  //fixes for S1255041
        // the view is dirty, we have removed the models
        that._setDirty(true);

        view.modelsTable.setSelectedIndex(-1);
        var isValid=models.length===0;
        model.setProperty("/listEnabled",true);
        view.deleteModelsButton.setEnabled(false);
        this.navChecker.setDirty(true);
    },
    _onAddBizAssumption:function(e){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var assumptionsjson=model.getData()["bizAssumptions"];

        //var bizAssumption=sas.stress.models.model.BizAssumptions.createBizAssumption(this.modelGroup);
        var uiAssumption={id:0,name:"",value:"",description:"",committed:false};
        uiAssumption.id=_.uniqueId();
        uiAssumption.name=sas.stress.models.model.BizAssumptions.getNewBizAssumptionName(this.modelGroup,assumptionsjson);
        if (assumptionsjson.length) {
            assumptionsjson.splice(0, 0, uiAssumption);
        } else{
            assumptionsjson.push(uiAssumption);
        }

        view.assumptionValue.fireLiveChange({id:view.assumptionValue.getId(),newValue:"",value:""});
        model.setProperty("/bizAssumptions",assumptionsjson);

        //the navchecker should be dirty.
        this._setDirty(true);
    },
    _onRemoveBizAssumption:function(e){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var committedDeleted=false;

        var table= view.bizVariablesTable;
        var selectedIndices=table.getSelectedIndices();
        var assumptionsjson=model.getData()["bizAssumptions"];
        _.each(selectedIndices,function(selectedIndex){
            if (!committedDeleted){
                committedDeleted=assumptionsjson[selectedIndex].committed;
            }
            // delete the assumption from the json
            delete assumptionsjson[selectedIndex];
        });
        // remove the undefineds'
        assumptionsjson=_.without(assumptionsjson,undefined);
        model.setProperty("/bizAssumptions",assumptionsjson);
        table.setSelectedIndex(-1);
        view.deletBizVarsButton.setEnabled(assumptionsjson.length>0&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.UPDATE));

        // fixes for S1252750.. un-saved changes notification
        this._setDirty(this.template&&committedDeleted);
    },
    _bizAssumptionNameLiveChange:function(e){
        var view=this.getView();
        var newName= e.getParameter("newValue");
        var nameInput=e.getSource();
        var isValid=true;
        var model=view.getModel();
        var assumptionsjson=model.getData()["bizAssumptions"];
        var input= e.getSource();   // fixes for S1268012

        if (newName == null||newName.length===0){
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            nameInput.setPlaceholder(this.i18N.getProperty("modelgroup.name.placeholder.txt"));
            isValid=false;
        } else if(!sas.stress.wkbench.StressUtil._isValidName(newName)){    // fixes for S1255068
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.invalidname.txt"));
            isValid=false;
        } else {
            //Check if Biz Assumption with this name already exists.
            /* fixes for S1268012 */
            var customData = _.find(input.getCustomData(), function (customData) {
                return customData.getKey() === "id";
            });
            var id = customData.getValue();
            /* fixes for S1268012 end */
            var exists = _.find(assumptionsjson,function(assumption){
                // fixes for S1261257.. case insensitive compare please
                return (assumption.name.toLowerCase()===newName.toLowerCase()&&id!==assumption.id);
            });
            if (exists) {
                //isValid = false;
                nameInput.setValueStateText(this.i18N.getProperty("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
                isValid=false;
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                isValid=true;
            }
        }
        //Toggle Save button based on the result of validation.
        this._toggleSaveAsModelGroupButton(isValid);
        this._toggleSaveCreateModelGroupButton(isValid);
        this.navChecker.setDirty(true);
    },
    _bizAssumptionValueLiveChange:function(e){
        var newValue= e.getParameter("newValue");
        var valueInput=e.getSource();
        var isValid=true;
        if (newValue == null/* || newValue.trim() === ""*/){
            valueInput.setValueState(sap.ui.core.ValueState.Error);
            valueInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            valueInput.setPlaceholder(this.i18N.getProperty("modelgroup.value.placeholder.txt"));
            isValid=false;
        } else {
            //Name is good.
            valueInput.setValueStateText("");
            valueInput.setValueState(sap.ui.core.ValueState.None);
        }
        //Toggle Save button based on the result of validation.
        this._toggleSaveAsModelGroupButton(isValid);
        this._toggleSaveCreateModelGroupButton(isValid);
        this.navChecker.setDirty(true);
    },
    _bizAssumptionDescLiveChange:function(/*e*/){
        this._setDirty(true);
    },
    _onModelsTableCellClick:function(event){
        var view=this.getView();
        var deletebutton=view.deleteModelsButton;
        var detailsButton=view.detailsButton;
        var table=view.modelsTable;
        var selectedIndices=table.getSelectedIndices();
        //S1266309.. the delete button is only active when the udl Editor is active
        deletebutton.setEnabled(selectedIndices.length>0&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.DELETE));
        detailsButton.setEnabled(selectedIndices.length===1&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.VIEW_MODELS));
    },
    _onBizAssumptionsSelectionChange:function(e){
        var view=this.getView();

        switch(e.getSource().getSelectedIndices().length){
            case 0:
                view.deletBizVarsButton.setEnabled(false);
                break;
            default:
                view.deletBizVarsButton.setEnabled(true&&
                    this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.DELETE));
                break;
        }
    },
    _onInsertClick:function(event){
        var view=this.getView();
        var udlCodeEditor=view.udlCodeEditor;
        var caretPosition=udlCodeEditor.caretPosition();

        var button=event.getSource();
        var customData = _.find(button.getCustomData(), function (customData) {
            return customData.getKey() === "id";
        });
        var id = customData.getValue();
        var ctx = _.find(view.getModel().getData().models, function (model) {
            return model.id === id;
        });
        if (ctx === undefined) {
            return;
        }else{
            udlCodeEditor.insert(this._runModelCode(ctx.name));
            setTimeout(function() {
                // goto the next line after the caret
                udlCodeEditor.gotoLine(caretPosition.line +2);
            }, 100);
            this._setDirty(true);
        }
    },
    _onAddVariables:function(e){
        var controller=this;
        var view=this.getView();
        var model=view.getModel();
        var _rb=this.i18N.getResourceBundle();

        var existinghierarchies=[];
        if (model) {
            var currentopars = model.getProperty("/outputvars");
            _.each(currentopars, function (currentopvar) {
                var existinghierarchy = {
                    key: currentopvar.selectedhierarchy.key,
                    name: currentopvar.selectedhierarchy.name
                };
                existinghierarchies.push(existinghierarchy);
            });
        }

        var addVariablesDlgView=sas.stress.wkbench.ViewManager.loadAddVariablesForGroupDialogView(this,_rb,undefined,existinghierarchies);
        /*
         * This is for safety.. the close of the dialog should fire "afterClose" event and
         * the dialog should be destroyed in the handler... if it's not we make sure to
         * destroy the dialog via this
         */
        var addVariablesDlg=sap.ui.getCore().byId("modelgroup.add.variables.dialog");
        if (addVariablesDlg){
            addVariablesDlg.destroy();
        }
        addVariablesDlg = new sas.hc.m.Dialog({
            id:"modelgroup.add.variables.dialog",
            title: _rb.getText("model.add.variables.dialog.header.title"),
            content: addVariablesDlgView,
            leftButton:addVariablesDlgView.beginbutton,
            endButton: addVariablesDlgView.endbutton,
            afterClose: function () {
                this.destroy();
            },
            width: "100%",
            height:"100%",
            initialFocus: addVariablesDlgView.endbutton,
            modal:true,
            preventDefault:true
        });
        addVariablesDlgView.attachClosing(function(e){
            addVariablesDlg.close();
            if (e.getParameter("action")!=='CANCEL'){
                var data= e.getParameter("data");
                controller._processOutputVariablesData(data);
            }
        },addVariablesDlg);
        addVariablesDlg.open();
    },
    /**
     * Searches the UDL code editor for the model
     *
     * @param {object} model the model
     * @returns {*} result
     * @private
     */
    _searchForModelNameInUDLCode:function(model){
        var that=this;
        var view=this.getView();
        var udlCodeEditor=view.udlCodeEditor;
        var searchmodelname=this._runModelCode(model.name);
        // search for the removed model in the code editor
        return udlCodeEditor.search(searchmodelname,{direction:"all",caseIgnored:true,wrapped:true,all:true});
    },
    /**
     * Toggle Save button based on the result of validation.
     * @param {boolean} isValid valid or not
     * @private
     */
    _toggleSaveCreateModelGroupButton: function (isValid) {
        this.getView().saveButton.setEnabled(isValid&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.CREATE));
    },
    /**
     *
     * @param {boolean} isValid valid or not
     * @private
     */
    _toggleSaveAsModelGroupButton: function (isValid) {
        this.getView().saveAsButton.setEnabled(isValid&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.CREATE));
        this.getView().saveAsButton.setVisible(this._saveAsVisible);
    },
    _setupBizAssumptionsModel:function(){
        var view=this.getView();
        var model=view.getModel();
        var bizAssumptions=model.getProperty("/bizAssumptions")||[];
        var bizAssumption;
        if (bizAssumptions&&bizAssumptions.length){
            bizAssumptions=[];
            model.setProperty("/bizAssumptions",[]);
        }
        _.each(this.modelGroup.getBizAssumptions(),function(assumption){
            bizAssumption={id:0,name:"",value:"",description:"",committed:true};

            bizAssumption.id=assumption.getId();
            bizAssumption.name=assumption.getName();
            bizAssumption.value=assumption.getValue();
            bizAssumption.description=assumption.getDescription();

            bizAssumptions.push(bizAssumption);
        });
        model.setProperty("/bizAssumptions",bizAssumptions);
    },
    _setupOutputVarsModel:function(hierarchies){
        var that=this;
        var view=this.getView();
        var nohierachies=false;
        if (hierarchies===undefined){
            nohierachies=true;
        } else if (hierarchies.length===0){
            nohierachies=true;
        }
        if (nohierachies){
            view.variablesTable.setNoData(new sas.stress.wkbench.widget.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                title: that.i18N.getResourceBundle().getText("modelgroup.no.hierarchies.defined.txt"),
                text: that.i18N.getResourceBundle().getText("modelgroup.variables.no.hierarchies.define.txt")
            }));

            view.addOutputVarsButton.setEnabled(false);
            return;
        } else {
            view.variablesTable.setNoData(new sas.stress.wkbench.widget.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                title: that.i18N.getResourceBundle().getText("modelgroup.no.variables.defined.txt"),
                text: that.i18N.getResourceBundle().getText("modelgroup.variables.create.button.tooltip"),
                buttons: [
                    new sas.hc.m.Button({
                        text: that.i18N.getResourceBundle().getText("model.groups.variables.create.button.txt"),
                        tooltip: that.i18N.getResourceBundle().getText("modelgroup.variables.create.button.tooltip"),
                        press: $.proxy(that._onAddVariables, that),
                        enabled:that.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.UPDATE)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));

            view.addOutputVarsButton.setEnabled(true &&
                that.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.UPDATE));
        }

        var model=view.getModel();
        var outputvars=model.getProperty("/outputvars")||[];
        var data={selectedhierarchy:{},levelnames:[],levels:[],levelmembers:{}};
        if (outputvars&&outputvars.length){
            outputvars=[];
            model.setProperty("/outputvars",outputvars);
        }
        if (this.modelGroup.getOutputVars().length) {
            _.each(this.modelGroup.getOutputVars(), function (outputVar) {
                data = {
                    selectedhierarchy: {},
                    classificationVars: [],
                    levelmembers: {}
                };
                var hierarchyId = outputVar.getHierarchyId();
                var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyId);

                var selectedhierarchy = {
                    key: hierarchy.getHierarchyKey(),
                    code: hierarchy.getHierarchyCode(),
                    name: hierarchy.getName(),
                    version: hierarchy.getVersion()
                };
                data.selectedhierarchy = selectedhierarchy;
                var levelnames = [];
                var levels=[];
                var levelNameInfo = {isEnabled: true, name: "", columnanme: ""};
                var hierarchylevelNames = hierarchy.getLevelNames();
                _.each(hierarchylevelNames, function (levelname, idx) {
                    levelNameInfo = {isEnabled: true, name: "", columnname: ""};
                    levelinfo={key:"",text:""};
                    if (outputVar.getClassificationVars()[idx]) {
                        levelNameInfo.isEnabled = true;
                        levelNameInfo.name = levelname;
                        levelNameInfo.columnname = outputVar.getClassificationVars()[idx].getName();
                    } else {
                        levelNameInfo.isEnabled = false;
                        levelNameInfo.name = levelname;
                        levelNameInfo.columnname = "";
                    }
                    levelnames.push(levelNameInfo);
                });
                data.classificationVars = levelnames;
                var mappings=_.reduce(hierarchy.getLineItems(),function(memo,lineitem){
                    var found=_.find(outputVar.getOutputVarMappings(),function(mapping){
                        return mapping.getMemberName().toLocaleLowerCase()===lineitem.getMemberName().toLocaleLowerCase();
                    });
                    if (found){
                        memo.push(found);
                    }
                    return memo;
                },[]);

                data.levelmembers=mappings;
                outputvars.push(data);
            });
        }

        model.setProperty("/outputvars",outputvars);
    },
    _processOutputVariablesData:function(data){
        var that=this;
        var view=this.getView();
        var model=view.getModel();

        var json=model.getProperty("/outputvars");
        var newdata={};
        var existingindex=-1;
        /*
        * All the other processing is not necessay... we should look for the existing
        * hierarchy in the json... json is populated, in case of, the variables table "cellClick" handler
        * if the hierarchy is found we should use that to update the selections..
        * else, it's a new definition and is added
        */
        var existing= _.find(json,function(eachvar,index){
            if (eachvar.selectedhierarchy.key===data.getData().selectedhierarchy.key){
                existingindex=index;
                return true;
            }
        });
        if (existing) {
            // update the existing with the latest changes
            existing.selectedhierarchy = data.getData().selectedhierarchy;
            existing.classificationVars = data.getData().classificationVars;
            existing.levelmembers = data.getData().selectoutputvarnames?_.filter(data.getData().levelmembers, function (member) {
                return member.columnname.length > 0;
            }):[];
            //update the json
            json.splice(existingindex, 1, existing);
        }else {
            newdata={
                selectedhierarchy:data.getData().selectedhierarchy,
                classificationVars:data.getData().classificationVars,
                levelmembers:[]
            };
            newdata.levelmembers = data.getData().selectoutputvarnames?_.filter(data.getData().levelmembers, function (member) {
                return member.columnname.length > 0;
            }):[];
            json.push(newdata);
        }
        model.setProperty("/outputvars",[]);
        model.setProperty("/outputvars",json);

        this._setDirty(true);   // fixes for S1252750
    },
    _onVariablesTableCellClick:function(e){
        var view=this.getView();
        var controller=this;
        var model=view.getModel();
        var table=view.variablesTable;

        var existinghierarchies=[];
        var ctx = e.getSource().getContextByIndex(e.getParameter("rowIndex"));
        var selected=ctx.getObject();
        console.log(selected);
        existinghierarchies.push(selected.selectedhierarchy.key);

        var _rb=this.i18N.getResourceBundle();
        /*
        * This is for safety.. the close of the dialog should fire "afterClose" event and
        * the dialog should be destroyed in the handler... if it's not we make sure to
        * destroy the dialog via this
        */
        var addVariablesDlg=sap.ui.getCore().byId("modelgroup.add.variables.dialog");
        if (addVariablesDlg){
            addVariablesDlg.destroy();
        }
        var addVariablesDlgView=sas.stress.wkbench.ViewManager.loadAddVariablesForGroupDialogView(this,_rb,selected,existinghierarchies);
        addVariablesDlg = new sas.hc.m.Dialog({
            id:"modelgroup.add.variables.dialog",
            title: _rb.getText("model.edit.variables.dialog.header.title"),
            content: addVariablesDlgView,
            leftButton:addVariablesDlgView.beginbutton,
            endButton: addVariablesDlgView.endbutton,
            afterClose: function () {
                this.destroy();
            },
            width: "100%",
            height:"100%",
            initialFocus: addVariablesDlgView.endbutton,
            modal:true,
            preventDefault:true
        });
        addVariablesDlgView.attachClosing(function(e){
            addVariablesDlg.close();
            if (e.getParameter("action")!=='CANCEL'){
                var data= e.getParameter("data");
                controller._processOutputVariablesData(data);
            }
        },addVariablesDlg);

        addVariablesDlg.open();
    },
    _setDirty: function (isDirty) {
        //This needs to be changed
        this.getView().saveButton.setEnabled(isDirty);  //Enable Save button
        this.getView().saveAsButton.setEnabled(true);   //Save As button is always enabled

        // Let Appswitcher know
        //var dirty=this.navChecker.getDirty(); // fixes for S1282173
        this.navChecker.setDirty(/*dirty||*/isDirty);   // make sure the navchecker's dirtiness is taken into accoun
    },
    _onUDLCodeTabClick:function(){  // fixes for S1266309
        // the buttons are now enabled because we are in the UDL editor
        this._toggleModelHeaderButtons();
    },
    _onPreviewCodeTabClick:function(){
        var view=this.getView();
        var controller=this;
        var model=view.getModel();
        this._toggleModelHeaderButtons();  // fixes for S1266309

        sas.stress.models.model.ModelGroup.previewCode(model,view.getSelectedInterval(),view.udlCodeEditor.getText())
            .done(function(preview){
                view.previewCodeEditor.setText(preview.code);
            })
            .fail(function(jqxhr){
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            });
    },
    _toggleModelHeaderButtons:function(){    // fixes for S1266309
        var view=this.getView();
        var modelsexist=view.getModel().getProperty("/modelsexist");
        var table=view.modelsTable;
        var selectedIndices=table.getSelectedIndices();

        // the buttons are enabled only if the UDL editor is active
        view.addModelsButton.setEnabled(/*view.udlActive&&*/modelsexist);
        //S1266309.. the delete button is only active when the udl Editor is active
        view.deleteModelsButton.setEnabled(selectedIndices.length>0&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.DELETE_MODELS));
    },
    _onRemoveOutputVars:function(){
        var view=this.getView();
        var table=view.variablesTable;
        var selectedindices=table.getSelectedIndices();
        if (selectedindices.length>0) {
            var model=view.getModel();
            var json=model.getProperty("/outputvars");
            _.each(selectedindices,function(selected){
                var opvar=json[selected];
                delete json[selected];
            });

            json= _.without(json,undefined);
            model.setProperty("/outputvars", json);

            table.setSelectedIndex(-1);
            this._setDirty(true);   // fixes for S1252750
        }
    },
    _onVariablesTableSelectionChange:function(e){
        var view=this.getView();
        var controller=this;
        var table=view.variablesTable;

        view.deleteOutputVarsButton.setEnabled(table.getSelectedIndices().length>0&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MODELGROUP.DELETE));
    },
    _enableSaveButtons:function(){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var modelsjson=model.getProperty("/models");
        /*var isValid=true;

        _.each(modelsjson,function(model){
            var search=that._searchForModelNameInUDLCode(model);
            if (search&&search.length) {    // if the search returns something? fixes for S1255041
                if (isValid) {  // if it's false in the loop keep it at false
                    isValid = search.length > 0;
                }
            }
        });*/
        // code for model inserted.. therefore, valid?
        this._toggleSaveAsModelGroupButton(true);
        this._toggleSaveCreateModelGroupButton(true);

    },
    _runModelCode:function(modelname){
        return "%sst_run_model(model_name="+modelname+");\n";   // fixes for S1251059
    },
    _onPrint:function(e) {
        var view=this.getView();
        var sourceBtn= e.getSource();
        var codeeditor;
        switch(sourceBtn.getId()){
            case "codeEditorPrint":
                codeeditor=view.udlCodeEditor;
                break;
            case "previewEditorPrint":
                codeeditor=view.previewCodeEditor;
                break;
        }
        var printDialog = window.open('about:blank', "printWindow", 'resizable=1,scrollbars=0,width=800,height=600');
        printDialog.document.write('<link type="text/css" rel="stylesheet" href="resources/themes/sas/hc/ui/commons/themes/sas_corporate/library.css">');
        printDialog.document.write(codeeditor.getHTML({lineNumber: true})); // fixes for S1250303
        printDialog.document.close();
        window.setTimeout(function(){
            printDialog.print();
        }, 100);
    },
    _udlUndoPressed:function() {
        var view=this.getView();
        var model=view.getModel();
        var modelsjson=model.getProperty("/models");

        view.udlCodeEditor.undo();
        var text=view.udlCodeEditor.getSelectedText();
        var searchstring="model_name=";
        var index=text.indexOf(searchstring);
        var modelname=text.substring(index+searchstring.length,text.length-3);  //
        var existsInUDL=false;
        _.each(modelsjson,function(model){
            if (model.name===modelname){
                existsInUDL=true;
            }
        });
        if (existsInUDL){
            //view.validationLabel.setText("The UDL code contains reference to a model " +modelname+ " that is not part of the model group");
            var rb=this.i18N.getResourceBundle();
            sas.hc.m.MessageBox.warning(rb.getText("modelgroup.codeeditor.undo.model.warning.fmt", modelname), sas.config.appName); // fixes for S1266279
        } else {
            view.validationLabel.setText("");
        }

        // code for model inserted.. therefore, valid?
        this._toggleSaveAsModelGroupButton(!existsInUDL);
        this._toggleSaveCreateModelGroupButton(!existsInUDL);
    },
    _udlRedoPressed:function() {
        var view=this.getView();
        var model=view.getModel();
        var modelsjson=model.getProperty("/models");

        view.udlCodeEditor.redo();
        var text=view.udlCodeEditor.getSelectedText();
        // code for model inserted.. therefore, valid?
        this._toggleSaveAsModelGroupButton(true);
        this._toggleSaveCreateModelGroupButton(true);
    },
    _validateUDLCode:function(){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var modelsjson=model.getProperty("/models");
        var isValid={isValid:true,modelname:""};

        _.each(modelsjson,function(model){
            var search=that._searchForModelNameInUDLCode(model);
            if (isValid) {  // if it's false in the loop keep it at false
                isValid.isValid = search.length > 0;
                isValid.modelname=model.name;
            }
        });

        return isValid;
    },
    _isValidName:function(name){
        var isValid=name.match(/[^\w]/gi);
        if (isValid&&isValid.length){
            return false;
        }
        return true;
    },
    _udlTextChanged:function(e){
        console.info(e);
        var removedText = e.getParameter("removedText");
        var text = e.getParameter("text");
        if ((removedText.length || text.length) && (removedText !== text)) {    // fixes for S1256396
            // something has changed
            this._setDirty(true);
        }
    },
    _onSegementedButtonPress:function(e){
        var that=this;
        var view=this.getView();
        var selectedKey=e.getSource().getSelectedKey();
        switch(selectedKey){
            case "1"/*"udlButton"*/:
            {
                view.editorTypeLabel.setText(this.i18N.getResourceBundle().getText("model.udl.code.editor.type.label"));
                setTimeout(function() {
                    view.codeLayout.removeContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, 0);
                    view.codeLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, view.udlCodeLayout);

                    that._onUDLCodeTabClick();
                    view.udlCodeEditor.refresh(); // need to refresh because of new UI since RFW2.3
                }, 100);
            }
                break;
            case "2"/*"previewButton"*/:
            {
                view.editorTypeLabel.setText(this.i18N.getResourceBundle().getText("model.preview.code.editor.type.label"));
                setTimeout(function() {
                    view.codeLayout.removeContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, 0);
                    view.codeLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, view.previewCodeLayout);

                    that._onPreviewCodeTabClick();
                }, 100);
            }
                break;
            default:
                // the initial call, therefore, ignore?
                break;
        }
    },
    onCellClick:function(event){
        var controller=this;
        var ctx = event.getSource().getContextByIndex(event.getParameter("rowIndex"));
        var cell = event.getParameter("cellControl");
        if (cell.getId().indexOf("addButton")===-1) {
            this._showModelDetailsDialog(event);
        }
    }
});