jQuery.sap.declare("sas.stress.packages.PackageUtil");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sas.stress.packages.PackageUtil = {
    getStatusIcon: function (status) {
        if (status) {
            status = status.toUpperCase();
        }
        switch (status) {
            case null:
                return null;
            case "COMPLETED":
            case "SUCCESS":
                return sas.icons.HC.COMPLETESTATUS;
            case "ERROR":
                return sas.icons.HC.ERRORSTATUS;
            case "EXECUTING":
                return sas.icons.HC.PROGRESSSTATUS;
            default:
                return null;
        }
    },

    getStatusText: function (rb, status) {
        if (status) {
            status = status.toUpperCase();
        }
        switch (status) {
            case null:
                return "";
            case "COMPLETED":
            case "SUCCESS":
                return rb.getText("packages.status.Completed.txt");
            case "ERROR":
                return rb.getText("packages.status.Error.txt");
            case "EXECUTING":
                return rb.getText("packages.status.Running.txt");
            case "PUBLISHED":
                return rb.getText("packages.status.Published.txt");
            default:
                return "";
        }
    },

    getWorksheetStatusText: function (rb, status, numValidationErrors) {
        if (status) {
            status = status.toUpperCase();
        }
        switch (status) {
            case null:
                return "";
            case "COMPLETED":
            case "SUCCESS":
            {
                if (0 === numValidationErrors) {
                    return rb.getText("packages.status.Completed.txt");
                } else {
                    return rb.getText("packages.status.CompletedWithErrors.fmt.txt", [numValidationErrors]);
                }
            }
            case "ERROR":
                return rb.getText("packages.status.WorksheetValidationFailed.txt");
            case "EXECUTING":
                return rb.getText("packages.status.Running.txt");
            default:
                return "";
        }
    },

    getReportStatusText: function (rb, status) {
        if (status) {
            status = status.toUpperCase();
        }
        switch (status) {
            case null:
                return "";
            case "COMPLETED":
            case "SUCCESS":
            case "PUBLISHED":
                return rb.getText("packages.status.Published.txt");
            default:
                return rb.getText("packages.status.NotPublished.txt");
        }
    },

    getReportStatusIcon: function (status) {
        if (status) {
            status = status.toUpperCase();
        }
        switch (status) {
            case null:
                return null;
            case "COMPLETED":
            case "SUCCESS":
            case "PUBLISHED":
                return sas.icons.HC.COMPLETESTATUS;
            default:
                return sas.icons.HC.NEUTRALSTATUS;
        }
    },

    setTaxonomyMap: function (rb, taxonomyData) {
        this.addHierarchies("entityHierarchy", taxonomyData.entityHierarchies);
        this.addTaxonomyOptions("legalEntity", taxonomyData.legalEntityIds);
        this.addTaxonomyOptions("currency", taxonomyData.currencies);
        this.addTaxonomyOptions("jurisdiction", taxonomyData.jurisdictions);
        this.addTaxonomyOptions("taxonomy", taxonomyData.taxonomies);
        this.addTaxonomyOptions("module", taxonomyData.modules);
        //this.addHierarchies("templates",taxonomyData.templates);
        this.addHierarchies("timeHierarchy", taxonomyData.timeHierarchies);
        this.addTaxonomyOptions("timeHierarchyLevel", taxonomyData.timeIds);

        var selectAll = [{
            "ownerCode": "",
            "code": "All",
            "name": rb.getText("newPackageDialog.selectAll.defaultItem.txt")
        }];
        this.addTaxonomyOptions(selectAll);
    },

    // deprecated
    createTimeHierarchyMap: function (timeHierarchies) {
        if (!this.time) {
            this.time = {};
        }

        if (timeHierarchies) {
            for (var i = 0; i < timeHierarchies.length; i++) {
                var hierarchy = timeHierarchies[i];
                if (hierarchy && hierarchy.code === "TIME") {
                    this.time[hierarchy.hierarchyKey] = hierarchy.name;
                    var levels = hierarchy.levelNames;
                    if (levels) {
                        for (var j = 0; j < levels.length; j++) {
                            this.time[hierarchy.hierarchyKey + "." + (j + 1)] = levels[j];
                        }
                    }
                }
            }
        }
        return this.time;
    },

    addHierarchies: function (hierarchyType, hierarchies) {
        if (!this[hierarchyType]) {
            this[hierarchyType] = {};
        }
        if (hierarchies) {
            for (var i = 0; i < hierarchies.length; i++) {
                var hierarchy = hierarchies[i];
                if (hierarchy) {
                    hierarchy.nameVersion = this.getHierarchyNameVersion(hierarchy);
                    this[hierarchyType][hierarchy.key] = hierarchy;
                }
            }
        }
    },
    getHierarchyNameVersion: function(hierarchy) {
        return sas.stress.wkbench.StressUtil.getNameVersion(hierarchy.name, hierarchy.version);
    },
    addTaxonomyOptions: function (mapType, options) {
        if (!this[mapType]) {
            this[mapType] = {};
        }
        if (options) {
            for (var i = 0; i < options.length; i++) {
                var entry = options[i];
                if (entry) {
                    this[mapType][entry.code] = entry; //.name;
                }
            }
        }
    },
    getTaxonomyValue: function (hierarchyType, key) {
        return this[hierarchyType][key].nameVersion || this[hierarchyType][key].name;
    },
    // S1341497
    getModuleSheetTemplates: function(key){ // return the sheet templates for the module
        return this["module"][key].sheetTemplates || [];
    },
    findHierarchyKey: function(hierarchyType, name, version) {

        var foundEntry = _.findWhere(this[hierarchyType], {name: name});
        if (version) {
            foundEntry = _.findWhere(this[hierarchyType], {nameVersion: sas.stress.wkbench.StressUtil.getNameVersion(name, version)});
        }
        return foundEntry? foundEntry.key || foundEntry.code : null;
        //var mapKey = null;
        //_.each(this[hierarchyType], function(value, key) {
        //    if (value === sas.stress.wkbench.StressUtil.getNameVersion(name, version)) {
        //        mapKey = key;
        //    }
        //});
        //return mapKey;
    }
};
