jQuery.sap.require('sas.hc.m.Input');
jQuery.sap.require('sas.hc.m.Select');
jQuery.sap.require('sas.hc.m.CheckBox');
jQuery.sap.require('sas.hc.ui.commons.TextView');
jQuery.sap.require('sas.hc.ui.commons.layout.VerticalLayout');
jQuery.sap.require("sas.hc.ui.commons.CascadeSelect");
jQuery.sap.require("sas.hc.ui.commons.CascadeSelectItem");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");

sap.ui.jsfragment("sas.stress.packages.view.NewPackageDialog", {
    createContent: function (controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var nameLabel = new sas.hc.m.Label({
            text: rb.getText("newPackageDialog.name.control.label"),
            required: true
        });

        var packageNameInput = new sas.hc.m.Input({
            id: controller.getView().createId("NewPackageDialog.packageNameInput"),
            type: sap.m.InputType.Text,
            disableValidation: false,
            maxLength: 64
        });
        nameLabel.setLabelFor(packageNameInput);

        var descriptionLabel = new sas.hc.m.Label({
            text: rb.getText("newPackageDialog.description.control.label")
        }).addStyleClass("sapUiTinyMarginTop");
        var packageDescriptionInput = new sas.hc.m.TextArea({
            id: controller.getView().createId("NewPackageDialog.packageDescriptionInput"),
            value: "{/description}",
            maxLength: 256
        });
        descriptionLabel.setLabelFor(packageDescriptionInput);

        var entityHierarchy = rb.getText("newPackageDialog.entityHierarchy.control.label");
        var legalEntityId = rb.getText("newPackageDialog.legalEntityIdentifier.control.label");
        var entityHierarchyCascadeSelect = new sas.hc.ui.commons.CascadeSelect({
            id: controller.getView().createId("NewPackageDialog.entityHierarchyCascadeSelect"),
            captions: [entityHierarchy, legalEntityId],
            orientation: "Horizontal",
            titlePosition: "Top",
            enabled: true,
            valueChange: $.proxy(controller.toggleOkButton, controller),
            widthOfSelect:"50%"
        }).addStyleClass("riskCascadeSelect").addStyleClass("riskCascadeSelectRequired");
        var taxonomyFactory = function (sId, oContext) {
            var oItem = new sas.hc.ui.commons.CascadeSelectItem({
                text: "{name}",
                key: "{code}"
            });
            oItem.setModel(oContext.getModel());
            var aItems = oContext.getProperty("items");
            if (aItems && aItems.length > 0) {
                oItem.bindAggregation("items", oContext.getPath() + "/items", taxonomyFactory);
            }
            return oItem;
        };
        var hierarchyItemFactory = function (sId, oContext) {
            var oItem = new sas.hc.ui.commons.CascadeSelectItem({
                text: "{nameVersion}",
                key: "{key}"
            });
            oItem.setModel(oContext.getModel());
            var aItems = oContext.getProperty("items");
            if (aItems && aItems.length > 0) {
                oItem.bindAggregation("items", oContext.getPath() + "/items", taxonomyFactory);
            }
            return oItem;
        };
        entityHierarchyCascadeSelect.bindItems("/entityHierarchies", hierarchyItemFactory);

        //var legalEntityLabel = new sas.hc.m.Label({
        //    text: rb.getText("newPackageDialog.legalEntityIdentifier.control.label"),
        //    required: true
        //});
        //var legalEntityIdentifierItemTemplate = new sas.hc.ui.core.Item({
        //    key: "{code}",
        //    text: "{name}"
        //});
        //var legalEntityIdentifierSelect = new sas.hc.m.Select({
        //    id: controller.getView().createId("NewPackageDialog.legalEntityIdentifierSelect"),
        //    width: "100%",
        //    items: {
        //        path: "/legalEntityIds",
        //        template: legalEntityIdentifierItemTemplate
        //    },
        //    change: $.proxy(controller.toggleOkButton, controller)
        //});
        //legalEntityLabel.setLabelFor(legalEntityIdentifierSelect);

        var currencyLabel = new sas.hc.m.Label({
            text: rb.getText("newPackageDialog.currency.control.label"),
            required: true
        }).addStyleClass("sapUiTinyMarginTop");
        var currencyItemTemplate = new sas.hc.ui.core.Item({ key: "{code}", text: "{name}" });
        var currencySelect = new sas.hc.m.Select({
            id: controller.getView().createId("NewPackageDialog.currencySelect"),
            width: "100%",
            items: {
                path: "/currencies",
                template: currencyItemTemplate
            },
            change: $.proxy(controller.toggleOkButton, controller)
        });
        currencyLabel.setLabelFor(currencySelect);

        var jurisdiction = rb.getText("newPackageDialog.jurisdiction.control.label");
        var taxonomy = rb.getText("newPackageDialog.taxonomy.control.label");
        var module = rb.getText("newPackageDialog.module.control.label");
        // S1341497
        var taxonomyCascadeSelect = new sas.hc.ui.commons.CascadeSelect({
            id: controller.getView().createId("NewPackageDialog.taxonomyCascadeSelect"),
            captions: [jurisdiction, taxonomy, module],
            orientation: "Vertical",
            titlePosition: "Top",
            enabled: true,
            valueChange: $.proxy(controller.toggleOkButton, controller)
        }).addStyleClass("riskCascadeSelect").addStyleClass("riskCascadeSelectRequired");
        taxonomyCascadeSelect.bindItems("/jurisdictions", taxonomyFactory);

        this.workSheetList = new sas.risk.common.widgets.SelectTable({
            id: this.createId("NewPackageDialog.workSheetList"),
            nameAttribute: "name",
            listItems: "{/moduleSheetTemplates}",
            listName:  rb.getText("newPackageDialog.sheetTemplate.control.label"),
            visibleRowCount: 4,
            //columnHeaderVisible: true,
            selectionChange: $.proxy(controller.sheetSelected, controller)
        });
        var workSheetListLabel = new sas.hc.m.Label({
            text: rb.getText("newPackageDialog.sheetTemplate.control.label"),
            labelFor: this.workSheetList,
            required: true
        }).addStyleClass("sapUiTinyMarginTop");

        this.workSheetList.bindRows("/moduleSheetTemplates");

        var timeHierarchy = rb.getText("newPackageDialog.timeHierarchy.control.label");
        var timeHierarchyLevel = rb.getText("newPackageDialog.timeHierarchyLevel.control.label");
        var timeHierarchyCascadeSelect = new sas.hc.ui.commons.CascadeSelect({
            id: controller.getView().createId("NewPackageDialog.timeHierarchyCascadeSelect"),
            captions: [timeHierarchy, timeHierarchyLevel],
            orientation: "Horizontal",
            titlePosition: "Top",
            enabled: true,
            valueChange: $.proxy(controller.toggleOkButton, controller),
            widthOfSelect:"50%"
        }).addStyleClass("riskCascadeSelect").addStyleClass("riskCascadeSelectRequired");
        //
        //var timeFactory = function (id, context) {
        //    var item = new sas.hc.ui.commons.CascadeSelectItem({
        //        text: "{timeHierarchyModel>name}",
        //        key: "{timeHierarchyModel>key}"
        //    });
        //    item.setModel(context.getModel(), "timeHierarchyModel");
        //    var sItems = context.getProperty("levelNames");
        //    if (sItems && sItems.length > 0) {
        //        item.bindAggregation("items", "timeHierarchyModel>" + context.getPath() + "/levelNames", timeFactory);
        //    }
        //
        //    return item;
        //};
        //timeHierarchyCascadeSelect.bindItems("timeHierarchyModel>/time", timeFactory);
        timeHierarchyCascadeSelect.bindItems("/timeHierarchies", hierarchyItemFactory);

        var statusLabel = new sas.hc.m.Label({
            text: rb.getText("newPackageDialog.status.control.label")
        }).addStyleClass("sapUiTinyMarginTop");
        var statusInput = new sas.hc.m.Input({
            id: controller.getView().createId("NewPackageDialog.statusInput"),
            value: "{/filingStatus}",
            type: sap.m.InputType.Text,
            disableValidation: false,
            maxLength: 256
        });
        statusLabel.setLabelFor(statusInput);

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsM: 2,
                columnsL: 2,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: nameLabel,
                            fields: [packageNameInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: descriptionLabel,
                            fields: [packageDescriptionInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            //label: legalEntityLabel,
                            fields: [entityHierarchyCascadeSelect] //legalEntityIdentifierSelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: currencyLabel,
                            fields: [currencySelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [taxonomyCascadeSelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: workSheetListLabel,
                            fields: [this.workSheetList]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [timeHierarchyCascadeSelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: statusLabel,
                            fields: [statusInput]
                        })
                    ]
                })
            ]
        });
        //form.addStyleClass("stressLargeForm");

        var dialog = new sas.hc.m.Dialog({
            id: controller.getView().createId("sas.stress.packages.view.NewPackageDialog"),
            initialFocus: packageNameInput,
            content: [form],
            beginButton: new sas.hc.m.Button({
                id: controller.getView().createId("NewPackageDialog.savePackageButton"),
                text: rb.getText("newPackageDialog.save.action.label"),
                press: $.proxy(controller.newPackageDialogOkButtonPress, controller),
                enabled: false
            }),
            endButton: new sas.hc.m.Button({
                text: rb.getText("button.cancel.action.label"),
                press: function () {
                    dialog.close();
                }
            }),
            afterClose: function () {
                dialog.destroy();
            },
            contentHeight: "100%",
            contentWidth: "45rem"
        }).allowTextSelection(false).addStyleClass("stressPreventTextSelection");

        return dialog;
    }
});
