jQuery.sap.require("sas.hc.m.MessageBox");
jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.packages.PackageAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");

sap.ui.controller("sas.stress.packages.view.PackageManager", {
    onInit: function () {
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.packages.messagebundle" });
        this.getView().setModel(i18N, "i18N");
        this._pollTimeOutIDs = [];
        this.editMode = false;
        this.taxonomy = "";
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.CREATE_PACKAGE, this.onAddPackages, this);
        new sas.risk.common.utils.LayoutService(this.getView().packageTable, "PackageManagerTable");
    },

    toggleButtons: function () {
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var selectedPackage = this._getFilingPackage();
        var view = this.getView();
        // The package exists and the status isn't null or executing.
        var packageStatusNullOrNotExecuting = (null !== selectedPackage) && (selectedPackage.packageStatus === null ||
            selectedPackage.packageStatus.toUpperCase() !== "EXECUTING");
        // The regenerate (run) button should only be enabled if a package is selected and the package isn't currently executing.
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        view.runPackageBtn.setEnabled(packageStatusNullOrNotExecuting &&
            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PACKAGE.RUN));
        view.addPackagesBtn.setEnabled(!project.isLocked() && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PACKAGE.CREATE));
        view.removeBtn.setEnabled(!project.isLocked() && null !== selectedPackage);
        view.runPackageBtn.setEnabled(!project.isLocked());
        view.editPackageBtn.setEnabled(!project.isLocked() && null !== selectedPackage);
        // The validate button should only be enabled if a package is selected, sheets exist, and the package isn't currently executing.
        var allowValidation = (packageStatusNullOrNotExecuting && undefined !== selectedPackage.sheets) &&
            (null !== selectedPackage.sheets && selectedPackage.sheets.length > 0);
        view.validatePackageBtn.setEnabled(!project.isLocked() && allowValidation);
    },

    onRowSelectionChanged: function (oEvent) {
        var view = this.getView();
        var idx = oEvent.getParameter('rowIndex');
        var filingPackage = {};
        this.toggleButtons();
        if (view.packageTable.isIndexSelected(idx)) {
            filingPackage = this._getFilingPackage(idx);
        }
        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME,
            sas.stress.wkbench.EventBusConstants.PACKAGE_SELECTION_CHANGED, filingPackage);
    },

    _getFilingPackage: function (index) {
        var view = this.getView();
        var filingPackage = null;
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.packageTable.getSelectedIndex();
        }
        if (index >= 0) {
            var cxt = view.packageTable.getContextByIndex(index);
            if (cxt) {
                filingPackage = view.packageTable.getModel().getProperty(cxt.sPath);
            }
        }
        return filingPackage;
    },

    /**
     * Select the package in the table that matches the given name
     * @param {string} packageName name of the package
     * @return {boolean} if package is found
     */
    selectPackageByName: function (packageName) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var view = this.getView();
        // Ensure no selection exists in the table already.
        view.packageTable.clearSelection();
        // We have to loop through the model because the row we want to select might not have been loaded yet
        var allPackages = view.packageTable.getModel().getData();
        var foundPackage = false;
        for (var i = 0; i < allPackages.length; i++) {
            var filingPackage = this._getFilingPackage(i);
            if (null !== filingPackage && filingPackage.name.toLocaleLowerCase() === packageName.toLocaleLowerCase()) {
                // Package found. Add it to the selection.
                view.packageTable.setSelectedIndex(i);
                // Scroll to the selected package.
                view.packageTable.setFirstVisibleRow(i);
                // We found the package, no need to keep searching.
                foundPackage = true;
                break;
            }
        }
        if (!foundPackage) {
            // Specified package wasn't found, throw an error.
            sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidPackage.fmt.txt",
                [packageName]), rb.getText("error.window.title"));
        }
        return foundPackage;
    },

    validateWorkSheetButtonPress: function (evt) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var view = this.getView();
        var projectKey = this.projectId;
        var packageObj = this._getFilingPackage();
        if (null !== packageObj) {
            //Show dialog to validate a package.
            var existingDialog = view.byId("sas.stress.packages.view.ValidateWorkSheetDialog");
            if (existingDialog) {
                existingDialog.destroy();
            }
            var dialog = sap.ui.jsfragment("sas.stress.packages.view.ValidateWorkSheetDialog", this);
            dialog.setTitle(rb.getText("validateWorkSheetDialog.window.validateOneFilingPackage.title.fmt.txt", [packageObj.name]));
            var packageData = {
                projectKey: projectKey,
                packageId: packageObj.id,
                sheets: packageObj.sheets
            };
            var model = new sas.stress.wkbench.model.RiskJSONModel(packageData);
            dialog.setModel(model);
            dialog.open();
            this._toggleValidateButton();

            // handle process orchestration parameters, if they exist
            if (this.validatePackageParams) {
                this.handleProcessOrchestration();
            }
        }
    },

    handleProcessOrchestration: function () {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var worksheetNames = this.validatePackageParams.worksheetNames;
        if (worksheetNames && worksheetNames.length > 0) {
            var worksheets = this.worksheetList.getListItems();
            var invalidWorksheetNames = [];
            var selectedWorksheets = [];
            var firstVisibleRow = -1;
            for (var paramIndex = 0; paramIndex < worksheetNames.length; ++paramIndex) {
                var foundWorksheet = false;
                for (var worksheetIndex = 0; worksheetIndex < worksheets.length; ++worksheetIndex) {
                    if (worksheets[worksheetIndex].sheetName.toLocaleLowerCase() === worksheetNames[paramIndex].toLocaleLowerCase()) {
                        // It matches, select it!
                        worksheets[worksheetIndex].selected = true;
                        selectedWorksheets.push(worksheets[worksheetIndex]);
                        // Need to manually fire the selection change, otherwise it won't validate in the UI.
                        //this.worksheetList.fireSelectionChange(
                        //    {
                        //        listItem: worksheets[worksheetIndex],
                        //        listItems: [worksheets[worksheetIndex]],
                        //        selected: true
                        //    }
                        //);
                        // Scroll to the selected index, needs to be in a timeout...
                        //setTimeout(function (index) {
                        //    controller.worksheetList.$().find('ul').find('li:nth-child(' + index + ')').focus();
                        //}, 100, worksheetIndex);
                        foundWorksheet = true;
                        if (firstVisibleRow < 0) {
                            firstVisibleRow = worksheetIndex;
                        }
                        break;
                    }
                }
                if (!foundWorksheet) {
                    // Specified worksheet does not exist.
                    invalidWorksheetNames.push(worksheetNames[paramIndex]);
                }
            }
            if (selectedWorksheets.length > 0) {
                this.worksheetList.fireSelectionChange({selectedItems: selectedWorksheets});
                if (firstVisibleRow > 0) {
                    this.worksheetList.setFirstVisibleRow(firstVisibleRow);
                }
            }
            // Were there any invalid worksheets specified?
            if (invalidWorksheetNames.length > 0) {
                // Initial string is just the first invalid worksheet name.
                var errorParams = invalidWorksheetNames[0];
                for (var invalidIndex = 1; invalidIndex < invalidWorksheetNames.length; ++invalidIndex) {
                    errorParams = rb.getText("packages.processOrchestration.invalidWorksheetsFormat.fmt.txt",
                        [errorParams, invalidWorksheetNames[invalidIndex]]);
                }
                if (invalidWorksheetNames.length === 1) {
                    // Error for a single invalid worksheet.
                    sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidWorksheet.fmt.txt",
                        [errorParams]), rb.getText('error.window.title'));
                } else {
                    // Error for 2 or more invalid worksheets.
                    sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidWorksheets.fmt.txt",
                        [errorParams]), rb.getText('error.window.title'));
                }
            }
        }
        delete this.validatePackageParams;
    },

    validateWorkSheetDialogValidateButtonPress: function (event) {
        var controller = this;
        var view = this.getView();
        var validateWorkSheetDialog = view.byId("sas.stress.packages.view.ValidateWorkSheetDialog");
        validateWorkSheetDialog.setBusy(true);
        var projectId = this.projectId;
        var worksheets = controller.worksheetList.getSelectedItems();
        var sheetIds = [];
        if (worksheets) {
            for (var i = 0; i < worksheets.length; i++) {
                var worksheet = worksheets[i];
                if (worksheet.sheetKey === undefined) {
                    worksheet = worksheets[i].getBindingContext().getObject();
                } else {
                    worksheet.editable = false; //disable if is already validating
                }
                if (worksheet.sheetKey) {
                    sheetIds.push(worksheet.sheetKey);
                }
            }
        }
        var validateData = {
            "ids": sheetIds
        };
        sas.stress.packages.PackageAPI.validatePackage(projectId, validateData).done(function () {
            validateWorkSheetDialog.setBusy(false);
            validateWorkSheetDialog.close();

            controller._poll(5); // Poll immediately in order to update any packages that contain the validating worksheets.
        }).fail(function (jqxhr) {
            validateWorkSheetDialog.setBusy(false);
            controller.onServerFailure(jqxhr);
        });
    },

    onCellClick: function (event) {
        var ctx = event.getSource().getContextByIndex(event.getParameter("rowIndex"));
        var cell = event.getParameter("cellControl");
        // Check if the click was in the messages cell, if not then we want to ignore it.
        if (-1 < cell.getId().indexOf("package-messages")) {
            var obj = ctx.getObject();
            // Only acknowledge the click if at least one message exists.
            if (obj.messages.length > 0) {
                //var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.wkbench");
                var existingDialog = this.getView().byId("sas.stress.reports.view.MessagesDialog");
                if (existingDialog) {
                    existingDialog.destroy();
                }
                var dialog = sap.ui.jsfragment("sas.stress.reports.view.MessagesDialog", this);

                var messagesStr="";
                if (obj.messages.length > 1) {
                    messagesStr = rb.getText("list.bullet.txt") + obj.messages[0];
                    for (var index = 1; index < obj.messages.length; ++index) {
                        messagesStr += "\n"+ rb.getText("list.bullet.txt") + obj.messages[index];
                    }
                }else{
                    messagesStr = obj.messages[0];
                }

                var messagesDialogJSON = {
                    messages: messagesStr
                };
                var model = new sas.stress.wkbench.model.RiskJSONModel(messagesDialogJSON);
                dialog.setModel(model);
                dialog.open();
                // Prevent message click event from also selecting row
                event.preventDefault();
            }
        }
    },

    validateWorkSheetDialogFilterSelectHandler: function () {
        this._toggleValidateButton();
    },

    _toggleValidateButton: function () {
        var sItems = this.worksheetList.getSelectedItems();
        var enableRunButtons = (sItems && sItems.length > 0);
        this.validateWorksheetButton.setEnabled(enableRunButtons);
    },

    getNewPackageName: function () {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var newPackageName = "";
        var count = 1;
        var view = this.getView();
        while (newPackageName === "") {
            var tempPackageName = rb.getText("package.name.fmt", [count++]);
            // check if package with this name exists
            var pkg = _.find(view.packageTable.getModel().getData(), function (pkg) {
                return (pkg.name !== null) && (pkg.name.trim().toLocaleLowerCase() === tempPackageName.trim().toLocaleLowerCase());
            });
            // if not, use this name
            if (pkg === undefined) {
                newPackageName = tempPackageName;
            }
        }
        return newPackageName;
    },

    onAddPackages: function (channel, event, data) {
        var controller = this;
        var view = this.getView();
        var pid = this.projectId;
        if (!pid && data) {
            pid = data.projectId;
            this.projectId = pid;
        }
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var existingDialog = view.byId("sas.stress.packages.view.NewPackageDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.packages.view.NewPackageDialog", this);
        dialog.setTitle(rb.getText("newPackageDialog.window.title"));
        dialog.open();
        dialog.setBusy(true);
        var packageName = this.getNewPackageName();
        var nameInput = view.byId("NewPackageDialog.packageNameInput");
        nameInput.attachLiveChange($.proxy(this.packageNameInputLiveChange, this, null));
        nameInput.setValue(packageName);
        //nameInput.fireLiveChange({ value: packageName });
        $.when(sas.stress.packages.PackageAPI.getTaxonomyValues(pid)) //, sas.stress.config.StressAPI.getHierarchies())
            .done(function (taxonomyValuesJSON) { //}, hierarchyValuesJSON) {
                if (taxonomyValuesJSON) {
                    _.each(taxonomyValuesJSON.entityHierarchies, function(entityHierarchy) {
                        entityHierarchy.nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(entityHierarchy.name, entityHierarchy.version);
                        entityHierarchy.items = [];
                        _.each(taxonomyValuesJSON.legalEntityIds, function(legalEntity) {
                            if (legalEntity && legalEntity.ownerCode === entityHierarchy.code) {
                                if (!entityHierarchy.items) {
                                    entityHierarchy.items = [];
                                }
                                entityHierarchy.items.push(legalEntity);
                            }
                        });
                    });
                    for (var k = 0; k < taxonomyValuesJSON.jurisdictions.length; k++) {
                        var jurisdiction = taxonomyValuesJSON.jurisdictions[k];
                        if (jurisdiction) {
                            jurisdiction.items = [];
                            for (var i = 0; i < taxonomyValuesJSON.taxonomies.length; i++) {
                                var taxonomy = taxonomyValuesJSON.taxonomies[i];
                                if (taxonomy && taxonomy.ownerCode === jurisdiction.code) {
                                    taxonomy.items = [];
                                    for (var j = 0; j < taxonomyValuesJSON.modules.length; j++) {
                                        var module = taxonomyValuesJSON.modules[j];
                                        module.sheetTemplates= []; // S1341497
                                        if (module && module.ownerCode === taxonomy.code) {
                                            taxonomy.items.push(module);
                                        }
                                        // S1341497
                                        for (var sti=0; sti < taxonomyValuesJSON.templates.length; ++sti){
                                            var sheetTemplate=taxonomyValuesJSON.templates[sti];
                                            if (sheetTemplate&&sheetTemplate.ownerCode===module.code){
                                                module.sheetTemplates.push(sheetTemplate);
                                            }
                                        }
                                    }
                                    jurisdiction.items.push(taxonomy);
                                }
                            }
                        }
                    }
                    _.each(taxonomyValuesJSON.timeHierarchies, function(timeHierarchy) {
                        timeHierarchy.nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(timeHierarchy.name, timeHierarchy.version);
                        timeHierarchy.items = [];
                        _.each(taxonomyValuesJSON.timeIds, function(timeId) {
                            if (timeId && timeId.ownerCode === timeHierarchy.code) {
                                timeHierarchy.items.push(timeId);
                            }
                        });
                    });

                    var selectEntity = {
                        "code": "",
                        "name": rb.getText("newPackageDialog.selectEntity.defaultItem.txt")
                    };
                    taxonomyValuesJSON.legalEntityIds.unshift(selectEntity);

                    var selectCurrency = {
                        "code": "",
                        "name": rb.getText("newPackageDialog.selectCurrency.defaultItem.txt")
                    };
                    taxonomyValuesJSON.currencies.unshift(selectCurrency);

                    taxonomyValuesJSON.editMode = false;
                }
                var taxonomyModel = new sas.stress.wkbench.model.RiskJSONModel(taxonomyValuesJSON);
                taxonomyModel.setProperty("/moduleSheetTemplates",[]);
                taxonomyModel.setProperty("/sheetTemplatesOwnerCode","");
                dialog.setModel(taxonomyModel);
                //var sheetTemplatesModel= new sas.stress.wkbench.model.RiskJSONModel({sheetTemplates:[]});
                //dialog.setModel(sheetTemplatesModel,"templates");

                // handle process orchestration parameters, if specified
                if (controller.createPackageParams) {
                    if (controller.createPackageParams.packageName && controller.createPackageParams.packageName.length > 0) {
                        var nameInput = view.byId("NewPackageDialog.packageNameInput");
                        nameInput.setValue(controller.createPackageParams.packageName);
                        nameInput.fireLiveChange({ value: controller.createPackageParams.packageName });
                    }
                    //if (!controller.createPackageParams.entityHierarchy && taxonomyValuesJSON.entityHierarchies) {
                    //    controller.createPackageParams.entityHierarchy = taxonomyValuesJSON.entityHierarchies[0].nameVersion;
                    //}

                    if (controller.createPackageParams.entityHierarchy && controller.createPackageParams.entityHierarchy.length > 0) {
                        var entityHierarchySelect = view.byId("NewPackageDialog.entityHierarchyCascadeSelect");
                        var entityKey = sas.stress.packages.PackageUtil.findHierarchyKey("entityHierarchy",
                            controller.createPackageParams.entityHierarchy, controller.createPackageParams.entityHierarchyVersion);
                        var selectedKeys = [entityKey];
                        if (controller.createPackageParams.legalEntityIdentifier && controller.createPackageParams.legalEntityIdentifier.length > 0) {
                            selectedKeys.push(controller.createPackageParams.legalEntityIdentifier);
                            //var legalEntityIdentifierSelect = view.byId("NewPackageDialog.legalEntityIdentifierSelect");
                            //legalEntityIdentifierSelect.setSelectedKey(controller.createPackageParams.legalEntityIdentifier);
                            //if (null === legalEntityIdentifierSelect.getSelectedItem()) {
                            //    sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidLegalEntityIdentifier.fmt.txt",
                            //        [controller.createPackageParams.legalEntityIdentifier]), rb.getText("error.window.title"));
                            //}
                        }
                        entityHierarchySelect.setSelectedKeys(selectedKeys);
                        var validationJson = controller.validateProvidedKeysForCascadeSelect(entityHierarchySelect, selectedKeys);
                        // Handle any errors that were found.
                        if (validationJson.valid === false) {
                            var errorMessages = ["packages.processOrchestration.invalidEntityHierarchy.fmt.txt",
                                "packages.processOrchestration.invalidLegalEntityIdentifier.fmt.txt"];
                            var nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(
                                controller.createPackageParams.entityHierarchy, controller.createPackageParams.entityHierarchyVersion);
                            var errorValues = [nameVersion, controller.createPackageParams.legalEntityIdentifier];
                            sas.hc.m.MessageBox.error(rb.getText(errorMessages[validationJson.invalidIndex],
                                [errorValues[validationJson.invalidIndex]]), rb.getText("error.window.title"));
                        }
                    }
                    if (controller.createPackageParams.currency && controller.createPackageParams.currency.length > 0) {
                        var currencySelect = view.byId("NewPackageDialog.currencySelect");
                        currencySelect.setSelectedKey(controller.createPackageParams.currency);
                        if (null === currencySelect.getSelectedItem()) {
                            sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidCurrency.fmt.txt",
                                [controller.createPackageParams.currency]), rb.getText("error.window.title"));
                        }
                    }
                    if (controller.createPackageParams.jurisdiction && controller.createPackageParams.jurisdiction.length > 0) {
                        var taxonomySelect = view.byId("NewPackageDialog.taxonomyCascadeSelect");
                        var selectedKeys = [controller.createPackageParams.jurisdiction];
                        if (controller.createPackageParams.taxonomy && controller.createPackageParams.taxonomy.length > 0) {
                            selectedKeys.push(controller.createPackageParams.taxonomy);
                            if (controller.createPackageParams.module && controller.createPackageParams.module.length > 0) {
                                selectedKeys.push(controller.createPackageParams.module);
                            }
                        }
                        taxonomySelect.setSelectedKeys(selectedKeys);
                        var validationJson = controller.validateProvidedKeysForCascadeSelect(taxonomySelect, selectedKeys);
                        // Handle any errors that were found.
                        if (validationJson.valid === false) {
                            var errorMessages = ["packages.processOrchestration.invalidJurisdiction.fmt.txt",
                                "packages.processOrchestration.invalidTaxonomy.fmt.txt", "packages.processOrchestration.invalidModule.fmt.txt"];
                            sas.hc.m.MessageBox.error(rb.getText(errorMessages[validationJson.invalidIndex],
                                [selectedKeys[validationJson.invalidIndex]]), rb.getText("error.window.title"));
                        }
                    }
                    if (controller.createPackageParams.timeHierarchy && controller.createPackageParams.timeHierarchy.length > 0) {
                        var timeHierarchySelect = view.byId("NewPackageDialog.timeHierarchyCascadeSelect");
                        var timeHierarchyKey = sas.stress.packages.PackageUtil.findHierarchyKey("timeHierarchy",
                            controller.createPackageParams.timeHierarchy, controller.createPackageParams.timeHierarchyVersion);
                        var selectedKeys = [timeHierarchyKey];
                        if (controller.createPackageParams.timeHierarchyLevel && controller.createPackageParams.timeHierarchyLevel.length > 0) {
                            var levelKey = sas.stress.packages.PackageUtil.findHierarchyKey("timeHierarchyLevel",
                                controller.createPackageParams.timeHierarchyLevel);
                            selectedKeys.push(levelKey);
                        }
                        timeHierarchySelect.setSelectedKeys(selectedKeys);
                        var validationJson = controller.validateProvidedKeysForCascadeSelect(timeHierarchySelect, selectedKeys);
                        // Handle any errors that were found.
                        if (validationJson.valid === false) {
                            var errorMessages = ["packages.processOrchestration.invalidTimeHierarchy.fmt.txt",
                                "packages.processOrchestration.invalidTimeHierarchyLevel.fmt.txt"];
                            var nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(
                                controller.createPackageParams.timeHierarchy, controller.createPackageParams.timeHierarchyVersion);
                            var errorValues = [nameVersion, controller.createPackageParams.timeHierarchyLevel];
                            sas.hc.m.MessageBox.error(rb.getText(errorMessages[validationJson.invalidIndex],
                                [errorValues[validationJson.invalidIndex]]), rb.getText("error.window.title"));
                        }
                    }
                }

                // Start of section for handling the time hierarchy information
                //var timeJSON = {
                //    "time": []
                //};
                //
                //var selectedTimeHierarchyKey;
                //var selectedTimeHierarchyLevelKey;
                //// Iterate through all hierarchies and find the time hierarchies.
                //var timeIndex = 0;
                //for (var hierarchyIndex = 0; hierarchyIndex < hierarchyValuesJSON.length; hierarchyIndex++) {
                //    if (hierarchyValuesJSON[hierarchyIndex].code === "TIME") {
                //        timeJSON.time[timeIndex] = hierarchyValuesJSON[hierarchyIndex];
                //        // process orchestration: get the key for the specified time hierarchy so it can be selected below
                //        if (controller.createPackageParams && controller.createPackageParams.timeHierarchy &&
                //            timeJSON.time[timeIndex].name === controller.createPackageParams.timeHierarchy) {
                //            selectedTimeHierarchyKey = timeJSON.time[timeIndex].key;
                //        }
                //        // Collect all level names for the time hierarchy.
                //        for (var levelNameIndex = 0; levelNameIndex < timeJSON.time[timeIndex].levelNames.length; levelNameIndex++) {
                //            var level = {};
                //            level.name = timeJSON.time[timeIndex].levelNames[levelNameIndex];
                //            level.key = levelNameIndex + 1;
                //            // Replace the name string with an object that has name and key
                //            timeJSON.time[timeIndex].levelNames[levelNameIndex] = level;
                //
                //            // process orchestration: get the key for the specified time hierarchy level so it can be selected below
                //            if (selectedTimeHierarchyKey && controller.createPackageParams.timeHierarchyLevel &&
                //                level.name === controller.createPackageParams.timeHierarchyLevel) {
                //                selectedTimeHierarchyLevelKey = level.key;
                //            }
                //        }
                //        ++timeIndex;
                //    }
                //}
                //var timeHierarchyModel = new sas.stress.wkbench.model.RiskJSONModel(timeJSON);
                //dialog.setModel(timeHierarchyModel, "timeHierarchyModel");
                //
                //// Handle process orchestration parameters
                //if (selectedTimeHierarchyKey) {
                //    var timeHierarchySelect = view.byId("NewPackageDialog.timeHierarchyCascadeSelect");
                //    var selectedTimeKeys = [selectedTimeHierarchyKey];
                //    if (selectedTimeHierarchyLevelKey) {
                //        selectedTimeKeys.push(selectedTimeHierarchyLevelKey);
                //    } else if (controller.createPackageParams.timeHierarchyLevel) {
                //        // It was specified, but it wasn't found. Show an error.
                //        sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidTimeHierarchyLevel.fmt.txt",
                //            [controller.createPackageParams.timeHierarchyLevel]), rb.getText("error.window.title"));
                //    }
                //    timeHierarchySelect.setSelectedKeys(selectedTimeKeys);
                //} else if (controller.createPackageParams && controller.createPackageParams.timeHierarchy) {
                //    // It was specified, but it wasn't found. Show an error.
                //    sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.invalidTimeHierarchy.fmt.txt",
                //        [controller.createPackageParams.timeHierarchy]), rb.getText("error.window.title"));
                //}

                delete controller.createPackageParams;
                controller.toggleOkButton();
                dialog.setBusy(false);
            }).fail(function (jqxhr) {
            dialog.setBusy(false);
            controller.onServerFailure(jqxhr);
        });
    },

    onEditPackage: function (event) {
        var view = this.getView();
        var selectedPackage = this._getFilingPackage();
        if (null !== selectedPackage) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
            var existingDialog = view.byId("sas.stress.packages.view.NewPackageDialog");
            if (existingDialog) {
                existingDialog.destroy();
            }
            var dialog = sap.ui.jsfragment("sas.stress.packages.view.NewPackageDialog", this);
            dialog.setTitle(rb.getText("editPackageDialog.window.title"));

            var entityHierarchyKey = selectedPackage.entityHierarchyKey;
            var legalEntityCode = selectedPackage.legalEntityCode;
            var currencyCode = selectedPackage.currencyCode;
            var moduleCode = (selectedPackage.moduleCode === "") ? "All" : selectedPackage.moduleCode;
            var taxonomyCode = selectedPackage.taxonomyCode;
            var jurisdictionCode = selectedPackage.jurisdictionCode;
            var timeHierarchy = selectedPackage.timeHierarchy;
            var timeHierarchyLevelKey = selectedPackage.timeHierarchyLevel;
            var entityHierarchySelect = view.byId("NewPackageDialog.entityHierarchyCascadeSelect");
            var currencySelect = view.byId("NewPackageDialog.currencySelect");
            var taxonomySelect = view.byId("NewPackageDialog.taxonomyCascadeSelect");
            var timeHierarchySelect = view.byId("NewPackageDialog.timeHierarchyCascadeSelect");
            var packageJSON = {
                "projectKey": selectedPackage.projectKey,
                "name": selectedPackage.name,
                "description": selectedPackage.description,
                "filingStatus": selectedPackage.filingStatus
            };

            var legalEntityIds = [{
                "code": legalEntityCode,
                "name": sas.stress.packages.PackageUtil.getTaxonomyValue("legalEntity", legalEntityCode)
            }];
            packageJSON.entityHierarchies = [{
                "key": entityHierarchyKey,
                "nameVersion": sas.stress.packages.PackageUtil.getTaxonomyValue("entityHierarchy", entityHierarchyKey),
                "items": legalEntityIds
            }];

            packageJSON.currencies = [{
                "code": currencyCode,
                "name": sas.stress.packages.PackageUtil.getTaxonomyValue("currency", currencyCode)
            }];
            var modules = [{
                "code": moduleCode,
                "name": sas.stress.packages.PackageUtil.getTaxonomyValue("module", moduleCode)
            }];
            var taxonomies = [{
                "code": taxonomyCode,
                "name": sas.stress.packages.PackageUtil.getTaxonomyValue("taxonomy", taxonomyCode),
                "items": modules
            }];
            packageJSON.jurisdictions = [{
                "code": jurisdictionCode,
                "name": sas.stress.packages.PackageUtil.getTaxonomyValue("jurisdiction", jurisdictionCode),
                "items": taxonomies
            }];
            var timeHierarchyLevels = [{
                "code": timeHierarchyLevelKey,
                "name": sas.stress.packages.PackageUtil.getTaxonomyValue("timeHierarchyLevel", timeHierarchyLevelKey)
            }];
            packageJSON.timeHierarchies = [{
                "key": timeHierarchy,
                "nameVersion": sas.stress.packages.PackageUtil.getTaxonomyValue("timeHierarchy", timeHierarchy),
                "items": timeHierarchyLevels
            }];
            packageJSON.editMode = true;
            packageJSON.key = selectedPackage.key;
            var packageModel = new sas.stress.wkbench.model.RiskJSONModel(packageJSON);
            dialog.setModel(packageModel);
            //var timeHierarchyName = sas.stress.packages.PackageUtil.getTaxonomyValue("time", timeHierarchy);
            //var timeHierarchyLevel = timeHierarchy + "." + timeHierarchyLevelKey;
            //var timeJSON = {
            //    "time": [{
            //        "key": timeHierarchy,
            //        "name": timeHierarchyName,
            //        "levelNames": [{
            //            "key": timeHierarchyLevelKey,
            //            "name": sas.stress.packages.PackageUtil.getTaxonomyValue("time", timeHierarchyLevel)
            //        }]
            //    }]
            //};
            //var timeModel = new sas.stress.wkbench.model.RiskJSONModel(timeJSON);
            //dialog.setModel(timeModel, "timeHierarchyModel");
            var nameInput = view.byId("NewPackageDialog.packageNameInput");
            nameInput.attachLiveChange($.proxy(this.packageNameInputLiveChange, this, selectedPackage.name));
            nameInput.setValue(selectedPackage.name);
            //nameInput.fireLiveChange({ value: selectedPackage.name });
            entityHierarchySelect.setSelectedKeys([entityHierarchyKey, legalEntityCode]);
            entityHierarchySelect.setEnabled(false);
            //legalEntityIdentifier.setSelectedKey(legalEntityCode);
            //legalEntityIdentifier.setEnabled(false);
            currencySelect.setSelectedKey(currencyCode);
            currencySelect.setEnabled(false);
            taxonomySelect.setSelectedKeys([jurisdictionCode, taxonomyCode, moduleCode]);
            taxonomySelect.setEnabled(false);
            timeHierarchySelect.setSelectedKeys([timeHierarchy, timeHierarchyLevelKey]);
            timeHierarchySelect.setEnabled(false);

            var sheetTemplates = sas.stress.packages.PackageUtil.getModuleSheetTemplates(moduleCode) || [];
            if (sheetTemplates.length) {
                var sortedDefs = _.sortBy(sheetTemplates, "name");
                _.each(sortedDefs,function(template){
                    template.editable=false;
                });
                dialog.getModel().setProperty("/moduleSheetTemplates", sortedDefs);
            }

            this.toggleOkButton();
            dialog.open();
        }
    },

    _executionsInProcess: function (filingPackages) {
        // Iterate through all given packages. Return true if any of the packages themselves are executing OR any child worksheets.
        for (var i = 0; i < filingPackages.length; i++) {
            var pkg = filingPackages[i];
            if (pkg) {
                if (null !== pkg.packageStatus && pkg.packageStatus.toUpperCase() === "EXECUTING") {
                    // Found a package that is currently executing, no need to continue looking.
                    return true;
                } else if (pkg.sheets) {
                    this._disableSheetsValidationInProcess(pkg.sheets);

                    // Iterate through all sheets for the package and determine if any are currently running
                    for (var sheetIndex = 0; sheetIndex < pkg.sheets.length; ++sheetIndex) {
                        if (pkg.sheets[sheetIndex].validationStatus && pkg.sheets[sheetIndex].validationStatus.toUpperCase() === "EXECUTING") {
                            // Found a worksheet that is currently validating, no need to continue looking.
                            return true;
                        }
                    }
                }
            }
        }
    },

    _updateValidationDialog: function (packages) {
        // Iterate through all given packages. update the worksheets in the validation dialog.
        if (this.worksheetList) {
            var packageData = this.worksheetList.getModel().getData();

            var matchingPackage = _.find(packages, function(package) {
                return (package.packageId = packageData.packageId);
            });

            if (matchingPackage) {
                _.each(packageData.sheets, function(sheet) {
                    if (sheet.validationStatus) {
                        var matchingSheet = _.find(matchingPackage.sheets, function(aSheet) {
                            return (sheet.name === aSheet.name);
                        });
                        // only copy the validationStatus so that we don't lose the selection state
                        sheet.validationStatus = matchingSheet.validationStatus;
                    }
                });
                // in case some of sheet is still validating, need to disable
                this._disableSheetsValidationInProcess(packageData.sheets);
                this.worksheetList.getModel().refresh();
            }
        }
    },

    _disableSheetsValidationInProcess: function (sheets) {
        //S1263752: don't allow validating again if validation is in process
        _.each(sheets, function(sheet) {
            if (sheet.validationStatus && sheet.validationStatus.toUpperCase() === "EXECUTING") {
                sheet.editable = false;
            } else {
                sheet.editable = true;
            }
        });
    },

    _poll: function (waitTime) {
        if (undefined === waitTime) {
            waitTime = 2500;
        }
        var controller = this;
        this._clearTimeout();
        controller._pollTimeOutIDs.push(setTimeout(function () {
            sas.stress.packages.PackageAPI.getPackages(controller.projectId).done(function (oData) {
                if (oData && oData.items) {
                    if (controller._executionsInProcess(oData.items)) {
                        controller._poll();
                    } else {
                        controller._clearTimeout();
                    }
                    // We always need to update the model, because SOME validations may have completed during polling.
                    // We don't want to wait until everything completes before updating the view.
                    controller.updatePackageTableData(oData);
                    // update the model of the validation dialog if it is up
                    controller._updateValidationDialog(oData.items);
                    // Get the selected filing package and fake its selection (for the worksheets and reports tables)
                    var filingPackage = controller._getFilingPackage();
                    sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME,
                        sas.stress.wkbench.EventBusConstants.PACKAGE_SELECTION_CHANGED, filingPackage);
                }
            }).fail(function (jqXHR) {
                controller._clearTimeout();
                var responseJSON = JSON.parse(jqXHR.responseText);
                sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
            });
        }, waitTime));
    },

    _clearTimeout: function () {
        // clear the timeout ID
        _.each(this._pollTimeOutIDs, function (timeoutid) {
            clearTimeout(timeoutid);
        });
        this._pollTimeOutIDs = [];
    },

    _calculateWorksheetAndReportProgressForPackage: function (filingPackage) {
        var notRunCount = 0;
        var errorCount = 0;
        var runningCount = 0;
        var validatedSuccessfully = 0;
        var validatedWithErrors = 0;
        if (filingPackage.sheets && null !== filingPackage.sheets) {
            for (var sheetIndex = 0; sheetIndex < filingPackage.sheets.length; ++sheetIndex) {
                var validationStatus = filingPackage.sheets[sheetIndex].validationStatus;
                if (undefined === validationStatus || null === validationStatus || "" === validationStatus) {
                    ++notRunCount;
                } else if ("EXECUTING" === validationStatus.toUpperCase()) {
                    ++runningCount;
                } else if ("ERROR" === validationStatus.toUpperCase()) {
                    ++errorCount;
                } else {
                    if (filingPackage.sheets[sheetIndex].validationErrorsCount === 0) {
                        ++validatedSuccessfully;
                    } else {
                        ++validatedWithErrors;
                    }
                }
            }
        }
        filingPackage.sheetData = {
            notRunCount: notRunCount,
            errorCount: errorCount,
            runningCount: runningCount,
            validatedSuccessfully: validatedSuccessfully,
            validatedWithErrors: validatedWithErrors
        };

        var notPublishedCount = 0;
        var publishedCount = 0;
        if (filingPackage.reportConfigs && null !== filingPackage.reportConfigs) {
            for (var reportIndex = 0; reportIndex < filingPackage.reportConfigs.length; ++reportIndex) {
                var reportStatus = filingPackage.reportConfigs[reportIndex].reportConfigStatus;
                // Checking for undefined/null first so that toUpperCase() in the next check doesn't fail.
                if (undefined === reportStatus || null === reportStatus || "" === reportStatus) {
                    ++notPublishedCount;
                } else if ("SUCCESS" === reportStatus.toUpperCase()) {
                    ++publishedCount;
                } else {
                    ++notPublishedCount;
                }
            }
        }
        filingPackage.reportData = {
            notPublishedCount: notPublishedCount,
            publishedCount: publishedCount
        };
        return filingPackage;
    },

    updatePackageTableData: function (data) {
        var view = this.getView();
        var modelData;
        if (data && data.items) {
            for (var packageIndex = 0; packageIndex < data.items.length; ++packageIndex) {
                this._calculateWorksheetAndReportProgressForPackage(data.items[packageIndex]);
                var sheetTemplates=sas.stress.packages.PackageUtil.getModuleSheetTemplates(data.items[packageIndex].moduleCode);
                var templateKeys={};
                _.each(data.items[packageIndex].templateCodes,function(code){
                    templateKeys[code]=true;
                });
                var selectedTemplates=_.filter(sheetTemplates,function(template){
                    return templateKeys[template.code];
                }, data.items[packageIndex].templateCodes);
                _.each(selectedTemplates,function(template){
                    template.selected=true;
                });

            }
            modelData = data.items;
        } else {
            modelData = [];
        }

        var packageModel = view.packageTable.getModel();
        if (packageModel instanceof sap.ui.model.json.JSONModel) {
            packageModel.setData(modelData);
            this.toggleButtons();
        } else {
            packageModel = new sas.stress.wkbench.model.RiskJSONModel(modelData);
            view.packageTable.setModel(packageModel);

            //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
            //var sorters = [new sap.ui.model.Sorter("name", false)];
            //view.packageTable.getBinding("rows").sort(sorters);
        }
    },

    _getPackageIndexInModelData: function (modelData, filingPackage) {
        if (null !== filingPackage) {
            // Iterate through all packages in the model to find the one with the same key. Note that because of sorting
            // on the table, the index in the table does not match the index in the model.
            for (var i = 0; i < modelData.length; i++) {
                if (modelData[i].key === filingPackage.key) {
                    return i;
                }
            }
        }
        // The package was not found in the view
        return -1;
    },

    onPackageRun: function () {
        var ctrl = this;
        var view = ctrl.getView();
        var selectedPackage = this._getFilingPackage();
        if (null !== selectedPackage) {
            var projectId = ctrl.projectId;
            sas.stress.packages.PackageAPI.regeneratePackage(projectId, selectedPackage.id).done(function (packageData) {
                var model = view.packageTable.getModel();
                var modelData = model.getData();
                var indexInModel = ctrl._getPackageIndexInModelData(modelData, selectedPackage);
                if (indexInModel > -1) {
                    ctrl._calculateWorksheetAndReportProgressForPackage(packageData);
                    modelData[indexInModel] = packageData;
                    model.refresh();

                    // Get the selected filing package and fake its selection (for the worksheets and reports tables)
                    // var filingPackage = controller._getFilingPackage();
                    sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME,
                        sas.stress.wkbench.EventBusConstants.PACKAGE_SELECTION_CHANGED, packageData);
                    ctrl.toggleButtons();
                    ctrl._poll();
                }
            }).fail(ctrl.onServerFailure);
        }
    },

    onRemovePackage: function () {
        var ctrl = this;
        var view = ctrl.getView();
        var selectedPackage = this._getFilingPackage();
        if (selectedPackage !== null) {
            var projectId = ctrl.projectId;
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
            var buttons = [sas.hc.m.MessageBox.Action.YES, sas.hc.m.MessageBox.Action.NO];
            var title = rb.getText("packageManager.remove.confirmation.window.title");
            var msg = rb.getText("packageManager.remove.confirmation.fmt.txt", [selectedPackage.name]);
            sas.hc.m.MessageBox.warning(msg, title, buttons, null, function (oAction) {
                if (oAction === sas.hc.m.MessageBox.Action.YES) {
                    var afterDeleteFn = function () {
                        var model = view.packageTable.getModel();
                        var modelData = model.getData();
                        var indexInModel = ctrl._getPackageIndexInModelData(modelData, selectedPackage);
                        if (indexInModel > -1) {
                            // Clear the selection and remove the package from the model.
                            view.packageTable.setSelectedIndex(-1);
                            modelData.splice(indexInModel, 1);
                            model.refresh();
                            var rows = view.packageTable.getBinding().getLength();
                            var vData = {
                                validate: (rows === 0),
                                zeroState: (rows === 0)
                            };
                            sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME,
                                sas.stress.wkbench.EventBusConstants.DELETE_PACKAGE,
                                vData);
                        }
                    };

                    sas.stress.packages.PackageAPI.deletePackage(projectId, selectedPackage.id).done(afterDeleteFn).fail(ctrl.onServerFailure);
                }
            });
        }
    },

    validateProvidedKeysForCascadeSelect: function (cascadeSelect, expectedKeys) {
        var view = this.getView();
        var validationJson = { valid: true, invalidIndex: -1 };
        var childSelects = cascadeSelect._getSelects();
        for (var selectIndex = 0; selectIndex < childSelects.length; ++selectIndex) {
            var keyOfSelectedItem = childSelects[selectIndex].getSelectedItem().getKey().toLocaleLowerCase();
            // If keys were specified, we need to validate that the selected key matches the given key. Otherwise
            // we will just ensure that the selected item has a key (not just a default).
            if (expectedKeys !== undefined && selectIndex < expectedKeys.length && keyOfSelectedItem !== String(expectedKeys[selectIndex]).toLocaleLowerCase()) {
                // The key doesn't match what was specified, it is invalid.
                validationJson.invalidIndex = selectIndex;
                validationJson.valid = false;
                break;
            } else if (keyOfSelectedItem.length === 0) {
                // No key specified, it is invalid.
                validationJson.invalidIndex = selectIndex;
                validationJson.valid = false;
                break;
            }
        }
        var dialog = view.byId("sas.stress.packages.view.NewPackageDialog");
        if (dialog){
            /*
             S1341497
             if the cascadeSelect is for taxonomy,
             if selections of the taxonomy cascade select are valid
             1. get the selection indices of each of the cascade select item
             2. look up the "jurisdictions" model for the module using the selected index
             3. get the sheetTemplates, and
             4. store on the model
             */
            if (cascadeSelect===view.byId("NewPackageDialog.taxonomyCascadeSelect")){   // S1341497
                if (validationJson.valid){
                    var sheetTemplates;
                    var selectedItemIndices = [];
                    var jurisdictions, sortedDefs, itemIndex, selectedItemIndex;

                    // get the existing property...
                    var moduleSheetTemplates=dialog.getModel().getProperty("/moduleSheetTemplates");
                    if (!moduleSheetTemplates.length) { // fixes... if the property is present do not update 2-way binding gets messed up
                        for (itemIndex = 0; itemIndex < childSelects.length; ++itemIndex) {
                            selectedItemIndex = childSelects[itemIndex].getSelectedIndex();
                            if (selectedItemIndex > 0) {
                                selectedItemIndices.push(selectedItemIndex - 1);
                            }
                        }
                        jurisdictions = dialog.getModel().getProperty("/jurisdictions");
                        sheetTemplates = jurisdictions[selectedItemIndices[0]].items[selectedItemIndices[1]].items[selectedItemIndices[2]].sheetTemplates || [];
                        if (sheetTemplates.length) {
                            sortedDefs = _.sortBy(sheetTemplates, "name");
                            dialog.getModel().setProperty("/moduleSheetTemplates", sortedDefs);
                            dialog.getModel().setProperty("/sheetTemplatesOwnerCode", cascadeSelect.getSelectedKeys()[2]);
                        }
                    }else{
                        // sheettemplates exist however, the module code has changed, therefore, the sheet templates change
                        var sheetTemplatesOwnerCode=dialog.getModel().getProperty("/sheetTemplatesOwnerCode");
                        var selectedModuleCode=cascadeSelect.getSelectedKeys()[2];
                        if (sheetTemplatesOwnerCode!==selectedModuleCode){
                            for (itemIndex = 0; itemIndex < childSelects.length; ++itemIndex) {
                                selectedItemIndex = childSelects[itemIndex].getSelectedIndex();
                                if (selectedItemIndex > 0) {
                                    selectedItemIndices.push(selectedItemIndex - 1);
                                }
                            }
                            jurisdictions = dialog.getModel().getProperty("/jurisdictions");
                            sheetTemplates = jurisdictions[selectedItemIndices[0]].items[selectedItemIndices[1]].items[selectedItemIndices[2]].sheetTemplates || [];
                            if (sheetTemplates.length) {
                                sortedDefs = _.sortBy(sheetTemplates, "name");
                                dialog.getModel().setProperty("/moduleSheetTemplates", []);
                                dialog.getModel().setProperty("/moduleSheetTemplates", sortedDefs);
                                dialog.getModel().setProperty("/sheetTemplatesOwnerCode", cascadeSelect.getSelectedKeys()[2]);
                            }
                        }
                    }
                }else{
                    // the taxonomy cascade selections are not valid, clear the sheets
                    dialog.getModel().setProperty("/moduleSheetTemplates", []);
                    dialog.getModel().setProperty("/sheetTemplatesOwnerCode", "");
                }
            }
        }
        return validationJson;
    },

    toggleOkButton: function () {
        var view = this.getView();
        var valid = true;
        // is the name valid?
        var nameInput = view.byId("NewPackageDialog.packageNameInput");
        valid = (valid && (nameInput.getValue().length > 0) && (sap.ui.core.ValueState.None === nameInput.getValueState()));
        // is the legal entity identifier specified?
        //var selectedLegalEntityIdentifier = view.byId("NewPackageDialog.legalEntityIdentifierSelect").getSelectedItem();
        //valid = (valid && (null !== selectedLegalEntityIdentifier) && selectedLegalEntityIdentifier.getKey().length > 0);
        // is the currency specified?
        var selectedCurrency = view.byId("NewPackageDialog.currencySelect").getSelectedItem();
        valid = (valid && (null !== selectedCurrency) && selectedCurrency.getKey().length > 0);
        // Handle the taxonomy cascade select
        var taxonomySelect = view.byId("NewPackageDialog.taxonomyCascadeSelect");
        valid = (valid && (true === this.validateProvidedKeysForCascadeSelect(taxonomySelect).valid));
        // Handle the entity hierarchy cascade select
        var entityHierarchySelect = view.byId("NewPackageDialog.entityHierarchyCascadeSelect");
        valid = (valid && (true === this.validateProvidedKeysForCascadeSelect(entityHierarchySelect).valid));
        // Handle the time hierarchy cascade select
        var timeHierarchySelect = view.byId("NewPackageDialog.timeHierarchyCascadeSelect");
        valid = (valid && (true === this.validateProvidedKeysForCascadeSelect(timeHierarchySelect).valid));

        var list=sap.ui.getCore().byId("NewPackageDialog.workSheetList");
        if (list){
            valid=valid && list.getSelectedItems().length>0;
        }
        this.getView().byId("NewPackageDialog.savePackageButton").setEnabled(valid);
    },

    onFilterChanged: function (e) {
        var value = e.getSource().getValue() || "";
        var oView = this.getView();
        var table = oView.packageTable;

        var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
        var descFilter = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, value);
        var createdByFilter = new sap.ui.model.Filter("createdByUser",
            sap.ui.model.FilterOperator.Contains, value);

        var taxonomyFilter = new sap.ui.model.Filter("", sap.ui.model.FilterOperator.Contains, value);
        taxonomyFilter.fnTest = function (data) {
            var filterTaxonomyValue = function (type, property, property2) {
                if (data && data.hasOwnProperty(property)) {
                    var code = data[property];
                    // the second property is only needed for Time Hierarchy Level to construct
                    // a compound code (eg 3.1 or 3.2) for the taxonomy map look up
                    if (property2 && data.hasOwnProperty(property2)) {
                        code += "." + data[property2];
                    }
                    var tValue = oView.getTaxonomyValue(type, code);
                    tValue = tValue ? tValue.toLowerCase() : code.toLowerCase();
                    var sValue = value.toLowerCase();
                    return (tValue.indexOf(sValue) >= 0);
                } else {
                    return false;
                }
            };
            var filterResult =
                filterTaxonomyValue("module", "moduleCode") ||
                filterTaxonomyValue("legalEntity", "legalEntityCode") ||
                filterTaxonomyValue("currency", "currencyCode") ||
                filterTaxonomyValue("jurisdiction", "jurisdictionCode") ||
                filterTaxonomyValue("taxonomy", "taxonomyCode") ||
                filterTaxonomyValue("time", "timeHierarchy") ||
                filterTaxonomyValue("time", "timeHierarchy", "timeHierarchyLevel");
            return filterResult;
        };

        var packageStatusFilter = new sap.ui.model.Filter("packageStatus", sap.ui.model.FilterOperator.Contains, value);
        packageStatusFilter.fnTest = function (status) {
            var filterResult = false;
            if (status) {
                var statusText = oView.getStatusText(status);
                statusText = statusText ? statusText.toLowerCase() : status.toLowerCase();
                var sValue = value.toLowerCase();
                filterResult = statusText.indexOf(sValue) >= 0;
            }
            return filterResult;
        };
        var filingStatusFilter = new sap.ui.model.Filter("filingStatus", sap.ui.model.FilterOperator.Contains, value);

        var createdDateFilter = new sap.ui.model.Filter("createdTimestamp", sap.ui.model.FilterOperator.Contains, value);
        createdDateFilter.fnTest = function (datetime) {
            var filterResult = false;
            if (datetime) {
                var datetimeText = oView.formatDateTme(datetime);
                var sValue = value.toLowerCase();
                filterResult = datetimeText.toLowerCase().indexOf(sValue) >= 0;
            }
            return filterResult;
        };

        var allFilters = new sap.ui.model.Filter([nameFilter, descFilter, createdByFilter, packageStatusFilter,
            filingStatusFilter, taxonomyFilter, createdDateFilter]);
        var binding = table.getBinding("rows");
        binding.filter(allFilters);
    },

    onExit: function () {
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME,
            sas.stress.wkbench.EventBusConstants.CREATE_PACKAGE, this.onAddPackages, this);
    },

    onServerFailure: function (jqxhr) {
        var responseJSON = (jqxhr.responseJSON) ? jqxhr.responseJSON : JSON.parse(jqxhr.responseText);
        sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
    },

    /* Called when save button is pressed */
    newPackageDialogOkButtonPress: function (event) {
        var ctrl = this;
        var view = this.getView();
        var dialog = view.byId("sas.stress.packages.view.NewPackageDialog");
        var newPackageDialogModelData = dialog.getModel().getData();
        var name = view.byId("NewPackageDialog.packageNameInput").getValue().trim();
        var status = view.byId("NewPackageDialog.statusInput").getValue();
        var description = view.byId("NewPackageDialog.packageDescriptionInput").getValue();
        var entityHierarchySelects = view.byId("NewPackageDialog.entityHierarchyCascadeSelect")._getSelects();
        //var legalEntityIdentifierSelected = view.byId("NewPackageDialog.legalEntityIdentifierSelect").getSelectedItem().getKey();
        var currencySelected = view.byId("NewPackageDialog.currencySelect").getSelectedItem().getKey();
        var taxonomySelects = view.byId("NewPackageDialog.taxonomyCascadeSelect")._getSelects();
        var moduleCode = (taxonomySelects[2].getSelectedItem().getKey() === "All") ? "" : taxonomySelects[2].getSelectedItem().getKey();
        var timeHierarchySelects = view.byId("NewPackageDialog.timeHierarchyCascadeSelect")._getSelects();
        var list=sap.ui.getCore().byId("NewPackageDialog.workSheetList");
        var selectedTemplates=list.getSelectedItems();
        var templateCodes=[];
        _.each(selectedTemplates,function(template){
            templateCodes.push(template.code);
        });
        var packageId = newPackageDialogModelData.key;
        var pid = this.projectId;
        var taxonomyPackage = {
            projectKey: pid,
            name: name,
            description: description,
            entityHierarchyKey: entityHierarchySelects[0].getSelectedItem().getKey(),
            legalEntityCode: entityHierarchySelects[1].getSelectedItem().getKey(),//legalEntityIdentifierSelected,
            currencyCode: currencySelected,
            jurisdictionCode: taxonomySelects[0].getSelectedItem().getKey(),
            taxonomyCode: taxonomySelects[1].getSelectedItem().getKey(),
            moduleCode: moduleCode,
            filingStatus: status,
            timeHierarchy: timeHierarchySelects[0].getSelectedItem().getKey(),
            timeHierarchyLevel: timeHierarchySelects[1].getSelectedItem().getKey(),
            templateCodes: templateCodes
        };
        var mode = newPackageDialogModelData.editMode;
        dialog.setBusy(true);
        if (!mode) { // Create mode
            sas.stress.packages.PackageAPI.createPackage(taxonomyPackage).done(function (packageData) {
                var rows = view.packageTable.getBinding().getLength();
                var vData = {
                    validate: false,
                    zeroState: false
                };
                if (rows === 0) {
                    vData.validate = true;
                    sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME,
                        sas.stress.wkbench.EventBusConstants.VALIDATE_PACKAGE_VIEW, vData);
                }
                ctrl._calculateWorksheetAndReportProgressForPackage(packageData);
                var oModel = view.packageTable.getModel();
                var oData = oModel.getData();
                oData.push(packageData);
                oModel.refresh();
                ctrl.selectPackageByName(packageData.name);
                ctrl.toggleButtons();
                dialog.setBusy(false);
                dialog.close();
                ctrl._poll();
            }).fail(function (jqxhr) {
                dialog.setBusy(false);
                ctrl.onServerFailure(jqxhr);
            });
        } else { //Edit Mode
            sas.stress.packages.PackageAPI.editPackage(taxonomyPackage, packageId).done(function (packageData) {
                var oModel = view.packageTable.getModel();
                var oData = oModel.getData();
                var indexInModel = ctrl._getPackageIndexInModelData(oData, packageData);
                if (indexInModel > -1) {
                    ctrl._calculateWorksheetAndReportProgressForPackage(packageData);
                    oData[indexInModel] = packageData;
                    oModel.refresh();
                    ctrl.selectPackageByName(packageData.name);
                    ctrl.toggleButtons();
                }
                dialog.setBusy(false);
                dialog.close();
            }).fail(function (jqxhr) {
                dialog.setBusy(false);
                ctrl.onServerFailure(jqxhr);
            });
        }
    },

    /*
     * @param: currentName is only not-null when in edit mode
     */
    packageNameInputLiveChange: function (currentName, event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        // Check if name is empty.
        if (newName === null || newName.trim() === "") {
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            // Check if Package with this name already exists.
            var pkg = _.find(view.packageTable.getModel().getData(), function (pkg) {
                // In case of edit, the new name may match the previous name.
                if (currentName !== null) {
                    return (pkg.name !== null) &&
                        (pkg.name.trim().toLocaleLowerCase() !== currentName.trim().toLocaleLowerCase()) &&
                        (pkg.name.trim().toLocaleLowerCase() === newName.trim().toLocaleLowerCase());
                } else {
                    return (pkg.name !== null) && (pkg.name.trim().toLocaleLowerCase() === newName.trim().toLocaleLowerCase());
                }
            });
            if (pkg) {
                nameInput.setValueStateText(rb.getText("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                // Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        this.toggleOkButton();
    },
    sheetSelected:function(e){
        // update the button.. S1341497
        this.toggleOkButton();//getView().byId("NewPackageDialog.savePackageButton").setEnabled(valid);
    }
});