jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.packages.PackageAPI");
jQuery.sap.require("sas.stress.packages.PackageUtil");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");

sap.ui.controller("sas.stress.packages.view.PackagePage", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("PackagePage", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
    },

    _init: function (event) {
        var controller = this;
        var view = controller.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.packages.messagebundle" });
        view.setModel(i18N, "i18N");
        var projectId = event.getParameter("arguments").projectId;
        this.projectId = projectId;
        controller.getView().setBusy(true);
        // S1273684: don't use the cached project since it might be stale or deleted by others
        //controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
        //if (!controller.project) {    // no it is not in the cache
        sas.stress.wkbench.model.Project.loadProject(projectId, true, false).done(function (project) {
            controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
            controller.afterProjectIsLoaded();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            controller.getView().setBusy(false);
            sas.stress.wkbench.ViewManager.loadProjectManager();
            //controller.getView().destroy();
        });
        //} else {
        //    controller.afterProjectIsLoaded();
        //}
    },

    afterProjectIsLoaded: function () {
        var controller = this;
        var view = controller.getView();
        sas.stress.packages.PackageAPI.getTaxonomyValues(this.projectId).done(function (taxonomyJson) {
            if (taxonomyJson) {
                /*
                 S1341497

                 we need to store the sheetTemplates based off the module
                 */
                for (var j = 0; j < taxonomyJson.modules.length; j++) {
                    var module = taxonomyJson.modules[j];
                    module.sheetTemplates= []; // S1341497
                    for (var sti=0; sti < taxonomyJson.templates.length; ++sti){
                        var sheetTemplate=taxonomyJson.templates[sti];
                        if (sheetTemplate&&sheetTemplate.ownerCode===module.code){
                            // store the sheet templates on the module based on the sheetTemplate.ownerCode===module.code
                            module.sheetTemplates.push(sheetTemplate);
                        }
                    }
                }
                sas.stress.packages.PackageUtil.setTaxonomyMap(view.getModel("i18N").getResourceBundle(), taxonomyJson);
                controller.getView().setModel(new sas.stress.wkbench.model.RiskJSONModel(taxonomyJson), "taxonomyOptions");
                controller.afterTaxonomyValuesAreLoaded();
            }
        }).fail(function (jqxhr) {
            controller.getView().setBusy(false);
            var responseJSON = (jqxhr.responseJSON) ? jqxhr.responseJSON : JSON.parse(jqxhr.responseText);
            sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
        });

    },
    // time hierarchy is included in taxonomy values now
    afterTaxonomyValuesAreLoaded: function () {
    //    var controller = this;
    //    sas.stress.config.StressAPI.getHierarchies().done(function (timeJSON) {
    //        controller.timeMap = sas.stress.packages.PackageUtil.createTimeHierarchyMap(timeJSON);
    //        controller.getView().setModel(new sas.stress.wkbench.model.RiskJSONModel(timeJSON), "timeOptions");
    //        controller.afterTimeHierarchiesAreLoaded();
    //    }).fail(function (jqxhr) {
    //        controller.getView().setBusy(false);
    //        var responseJSON = (jqxhr.responseJSON) ? jqxhr.responseJSON : JSON.parse(jqxhr.responseText);
    //        sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
    //    });
    //},
    //
    //afterTimeHierarchiesAreLoaded: function () {
        var ctrl = this;
        this.getView().setModel(new sas.stress.wkbench.model.RiskJSONModel({ project: this.project }), "ctxt");
        sas.stress.packages.PackageAPI.getPackages(this.projectId).done(function (oData) {
            if (oData && oData.items) {
                ctrl.getView().validatePackageView(ctrl, oData, ctrl.projectId);

                //_.each(oData.items, function (pkg) {
                //    if (pkg.timeHierarchy && pkg.timeHierarchyLevel && ctrl.timeMap) {
                //        pkg.timeHierarchyName = ctrl.timeMap[pkg.timeHierarchy];
                //        pkg.timeHierarchyLevelName = ctrl.timeMap[pkg.timeHierarchy + "." + pkg.timeHierarchyLevel];
                //    }
                //});
                ctrl.handleProcessOrchestration();
            }
        }).fail(function (jqxhr) {
            ctrl.getView().validatePackageView(ctrl, null, ctrl.projectId);
            ctrl.getView().setBusy(false);
            var responseJSON = (jqxhr.responseJSON) ? jqxhr.responseJSON : JSON.parse(jqxhr.responseText);
            sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
        });

        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VALIDATE_PACKAGE_VIEW, this._validatePackageView, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.DELETE_PACKAGE, this.onRemovePackage, this);
    },

    handleProcessOrchestration: function () {
        var view = this.getView();
        var packageManagerController = view.packageManager.getController();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");
        // handle process orchestration commands, if they exist
        if (sap.ui.getCore().getModel("CreatePackageParams")) {
            packageManagerController.createPackageParams = sap.ui.getCore().getModel("CreatePackageParams").getData();
            if (view.addPackagesBtn.getEnabled()) {
                view.addPackagesBtn.firePress();
            } else {
                sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.createPackageNotAllowed.txt"), rb.getText("error.window.title"));
                delete packageManagerController.createPackageParams;
            }
            sap.ui.getCore().setModel(undefined, "CreatePackageParams");
        } else if (sap.ui.getCore().getModel("ValidatePackageParams")) {
            packageManagerController.validatePackageParams = sap.ui.getCore().getModel("ValidatePackageParams").getData();
            if (packageManagerController.validatePackageParams && packageManagerController.validatePackageParams.packageName && packageManagerController.validatePackageParams.packageName.length > 0) {
                if (packageManagerController.selectPackageByName(packageManagerController.validatePackageParams.packageName)) {
                    if (view.packageManager.validatePackageBtn.getEnabled()) {
                        view.packageManager.validatePackageBtn.firePress();
                    } else {
                        sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.validatePackageNotAllowed.txt"), rb.getText("error.window.title"));
                        delete packageManagerController.validatePackageParams;
                    }
                }
            } else {
                // A package must be specified in order to run validation, display an error to the user.
                sas.hc.m.MessageBox.error(rb.getText("packages.processOrchestration.validatePackageNotAllowedNoSelection.txt"), rb.getText("error.window.title"));
                delete packageManagerController.validatePackageParams;
            }
            sap.ui.getCore().setModel(undefined, "ValidatePackageParams");
        }
        view.setBusy(false);
    },

    onAddPackages: function () {
        var projectId = this.projectId;
        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.CREATE_PACKAGE, { projectId: projectId });

    },
    onRemovePackage: function (channel, event, data) {
        if (data.validate) {
            this.getView().validatePackageView();
        }
        // fixes for S1313310.. remove some of the items from the user preferences related to the project
        var multiDelete={};
        var project=sas.stress.wkbench.model.Project.getProject(this.projectId);
        if (project){
            var multiDelete={};
            _.each(project.getSheetDefinitions(),function(sheetDefn){
                // clear the sheet column widths.. the sheet/s is/are deleted
                multiDelete["sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths"]=undefined;
            });
        }
        // clear the project filters
        multiDelete["sas.stress.wkbench.project_filters"]=undefined;
        // clear the sheet key
        multiDelete["sas.stress.wkbench.sheet_key"]=undefined;
        // clear the category key
        multiDelete["sas.stress.wkbench.category_key"]=undefined;
        // delete the preferences for the project related to sheets and filters
        sas.stress.wkbench.UserPrefs.delete(multiDelete,this.projectId);
    },
    _validatePackageView: function (channel, event, data) {
        if (data.validate) {
            this.getView().validatePackageView();
        }
    },

    onExit: function () {
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VALIDATE_PACKAGE_VIEW, this._validatePackageView, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.DELETE_PACKAGE, this.onRemovePackage, this);
        this.router.detachExactRouteMatched("PackagePage", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.packages.view.PackagePage", null);
        this.navChecker.stop();
    },

    onServerFailure: function (jqxhr) {
        var responseJSON = (jqxhr.responseJSON) ? jqxhr.responseJSON : JSON.parse(jqxhr.responseText);
        sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
    }
});

