jQuery.sap.require("sap.ui.core.Orientation");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.stress.wkbench.widget.CustomBreadcrumb");
jQuery.sap.require("sas.stress.wkbench.widget.StressGrooveBar");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");
jQuery.sap.require("sas.stress.wkbench.widget.ZeroStateLayout");

sap.ui.jsview("sas.stress.packages.view.PackagePage", {

    getControllerName: function () {
        return "sas.stress.packages.view.PackagePage";
    },

    createContent: function (controller) {

        this.setHeight("100%");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.packages");

        this.tableHeader = this._createTableHeader(controller);
        this.packageManager = this._createPackageManager(controller);
        this.packageViewer = this._createPackageViewer(controller);

        this.splitter = new sas.hc.ui.layout.Splitter({
            contentAreas: [],
            orientation: sap.ui.core.Orientation.Vertical,
            width: "100%",
            height: "100%"
        });

        this.breadCrumb = new sas.stress.wkbench.widget.CustomBreadcrumb({
            customData: new sap.ui.core.CustomData({key: "size", value: "3rem"})
        }).addStyleClass("innerBreadCrumb");

        this.breadCrumb.addBreadcrumbItem("Top", rb.getText("packageManager.header.title"));

        this.viewLayout = new sas.risk.common.widgets.RiskPage({
            iconBar: new sas.stress.wkbench.widget.StressGrooveBar({
                section: sas.stress.wkbench.widget.StressGrooveBar.PROJECTS
            }),
            mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                type: sas.risk.common.widgets.WidgetFactory.OBJECT,
                top: new sas.risk.common.widgets.RiskToolbar({
                    homeFunction: function () {
                        sas.stress.wkbench.ViewManager.loadProjectManager();
                    },
                    headerContent: [
                        new sas.hc.m.Label({text: sas.stress.wkbench.StressUtil.getBundle().getText("iteration.label.txt")}).addStyleClass("toolbarLabel"),
                        new sas.hc.m.Label({text: "{ctxt>/project/currentIterationNo}"}).addStyleClass("toolbarValue"),
                        new sas.hc.m.Label({text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.asOfDate.label")}).addStyleClass("toolbarLabel"),
                        new sas.hc.m.Label({
                            text: {
                                path: "ctxt>/project/asOfDate",
                                formatter: function (asOfDate) {
                                    var dateFormat = new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}}, {target: {pattern: "MMM dd, yyyy"}}, asOfDate);
                                    return dateFormat.formatValue(asOfDate, "string");
                                }
                            }
                        }).addStyleClass("toolbarValue")
                    ],
                    title: new sas.hc.m.Label({text: "{ctxt>/project/name}"})
                }),
                content: new sas.risk.common.widgets.RiskPage({
                    mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                        top: this.breadCrumb,
                        content: this.splitter
                    }),
                    iconBar: new sas.stress.wkbench.widget.StressIconBar({
                        section: sas.stress.wkbench.widget.StressIconBar.PACKAGES
                    })
                })
            })
        });

        return this.viewLayout;
    },

    _createTableHeader: function (controller) {
        this.addPackagesBtn = new sas.hc.m.Button({
            id: controller.getView().createId("zeroStateBox-addPackagesBtn"),
            icon: sas.icons.HC.NEWSTARBURST,
            tooltip: "{i18N>packageManager.addPackagesBtn.tip}",
            press: $.proxy(controller.onAddPackages, controller)
        });

        this.runPackageBtn = new sas.hc.m.Button({
            id: controller.getView().createId("zeroStateBox-runPackageBtn"),
            icon: sas.icons.HC.SYNC,
            tooltip: "{i18N>packageManager.runPackageBtn.tip}",
            enabled: false
        });

        this.editPackageBtn = new sas.hc.m.Button({
            id: controller.getView().createId("zeroStateBox-editPackageBtn"),
            icon: sas.icons.HC.EDIT,
            tooltip: "{i18N>packageManager.editPackageBtn.tip}",
            enabled: false
        });
        this.validatePackageBtn = new sas.hc.m.Button({
            id: controller.getView().createId("zeroStateBox-validatePackageBtn"),
            icon: sas.icons.HC.VALIDATEITEM,
            tooltip: "{i18N>packageManager.validatePackageBtn.tip}",
            enabled: false
        });
        this.removeBtn = new sas.hc.m.Button({
            id: controller.getView().createId("zeroStateBox-removeBtn"),
            icon: sas.icons.HC.DELETE,
            tooltip: "{i18N>packageManager.removeBtn.tip}",
            enabled: false
        });

        return new sas.hc.m.Toolbar({
            content: [
                this._createSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                this.addPackagesBtn,
                this.runPackageBtn,
                this.editPackageBtn,
                this.validatePackageBtn,
                this.removeBtn
            ]
        });

    },

    _createSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: controller.getView().createId("zeroStateBox-searchFld"),
            placeholder: "{i18N>packageManager.searchField.prompt.txt}"
        });
    },

    validatePackageView: function (packageManagerController, data, projectId) {
        if (projectId) {
            this.projectId = projectId;
        }
        this.splitter.removeAllContentAreas();

        if (!this.packageManager) {
            this.packageManager = this._createPackageManager(packageManagerController);

        }
        if (this.projectId) {
            this.packageManager.setProjectId(this.projectId);
        }
        if (!this.packageViewer) {
            this.packageViewer = this._createPackageViewer(packageManagerController);
        }
        this.splitter.addContentArea(this.packageManager);
        this.splitter.addContentArea(this.packageViewer);
        this.packageManager.getController().updatePackageTableData(data);
        // Make an initial call to poll, just in case any packages or sheets are already executing/validating.
        //  Srini.. make sure we have some executions in progress for the current data.. if so start polling ... S1321628
        if (data && data.items) {
            if (this.packageManager.getController()._executionsInProcess(data.items)) {
                this.packageManager.getController()._poll();
            }
        }
    },

    _createPackageManager: function (controller) {
        var packageMgr = sap.ui.jsview(controller.createId("packageManager"), "sas.stress.packages.view.PackageManager");
        packageMgr.setModel(this.getModel("ctxt"), "ctxt");
        packageMgr.getController().updatePackageTableData();
        return packageMgr;
    },

    _createPackageViewer: function (controller) {
        var packageView = sap.ui.jsview(controller.createId("workSheetReportViewer"), "sas.stress.packages.view.WorkSheetReportViewer");
        packageView.setModel(this.getModel("ctxt"), "ctxt");
        return packageView;
    },

    destroy: function () {
        if (this.zeroStateBox) {
            this.zeroStateBox.destroy();
        }

        this.tableHeader.destroy();
        if (this.packageManager) {
            this.packageManager.destroy();
        }
        if (this.packageViewer) {
            this.packageViewer.destroy();
        }
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    }
});
