jQuery.sap.declare("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sas.stress.reports.StressAPI = {

    getReportConfigs: function (projectKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/projects/" + projectKey + "/reportConfigs", function (data) {
            deferred.resolve(data.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getLatestDataForReportConfig: function (projectKey, reportCfgId) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/projects/" + projectKey + "/reportConfigs/" + reportCfgId, function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    listRequiredFilesFrom: function (projectId) {
        var deferred = $.Deferred();
        var url = "rest/packages/" + projectId + "/landingAreaData";
        $.getJSON(url, function (result) {
            deferred.resolve(result.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    startExportOfSheetFilters: function (projectKey, filters) { // SOPHIE TODO: when are we going to be using this?
        // Request Body: List of Sheet Filter:
        // {“hierarchyKey” : “#”,
        // “hierarchyCategoryCode”: “<code>”,
        // “liKeys” : [“<key>”] } – Technically a list, but currently only 1 value supported.

        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "POST",
                   url: "rest/reports/" + projectKey + "/export",
                   data: JSON.stringify(filters),
                   dataType: "json"
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    /* Creates reports and exports them to the report datamart */
    startFinalReportGeneration: function (projectKey, filters, comment, reportConfigIds) {
        // Request Body: Report Group (comment, sheet filters, report configuration ids)
        //{“comment” : “<user supplied comment>”,
        //“filters:” : [{“hierarchyKey” : “#”,
        //“hierarchyCategoryCode”: “<code>”,
        //“liKeys” : [“<key>”] }], – Technically a list, but only 1 value supported.
        //“ids” : [<1 or more integer ids>]}

        var requestBody = { comment: comment, filters: filters, ids: reportConfigIds };

        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "POST",
                   url: "rest/reports/" + projectKey + "/report",
                   data: JSON.stringify(requestBody),
                   dataType: "json"
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    /* Creates reports, but does not export them to the report datamart */
    startReportGeneration: function (projectKey, filters, comment, reportConfigIds, fileId) {
        var requestBody = { comment: comment, filters: filters, ids: reportConfigIds, fileId: fileId };
        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "POST",
                   url: "rest/reports/" + projectKey + "/preview",
                   data: JSON.stringify(requestBody),
                   dataType: "json"
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    publishReport: function (projectKey, reportConfigHistoryKey) {
        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "POST",
                   url: "rest/reports/" + projectKey + "/publish/" + reportConfigHistoryKey
               })
         .done(function (reportConfigCategory) {
             deferred.resolve(reportConfigCategory);
         }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    addReportConfig: function (reportConfig) {
        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "POST",
                   url: "rest/config/projects/" + reportConfig.getProjectKey() + "/reportConfigs",
                   data: JSON.stringify(reportConfig),
                   dataType: "json"
               })
         .done(function (result) {
             deferred.resolve(result);
         }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    updateReportConfig: function (reportConfig) {
        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "PUT",
                   url: "rest/config/projects/" + reportConfig.getProjectKey() + "/reportConfigs/" + reportConfig.getKey(),
                   data: JSON.stringify(reportConfig),
                   dataType: "json"
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteReportConfig: function (projectKey, reportCfgId) {
        var deferred = $.Deferred();
        $.ajax({
                   type: "DELETE",
                   url: "rest/config/projects/" + projectKey + "/reportConfigs/" + reportCfgId
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteReportConfigs: function (projectKey, reportCfgIds) {
        var deferred = $.Deferred();
        $.ajax({
                type: "DELETE",
                url: "rest/config/projects/" + projectKey + "/reportConfigs",
                data: JSON.stringify(reportCfgIds),
                dataType: "json"

            })
            .done(function () {
                deferred.resolve();
            }).fail(function (jqxhr, textStatus, error) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
        });
        return deferred;
    },

    downloadReport: function (attachmentKey) {
        // fixes for S1188266...
        return sas.stress.wkbench.StressUtil._downloadAttachment(attachmentKey);
    },

    listReportTemplates: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/config/reportTemplates", function (result) {
            deferred.resolve(result.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    listValidFilters: function (projectKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/projects/" + projectKey + "/sheets/filterValues", function (result) {
            deferred.resolve(result);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    listReportConfigCategories: function (projectKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/projects/" + projectKey + "/reportConfigs/categories", function (categories) {
            deferred.resolve(categories.items);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    addReportConfigCategory: function (projectKey, reportConfigCategory) {
        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "POST",
                   url: "rest/config/projects/" + projectKey + "/reportConfigs/categories",
                   data: JSON.stringify(reportConfigCategory),
                   dataType: "json"
               })
         .done(function (reportConfigCategory) {
             deferred.resolve(reportConfigCategory);
         }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    updateReportConfigCategory: function (projectKey, reportConfigCategory) {
        var deferred = $.Deferred();
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json"
                   },
                   type: "PUT",
                   url: "rest/config/projects/" + projectKey + "/reportConfigs/categories/" + reportConfigCategory.getKey(),
                   data: JSON.stringify(reportConfigCategory),
                   dataType: "json"
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteReportConfigCategory: function (projectKey, categoryKey) {
        var deferred = $.Deferred();
        $.ajax({
                   type: "DELETE",
                   url: "rest/config/projects/" + projectKey + "/reportConfigs/categories/" + categoryKey
               })
         .done(function () {
             deferred.resolve();
         }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    drillDown: function (projectId, data,iterationNum) {
        var deferred = $.Deferred();
        var self = this;
        var exportURL = 'rest/reports/' + projectId + '/drilldown?iteration=' + iterationNum;     // fixes for S1302840

        var xhr = new XMLHttpRequest();
        xhr.onreadystatechange=function(){
            switch(xhr.readyState){
                case 1: // OPENED
                    // set up request header and send the request
                    xhr.setRequestHeader("Content-type", "application/json;charset=utf-8");
                    xhr.send(JSON.stringify(data)); //
                    break;
                case 2: // HEADERS_RECEIVED.. equivalent to HEAD request and response
                    switch(xhr.status){
                        case 200:   // OK
                            // the response will come back.. therefore, setup the "blob"
                            xhr.responseType = "blob";
                            break;
                        default:
                            // the response will have failure..
                            xhr.responseType="json";
                            break;
                    }
                    break;
                case 4: // DONE
                    switch(xhr.status){
                        case 200:   // download the blob to a file
                        {
                            var contentDispositionHeader = xhr.getResponseHeader("content-disposition");
                            var fileName = contentDispositionHeader.slice(contentDispositionHeader.indexOf("filename")).replace("filename=\"", "").replace("\"", "");
                            // fixes for S1241564.. trim the "filename="
                            //fileName = fileName.substring("filename=".length);
                            sas.stress.wkbench.StressUtil._downloadHandler(this.response, fileName);
                            deferred.resolve();
                        }
                            break;
                        default:    // failed
                            deferred.reject(xhr);
                            break;
                    }
                    break;
            }
        };
        xhr.open("POST", exportURL, true);

        return deferred;
    }


};