jQuery.sap.declare("sas.stress.reports.model.ReportConfig");
jQuery.sap.require("sas.stress.reports.model.ReportHistory");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.wkbench.widget.StatusLabel");
jQuery.sap.require("sas.stress.wkbench.model.Project");

sap.ui.base.Object.extend("sas.stress.reports.model.ReportConfig", {
    constructor: function (key, projectKey, reportConfigCategoryKey, reportConfigCategory, reportMapKey, reportName, template, history, generatorName,
                           dataName, dependentTemplate, dependentConfigKey, requiresFileFrom, taxonomyPackageOwners,
                           createdByUser, updatedByUser, createdTime, updatedTime) {
        this.key = key;
        this.projectKey = projectKey;
        this.reportConfigCategoryKey = reportConfigCategoryKey;
        this.reportConfigCategory = reportConfigCategory;
        this.reportMapKey = reportMapKey;
        this.reportName = reportName;
        this.template = template;
        this.history = history; //Array of ReportHistory
        this.generatorName = generatorName;
        this.dataName = dataName;
        this.dependentTemplate = dependentTemplate;
        this.dependentConfigKey = dependentConfigKey;
        this.getReportStatus(); // Sets up the reportStatus based on the latest history item
        this.getMessages(); // Setups up the messages based on the latest history item
        this.requiresFileFrom = requiresFileFrom;
        this.taxonomyPackageOwners = taxonomyPackageOwners;
        this.createdByUser = createdByUser;
        this.updatedByUser = updatedByUser;
        this.createdTime = createdTime;
        this.updatedTime = updatedTime;

        /*
         project locking enforcement
         1.	Reports created before locking cannot be deleted.
         3.	Cannot modify report definition created before locking.
         */
        this.canBeEditedIfProjectIsLocked=sas.stress.wkbench.model.Project.getCurrentProject().isLocked()?createdTime>=sas.stress.wkbench.model.Project.getCurrentProject().getLastModified():true;
    }
});

sas.stress.reports.model.ReportConfig.prototype.getKey = function () {
    return this.key;
};

sas.stress.reports.model.ReportConfig.prototype.getProjectKey = function () {
    return this.projectKey;
};

sas.stress.reports.model.ReportConfig.prototype.getReportMapKey = function () {
    return this.reportMapKey;
};

sas.stress.reports.model.ReportConfig.prototype.getCategoryKey = function () {
    return this.reportConfigCategoryKey;
};

sas.stress.reports.model.ReportConfig.prototype.getCategoryName = function () {
    return this.reportConfigCategory;
};

sas.stress.reports.model.ReportConfig.prototype.getReportName = function () {
    return this.reportName;
};

sas.stress.reports.model.ReportConfig.prototype.getTemplate = function () {
    return this.template;
};

sas.stress.reports.model.ReportConfig.prototype.getHistory = function () {
    return this.history;
};

sas.stress.reports.model.ReportConfig.prototype.getGeneratorName = function () {
    return this.generatorName;
};

sas.stress.reports.model.ReportConfig.prototype.getDataName = function () {
    return this.dataName;
};

sas.stress.reports.model.ReportConfig.prototype.getDependentTemplate = function () {
    return this.dependentTemplate;
};

sas.stress.reports.model.ReportConfig.prototype.getDependentConfigKey = function () {
    return this.dependentConfigKey;
};

sas.stress.reports.model.ReportConfig.prototype.getReportStatus = function () {
    this.reportStatus = "";
    var historyItems = this.history.length;
    // Does at least one history item exist?
    if (historyItems > 0) {
        // Grab the last history item's report status (midtier returns the history items sorted by descending date)
        this.reportStatus = this.history[0].getReportStatus();
    }
    return this.reportStatus;
};

sas.stress.reports.model.ReportConfig.prototype.getReportStatusPropertyStr = function () {
    // Status could be: <empty string>, Success, Error, Executing.
    switch (this.reportStatus) {
        case "Success":
            return "report.status.success.txt";
        case "Error":
            return "report.status.error.txt";
        case "Executing":
            return "report.status.executing.txt";
        default:
            return "report.status.default.txt";
    }
};

sas.stress.reports.model.ReportConfig.getReportStatusIcon = function (status) {
    // Status could be: <empty string>, Success, Error, Executing.
    switch (status) {
        case "Success":
            return sas.icons.HC.COMPLETESTATUS;
        case "Error":
            return sas.icons.HC.ERRORSTATUS;
        case "Executing":
            return sas.icons.HC.PROGRESSSTATUS;
        default:
            return sas.icons.HC.NEUTRALSTATUS;
    }
};

sas.stress.reports.model.ReportConfig.prototype.getMessages = function () {
    this.messages = [];
    var historyItems = this.history.length;
    // Does at least one history item exist?
    if (historyItems > 0) {
        // Grab the last history item's messages (midtier returns the history items sorted by descending date)
        this.messages = this.history[0].getMessages();
    }
    return this.messages;
};

sas.stress.reports.model.ReportConfig.prototype.getRequiresFileFrom = function () {
    return this.requiresFileFrom;
};

sas.stress.reports.model.ReportConfig.prototype.getTaxonomyPackageOwners = function () {
    return this.taxonomyPackageOwners;
};

sas.stress.reports.model.ReportConfig.prototype.getCreatedByUser = function () {
    return this.createdByUser;
};

sas.stress.reports.model.ReportConfig.prototype.getUpdatedByUser = function () {
    return this.updatedByUser;
};

sas.stress.reports.model.ReportConfig.prototype.getCreatedTime = function () {
    return this.createdTime;
};

sas.stress.reports.model.ReportConfig.prototype.getUpdatedTime = function () {
    return this.updatedTime;
};

/*
 project locking enforcement
 1.	Reports created before locking cannot be deleted.
 3.	Cannot modify report definition created before locking.
 */
sas.stress.reports.model.ReportConfig.prototype.getCanBeEditedIfProjectIsLocked = function () {
    return this.canBeEditedIfProjectIsLocked;
};

sas.stress.reports.model.ReportConfig._reportConfigs = {};

sas.stress.reports.model.ReportConfig.getReportConfig = function (key) {
    return sas.stress.reports.model.ReportConfig._reportConfigs[key];
};

sas.stress.reports.model.ReportConfig.getReportConfigs = function (projectKey) {
    var deferred = $.Deferred();
    sas.stress.reports.StressAPI.getReportConfigs(projectKey).done(function (reportConfigsJson) {
        // Clear any existing report configs
        sas.stress.reports.model.ReportConfig._reportConfigs = {};
        var reportConfigs = [];
        $.each(reportConfigsJson, function (index, reportConfigJson) {
            var reportConfig = sas.stress.reports.model.ReportConfig._getReportConfigFromJson(reportConfigJson);
            reportConfigs.push(reportConfig);
            sas.stress.reports.model.ReportConfig._reportConfigs[reportConfig.getKey()] = reportConfig;
        });
        deferred.resolve(reportConfigs);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.reports.model.ReportConfig.getLatestDataForReportConfig = function (projectKey, reportCfgId) {
    var deferred = $.Deferred();
    sas.stress.reports.StressAPI.getLatestDataForReportConfig(projectKey, reportCfgId).done(function (reportConfigJson) {
        var reportConfig = sas.stress.reports.model.ReportConfig._getReportConfigFromJson(reportConfigJson);
        sas.stress.reports.model.ReportConfig._reportConfigs[reportConfig.getKey()] = reportConfig;
        deferred.resolve(reportConfig);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.reports.model.ReportConfig.addReportConfig = function (reportConfig) {
    var deferred = $.Deferred();
    sas.stress.reports.StressAPI.addReportConfig(reportConfig).done(function (reportConfigJson) {
        var reportConfig = sas.stress.reports.model.ReportConfig._getReportConfigFromJson(reportConfigJson);
        sas.stress.reports.model.ReportConfig._reportConfigs[reportConfig.getKey()] = reportConfig;
        deferred.resolve(reportConfig);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.reports.model.ReportConfig.updateReportConfig = function (reportConfig) {
    var deferred = $.Deferred();
    sas.stress.reports.StressAPI.updateReportConfig(reportConfig).done(function () {
        // delete the report in the cache
        sas.stress.reports.model.ReportConfig._reportConfigs[reportConfig.getKey()] = undefined;
        delete sas.stress.reports.model.ReportConfig._reportConfigs[reportConfig.getKey()];
        // set the report in the cache
        sas.stress.reports.model.ReportConfig._reportConfigs[reportConfig.getKey()] = reportConfig;
        deferred.resolve(reportConfig);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.reports.model.ReportConfig.deleteReportConfig = function (projectKey, reportCfgId) {
    var deferred = $.Deferred();
    sas.stress.reports.StressAPI.deleteReportConfig(projectKey, reportCfgId).done(function () {
        sas.stress.reports.model.ReportConfig._reportConfigs[reportCfgId] = undefined;
        delete sas.stress.reports.model.ReportConfig._reportConfigs[reportCfgId];
        deferred.resolve();
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.reports.model.ReportConfig.deleteReportConfigs = function (projectKey, reportCfgIds) {
    var deferred = $.Deferred();
    sas.stress.reports.StressAPI.deleteReportConfigs(projectKey, reportCfgIds).done(function () {
        _.each(reportCfgIds, function (reportCfgId) {
            sas.stress.reports.model.ReportConfig._reportConfigs[reportCfgId] = undefined;
            delete sas.stress.reports.model.ReportConfig._reportConfigs[reportCfgId];
        });
        deferred.resolve();
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

/* Used to initially set the Name input in the New Report dialog with a unique name. */
sas.stress.reports.model.ReportConfig.getNewReportName = function () {
    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
    var newReportName = "";
    var count = 1;
    while (newReportName === "") {
        var tempReportName = rb.getText("report.name.fmt", [count++]);
        var isNameAvailable = sas.stress.reports.model.ReportConfig.isNameAvailable(tempReportName);
        // if available, use the temp name.
        if (isNameAvailable) {
            newReportName = tempReportName;
        }
    }
    return newReportName;
};

sas.stress.reports.model.ReportConfig.isNameAvailable = function (newName, reportBeingEdited) {
    //check if Report with this name exists
    var report = _.find(sas.stress.reports.model.ReportConfig._reportConfigs, function (reportConfig) {
        if (reportBeingEdited) {
            // In the case of edit, the new name may match the previous name.
            return (reportConfig.getReportName() !== null) &&
                (reportConfig.getReportName().trim().toLocaleLowerCase() !== reportBeingEdited.getReportName().trim().toLocaleLowerCase()) &&
                (reportConfig.getReportName().trim().toLocaleLowerCase() === newName.trim().toLocaleLowerCase());
        } else {
            return (reportConfig.getReportName() !== null) && (reportConfig.getReportName().trim().toLocaleLowerCase() === newName.trim().toLocaleLowerCase());
        }
    });
    return (report === undefined);
};

sas.stress.reports.model.ReportConfig._getReportConfigFromJson = function (reportConfigJson) {
    var reportHistories = sas.stress.reports.model.ReportConfig._getReportHistoriesFromJson(reportConfigJson);
    var reportConfig = new sas.stress.reports.model.ReportConfig(reportConfigJson.key, reportConfigJson.projectKey,
        reportConfigJson.reportConfigCategoryKey, reportConfigJson.reportConfigCategory, reportConfigJson.reportMapKey,
        reportConfigJson.reportName, reportConfigJson.template, reportHistories, reportConfigJson.generatorName,
        reportConfigJson.dataName, reportConfigJson.dependentTemplate, reportConfigJson.dependentConfigKey,
        reportConfigJson.requiresFileFrom, reportConfigJson.taxonomyPackageOwners, reportConfigJson.createdByUser,
        reportConfigJson.updatedByUser, reportConfigJson.createdTime, reportConfigJson.updatedTime);
    return reportConfig;
};

sas.stress.reports.model.ReportConfig._getReportHistoriesFromJson = function (reportConfigJson) {
    var reportHistories = [];
    if (reportConfigJson.history) {
        $.each(reportConfigJson.history, function (index, history) {
            var reportHistory = sas.stress.reports.model.ReportHistory.fromJson(history);
            reportHistories.push(reportHistory);
        });
    }
    return reportHistories;
};

sas.stress.reports.model.ReportConfig.toJson = function (reportConfig) {
    // Calculate the status label
    var status = {
        label: reportConfig.getReportStatusPropertyStr(),
        icon: null, // Not needed here (calculated in formatter)
        date: null // Not needed for reports at this time
    };
    var statusLabel = sas.stress.wkbench.widget.StatusLabel.calculateStatusLabel(status);

    /*
    project locking enforcement
     1.	Reports created before locking cannot be deleted.
     3.	Cannot modify report definition created before locking.
     */
    var canBeEditedIfProjectIsLocked=sas.stress.wkbench.model.Project.getCurrentProject().isLocked()?reportConfig.getCreatedTime()>=sas.stress.wkbench.model.Project.getCurrentProject().getLastModified():true;

    var reportConfigJson = {
        key: reportConfig.getKey(),
        projectKey: reportConfig.getProjectKey(),
        reportName: reportConfig.getReportName(),
        template: reportConfig.getTemplate(),
        dependentTemplate: reportConfig.getDependentTemplate(),
        reportConfigCategory: reportConfig.getCategoryName(),
        reportConfigCategoryKey: reportConfig.getCategoryKey(),
        reportMapKey: reportConfig.getReportMapKey(),
        history: reportConfig.getHistory(),
        requiresFileFrom: reportConfig.getRequiresFileFrom(),
        generatorName: reportConfig.getGeneratorName(),
        dataName: reportConfig.getDataName(),
        reportStatus: reportConfig.getReportStatus(),
        createdByUser:reportConfig.getCreatedByUser(),
        createdTime:reportConfig.getCreatedTime(),
        updatedByUser:reportConfig.getUpdatedByUser(),
        updatedTime:reportConfig.getUpdatedTime(),
        statusLabel: statusLabel,
        messages: reportConfig.getMessages(),
        canBeEditedIfProjectIsLocked:canBeEditedIfProjectIsLocked
    };
    return reportConfigJson;
};