jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.TextArea");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.stress.wkbench.widget.ValidFiltersWindowShade");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");

sap.ui.jsfragment("sas.stress.reports.view.RunReportDialog", {
    createContent: function (controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");

        var commentLabel = new sas.hc.m.Label({
            text: rb.getText("runReportDialog.comment.control.label"),
            visible: {
                path:"/reportHasNoComments",
                formatter:function(reportHasNoComments){
                    return false === reportHasNoComments;
                }
            }//(false === controller.reportHasNoComments)
        });
        var commentInput = new sas.hc.m.TextArea({
            id: controller.getView().createId("runReportCommentInput"),
            visible: {
                path:"/reportHasNoComments",
                formatter:function(reportHasNoComments){
                    return false === reportHasNoComments;
                }
            },//(false === controller.reportHasNoComments),
            maxLength: 256
        });
        commentInput.addAriaLabelledBy(commentLabel);

        controller.validFiltersWindowShade = new sas.stress.wkbench.widget.ValidFiltersWindowShade();
        controller.validFiltersWindowShade.setWidth('100%');

        var validFiltersLabel = new sas.hc.m.Label({
            text: rb.getText("runReportDialog.filters.title.txt"),
            visible: {
                path:"/reportHasNoFilters",
                formatter:function(reportHasNoFilters){
                    return false === reportHasNoFilters;
                }
            },//"{/reportHasNoFilters}"//(false === controller.reportHasNoFilters)
            required: true
        });
        var scrollContainer = new sas.hc.m.ScrollContainer({
            content: controller.validFiltersWindowShade,
            height: "20rem",
            //width: "100%",
            horizontal: false,
            vertical: true,
            visible: {
                path: "/reportHasNoFilters",
                formatter:function(reportHasNoFilters){
                    return false === reportHasNoFilters;
                }
            }//"{/reportHasNoFilters}"//(false === controller.reportHasNoFilters)
        });

        controller.landingAreaDataList = this.createDataSelectList("/landingAreaFiles", "timestamp", controller);
        controller.landingAreaDataList.attachSelectionChange(controller.landingAreaDataListChange, controller);

        var fileSelectLabel = new sas.hc.m.Label({
            text: rb.getText("runReportDialog.landingArea.title.txt"),
            visible: {
                path: "/reportHasLandingAreaFile",
                formatter:function(reportHasLandingAreaFile){
                    return true === reportHasLandingAreaFile;
                }
            }//(true === controller.reportHasLandingAreaFile)
        });

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: validFiltersLabel,
                            fields: [scrollContainer]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: fileSelectLabel,
                            fields: [controller.landingAreaDataList]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: commentLabel,
                            fields: [commentInput]
                        })
                    ]
                })
            ]
        });
        form.addStyleClass("stressXLargeForm");

        controller.runReportButton = new sas.hc.m.Button({
            id: controller.getView().createId("runReportButton"),
            text: rb.getText("button.runReport.action.label"),
            press: $.proxy(controller.runReportDialogRunButtonPress, controller)
        });

        controller.runAndExportReportButton = new sas.hc.m.Button({
            id: controller.getView().createId("runAndExportReportButton"),
            text: rb.getText("button.runAndExportReport.action.label"),
            press: $.proxy(controller.runReportDialogRunAndExportButtonPress, controller)
        });

        var dialog = new sas.hc.m.Dialog({
            id: controller.getView().createId("sas.stress.reports.view.RunReportDialog"),
            initialFocus: commentInput,
            content: form,
            buttons: [
                controller.runReportButton,
                controller.runAndExportReportButton,
                new sas.hc.m.Button({
                    text: rb.getText("button.cancel.action.label"),
                    press: function () {
                        dialog.close();
                    }
                })
            ],
            afterClose: function () {
                dialog.destroy();
            }
        });

        var reportHasNoFilters = dialog.getModel()?dialog.getModel().getProperty("/reportHasNoFilters"):controller.reportHasNoFilters;
        if (false === reportHasNoFilters) {
            this._createFilters(controller, controller.validFiltersWindowShade, dialog);

            controller.validFiltersWindowShade.addEventDelegate(
                {
                    onAfterRendering: function () {
                        var capabilityManager=sas.stress.wkbench.CapabilityManager.getInstance();
                        // If all values have default/preset values, then we need to ensure the run buttons are enabled immediately.
                        var allFiltersSet = !controller.validFiltersWindowShade.hasUnsetFilters();
                        controller.runReportButton.setEnabled(allFiltersSet);
                        controller.runAndExportReportButton.setEnabled(allFiltersSet &&
                            capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.EXPORT_TO_RDM)); //defect S1294748)

                        dialog.setBusy(false);
                    }
                },
                controller
            );
        }

        return dialog;
    },

    createDataSelectList: function (itemsPath, timePath, controller) {
        var singleSelectTable = new sas.risk.common.widgets.SelectTable({
            listItems: "{" + itemsPath + "}",
            listName: "{i18N>runReportDialog.fileList.name.txt}",
            selectionMode: sap.ui.table.SelectionMode.Single,
            //selectionBehavior: sap.ui.table.SelectionBehavior.RowSelector,
            visibleRowCount: 5,
            visible: {
                path: "/reportHasLandingAreaFile",
                formatter:function(reportHasLandingAreaFile){
                    return true === reportHasLandingAreaFile;
                }
            },/*(true === controller.reportHasLandingAreaFile),*/
            width:"100%"
        });

        singleSelectTable.addColumn(new sas.hc.ui.table.Column({
            template: new sas.hc.m.Text({
                text: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                tooltip: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                wrapping: false
                //maxLines: 1
            }),
            width: "35%"
        }));
        singleSelectTable.bindRows(itemsPath);
        return singleSelectTable;
    },

    _createFilters: function (controller, oWindowShade, dialog) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        sas.stress.reports.StressAPI.listValidFilters(controller.project.getId()).done(function (validFilters) {
            if (validFilters.items.length > 0) {
                var preselectedFilters = undefined;
                if (controller.runReportParams && controller.runReportParams.filters) {
                    preselectedFilters = controller.runReportParams.filters;
                }
                for (var index = 0; index < validFilters.items.length; ++index) {
                    oWindowShade.createSectionForHierarchy(validFilters.items[index], controller.runReportDialogFilterSelectHandler, controller, preselectedFilters, dialog);
                }

                // Are any filters left? If so, it means they were not found and are therefore invalid in the current setup.
                if (preselectedFilters && preselectedFilters.length && preselectedFilters.length > 0) {
                    sas.hc.m.MessageBox.error(rb.getText("reports.processOrchestration.invalidFilter.fmt.txt", [preselectedFilters[0]]));
                }

                delete controller.runReportParams;
            }
            // DD: Newer requirement: Having no filters is a valid condition
            //else {
                // Having no filters is invalid, throw an error. // SOPHIE TODO: throw a more helpful error here...
                //sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            //}
        }).fail(function (jqxhr) {
            if (jqxhr && jqxhr.responseJSON) {
                sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
            } else {
                sas.hc.m.MessageBox.error(rb.getText('error.message.txt'), rb.getText('error.window.title'));
            }
        });
    }
});
