jQuery.sap.require("sas.stress.scenarios.ScenarioAPI");

sap.ui.controller("sas.stress.scenarios.view.AddScenarios", {
    onBeforeRendering: function () {
        var view = this.getView();
        var controller = this;
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.scenarios.messagebundle" });
        view.setModel(i18N, "i18N");
        var scenariosModel = new sas.stress.wkbench.model.RiskJSONModel();
        view.setModel(scenariosModel);
        controller.projectId = view.getViewData().projectId;
        //Get scenarios
        var scenarios = controller._getScenarios(controller.projectId);
        //Get master scenarios
        var masterScenarios = sas.stress.scenarios.ScenarioAPI.getMasterScenarios();

        view.table.setBusy(true);
        view.capabilitiesWarning.setVisible(false);
        $.when(scenarios, masterScenarios).done(function (scenarios, masterScenarios) {
            // Filter masterScenarios to remove
            // -- the scenarios that are in the scenarios list.
            // -- scenarios that do not have a supportable period type
            var filteredScenarios = _.filter(masterScenarios, function (masterScenario) {
                // These rules are also found in GlobalScenarioPage.view and must be updated there.
                // We should extract these to a commom method
                if (!masterScenario.periodType) {
                    return false;
                }
                // S1349000
                if (["YEAR","SEMIYEAR", "QUARTER", "MONTH", "WEEK"].indexOf(masterScenario.periodType.toUpperCase()) < 0) {
                    return false;
                }

                if (masterScenario.periods && masterScenario.periods[0].horizon > 0) {
                    return false;
                }

                if (_.find(masterScenario.stresses, function (stress) {
                        return stress.perturbType !== "VALUE";
                    })) {
                    return false;
                }

                var scenarioExists = _.find(scenarios, function (scenario) {
                    // For now, we don't allow multiple versions of a given scenario in the same project.
                    return masterScenario.id === scenario.id ||
                           masterScenario.name.toUpperCase() === scenario.name.toUpperCase();
                });

                return scenarioExists === undefined;
            });

            view.getModel().setProperty("/scenarios", filteredScenarios);

            //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
            //var sorters = [new sap.ui.model.Sorter("label", false), new sap.ui.model.Sorter("scenarioVersion", false)];
            //view.table.getBinding("rows").sort(sorters);

            view.table.setBusy(false);

            try {
                controller.handleProcessOrchestration();
            } catch (ex) {
                sas.hc.m.MessageBox.error(ex.message, i18N.getResourceBundle().getText("error.window.title"));
            } finally {
                sap.ui.getCore().setModel(undefined, "AddScenariosParams");
            }
        }).fail(function (jqxhr) {
            view.table.setBusy(false);
            // Don't display the error directly to the user, instead we will make the capabilities warning visible above the table.
            view.capabilitiesWarning.setVisible(true);
        });
        controller._toggleOKButton();
    },

    handleProcessOrchestration: function () {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.scenarios");
        // handle process orchestration commands, if they exist
        if (sap.ui.getCore().getModel("AddScenariosParams")) {
            this.addScenariosParams = sap.ui.getCore().getModel("AddScenariosParams").getData();
            if (this.addScenariosParams && this.addScenariosParams.scenarios) {
                var allScenarios = view.getModel().getProperty("/scenarios");
                var errorMessage = "";
                // Find the scenario that matches the parameter
                for (var scenarioIndex = 0; scenarioIndex < this.addScenariosParams.scenarios.length; ++scenarioIndex) {
                    var specifiedName = this.addScenariosParams.scenarios[scenarioIndex].name;
                    var specifiedVersion = this.addScenariosParams.scenarios[scenarioIndex].version;
                    if (specifiedName) {
                        var found = false;
                        var foundWrongVersion = false;
                        // We have to loop through the model because the row we want to select might not have been loaded yet
                        for (var i = 0; i < allScenarios.length; i++) {
                            var tableContext = view.table.getContextByIndex(i);
                            if (tableContext) {
                                var path = tableContext.getPath();
                                var scenario = view.table.getModel().getProperty(path);
                                // Does the model in the table match the current model that was passed in?
                                if (scenario.label.toLocaleLowerCase() === specifiedName.toLocaleLowerCase()) {
                                    // If the scenario has version information, then what was specified needs to match exactly.
                                    // If the scenario doesn't have version information, then no version information should've been specified.
                                    // Anything else is considered a miss and will generate an error (if no exact match is found later in the data).
                                    if ((!!scenario.scenarioVersion && !!specifiedVersion && scenario.scenarioVersion.toLocaleLowerCase() === specifiedVersion.toLocaleLowerCase())
                                        || (!scenario.scenarioVersion && !specifiedVersion)) {
                                        view.table.addSelectionInterval(i, i);
                                        found = true;
                                        break;
                                    } else {
                                        // A match was found by name, but not by version.
                                        foundWrongVersion = true;
                                    }
                                }
                            }
                        }
                        if (!found) {
                            if (errorMessage.length > 0) {
                                // more than one error
                                errorMessage += "\n";
                            }
                            if (foundWrongVersion) {
                                // The scenario wasn't found as specified, but existed with mismatching version information.
                                errorMessage += rb.getText("scenarios.processOrchestration.scenarioExistsWithDifferentVersion.fmt.txt", [specifiedName]);
                            } else if (specifiedVersion) {
                                // The scenario wasn't found as specified, version information was specified.
                                errorMessage += rb.getText("scenarios.processOrchestration.invalidScenarioHasVersion.fmt.txt", [specifiedName, specifiedVersion]);
                            } else {
                                // The scenario wasn't found as specified, no version information was specified.
                                errorMessage += rb.getText("scenarios.processOrchestration.invalidScenarioNoVersion.fmt.txt", [specifiedName]);
                            }
                        }
                    }
                }
                //S1276274: throw exception after all scenarios are processed
                if (errorMessage.length > 0) {
                    throw new Error(errorMessage);
                }
            }
        }
    },

    _toggleOKButton: function () {
        var view = this.getView();
        var items = view.getSelectedItems();
        if (items.length === 0) {
            view.okButton.setEnabled(false);
            return;
        }

        var nameCache = {};
        _.each(items, function (item) {
            nameCache[item.name] = item;
        });

        view.okButton.setEnabled(items.length === _.size(nameCache));
    },

    _getScenarios: function (projectId) {
        var deferred = $.Deferred();
        sas.stress.scenarios.ScenarioAPI.getScenarios(projectId).done(function (scenarios) {
               deferred.resolve(scenarios);
           })
           .fail(function () {
               var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.scenarios");
               sas.hc.m.MessageBox.error(rb.getText('scenarioManager.server.error.message.txt'), rb.getText('scenarioManager.server.error.window.title'));
               deferred.reject();
           });
        return deferred;
    },

    search: function (event) {
        var view = this.getView();
        var value = event.getParameter("value") || event.getParameter("newValue") || event.getParameter("query") || "";
        var labelFilter = new sap.ui.model.Filter("label", sap.ui.model.FilterOperator.Contains, value);
        var descFilter = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, value);
        var userModifiedFilter = new sap.ui.model.Filter("modifiedBy", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([labelFilter, descFilter, userModifiedFilter]);
        var binding = view.table.getBinding("rows");
        if (binding !== undefined) {
            binding.filter(allFilters);
        }
    },

    tableRowSelectionChange: function (event) {
        this._toggleOKButton();
    },

    cancelButtonPress: function () {
        var view = this.getView();
        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "CANCEL" });
    },

    okButtonPress: function () {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.scenarios");
        var selectedScenariosCnt = view.table.getSelectedIndices().length;
        if (selectedScenariosCnt === 0) {
            return;
        }

        sas.stress.scenarios.ScenarioAPI.saveScenarioMetas(controller.projectId, view.getSelectedItems()).done(function (data) {
            view.fireEvent(view.getId() + "_SCENARIOS_ADDED", { "view": view, "action": "_SCENARIO_ADDED" });
            view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "OK" });
        }).fail(function (json) {
            sas.hc.m.MessageBox.error(json.message, rb.getText('scenarioManager.server.error.window.title'));
        });

    }
});
