jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.widget.ZeroStateLayout");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.ui.commons.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.m.Image");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.ui.layout.Splitter");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.StressGrooveBar");

sap.ui.jsview("sas.stress.scenarios.view.GlobalScenarioPage", {

    getControllerName: function () {
        return "sas.stress.scenarios.view.GlobalScenarioPage";
    },

    createContent: function (controller) {

        this.capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        this.setHeight("100%");

        return new sas.risk.common.widgets.RiskPage({
            iconBar: new sas.stress.wkbench.widget.StressGrooveBar({
                section: sas.stress.wkbench.widget.StressGrooveBar.SCENARIOS
            }),
            mainContent: new sas.risk.common.widgets.RiskPage({
                mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                    content: new sas.hc.ui.layout.Splitter({
                        contentAreas: [
                            this._createScenarioManager(controller),
                            this._createScenarioViewer(controller)
                        ],
                        orientation: sap.ui.core.Orientation.Vertical,
                        width: "100%",
                        height: "100%"
                    })
                }),
                toolbar: new sas.risk.common.widgets.RiskToolbar({
                    title: new sas.hc.m.Label({ text: "{i18N>scenarioManager.header.title}" })
                })
            })
        });

    },

    _createScenarioManager: function (controller) {
        this.scenarioTable = this._createScenarioTable(controller);
        return sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({content: this.scenarioTable});
    },

    _createScenarioViewer: function (controller) {
        this.perturbationTable = this._createPerturbationTable(controller);
        return sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({content: this.perturbationTable});
    },

    _createScenarioTableHeader: function (controller) {
        this.capabilitiesWarning = new sas.hc.ui.commons.layout.HorizontalLayout({
            visible: false,
            content: [
                new sas.hc.m.Image({ src: sas.icons.HC.WARNINGSTATUS, width: "16px", height: "16px" }),
                new sas.hc.m.Text({
                    text: "{i18N>scenarioManager.capabilitiesWarning.txt}",
                    wrapping: true,
                    tooltip: "{i18N>scenarioManager.capabilitiesWarning.txt}"
                })
            ]
        }).addStyleClass("stressInline").addStyleClass("stressInlineWarn");

        this.importExcelBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.IMPORT,
            press: $.proxy(controller.onImportExcel, controller),
            enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCENARIO.IMPORT),
            tooltip: "{i18N>scenarioManager.importExcelBtn.tip}"
        });

        this.refreshBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.REFRESH,
            press: $.proxy(controller.onRefreshScenario, controller),
            enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCENARIO.READ),
            tooltip: "{i18N>scenarioManager.refreshBtn.tip}"
        });

        this.removeBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.DELETE,
            press: $.proxy(controller.onDeleteScenario, controller),
            tooltip: "{i18N>scenarioManager.deleteBtn.tip}"
        });

        return new sas.hc.m.Toolbar(
            {
                content: [
                    this._createScenarioTableSearchField(controller),
                    new sas.hc.m.ToolbarSpacer(),
                    this.capabilitiesWarning,
                    this.importExcelBtn,
                    this.refreshBtn,
                    this.removeBtn
                ]
            }
        );
    },

    _createScenarioTableSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            placeholder: "{i18N>scenarioManager.searchField.prompt.txt}",
            liveChange: $.proxy(controller.scenarioTableFilterChanged, controller),
            search: $.proxy(controller.scenarioTableFilterChanged, controller)
        });
    },

    _createScenarioTable: function (controller) {
        var view = this;
        var table = new sas.hc.ui.table.Table({
            showTableOptions: false,
            visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionBehavior: sap.ui.table.SelectionBehavior.Row,
            selectionMode: sap.ui.table.SelectionMode.Single,
            useRadioForSingleSelection: true,
            rowSelectionChange: $.proxy(controller.scenarioTableRowSelectionChanged, controller),
            toolbar: this._createScenarioTableHeader(controller),
            noData: new sas.stress.wkbench.widget.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEALERT,
                title: "{i18N>scenarioManagerTable.noData.title}"
            })
        });

        // Legacy name/label problem.  In HPRisk & the flex scenario editor, the scenario's name is assigned
        // by the system from its label, and is not updated.  We generally showed the label value (even if we
        // called the field 'name'); there was a configuration setting that swapped that for the entire HPRisk
        // UI.  In this case, we seem to be mapping repository labels to name inside the workbench somewhere,
        // so here, we're going to do the same switch the legacy systems did.
        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>scenarioManagerTable.nameColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "label"),
            sortProperty: "label",
            width: "10em"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: '{i18N>scenarioManagerTable.versionColumn.header.title}' }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty('text', 'scenarioVersion'),
            sortProperty: 'scenarioVersion',
            width: "7em"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>scenarioManagerTable.periodColumn.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: {
                    path: "periodType",
                    formatter: function (periodType) {
                        var resourceModel = view.getModel("i18N");
                        if ($.type(periodType) !== "string") {
                            return "";
                        }
                        if (resourceModel === undefined) {
                            return periodType;
                        }
                        var key = "scenario.period." + periodType.toLowerCase() + ".label.txt";
                        var value=resourceModel.getProperty(key);
                        if (value===key){
                            return value;
                        }else{
                            return periodType;
                        }
                        //return (resourceModel.hasOwnProperty(key)) ? resourceModel.getProperty(key) : periodType;
                    }
                }
            }),
            sortProperty: "periodType",
            width: "7em"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>scenarioManagerTable.compatColumn.header.title}" }),
            template: new sas.hc.m.Button({
                icon: sas.icons.HC.CHECKMARK,
                tooltip: "{i18N>scenarioManagerTable.compatColumn.tip}",
                visible: {
                    parts: ["periodType", "periods", "stresses"],
                    formatter: function (periodType, periods, stresses) {
                        // These rules are also found in AddScenarios.controller and must be updated there.
                        // We should extract these to a commom method
                        if (!periodType || periodType === null) {
                            return false;
                        }
                        // S1349000
                        if (["YEAR", "SEMIYEAR", "QUARTER", "MONTH", "WEEK"].indexOf(periodType.toUpperCase()) < 0) {
                            return false;
                        }

                        if (periods && periods[0].horizon > 0) {
                            return false;
                        }

                        if (_.find(stresses, function (stress) {
                                return stress.perturbType !== "VALUE";
                            })) {
                            return false;
                        }

                        return true;

                    }
                }
            }),
            hAlign: sap.ui.core.HorizontalAlign.Center,
            width: "7.5rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: '{i18N>scenarioManagerTable.baselineColumn.header.title}' }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "baselineName"),
            sortProperty: 'baselineName',
            width: "10em"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: '{i18N>scenarioManagerTable.baselineVersionColumn.header.title}' }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "baselineVersion"),
            sortProperty: 'baselineVersion',
            width: "10em"
        }));


        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>scenarioManagerTable.descriptionColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "description"),
            sortProperty: "description",
            width: "20em"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>scenarioManagerTable.dateModifiedColumn.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: {
                    path: "modifiedTimeStamp",
                    formatter: function (dttm) {
                        if (!dttm) {
                            return "";
                        }
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                        // Maybe if we determine that space is at a premium...
//                      var fmt = sas.hc.nls.format.DateFormat.getDateTimeInstance({ style:"short", notzone:true });
//                      return fmt.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "modifiedTimeStamp",
            width: "12em"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>scenarioManagerTable.modifiedUserColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "modifiedBy"),
            sortProperty: "modifiedBy",
            width: "7em"
        }));

        table.bindRows("/scenarios");
        //S1256573: Default sort by Name/Version asc.
        var bindingInfo = table.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [new sap.ui.model.Sorter("label", false), new sap.ui.model.Sorter("scenarioVersion", false)];
            bindingInfo.sorter = sorters;
        }
        return table;
    },

    _createPerturbationTableSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: "GlobalScenarioPage.perturbationTableSearchField",
            placeholder: "{i18N>perturbationTable.searchField.prompt.txt}",
            liveChange: $.proxy(controller.perturbationTableFilterChanged, controller),
            search: $.proxy(controller.perturbationTableFilterChanged, controller)
        });
    },

    _createPerturbationTable: function (controller) {

        this.editBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.SCENARIOMANAGER,
            enabled: false,
            tooltip: "{i18N>scenarioManager.editorBtn.tip}",
            press: function (event) {
                sas.stress.wkbench.StressUtil.launchSASScenarioEditor(event.getSource().getModel().getProperty("/scenario/id"));
            }
        });

        this.sizeWarning = new sas.hc.ui.commons.layout.HorizontalLayout({
            visible: false,
            content: [
                new sas.hc.m.Image({ src: sas.icons.HC.WARNINGSTATUS, width: "16px", height: "16px" }),
                new sas.hc.m.Text({
                    text: "{i18N>perturbationTable.sizeWarning.txt}",
                    wrapping: true,
                    tooltip: "{i18N>perturbationTable.sizeWarning.txt}"
                })
            ]
        }).addStyleClass("stressInline").addStyleClass("stressInlineWarn");

        // TODO: Sortable will be false -- order is significant and we're only supporting view for now.
        var table = new sas.hc.ui.table.Table({
            editable: false,
            width: "100%",
            showTableOptions: false,
            enableColumnReordering: false,
            fixedColumnCount: 1,
            title: {
                parts: ['/scenario/label', '/scenario/scenarioVersion'],
                formatter: function (label, version) {
                    return version ? label + "/" + version : label;
                }
            },
            visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
            selectionMode: sap.ui.table.SelectionMode.Single,
            toolbar: new sas.hc.m.Toolbar(
                {
                    content: [
                        this._createPerturbationTableSearchField(controller),
                        new sas.hc.m.ToolbarSpacer(),
                        this.sizeWarning,
                        this.editBtn
                    ]
                }
            ),
            noData: new sas.stress.wkbench.widget.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEALERT,
                title: "{i18N>perturbationTable.noData.title}"
            })
        });

        table.bindRows("/scenario/stresses");
        table.addStyleClass("perturbTable");
        return this._configureTable(table);
    },

    _configureTable: function (table, scenario) {
        table.destroyColumns();

        table.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: "{i18N>perturbationTable.variableColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "variable"),
            width: "250px"
        }));

        if (scenario) {

            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.scenarios");

            table.addColumn(new sas.hc.ui.table.Column({
                label: new sas.hc.m.Label({ text: "{i18N>perturbationTable.perturbTypeColumn.header.title}" }),
                template: new sas.hc.m.Text({
                    wrapping: false,
                    text: {
                        path: "perturbType",
                        formatter: function (ptype) {
                            return ptype ? rb.getText("perturbType." + ptype.toLowerCase() + ".label.txt") : "";
                        }
                    }
                }),
                width: "140px"
            }));

            // Limit the number of columns in the table to 360 to avoid slowdown
            this.maxHorizons = 360;
            this.sizeWarning.setVisible(scenario.periods.length > this.maxHorizons);

            var dateFormat = new sap.ui.model.type.Date({ source: { pattern: "yyyy-MM-dd" } });
            $.each(scenario.periods.slice(0, this.maxHorizons), function (idx, period) {
                var headerLbl = new sas.hc.m.Label({
                    text: (period.horizon > 0) ?
                          period.horizon + " " + period.periodType :
                          (period.date === undefined) ?
                          "{i18N>perturbationTable.changeColumn.header.title}" :
                          dateFormat.formatValue(period.date, "string"),
                    textAlign: sap.ui.core.TextAlign.Right
                });
                headerLbl.addStyleClass("pertTableValueHdr");

                var renderText = new sas.hc.m.Text({ wrapping: false });
                renderText.bindProperty("text", "perturbations", function (perts) {
                    if (perts && $.isNumeric(perts[idx])) {
                        var fmt = sap.ui.core.format.NumberFormat.getFloatInstance();
                        return fmt.format(perts[idx]);
                    }
                    return "";
                });

                table.addColumn(new sas.hc.ui.table.Column({
                    label: headerLbl,
                    template: renderText,
                    hAlign: sap.ui.core.TextAlign.Right,
                    width: "140px"
                }));
            });
        } else {
            // If no scenario is to be displayed, ensure no size warning message is displayed.
            this.sizeWarning.setVisible(false);
        }

        return table;
    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    }
});