jQuery.sap.declare('sas.stress.wkbench.CellEvaluationUtil');
jQuery.sap.require("sas.stress.wkbench.parser.Parser");
jQuery.sap.require("sas.stress.wkbench.parser.Lexer");
jQuery.sap.require("sas.stress.wkbench.parser.ExpressionUtil");
jQuery.sap.require("sas.stress.wkbench.model.CellRef");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sas.stress.wkbench.CellEvaluationUtil = {
	getLatestValue:function(cellRef){
		var value="";
		if(cellRef){
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var infoData=cellRef.getInfoData();
			var row=cellRef.row;
			var col=cellRef.column;
			var key="$"+infoData.getLongColumn(col)+"$"+infoData.getLongRow(row);
			var overlay=data.ncFormulas[key];
			if(!overlay){//pure value
				value=data.data[row][col];
			}else if(overlay.error){
				value=overlay.txt;
			}else if(!overlay.timestamp&&(overlay.source&&overlay.source!=="allocationOverride")){//not committed overlay and not becuase of "allocationRule"
				value=overlay.origtxt;
			}else{//committed overlay
				var showEquals = (overlay.expr.type !== "NAME" && overlay.expr.type !== "PREFIX") || (overlay.expr.name && overlay.expr.name.indexOf("$") !== -1);
				try {
					value = (showEquals ? "=" : "") + sas.stress.wkbench.parser.ExpressionUtil.print(overlay.expr, $.proxy(data.printShortContext, data));
					if (showEquals && !_.isNull(data.data[row][col]) &&
						(!_.isNumber(data.data[row][col]) && !_.isBoolean(data.data[row][col])&& !_.isString(data.data[row][col]))) {	//fixes for S1301860.. ignore numeric, text and boolean values
						value = data.data[row][col] + " " + value;
					}
				} catch (e) {
					console.error("Unable to evaluate expression " + sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr), overlay);
					value = "#NAME? " + overlay.txt;
				}
			}
		}
		return value;
	},
	getLatestExpr:function(cellRef){
		var expr=null;
		if(cellRef){
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var infoData=cellRef.getInfoData();
			var row=cellRef.row;
			var col=cellRef.column;
			var key="$"+infoData.getLongColumn(col)+"$"+infoData.getLongRow(row);
			var overlay=data.ncFormulas[key];
			if(overlay){//not committed overlay
				expr=overlay.expr;
			}else{
				overlay=data.cFormulas[key];
				if(overlay){//committed overlay
					expr=overlay.expr;
				}//else pure value, no expr
			}
		}
		return expr;
	},
	validateDependencies:function(references,scenario){
		var results=[];
		var getChildren=function(refId){
			var data=refId.getFilterData()/*.getScenarioData()*/;
			var key="$"+refId.colLong+"$"+refId.rowLong;
			var overlay=data.ncFormulas[key]||data.cFormulas[key];
			if(overlay){
				var expression=overlay.expr;
				return sas.stress.wkbench.CellEvaluationUtil.getCellRefs(expression,refId.getSheetFilters()/*getScenario()*/);
			} else {
				return [];
			}
		};
		_.each(references,function(refId){
			sas.stress.wkbench.CellEvaluationUtil._traverseGraph(getChildren,refId,results);
			results.push(refId);
		});
		return results;
	},
	_traverseGraph:function(getChildren,refId,results){
		_.each(getChildren(refId)||[],function(dep){
			if(results.indexOf(dep)===-1){
				results.push(dep);
				sas.stress.wkbench.CellEvaluationUtil._traverseGraph(getChildren,dep,results);
			}
		});
		return results;
	},
	getCellRefs:function(expression,scenario,errorobj){
		if(expression){
			var refArrays=[];
			var nameExps=sas.stress.wkbench.parser.ExpressionUtil.collect(expression,"NAME");
			var i,nameExp;
			for(i=0;i<nameExps.length;i++){
				nameExp=nameExps[i];
				if(sas.stress.wkbench.parser.ExpressionUtil._getType(nameExp.name)==="REFERENCE"){
					refArrays.push(sas.stress.wkbench.model.CellRef.getCellRefs(nameExp.name,scenario));
				}
			}
			var refs=[];
			_.each(refArrays,function(refArray){
				refs=_.union(refs,refArray);
			});
			var nullfound=_.find(refs,function(cellRef){
				return cellRef===null ;
			});
			if (nullfound!==null){
				return _.uniq(refs,false,function(cellRef){
					return cellRef.toIdString();
				});
			}else{
				if (errorobj!==undefined) {
					errorobj.error = true;
				}
				return [];
			}
			return _.uniq(refs,false,function(cellRef){
				return cellRef.toIdString();
			});
		}
		return [];//none found
	},
	getCellRefStrs:function(expression,scenario){
		if(expression){
			var refArrays=[];
			var nameExps=sas.stress.wkbench.parser.ExpressionUtil.collect(expression,"NAME");
			var i,nameExp;
			for(i=0;i<nameExps.length;i++){
				nameExp=nameExps[i];
				if(sas.stress.wkbench.parser.ExpressionUtil._getType(nameExp.name)==="REFERENCE"){
					try{
						refArrays.push(sas.stress.wkbench.model.CellRef.getCellRefStrs(nameExp.name,scenario));
					}catch(e){
						console.error(e);
					}
				}
			}
			var refs=[];
			_.each(refArrays,function(refArray){
				refs=_.union(refs,refArray);
			});
			return _.uniq(refs);//,false,function(cellRef){return cellRef.toIdString();});
		}
		return [];//none found
	},
	getFunctionReferences:function(expression){
		var refs=[];
		if(expression){
			var callExps=sas.stress.wkbench.parser.ExpressionUtil.collect(expression,"CALL");
			_.each(callExps,function(callExp){
				refs.push(callExp.func.name);
			});
		}
		return refs;
	},
	parseFormula:function(formulaStr){
		formulaStr=(formulaStr[0]==='=')?formulaStr.substring(1):formulaStr;
		var lexer=new sas.stress.wkbench.parser.Lexer(formulaStr);
		var parser=new sas.stress.wkbench.parser.Parser(lexer);
		return parser.parseExpression();
	},
	//Leaving this in for function name validation
	parseFunction:function(formula){
		if(formula===null||formula===undefined){
			return null;
		}
		if(formula.indexOf("=")===0){
			formula=formula.substring(1);
		}
		var lexer=new sas.stress.wkbench.parser.Lexer(formula);
		var parser=new sas.stress.wkbench.parser.Parser(lexer);
		var expression=parser.parseExpression();
		var funcRefs=sas.stress.wkbench.CellEvaluationUtil.getFunctionReferences(expression);//this changes things, now the function name can't be a dynamic expression
		var invalidNames=sas.stress.wkbench.CellEvaluationUtil.checkForInvalidFunctionNames(funcRefs);
		if(invalidNames.length){
			console.error("Invalid function name(s) found: "+funcRefs.join());
			throw "#NAME?";
		}
		return expression;
	},
	checkForInvalidFunctionNames:function(funcName){
		var funcNames=_.isArray(funcName)?funcName:[funcName];
		return _.filter(funcNames,function(name){
			return !sas.stress.wkbench.CellEvaluationUtil.functions[name.toLowerCase()];
		});
	},
	updateDependencies:function(scenario,thisCellRef,before,after, ignoreDependencies){
		var thisCellRefStr=thisCellRef.toIdString();
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var projectId=scenario.getSheetInfo().getSheet().getProject().getId();
		var project=sas.stress.wkbench.model.Project.getProject(projectId);
		var deps=project._overlayDeps;
		var refs=project._overlayRefs;
		var beforeRefs=[];
		try{
			beforeRefs=CellEvaluationUtil.getCellRefs(before||null,scenario);
		}catch(e){
			console.error("ignoring before refs problem!!");
		}
		var afterRefs=CellEvaluationUtil.getCellRefs(after,scenario);
		var refsToAdd=sas.stress.wkbench.StressUtil._differenceObj(afterRefs,beforeRefs);
		var refsToRemove=sas.stress.wkbench.StressUtil._differenceObj(beforeRefs,afterRefs);
		var i,cellRefStr;
		for(i=0;i<refsToAdd.length;i++){//update map cell->direct dependents. add itself as dependent
			cellRefStr=refsToAdd[i].toIdString();
			deps[cellRefStr]=_.union(deps[cellRefStr]||[],[thisCellRefStr]);
		}
		for(i=0;i<refsToRemove.length;i++){//update map cell->direct dependents. remove itself as dependent
			cellRefStr=refsToRemove[i].toIdString();
			deps[cellRefStr]=_.without(deps[cellRefStr]||[],thisCellRefStr);
		}
		refs[thisCellRefStr]=afterRefs;//update map cell->direct references to latest list of references
		var refsToRecalculate=sas.stress.wkbench.CellEvaluationUtil._collectReferences(deps,thisCellRefStr,{},ignoreDependencies);
		refsToRecalculate=sas.stress.wkbench.CellEvaluationUtil._removeRedundants(refs,refsToRecalculate);
		_.each(refsToRecalculate,function(cellRef){//reset all dependents
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var overlay=data.ncFormulas[key]||data.cFormulas[key];
			overlay.status=0;
		});
		_.each(refsToRecalculate,function(cellRef){//re-evaluate all dependents
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var overlay=data.ncFormulas[key]||data.cFormulas[key];
			data.evalOverlay(overlay);
		});
	},
	_removeRedundants:function(refs,refIdMap){
		var redundants={};
		_.each(refIdMap,function(refIdStr){
			_.each(refs[refIdStr]||[],function(referencedRefStr){
				redundants[referencedRefStr]=true;
			});
		});
		return _.filter(refIdMap,function(value,key){
			return !redundants[key];
		});
	},
	_collectReferences:function(deps,currentRefStr,set,ignoreDependencies){//collect all references
		var i;
		set=set||{};//Map to add unique cellRefs
		var dependents=deps[currentRefStr]||[];
		dependents=_.difference(dependents,ignoreDependencies);
		var dependentsLength=dependents.length;
		for(i=0;i<dependentsLength;i++){
			var cellRefStr=dependents[i];
			if(!set[cellRefStr]){
				set[cellRefStr]=sas.stress.wkbench.model.CellRef.parse(cellRefStr);
				sas.stress.wkbench.CellEvaluationUtil._collectReferences(deps,cellRefStr,set);
			}
		}
		return set;
	},
	saveNewFormula:function(formula,cellRef,save,ignoreDependencies){	// fixes for S1201112... added new parameter
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var newExpr=undefined;
		if(formula!==null){
			newExpr=CellEvaluationUtil.parseFormula(formula);
		} else{
			newExpr=CellEvaluationUtil.parseFormula("0.00");	// fummy value for not failing in the next lines
		}
		var newExprStr=JSON.stringify(newExpr);
		var scenario=cellRef.getSheetFilters()/*.getScenario()*/;
		var oldExpr=CellEvaluationUtil.getLatestExpr(cellRef);
		var oldExprStr=JSON.stringify(oldExpr);
		if(newExprStr!==oldExprStr||!save){	// fixes for S1203544... if new != old OR NOT save (log dialog discard)
			var errorObj={error:false};
			var error=false;
			try {
				var references = CellEvaluationUtil.getCellRefs(newExpr, scenario,errorObj);
				error=errorObj.error;
			}catch (e){
				error=true;
			}
			var dependencies=CellEvaluationUtil.validateDependencies(references,scenario);
			if(_.some(dependencies,cellRef.equals,cellRef)){
				throw "CYCLICAL_DEP";
			}
			var data=scenario.getData();
			var userName = sas.stress.wkbench.StressUtil.getUserName();
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var txt="",showEquals=false;
			try {
				txt = (newExpr) ? sas.stress.wkbench.parser.ExpressionUtil.print(newExpr, $.proxy(data.printLongContext, data)) : null;
				if (txt){
					newExpr=CellEvaluationUtil.parseFormula(txt);	// store the long expr values for other sheets to evaluate
				}
				showEquals=isNaN(txt);
			}catch(e){
				txt=formula;
			}
			var prevTxt=data.ncFormulas.hasOwnProperty(key)?data.ncFormulas[key].prevtxt:((data.cFormulas.hasOwnProperty(key)?data.cFormulas[key]:data.data[cellRef.row][cellRef.column]));
			var dataKey;
			if(data.ncFormulas.hasOwnProperty(key)){
				dataKey=data.ncFormulas[key].dataKey;
			}else{
				dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var source="sheetOverride";
			var canBeDiscarded=true;
			if (ignoreDependencies){
				source="allocationOverride";	// rb.getText
				canBeDiscarded=false;
			}
			var overlay={
				user:userName,
				txt:(showEquals?"=":"")+txt,
				origtxt:formula,
				expr:newExpr,
				rowIndex:cellRef.row,
				columnIndex:cellRef.column,
				status:0,
				prevtxt:prevTxt,/*data.ncFormulas.hasOwnProperty(key)?data.ncFormulas[key].txt:data.data[cellRef.row][cellRef.column]*/
				error:error,
				dataKey:dataKey,
				source:source,
				canBeDiscarded:canBeDiscarded
			};
			if (save){	//S1201112.. do we need to create un-committed overlays?
				data.ncFormulas[key]=overlay;
			}
			data.evalOverlay(overlay);
			CellEvaluationUtil.updateDependencies(scenario, cellRef, oldExpr, newExpr, ignoreDependencies||[]);
		}
	},
	performAllocations:function(value,cellRef,itemid){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
		var dataKey;
		if(cellRef.getSheetFilters().getData().ncFormulas.hasOwnProperty(key)){
			dataKey=cellRef.getSheetFilters().getData().ncFormulas[key].dataKey;
		}else{
			dataKey=cellRef.getSheetFilters().getSheetInfo().getData().getDatakeyFromRowColumn(key);
		}
		var oldExpr=CellEvaluationUtil.getLatestExpr(cellRef);
		var childCellRefs=[],eachCellRefs,idx;
		if (oldExpr.args){
			if (oldExpr.args.length>1){
				for (idx=0;idx<oldExpr.args.length;++idx) {
					if (oldExpr.args[idx].name){
						eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(oldExpr.args[idx].name);
					}else{
						switch (oldExpr.args[idx].type){
							case "PREFIX":
								eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(oldExpr.args[idx].right.name);
								break;
							case "POSTFIX":
								eachCellRefs= sas.stress.wkbench.model.CellRef.getCellRefs(oldExpr.args[idx].left.name);
								break;
						}

					}
					childCellRefs.push(eachCellRefs[0]);
				}
			}else {
				childCellRefs=sas.stress.wkbench.model.CellRef.getCellRefs(oldExpr.args[0].name);
			}
		}else{
			childCellRefs=sas.stress.wkbench.model.CellRef.getCellRefs(cellRef.getSheetFilters().getData().printShortContext(oldExpr.name),cellRef.getSheetFilters());
		}
		var newExpr=CellEvaluationUtil.parseFormula(value);
		var filtersData=cellRef.getSheetFilters().getData();
		var parsedValue=sas.stress.wkbench.parser.ExpressionUtil.evaluate(newExpr, $.proxy(filtersData.evalContext, filtersData));
		if (isFinite(parsedValue)){
			var previousValue=cellRef.getSheetFilters().sheetInfo.sheet.project.getCellValueByDataKey(dataKey);
			CellEvaluationUtil._performAllocationsRecurse(parsedValue,cellRef,childCellRefs,itemid,false);
			var currentValue=cellRef.getSheetFilters().sheetInfo.sheet.project.getCellValueByDataKey(dataKey)||Number(value);
			var userName = sas.stress.wkbench.StressUtil.getUserName();
			var rb=sas.stress.wkbench.StressUtil.getBundle();
			switch (itemid){
				case "even":
					cellRef.getSheetFilters().getData().addComment(key,cellRef.row,cellRef.column,userName,
						rb.getText("cell.editor.even.allocation.default.comment.fmt",[previousValue,currentValue]),"allocationOverride");
					break;
				case "equally":
					cellRef.getSheetFilters().getData().addComment(key,cellRef.row,cellRef.column,userName,
						rb.getText("cell.editor.equal.allocation.default.comment.fmt",[previousValue,currentValue]),"allocationOverride");
					break;
				case "proportional":
					cellRef.getSheetFilters().getData().addComment(key,cellRef.row,cellRef.column,userName,
						rb.getText("cell.editor.proportional.allocation.default.comment.fmt",[previousValue,currentValue]),"allocationOverride");
					break;
			}
		}else{
			throw CellEvaluationUtil.INVALID_ALLOCATION_VALUE;
		}

		// we need to repeat this for all sheets where the cellref key is found ${column liKEY}${row liKey}
		/*var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
		 var dataKey;
		 if(cellRef.getSheetFilters().getData().ncFormulas.hasOwnProperty(key)){
		 dataKey=cellRef.getSheetFilters().getData().ncFormulas[key].dataKey;
		 }else{
		 dataKey=cellRef.getSheetFilters().getSheetInfo().getData().getDatakeyFromRowColumn(key);
		 }
		 var currentSheet=cellRef.getSheetFilters().getSheetInfo().getSheet();
		 var restOfSheets= _.filter(cellRef.getSheetFilters().getSheetInfo().getSheet().getProject().getSheets(),function(sheet){
		 return sheet.getId()!==cellRef.sheetId;
		 });

		 CellEvaluationUtil._performAllocationsAcrossSheets(restOfSheets,key,dataKey,value,itemid);*/
	},
	_performAllocationsRecurse:function(value,cellRef,childRefs,itemid,evenAllocationAmountEvaluated){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var key, idx, subtotalChildCellRef, childCellRefs=[], subtotalChildCellRefs,filteredNumericChildrenRefs;
		var currentExpr;
		var cellValue,subtotalCellValue, differencevalue,evenlyDistributedAmount,equalDistributeValue;
		var parentDetails,notToBeUpdatedDependentCellRefStrs=[];

		// look for the children Refs that have a formula and are not subtotal nodes
		var subtotalChildCellRefs= _.filter(childRefs,function(childCellRef){
				currentExpr=CellEvaluationUtil.getLatestExpr(childCellRef);
				var isFunc=currentExpr?currentExpr.type==="CALL":false;
				return childCellRef.getRowNode().subtotallikey!==undefined || isFunc;
			})||[];

		if (subtotalChildCellRefs.length>0){
			key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			cellValue=cellRef.getSheetFilters().sheetInfo.sheet.project.getCellValueByDataKey(cellRef.getSheetFilters().getData().ncFormulas[key].dataKey);   // undefined is not finite
			parentDetails = CellEvaluationUtil._getSubTotalNodeDetails(subtotalChildCellRefs);
			// get only the numerics from the children references
			filteredNumericChildrenRefs=CellEvaluationUtil._getFilteredChildRefsThatAreNumeric(subtotalChildCellRefs);

			differencevalue=Number(value)-cellValue;
			for (idx = 0; idx < filteredNumericChildrenRefs.length;idx++) { // S1306783... only look at the children that are numeric...
				subtotalChildCellRef = filteredNumericChildrenRefs[idx];    // S1306783
				key = "$" + subtotalChildCellRef.colLong + "$" + subtotalChildCellRef.rowLong;
				subtotalCellValue = subtotalChildCellRef.getSheetFilters().sheetInfo.sheet.project.getCellValueByDataKey(subtotalChildCellRef.getSheetFilters().getData().ncFormulas[key].dataKey);  // undefined is not finite

				currentExpr = CellEvaluationUtil.getLatestExpr(subtotalChildCellRef);
				// get the childCellRefs
				if (currentExpr.args){
					if (currentExpr.args.length>1){
						for (var i=0;i<currentExpr.args.length;++i) {
							if (currentExpr.args[i].name){
								eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(currentExpr.args[i].name);
							}else{
								switch (currentExpr.args[i].type){
									case "PREFIX":
										eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(currentExpr.args[i].right.name);
										break;
									case "POSTFIX":
										eachCellRefs= sas.stress.wkbench.model.CellRef.getCellRefs(currentExpr.args[i].left.name);
										break;
								}
							}
							childCellRefs.push(eachCellRefs[0]);
						}
					}else{
						childCellRefs=sas.stress.wkbench.model.CellRef.getCellRefs(currentExpr.args[0].name);
					}
				}else{
					// if expression's name (which is value) finite...
					if (isFinite(currentExpr.name)){
						// get the reason for the finite-ness
						var expr=CellEvaluationUtil.getLatestExpr(subtotalChildCellRef);
						if (expr.args){
							for (var idx=0;idx<expr.args.length;++idx) {
								var eachCellRefs=[];
								if (expr.args[idx].name){
									eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(expr.args[idx].name);
								}else{
									switch (expr.args[idx].type){
										case "PREFIX":
											eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(expr.args[idx].right.name);
											break;
										case "POSTFIX":
											eachCellRefs= sas.stress.wkbench.model.CellRef.getCellRefs(expr.args[idx].left.name);
											break;
									}

								}
								childCellRefs.push(eachCellRefs[0]);
							}
						}else{
							// the expression does not have arguments, is a subtotal node and has finite value
							if (subtotalChildCellRef.subtotallikey&&!isNaN(expr.name)){
								console.warn("DID WE MISS PROCESSING: " + expr.name);
								childCellRefs=sas.stress.wkbench.model.CellRef.getCellRefs(cellRef.getSheetFilters().getData().printShortContext(expr.name),cellRef.getSheetFilters());
							}else{
								console.warn("no formula found!! maybe overridden via allocation!!");
							}
						}
						//childCellRefs=sas.stress.wkbench.model.CellRef.getCellRefs(CellEvaluationUtil.getLatestExpr(cellRef),cellRef.getSheetFilters());
					}else{
						childCellRefs=sas.stress.wkbench.model.CellRef.getCellRefs(cellRef.getSheetFilters().getData().printShortContext(currentExpr.name),cellRef.getSheetFilters());   // empty array otherwise
					}
				}

				if (isFinite(subtotalCellValue)) {
					switch (itemid) {
						case "even":
							evenlyDistributedAmount = subtotalCellValue + differencevalue / (filteredNumericChildrenRefs.length?filteredNumericChildrenRefs.length:1);
							CellEvaluationUtil._performAllocationsRecurse(evenlyDistributedAmount, subtotalChildCellRef, childCellRefs, itemid, false);
							break;

						case "proportional":
							var eachkey = "$" + subtotalChildCellRef.colLong + "$" + subtotalChildCellRef.rowLong;
							var previousvalue = subtotalChildCellRef.getSheetFilters().sheetInfo.sheet.project.getCellValueByDataKey(subtotalChildCellRef.getSheetFilters().getData().ncFormulas[eachkey].dataKey);

							var toBeDistributedValue;
							if (subtotalCellValue === 0) {
								if (!parentDetails.allNullNodes){
									toBeDistributedValue = previousvalue + (Math.abs(previousvalue) / (parentDetails.absoluteTotalOfSubtotalNode?parentDetails.absoluteTotalOfSubtotalNode:1)) * Number(value);
								}else{
									toBeDistributedValue=Number(value)/(filteredNumericChildrenRefs.length?filteredNumericChildrenRefs.length:1);
								}
							} else {
								toBeDistributedValue = (previousvalue / cellValue) * Number(value);	// cannot divide by 0... therefore adding 1
							}
							CellEvaluationUtil._performAllocationsRecurse(toBeDistributedValue, subtotalChildCellRef, childCellRefs, itemid);
							break;

						case "equally":
							evenlyDistributedAmount = value / (filteredNumericChildrenRefs.length?filteredNumericChildrenRefs.length:1);
							CellEvaluationUtil._performAllocationsRecurse(evenlyDistributedAmount, subtotalChildCellRef, childCellRefs, itemid);
							break;
					}
				}
				childCellRefs=[];	// reset for the next child
			}
		} else {
			key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			subtotalCellValue=cellRef.getSheetFilters().sheetInfo.sheet.project.getCellValueByDataKey(cellRef.getSheetFilters().getData().ncFormulas[key].dataKey);
			//get the children that are NUMERIC...
			filteredNumericChildrenRefs = CellEvaluationUtil._getFilteredChildRefsThatAreNumeric(childRefs);

			// remove the children Refs that have a formula and are not subtotal nodes
			var filteredChildRefs= _.filter(filteredNumericChildrenRefs,function(childCellRef){
				var currentExpr=CellEvaluationUtil.getLatestExpr(childCellRef);
				var isFunc=currentExpr?currentExpr.type!=="NAME":false;
				return childCellRef.getRowNode().subtotallikey===undefined && isFunc;
			});

			_.each(filteredChildRefs,function(childRef){
				notToBeUpdatedDependentCellRefStrs.push(childRef.toIdString());
			});

			switch(itemid){
				case "even":
					parentDetails = CellEvaluationUtil._getSubTotalNodeDetails(childRefs);
					if (!evenAllocationAmountEvaluated) {
						if (parentDetails.absoluteTotalOfSubtotalNode !== subtotalCellValue){
							differencevalue=Number(value)-subtotalCellValue;
							evenlyDistributedAmount=differencevalue/(filteredNumericChildrenRefs.length?filteredNumericChildrenRefs.length:1);
						}else {
							differencevalue = Number(value) - subtotalCellValue;
							evenlyDistributedAmount = differencevalue / (filteredNumericChildrenRefs.length ? filteredNumericChildrenRefs.length : 1);
						}
					}else{
						evenlyDistributedAmount =  (Number(value)/*-subtotalCellValue*/) / (filteredNumericChildrenRefs.length?filteredNumericChildrenRefs.length:1);
					}
					CellEvaluationUtil._performEvenAllocation(evenlyDistributedAmount,filteredNumericChildrenRefs,notToBeUpdatedDependentCellRefStrs);
					break;

				case "proportional":
					//var totalProportionalValue=Number(value)-subtotalCellValue;
					if (subtotalCellValue===0) {
						parentDetails = CellEvaluationUtil._getSubTotalNodeDetails(filteredNumericChildrenRefs);
						if (!parentDetails.allNullNodes) {
							CellEvaluationUtil._performProportionalAllocation(Number(value), subtotalCellValue,
								parentDetails.absoluteTotalOfSubtotalNode ? parentDetails.absoluteTotalOfSubtotalNode : 1, filteredNumericChildrenRefs, notToBeUpdatedDependentCellRefStrs);
						}
					}else{
						CellEvaluationUtil._performProportionalAllocation(Number(value), subtotalCellValue,0, filteredNumericChildrenRefs, notToBeUpdatedDependentCellRefStrs);
					}
					break;

				case "equally":
					equalDistributeValue = Number(value) / (filteredNumericChildrenRefs.length ? filteredNumericChildrenRefs.length : 1);
					CellEvaluationUtil._performEqualAllocation(equalDistributeValue,filteredNumericChildrenRefs,notToBeUpdatedDependentCellRefStrs);
					break;
			}
		}
		// update the cell to indicate it's now contributing to allocation
		var userName = sas.stress.wkbench.StressUtil.getUserName();
		var txt = "", showEquals = false;
		var scenario = cellRef.getSheetFilters()/*.getScenario()*/;
		var data = scenario.getData();

		if ("proportional"===itemid&&(parentDetails&&parentDetails.allNullNodes)){	// if subtotal cell's children are all null or 0... use value and override formula
			CellEvaluationUtil.saveNewFormula(value.toString(),cellRef,true,[]);	// fixes for S1299281... need to send string for parsing the formula
		}else if (filteredNumericChildrenRefs.length===0){	// if all the children are NON-numeric.. the value is on the root
			CellEvaluationUtil.saveNewFormula(value.toString(),cellRef,true,[]);	// fixes for S1299281... need to send string for parsing the formula
		}else {
			// TODO: Srini---- should be update the existing overlay? maybe not needed because the datakey will have the value
			/*var dataKey;
			 if (data.ncFormulas.hasOwnProperty(key)) {
			 dataKey = data.ncFormulas[key].dataKey;
			 } else {
			 dataKey = scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			 }*/

		}
	},
	_performEvenAllocation:function(value, childRefs, notToBeUpdatedDependentCellRefStrs){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var newExpr =CellEvaluationUtil.parseFormula(value.toString());

		_.each(childRefs,function(cellRef){
			var isContra=cellRef.getRowNode().contra;

			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var scenario=cellRef.getSheetFilters()/*.getScenario()*/;
			var data=scenario.getData();
			var dataKey;
			if(data.ncFormulas.hasOwnProperty(key)){
				dataKey=data.ncFormulas[key].dataKey;
			}else{
				dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var previousvalue=scenario.getSheetInfo().getSheet().getProject().getCellValueByDataKey(dataKey);
			if (previousvalue===undefined||previousvalue===null){
				CellEvaluationUtil.saveNewFormula(value.toString(), cellRef, true, notToBeUpdatedDependentCellRefStrs);
			}
			if (isFinite(previousvalue)) {
				if (!isContra) {
					CellEvaluationUtil.saveNewFormula((Number(previousvalue) + value).toString(), cellRef, true, notToBeUpdatedDependentCellRefStrs);
				}else{
					CellEvaluationUtil.saveNewFormula((Number(previousvalue) - value).toString(), cellRef, true, notToBeUpdatedDependentCellRefStrs);
				}
			}

		});
	},
	_performProportionalAllocation:function(userValue,subtotalCellCurrentValue,subtotalAbsoluteValue, childRefs, notToBeUpdatedCellRefStrs){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var newExpr =CellEvaluationUtil.parseFormula(subtotalCellCurrentValue.toString());

		_.each(childRefs,function(cellRef){
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var scenario=cellRef.getSheetFilters()/*.getScenario()*/;
			var data=scenario.getData();
			var dataKey;
			if(data.ncFormulas.hasOwnProperty(key)){
				dataKey=data.ncFormulas[key].dataKey;
			}else{
				dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var currentCellValue=scenario.getSheetInfo().getSheet().getProject().getCellValueByDataKey(dataKey);
			var value;
			if (currentCellValue===undefined||currentCellValue===null||!isFinite(currentCellValue)){
				value=0;
				//CellEvaluationUtil.saveNewFormula(value.toString(),cellRef,true,notToBeUpdatedCellRefStrs);
			}else {
				if (subtotalCellCurrentValue===0) {
					value = currentCellValue + ((Math.abs(currentCellValue)) / subtotalAbsoluteValue) * userValue;
				}else {
					value = (currentCellValue / subtotalCellCurrentValue) * userValue;
					//value+=currentCellValue;
				}
				CellEvaluationUtil.saveNewFormula(value.toString(),cellRef,true,notToBeUpdatedCellRefStrs);
			}
		});
	},
	_performEqualAllocation:function(value,childRefs,notToBeUpdatedDependentCellRefStrs){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		_.each(childRefs,function(cellRef){
			var isContra=cellRef.getRowNode().contra;

			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var scenario=cellRef.getSheetFilters()/*.getScenario()*/;
			var data=scenario.getData();
			var dataKey;
			if(data.ncFormulas.hasOwnProperty(key)){
				dataKey=data.ncFormulas[key].dataKey;
			}else{
				dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var currentCellValue=scenario.getSheetInfo().getSheet().getProject().getCellValueByDataKey(dataKey);
			if (currentCellValue === undefined || currentCellValue === null || isFinite(currentCellValue)) {
				//if (!isContraed) {
				CellEvaluationUtil.saveNewFormula(value.toString(), cellRef, true, notToBeUpdatedDependentCellRefStrs);	//(currentValue+equalValue).toString()
				/*}else{
				 if(!isContra){
				 CellEvaluationUtil.saveNewFormula((Number(currentCellValue) + value).toString(), cellRef, true, notToBeUpdatedDependentCellRefStrs);	//(currentValue+equalValue).toString()
				 }else{
				 CellEvaluationUtil.saveNewFormula((Number(currentCellValue) - value).toString(), cellRef, true, notToBeUpdatedDependentCellRefStrs);	//(currentValue+equalValue).toString()
				 }
				 }*/
			}
		});
	},
	_getSubTotalNodeDetails:function(childRefs){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var key, scenario, data, dataKey;
		var absoluteTotalOfSubTotalNode=0;
		var cellExpr, nullnodes=[], nullnode;
		var details={absoluteTotalOfSubtotalNode: absoluteTotalOfSubTotalNode, allNullNodes:false};
		_.each(childRefs,function(cellRef){
			cellExpr=CellEvaluationUtil.getLatestExpr(cellRef);
			nullnode=cellExpr?cellExpr.type==="NAME":true;
			key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			scenario=cellRef.getSheetFilters()/*.getScenario()*/;
			data=scenario.getData();
			if(data.ncFormulas.hasOwnProperty(key)){
				dataKey=data.ncFormulas[key].dataKey;
			}else{
				dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var currentValue=scenario.getSheetInfo().getSheet().getProject().getCellValueByDataKey(dataKey);
			if (currentValue===undefined||currentValue===null){
				nullnodes.push(nullnode);
			}else if (0===currentValue||"0"===currentValue){
				nullnodes.push(nullnode);
			} else if (isFinite(currentValue)){
				absoluteTotalOfSubTotalNode+=Math.abs(currentValue);
			} else {
				absoluteTotalOfSubTotalNode+=0;
			}
		});
		details.absoluteTotalOfSubtotalNode=absoluteTotalOfSubTotalNode;
		if(nullnodes.length===childRefs.length){
			details.allNullNodes=true;
		}
		return details;
	},
	_getFilteredChildRefsThatAreNumeric:function(childRefs){
		var key, scenario, data, dataKey;
		var filteredNumericChildren=_.filter(childRefs,function(cellRef){
			key = "$" + cellRef.colLong + "$" + cellRef.rowLong;
			scenario = cellRef.getSheetFilters()/*.getScenario()*/;
			data = scenario.getData();
			if (data.ncFormulas.hasOwnProperty(key)) {
				dataKey = data.ncFormulas[key].dataKey;
			} else {
				dataKey = scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var currentValue = scenario.getSheetInfo().getSheet().getProject().getCellValueByDataKey(dataKey);

			if (currentValue === undefined || currentValue === null) {
				return false;
			}else if (isFinite(currentValue)){
				return true;
			} else{
				return false;
			}
		});
		return filteredNumericChildren;
	},
	_performAllocationsAcrossSheets:function(restOfSheets,originalKey,dataKey,value,itemId){
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		_.each(restOfSheets,function(sheet){
			var found=false;
			var sheetInfoData=sheet.getInfo().getFilters().getData();
			var overlay;
			for(var key in sheetInfoData.cFormulas) {
				if (sheetInfoData.cFormulas.hasOwnProperty(key)) {
					overlay = sheetInfoData.cFormulas[originalKey];
					if (overlay&&overlay.dataKey===dataKey){
						found=true;
						break;
					}
				}
			}
			if (found) {
				var cellRef = new sas.stress.wkbench.model.CellRef(overlay.columnIndex, overlay.rowIndex, sheet.getInfo().getFilters());
				var oldExpr = CellEvaluationUtil.getLatestExpr(cellRef);
				var childCellRefs = [], eachCellRefs, idx;
				if (oldExpr.args.length > 1) {
					for (idx = 0; idx < oldExpr.args.length; ++idx) {
						eachCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(oldExpr.args[idx].name);
						childCellRefs.push(eachCellRefs[0]);
					}
				} else {
					childCellRefs = sas.stress.wkbench.model.CellRef.getCellRefs(oldExpr.args[0].name);
				}

				CellEvaluationUtil._performAllocationsRecurse(value, cellRef, childCellRefs, itemId);
			}
		});
	},
	INVALID_ALLOCATION_VALUE:"invalid allocation value",
	functions:{
		"sum":function sum(){
			var values=Array.prototype.slice.call(arguments);
			var value=0;
			var errorvalue="";
			var sumItUp=function(num){
				if($.isArray(num)){
					_.each(num,sumItUp);
				}/*else if (isFinite(num)) {
				 value+=Number(num);
				 }*/else if(_.isNumber(num)){
					value+=num;
				}else if(num===null){
					value+=0;
				}else{
					errorvalue="#NAME?";
				} /*if(num==="#NAME?"||num==="#DIV/0!"||num==="#REF!"){

				 }else if(_.isNumber(num)){
				 value+=num;
				 }*/
			};
			_.each(values,sumItUp);
			return errorvalue.length?errorvalue:value;
		},
		"if":function if_func(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>1 && values.length<4 && typeof values[0]==="boolean"){
				if(values[0]){
					return values[1];
				}else{
					return values.length>2?values[2]:false;
				}
			}
			console.error("Error:Invalid syntax for If");
			throw "Error:Invalid syntax for If";
		},
		"abs":function abs(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>0 && _.isNumber(values[0])/*isFinite(values[0])*/){
				return Math.abs(values[0]);
			}else{
				console.error("Error:Invalid syntax for Abs");
				throw "Error:Invalid syntax for Abs";
			}
		},
		"and":function and(){
			var values=Array.prototype.slice.call(arguments);
			var result=null;
			for(var i=0;i<values.length&&result!==false;i++){
				var val=values[i];
				if(_.isNumber(val)/*isFinite(val)*/){
					result=!!val;
				}else if(_.isBoolean(val)){
					result=val;
				}
			}
			if(result===null){
				console.error("Error:Invalid syntax for And");
				throw "Error:Invalid syntax for And";
			}
			return result;
		},
		"or":function or(){
			var values=Array.prototype.slice.call(arguments);
			var result=null;
			for(var i=0;i<values.length&&result!==true;i++){
				var val=values[i];
				if(_.isNumber(val)){
					result=!!val;
				}else if(_.isBoolean(val)){
					result=val;
				}
			}
			if(result===null){
				console.error("Error:Invalid syntax for And");
				throw "Error:Invalid syntax for And";
			}
			return result;
		},
		"average":function average(){
			var values=Array.prototype.slice.call(arguments);
			var value=0;
			var count=0;
			var nullcount=0;
			var sumItUp=function(num){
				if($.isArray(num)){
					_.each(num,sumItUp);
				}else if(_.isNull(num)){
					value+=num;
					nullcount++;
				}else if(_.isNumber(num)){
					value+=num;
					count++;
				}
			};
			_.each(values,sumItUp);
			if(count===0){
				if(nullcount===0) {
					console.error("Error:Invalid syntax for Average");
					throw "Error:Invalid syntax for Average";
				} else{
					console.error("Error:All values NULL for Average");
					throw {message:"#DIV/0!"};
				}
			}
			return value/count;
		},
		"var":function var_func(){
			var values=Array.prototype.slice.call(arguments);
			try{
				var mean=sas.stress.wkbench.CellEvaluationUtil.functions.average.apply(this,arguments);
			}catch(e){
				console.error("Error:Invalid syntax for Var");throw "Error:Invalid syntax for Var";
			}
			var meanSquared=mean*mean;
			var meanDouble=2*mean;
			var result=0;
			var count=0;
			var varItUp=function(val){
				if($.isArray(val)){
					_.each(val,varItUp);
				}else if(_.isNumber(val)){
					result+=val*(val-meanDouble)+meanSquared;
					count++;
				}
			};
			_.each(values,varItUp);
			return result/(count-1);
		},
		"stdev":function stdev(){
			var values=Array.prototype.slice.call(arguments);
			try{
				var variance=sas.stress.wkbench.CellEvaluationUtil.functions.var.apply(this,arguments);
			}catch(e){
				console.error("Error:Invalid syntax for Stdev");throw "Error:Invalid syntax for Stdev";
			}
			return Math.sqrt(variance);
		},
		"min":function min(){
			var values=Array.prototype.slice.call(arguments);
			var numbers=[];
			var pushIt=function(value){
				if($.isArray(value)){
					_.each(value,pushIt);
				}else if(_.isNumber(value)){
					numbers.push(value);
				}else if(_.isBoolean(value)){
					numbers.push(value?1:0);
				}
			};
			_.each(values,pushIt);
			if(numbers.length){
				return Math.min.apply(null,numbers);
			}else{
				console.error("Error:Invalid syntax for Min");
				throw "Error:Invalid syntax for Min";
			}
		},
		"max":function max(){
			var values=Array.prototype.slice.call(arguments);
			var numbers=[];
			var pushIt=function(value){
				if($.isArray(value)){
					_.each(value,pushIt);
				}else if(_.isNumber(value)){
					numbers.push(value);
				}else if(_.isBoolean(value)){
					numbers.push(value?1:0);
				}
			};
			_.each(values,pushIt);
			if(numbers.length){
				return Math.max.apply(null,numbers);
			}else{
				console.error("Error:Invalid syntax for Max");
				throw "Error:Invalid syntax for Max";
			}
		},
		"sumproduct":function sumproduct(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length<1 || !$.isArray(values[0])){
				console.error("Error:Invalid syntax for Sumproduct");
				throw "Error:Invalid syntax for Sumproduct";
			}
			var prev=values.shift();
			var length=prev.length;
			_.each(values,function(array){
				if(!$.isArray(array)||array.length!==length){
					console.error("Error:Invalid syntax for Sumproduct");
					throw "Error:Invalid syntax for Sumproduct";
				}
				for(var i=0;i<length;i++){
					prev[i]*=array[i];
				}
			});
			var result=0;
			for(var i=0;i<length;i++){
				result+=prev[i];
			}
			return result;
		},
		"normdist":function normdist(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length<3 || !_.isNumber(values[0]) || !_.isNumber(values[1]) || !_.isNumber(values[2])){
				console.error("Error:Invalid syntax for Normdist");
				throw "Error:Invalid syntax for Normdist";
			}
			var value=values[0];
			var mean=values[1];
			var stdev=values[2];
			var variance=stdev*stdev;
			var cumulative=values.length>=4 && _.isBoolean(values[3]) && values[3];
			if(!cumulative){
				return 1/(Math.sqrt(2*Math.PI)*Math.sqrt(variance))*Math.exp(-1*(value-mean)*(value-mean)/(2*variance));
			}
			var x=(value-mean)/(Math.sqrt(2*variance));
			// save the sign of x
			var sign = (x >= 0) ? 1 : -1;
			x = Math.abs(x);
			// constants
			var a1 =  0.254829592;
			var a2 = -0.284496736;
			var a3 =  1.421413741;
			var a4 = -1.453152027;
			var a5 =  1.061405429;
			var p  =  0.3275911;
			// A&S formula 7.1.26
			var t = 1.0/(1.0 + p*x);
			var y = 1.0 - (((((a5 * t + a4) * t) + a3) * t + a2) * t + a1) * t * Math.exp(-x * x);
			var erf=sign * y; // erf(-x) = -erf(x);
			return 0.5*(1+erf);
		},
		"norminv":function norminv(){
			/// Original C++ implementation found at http://www.wilmott.com/messageview.cfm?catid=10&threadid=38771
			/// C# implementation found at http://weblogs.asp.net/esanchez/archive/2010/07/29/a-quick-and-dirty-implementation-of-excel-norminv-function-in-c.aspx
			/*
			 *     Compute the quantile function for the normal distribution.
			 *
			 *     For small to moderate probabilities, algorithm referenced
			 *     below is used to obtain an initial approximation which is
			 *     polished with a final Newton step.
			 *
			 *     For very large arguments, an algorithm of Wichura is used.
			 *
			 *  REFERENCE
			 *
			 *     Beasley, J. D. and S. G. Springer (1977).
			 *     Algorithm AS 111: The percentage points of the normal distribution,
			 *     Applied Statistics, 26, 118-121.
			 *
			 *      Wichura, M.J. (1988).
			 *      Algorithm AS 241: The Percentage Points of the Normal Distribution.
			 *      Applied Statistics, 37, 477-484.
			 */
			var values=Array.prototype.slice.call(arguments);
			if(values.length<3 || !_.isNumber(values[0]) || !_.isNumber(values[1]) || !_.isNumber(values[2]) || values[0]<=0 || values[0]>=1 || values[2]<=0){
				console.error("Error:Invalid syntax for Norminv");
				throw "Error:Invalid syntax for Norminv";
			}
			var probability=values[0];
			var mean=values[1];
			var stdev=values[2];
			if(stdev===0){
				return mean;
			}
			var r,val,q = probability - 0.5;
			/*-- use AS 241 --- */
			/* double ppnd16_(double *p, long *ifault)*/
			/*      ALGORITHM AS241  APPL. STATIST. (1988) VOL. 37, NO. 3

			 Produces the normal deviate Z corresponding to a given lower
			 tail area of P; Z is accurate to about 1 part in 10**16.
			 */
			if (Math.abs(q) <= .425){
				/* 0.075 <= p <= 0.925 */
				r = .180625 - q * q;
				val =
					q * (((((((r * 2509.0809287301226727 +
					33430.575583588128105) * r + 67265.770927008700853) * r +
					45921.953931549871457) * r + 13731.693765509461125) * r +
					1971.5909503065514427) * r + 133.14166789178437745) * r +
					3.387132872796366608)
					/ (((((((r * 5226.495278852854561 +
					28729.085735721942674) * r + 39307.89580009271061) * r +
					21213.794301586595867) * r + 5394.1960214247511077) * r +
					687.1870074920579083) * r + 42.313330701600911252) * r + 1);
			}else{
				/* closer than 0.075 from {0,1} boundary */

				/* r = min(p, 1-p) < 0.075 */
				if (q > 0)
					r = 1 - p;
				else
					r = p;

				r = Math.Sqrt(-Math.log(r));
				/* r = sqrt(-log(r))  <==>  min(p, 1-p) = exp( - r^2 ) */

				if (r <= 5){
					/* <==> min(p,1-p) >= exp(-25) ~= 1.3888e-11 */
					r += -1.6;
					val = (((((((r * 7.7454501427834140764e-4 +
						.0227238449892691845833) * r + .24178072517745061177) *
						r + 1.27045825245236838258) * r +
						3.64784832476320460504) * r + 5.7694972214606914055) *
						r + 4.6303378461565452959) * r +
						1.42343711074968357734)
						/ (((((((r *
						1.05075007164441684324e-9 + 5.475938084995344946e-4) *
						r + .0151986665636164571966) * r +
						.14810397642748007459) * r + .68976733498510000455) *
						r + 1.6763848301838038494) * r +
						2.05319162663775882187) * r + 1);
				}else{
					/* very close to  0 or 1 */
					r += -5;
					val = (((((((r * 2.01033439929228813265e-7 +
						2.71155556874348757815e-5) * r +
						.0012426609473880784386) * r + .026532189526576123093) *
						r + .29656057182850489123) * r +
						1.7848265399172913358) * r + 5.4637849111641143699) *
						r + 6.6579046435011037772)
						/ (((((((r *
						2.04426310338993978564e-15 + 1.4215117583164458887e-7) *
						r + 1.8463183175100546818e-5) * r +
						7.868691311456132591e-4) * r + .0148753612908506148525)
						* r + .13692988092273580531) * r +
						.59983220655588793769) * r + 1);
				}

				if (q < 0.0){
					val = -val;
				}
			}

			return mean + stdev * val;
		}
	}
};