jQuery.sap.declare("sas.stress.wkbench.model.Process");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.model.Sheet");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.base.Object.extend("sas.stress.wkbench.model.Process", {
    constructor: function (project, id, name, description, startId, endId) {
        this.project = project;
        this.id = id;
        this.name = name;
        this.description = description;
        this.startId = startId;
        this.endId = endId;
        this.tasks = {};
        this.transitions = {};
        this.transGroups = {};
        this.transGroupsBySource = {};
        this.plannedStartDates = {};
        this.plannedEndDates = {};
        this.startDate = sap.ui.core.format.DateFormat.getDateInstance().parse(project.getProcessStartDate()).getTime();

        // S1403521
        this.storedDatesStr=localStorage.getItem("TaskDates-"+project.getProcessId());
        if (this.storedDatesStr){
            this.storedDatesJSON=JSON.parse(this.storedDatesStr);
        }
    }
});

sas.stress.wkbench.model.Process.prototype._calcPlannedDates = function () {
    this.plannedStartDates = {};
    this.plannedEndDates = {};
    var that = this;

    var lineage = [];
    var calculateDates = function (task, startDate) {
        // Verify the task exists
        if (!task) {
            // The task doesn't exist - get out.
            return;
        }
        // lineage check
        if (-1 !== lineage.indexOf(task.getId())) {
            // We've already calculated this node's dates, we do not want to overwrite those calculations. (a loop)
            return;
        }

        // check if the start date exists and is greater than what was proposed this time
        var currentStartDate = that.plannedStartDates[task.getId()];
        if ((currentStartDate !== undefined && currentStartDate > startDate) && that.plannedEndDates[task.getId()] !== undefined) {
            // We've already calculated this node's dates and they're greater than what is being proposed. Get out.
            return;
        } else {
            that.plannedStartDates[task.getId()] = startDate;
        }

        var delta = task.getDueDays() * 86400000;//24*60*60*1000
        var endDate = startDate + delta;
        that.plannedEndDates[task.getId()] = endDate;
        lineage.push(task.getId());

        var transGroups = that.transGroupsBySource[task.getId()];
        _.each(transGroups, function (group) {
            _.each(group, function (transition) {
                var targetTaskId = transition.getTargetId();
                if (targetTaskId !== undefined) {
                    // The end date of the current task is proposed as the start date for the subsequent task.
                    calculateDates(that.tasks[targetTaskId], endDate);
                } else {
                    // I don't really think this should happen, but to be safe we will clean up the lineage and log it to the console.
                    lineage.pop();
                    console.log("Stress error: targetId for a transition was undefined in task " + task.getId() + ")");
                    return;
                }
            });
        });
        lineage.pop();
    };
    if (this.tasks[this.startId] && this.tasks[this.endId]) {
        calculateDates(this.tasks[this.startId], this.startDate);
    }
};
sas.stress.wkbench.model.Process.prototype.getPlannedStartDate = function (taskId) {
    return this.plannedStartDates[taskId];
};
sas.stress.wkbench.model.Process.prototype.getPlannedEndDate = function (taskId) {
    return this.plannedEndDates[taskId];
};
sas.stress.wkbench.model.Process.prototype.getId = function () {
    return this.id;
};
sas.stress.wkbench.model.Process.prototype.getName = function () {
    return this.name;
};
sas.stress.wkbench.model.Process.prototype.getProject = function () {
    return this.project;
};
sas.stress.wkbench.model.Process.prototype.getProcessProject = function () {
    return this.project;
};
sas.stress.wkbench.model.Process.prototype.addTask = function (task) {
    this.tasks[task.getId()] = task;
    //var datesStr=localStorage.getItem("TaskDates-"+this.getId());
    // S1403521 make sure the cache of dates is present and the start date matches
    if (this.storedDatesStr&&this.storedDatesJSON.processStartDate===this.getProject().getProcessStartDate()){
        this.plannedStartDates[task.getId()]=this.storedDatesJSON.startDates[task.getId()];
        this.plannedEndDates[task.getId()]=this.storedDatesJSON.endDates[task.getId()];
    }else{
        this._calcPlannedDates();
        localStorage.removeItem("TaskDates-"+this.getProject().getProcessId());
        if (_.size(this.plannedStartDates)>0 && _.size(this.plannedEndDates)>0){
            this.storedDatesStr= JSON.stringify({
                processStartDate: this.getProject().getProcessStartDate(),
                startDates: this.plannedStartDates,
                endDates: this.plannedEndDates
            });
            localStorage.setItem("TaskDates-"+this.getProject().getProcessId(), this.storedDatesStr);
        }
    }
};
sas.stress.wkbench.model.Process.prototype._getObj = function (obj, key, isArray) {
    var holder = obj[key];
    if (!holder) {
        holder = isArray ? [] : {};
        obj[key] = holder;
    }
    return holder;
};
sas.stress.wkbench.model.Process.prototype.addTransition = function (transition) {
    this.transitions[transition.getId()] = transition;
    // By Source ID
    var groupsBySource = this._getObj(this.transGroupsBySource, transition.getSourceId());
    var groupBySource = this._getObj(groupsBySource, transition.getSyncGroup(), true);
    groupBySource.push(transition);//eg:transGroups:{ sourceId:{sync1:[t1,t2,t3],sync2:[t4,t6]}, sourceId2:{...}}

    // By Target ID
    // TODO: would be nice to remove need for this second section (see getNodes and getLinks for usage of this style)
    var groupsByTarget = this._getObj(this.transGroups, transition.getTargetId());
    var groupByTarget = this._getObj(groupsByTarget, transition.getSyncGroup(), true);
    groupByTarget.push(transition);//eg:transGroups:{ sourceId:{sync1:[t1,t2,t3],sync2:[t4,t6]}, sourceId2:{...}}
};
sas.stress.wkbench.model.Process.prototype.getNodes = function () {
    var nodes = [];
    _.each(this.tasks, function (task) {
        nodes.push(
            {
                key: task.getId(),
                text: task.getName(),
                category: task.getType(),
                status: task.getStatusValue(),
                commentCount: task.getCommentCount(),
                icon: task.getStatusIcon(),
                label: task.getStatusLabel()
            }
        );
    });
    _.each(this.transGroups, function (groups, targetId) {
        _.each(groups, function (ts, syncGroupId) {
            if (ts.length > 1) {
                nodes.push(
                    {
                        key: targetId + "_BAR",
                        text: '',
                        category: '_SYNCBAR_'
                    }
                );
            }
        });
    });
    var rb = sas.stress.wkbench.StressUtil.getBundle();
    nodes.push({ key: -1, text: rb.getText('task.activity.start.txt'), category: '_START_' });
    nodes.push({ key: -2, text: rb.getText('task.activity.end.txt'), category: '_END_' });
    return nodes;
};

sas.stress.wkbench.model.Process.prototype.getLinks = function () {
    var links = [];
    var syncBars = {};
    var that = this;
    _.each(this.transitions, function (transition) {
        var mustSync = that.transGroups[transition.getTargetId()][transition.getSyncGroup()].length > 1;
        links.push(
            {
                from: transition.getSourceId(),
                to: transition.getTargetId() + (mustSync ? "_BAR" : ""),
                text: transition.getExpression()
            }
        );
        if (mustSync) {
            syncBars[transition.getTargetId()] = true;
        }
    });
    _.each(syncBars, function (val, targetId) {
        links.push(
            {
                from: targetId + "_BAR",
                to: targetId
            }
        );
    });
    _.each(this.tasks, function (task) {
        if (task.getType() === 'START') {
            links.push({ from: -1, to: task.getId() });
        } else if (task.getType() === 'END') {
            links.push({ from: task.getId(), to: -2 });
        }
    });
    return links;
};
