jQuery.sap.declare("sas.stress.wkbench.model.Sheet");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.model.SheetInfo");
jQuery.sap.require("sas.stress.scenarios.ScenarioAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.base.Object.extend("sas.stress.wkbench.model.Sheet", {
    constructor: function (project, sheetCategory, id, name,version) {
        this.project = project;
        this.sheetCategory = sheetCategory;
        this.id = id;
        this.name = name;
        this._queue = [];
        this.sheetFilterKey = "sas.stress.wkbench.sheet_" + id + ".filters";
        //this.setPreferredSheetFilters();

        this.allFilters = {};
        this.meta = undefined;

        this.version=version;

        this.errored=false; // there are cases where the mid-tier returns true and UI fails S1266931
        this.preferredFilters=[];   // using S1272998...

        this.nameVersion=sas.stress.wkbench.StressUtil.getNameVersion(this.name,this.version);
        this.noVisibilityReasons=undefined;
    }
});
sas.stress.wkbench.model.Sheet.prototype.getProject = function () {
    return this.project;
};
sas.stress.wkbench.model.Sheet.prototype.getSheetCategory = function () {
    return this.sheetCategory;
};
sas.stress.wkbench.model.Sheet.prototype.getId = function () {
    return this.id;
};
sas.stress.wkbench.model.Sheet.prototype.getName = function () {
    return this.name;
};
sas.stress.wkbench.model.Sheet.prototype.getSheetScenario = function (scenarioId) {
    var sheetInfo = this.getInfo();
    return sheetInfo.getScenario(scenarioId);
};
sas.stress.wkbench.model.Sheet.prototype.getErrored = function () {
    return this.errored;
};
sas.stress.wkbench.model.Sheet.prototype.getNameVersion=function(){
    return this.nameVersion;
};
// fixes for S1228592
// send in the iterationID from the project when loading the sheetInfo
sas.stress.wkbench.model.Sheet.prototype.loadSheetInfo = function (sheetinfo) {
    if (this.meta){
        this.meta.destroy();
    }
    var that=this;
    var projectId=this.project.getId();

    try {
        // using S1272998... set the filterhierarchies first.. this will ensure the preferredfilters is set even if failure detected
        that.setPreferredSheetFilters(sheetinfo.filterHierarchies);

        that.meta = new sas.stress.wkbench.model.SheetInfo(that,sheetinfo);
        that.meta.loadInfo(sheetinfo);
    } catch (e) {
        throw e;
    }
};
sas.stress.wkbench.model.Sheet.prototype.loadSheetFilters = function (projectId) {
    var deferred = $.Deferred();
    var size = _.size(this.allFilters);
    if (size > 0) {
        deferred.resolve(this.allFilters);
    }

    var that = this;
    sas.stress.wkbench.StressAPI.loadSheetFilters(projectId, this.id)
        .done(function (items) {
            that.setSheetFilters(items);
            deferred.resolve(items);
        })
        .fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });

    return deferred;
};
sas.stress.wkbench.model.Sheet.prototype.getInfo = function () {
    return this.meta;
};
sas.stress.wkbench.model.Sheet.prototype.getSheetLogUpdates = function () {
    var deferred=$.Deferred();
    var updates = [];
    var filters = this.getProject().getFilters();
    var payloadEntry={
        // send in the iteration from the user_preferences if defined, else it is the current iteration
        iterationNum:Number(sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id",this.getProject().getId()))||this.getProject().getCurrentIterationNo(),
        filters: filters
    };

    sas.stress.wkbench.StressAPI.loadSheetHistory(this.getProject().getId(),this.getId(),payloadEntry)
        .done(function(history){
            var filters=sheetInfo.getFilters();
            updates=filters.getLogUpdates();
            updates = _.union(history.historyItems, updates);
            //updates=history.historyItems;
            deferred.resolve(updates);
        })
        .fail(function(jqxhr){
            deferred.reject(jqxhr);
        });
    var sheetInfo = this.getInfo();


    //return updates;
    return deferred;
};

sas.stress.wkbench.model.Sheet.prototype.getSheetFilterHierarchies=function(){
    return this.preferredFilters;
};

sas.stress.wkbench.model.Sheet.prototype.setPreferredSheetFilters = function (sheetFilters) {
    this.preferredFilters = sheetFilters;
};

/* this is required when the sheet template is edited. All the sheets that have the template need to have the preferred filters cleared */
sas.stress.wkbench.model.Sheet.prototype.clearPreferredSheetFilters = function () {
    sas.stress.wkbench.UserPrefs.delete(this.sheetFilterKey, this.getProject().getId());
    if (this.meta) {
        this.meta.clearData();
    }
    this.preferredFilters = [];
};

sas.stress.wkbench.model.Sheet.prototype.getPreferredSheetFilter = function () {
    var sheetFilters=[];
    var projectFilters=this.getProject().getFilters();
    for (var index=0;index<this.preferredFilters.length;index++){
        var sheetfilter=this.preferredFilters[index];
        var preferredFilter=_.find(projectFilters,function(filter){
            return filter.hierarchyKey===sheetfilter;
        });
        if (preferredFilter){
            sheetFilters.push(preferredFilter.liKey);
        }
    }
    return sheetFilters;
};

sas.stress.wkbench.model.Sheet.prototype.setSheetFilters = function (items) {
    this.allFilters = items;
};

sas.stress.wkbench.model.Sheet.prototype.getSheetFilters = function () {
    return this.allFilters;
};

sas.stress.wkbench.model.Sheet.prototype.destroy = function () {
    if (this.meta !== undefined) {
        this.preferredFilters = [];
        this.meta.destroy();
        this.meta = undefined;
        delete this.meta;
    }

    this.project = undefined;
    delete this.project;
    this.sheetCategory = undefined;
    delete this.sheetCategory;
    this.allFilters = {};

    this.destroyed = true;
    return this;
};

sas.stress.wkbench.model.Sheet.prototype.setNoVisibilityReasons=function(noVisibilityReasons){
    this.noVisibilityReasons=noVisibilityReasons;
};

sas.stress.wkbench.model.Sheet.prototype.getNoVisibilityReasons=function(noVisibilityReasons){
    return this.noVisibilityReasons;
};

sas.stress.wkbench.model.Sheet.NO_PERMISSONS="no permissions";
sas.stress.wkbench.model.Sheet.prototype._generateColumnHeaders = function (sheetjson) {
    var columnHeaders = {};                       // map key="level" and value="Array[] of column names"

    var _addColumnHeader = function (column, key) {
        var rootkey = key;
        // column has children
        $.each(column.children || [], function (idx, childColumn) {
            var parent = column;

            // the column at its level
            var childrenLength = childColumn.children.length;
            var levelColumnHdrs = columnHeaders[childColumn.level] || {labels: [], liKeys: []};

            levelColumnHdrs.labels.push(childColumn.name);
            levelColumnHdrs.liKeys.push(childColumn.liKey);
            //levelColumnHdrs.spans.push((childrenLength===0)?childrenLength+1:childrenLength);
            for (indexOfHdr = 1; indexOfHdr < childrenLength; indexOfHdr++) {
                levelColumnHdrs.labels.push(childColumn.name);
                levelColumnHdrs.liKeys.push(childColumn.liKey);
            }
            // add the column header to the array if not present
            if (!columnHeaders[childColumn.level]) {
                columnHeaders[childColumn.level] = levelColumnHdrs;
            }

            // need to append the parent too
            if (childrenLength > 0) {
                for (var lvl = 0; lvl < childColumn.level; lvl++) {
                    var parenthdr = columnHeaders[lvl];
                    for (indexOfHdr = 1; indexOfHdr < childrenLength; indexOfHdr++) {
                        parenthdr.labels.push((parent && parent.parent && lvl !== (childColumn.level - 1)) ? parent.parent : childColumn.parent);
                    }
                }
            }

            // look for child column's children
            _addColumnHeader(childColumn, rootkey);
        });

        return;
    };
    _.each(sheetjson.columnhierarchy, function (column, idx) {
        // the children
        var headerinfo = columnHeaders[column.level] || {labels: [], liKeys: []};
        headerinfo.labels.push(column.name);
        headerinfo.liKeys.push(column.liKey);

        if (!columnHeaders[column.level]) {
            columnHeaders[column.level] = headerinfo;
        }
        for (var i = 0; i < (column.children.length - 1); ++i) {  // for the children.. one less becuase we have added one earlier
            headerinfo.labels.push(column.name);
            headerinfo.liKeys.push(column.liKey);
        }
        _addColumnHeader(column, idx);
    });
    // TODO: need to make sure all the column headers have the same size
    // TODO: need to add the measures information in.. the final structure will change
    sheetjson.columnsHeadersInfo = {headers: columnHeaders};

    return;
};