jQuery.sap.declare("sas.stress.wkbench.model.SheetFiltersData");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.parser.Lexer");
jQuery.sap.require("sas.stress.wkbench.parser.Parser");
jQuery.sap.require("sas.stress.wkbench.parser.ExpressionUtil");
jQuery.sap.require("sas.stress.wkbench.CellEvaluationUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");

sas.stress.wkbench.model.SheetFiltersData=function(scenario){
	this.scenario=scenario;
	this.sheetInfo=scenario.getSheetInfo();
	var sheet=this.sheetInfo.getSheet();
	this.sheetName=sheet.getName();
	this.sheetId=sheet.getId();
	this.project=sheet.getProject();
	this.scenarioName=scenario.getName();
	this.data=[];
	this.cComments={};
	var Comments=function(){};
	Comments.prototype=this.cComments;
	this.ncComments=new Comments();
	this.cFormulas={};
	var Formulas=function(){};
	Formulas.prototype=this.cFormulas;
	this.ncFormulas=new Formulas();

	this.cValidationErrors={};
	var ValidationErrors=function(){};
	ValidationErrors.prototype=this.cValidationErrors;
	//this.ncValidationErrors=new ValidationErrors();

	this.cDrillDowns={};
	var Drilldowns=function(){};
	Drilldowns.prototype=this.cDrillDowns;
};

sas.stress.wkbench.model.SheetFiltersData.prototype={
	loadData:function(rowdata){
		this.data=rowdata;
	},
	loadOverlays:function(data){
		var that=this;
		if(!data){
			return;
		}
		var i,j,overlay,key,rowIndex,colIndex,expr,refsToAdd,thisCellRefStr,cellRefStr,validationkey/*keep this separate please.. pushing with S1248704*/;
		var columnId,rowId;
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var deps=this.project._overlayDeps;
		var infoData=this.sheetInfo.getData();
        var length=data.length /*_.size(overlays)*/;
		for(i=length-1;i>=0;i--){//assumed that server sends overlays ordered by timestamp (for a cell)
			var row = data[i]; 		//overlay=overlays[i];
			_.each(row,function(datakey,key){
				overlay=that.project.getCellOverlayByDataKey(datakey);
				if (overlay) {
					rowId = infoData.getRowLongFromDatakey(overlay.dataKey);
					columnId = infoData.getColLongFromDatakey(overlay.dataKey);
					rowIndex = infoData.getRowIndexByLongUsingDatakey(overlay.dataKey);
					colIndex = infoData.getColumnIndexByLongUsingDatakey(overlay.dataKey);
					key = "$" + columnId + "$" + rowId;
					if (overlay.comment!== undefined) {
						if (!that.cComments[key]) {
							that.cComments[key] = {
								user: overlay.comment.user,
								timestamp: overlay.comment.timestamp,
								comment: overlay.comment.value,
								rowIndex: rowIndex,
								columnIndex: colIndex,
								dataKey: overlay.dataKey
							};
						}
					}
					if (overlay.validationError!== undefined) {
						validationkey = "$" + colIndex + "$" + rowIndex;
						if (!that.cValidationErrors[validationkey]) {
							that.cValidationErrors[validationkey] = {
								user: overlay.validationError.user,
								timestamp: overlay.validationError.timestamp,
								validationError: overlay.validationError.value,
								rowIndex: rowIndex,
								columnIndex: colIndex
							};
						}
					}
					if (overlay.value!== undefined && (overlay.value.value!==undefined&&overlay.value.value!== null)/*&&overlay.value.length>0*/) {
						var propertyKey=overlay.value.propertyKey;
						if (propertyKey>0&&!that.cDrillDowns[key]){
							that.cDrillDowns[key]={
								rowIndex: rowIndex,
								columnIndex: colIndex,
								propertyKey: propertyKey,
								dataKey: overlay.dataKey
							};
						}
						if (!that.cFormulas[key]) {
							expr = overlay.value.parsedFormula || CellEvaluationUtil.parseFormula(overlay.value.value);
							that.cFormulas[key] = {
								user: overlay.value.user,
								timestamp: overlay.value.timestamp,
								txt: overlay.value.value,
								expr: expr,
								rowIndex: rowIndex,
								columnIndex: colIndex,
								status: 0, //0=parsed only, 1=evaluating, 2=evaluated
								dataKey: overlay.dataKey,
								propertyKey: propertyKey,
								systemGenerated: overlay.value.systemGenerated
							};
							refsToAdd = CellEvaluationUtil.getCellRefStrs(expr, that.scenario);
							thisCellRefStr = colIndex + ":" + rowIndex + ":" + that.scenario.id + ":" + that.sheetId;
							for (j = 0; j < refsToAdd.length; j++) {//update map cell->direct dependents. add itself as dependent
								cellRefStr = refsToAdd[j];
								deps[cellRefStr] = _.union(deps[cellRefStr] || [], [thisCellRefStr]);
							}
						}
					}
				}
			});
		}
	},
	addComment:function(key,rowIndex,colIndex,user,comment,source){
		var theSource="sheetOverride";
		if (source!==undefined){
			theSource="allocationOverride"; // TODO: could be theSource=source
		}
		this.ncComments[key]={
			user:user,
			comment:comment,
			rowIndex:rowIndex,
			columnIndex:colIndex,
			source:theSource
		};
		var datakey=this.scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
		this.project.setCellCommentByDataKey(datakey,{user:user,comment:comment});
	},
	processOverlays:function(){
		for(var key in this.cFormulas){
			var overlay=this.cFormulas[key];
			this.evalOverlay(overlay);
		}
	},
	evalOverlay:function(overlay){
		var value,dataKey,celloverlay;
		dataKey=overlay.dataKey;
		switch(overlay.status){
		case 0://parsed only
			overlay.status=1;
			try{
				if (!!overlay.error){
					value=overlay.txt;
				}else {
					value = sas.stress.wkbench.parser.ExpressionUtil.evaluate(overlay.expr, $.proxy(this.evalContext, this));
				}
			}catch(e){
                if (e.message&& e.message==="#DIV/0!"){
                    console.error("Unable to evaluate expression:\n"+sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr),overlay);
                    value= e.message;
                }else if(e.message&& e.message==="#REF!"){
					console.error("Invalid cross-sheet cell reference:\n"+sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr),overlay);
					value= e.message;
				}else {
                    console.error("Unable to evaluate expression:\n" + sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr), overlay);
                    value = "#NAME?";
                }
			}
			// it's a formula(=F10) and the value evaluated to null
			var showEquals=overlay.txt.indexOf("=")===0;
			if (showEquals&&value===null){
				value=0;	// set the value to 0..
			}
			this.data[overlay.rowIndex][overlay.columnIndex] = value;
			celloverlay=this.project.getCellOverlayByDataKey(dataKey);
			if (!celloverlay){
				// create a new on the global list
				this.project.setCellDataByDataKey(dataKey,overlay);
				this.project.setCellValueByDataKey(dataKey,value);
			}else {
				//update the global value
				celloverlay.status=2;		// evaluated
				if(celloverlay.value&&celloverlay.value.value!==overlay.txt) {	// if the value for the formula specified ignore
					celloverlay.value.parsedFormula = overlay.expr;
					celloverlay.value.value = overlay.txt;
					celloverlay.value.user = overlay.user;
				}

				this.project.setCellValueByDataKey(dataKey,value);
			}
			overlay.status=2;
			return value;
		case 1://evaluating
			console.error("ERROR:Circular Dependency",overlay,this);
			throw "ERROR!";
		case 2://evaluated
			// however, the value indexed by the "datakey" could have changed due to updates in other sheet
			//	use that value and update the array because the slimsheet does not have idea of the "datakey"
			//	to get the current value.. some bug Jeff, S found
			value=this.project.getCellValueByDataKey(dataKey);
			this.data[overlay.rowIndex][overlay.columnIndex]=value;
			return value/*this.data[overlay.rowIndex][overlay.columnIndex]   this.project.getCellValueByDataKey(dataKey)*/;
		}
	},
	/*
	 The [^'\-] needs to be [^']. We had a case where the sheet and scenario name were separated with a -
	 for eg sheet1 - baseline
	 This is no longer the case
	 this is the original regex
	/^(?:'([^'\-]*)(?:\s\-\s)?([^'\-]*)'!){0,1}\$([\w|\@]*)\$([\w|\@]*)/
	*/
	RE_LONG_REFERENCE:/^(?:'([^']*)(?:\s\-\s)?([^']*)'!){0,1}\$([\w|\@]*)\$([\w|\@]*)/,		/* /^(?:'([^'\-]*)(?:\s\-\s)?([^'\-]*)'!){0,1}\$([\w]*)\$(\w*)$/,*/
	RE_SHORT_REFERENCE:/^(?:'([^']*)(?:\s\-\s)?([^']*)'!){0,1}([A-Za-z]*)(\d*)$/,		/* /^(?:'([^'\-]*)(?:\s\-\s)?([^'\-]*)'!){0,1}([A-Za-z]*)(\d*)$/, */
	evalContext:function(incoming){
		var func=sas.stress.wkbench.CellEvaluationUtil.functions[incoming.toLowerCase()];
		if(func){
			return func;
		}
		var range=incoming.split(":");
		var isLong=true;
		var trimmedstr=range[0].trim();
		var parsed=this.RE_LONG_REFERENCE.exec(trimmedstr);
		if(!parsed){
			parsed=this.RE_SHORT_REFERENCE.exec(trimmedstr);
			isLong=false;
		}
		var sheetName=parsed[1];
		if(sheetName&&sheetName!==this.sheetName){//range is from a different sheet, redirect to the other scenario
			var sheet=this.scenario.getSheetInfo().getSheet().getProject().getSheetByName(sheetName);
			var scenario=sheet.getInfo().getFilters();/*getScenarioByName(this.scenarioName)*/
			return scenario.getData().evalContext(incoming);
		}
		if(range.length===2){
			var infoData=this.sheetInfo.getData();
			if(isLong){
				var key="$"+parsed[3]+"$"+parsed[4];
				var datakey=infoData.getDatakeyFromRowColumn(key);
				var colIndex=infoData.getColumnIndexByLong(parsed[3]);
				var rowIndex=infoData.getRowIndexByLong(parsed[4]);
				trimmedstr=range[1].trim();
				parsed=this.RE_LONG_REFERENCE.exec(trimmedstr);
				var key2="$"+parsed[3]+"$"+parsed[4];
				var datakey2=infoData.getDatakeyFromRowColumn(key);
				var colIndex2=infoData.getColumnIndexByLong(parsed[3]);
				var rowIndex2=infoData.getRowIndexByLong(parsed[4]);
			}else{
				var rowIndex=infoData.getRowIndexByShort(parsed[4]);
				var colIndex=infoData.getColumnIndexByShort(parsed[3]);
				trimmedstr=range[1].trim();
				parsed=this.RE_SHORT_REFERENCE.exec(trimmedstr);
				var rowIndex2=infoData.getRowIndexByShort(parsed[4]);
				var colIndex2=infoData.getColumnIndexByShort(parsed[3]);
			}
			var i,j;
			var values=[];
			for(i=Math.min(rowIndex,rowIndex2);i<=Math.max(rowIndex,rowIndex2);i++){
				if(this.data[i]){//is it a data row?
					for (j = Math.min(colIndex, colIndex2); j <= Math.max(colIndex, colIndex2); j++) {
						values.push(this._evalCellByIndeces(i, j));
					}
				}
			}
			return values;
		}
		return isLong?this._evalCellByLongKey(parsed[3],parsed[4]):this._evalCellByShortKey(parsed[3],parsed[4]);
	},
	_evalCellByLongKey:function(colLong,rowLong){
		var key="$"+colLong+"$"+rowLong;
		var overlay=this.ncFormulas[key]||this.cFormulas[key];
		if(overlay){
			return this.evalOverlay(overlay);
		}
		var infoData=this.sheetInfo.getData();
		var dataKey=infoData.getDatakeyFromRowColumn(key);
		var colIndex=infoData.getColumnIndexByLong(colLong);
		var rowIndex=infoData.getRowIndexByLong(rowLong);
		if (colIndex===-1||rowIndex===-1){
			colIndex=infoData.getColumnIndexByLongUsingDatakey(dataKey);
			rowIndex=infoData.getRowIndexByLongUsingDatakey(dataKey);
		}
		return this.project._values[dataKey]?this.project._values[dataKey]:this.data[rowIndex][colIndex];	// give preference to the project values
	},
	_evalCellByShortKey:function(colShort,rowShort){
		var infoData=this.sheetInfo.getData();
		var colIndex=infoData.getColumnIndexByShort(colShort);
		var rowIndex=infoData.getRowIndexByShort(rowShort);
		// start fixes for S1267198
		var rowId=this.sheetInfo.getRowLong(rowIndex);
		var colId=this.sheetInfo.getColumnLong(colIndex);
		var key="$"+colId+"$"+rowId;
		var overlay=this.ncFormulas[key]||this.cFormulas[key];
		// end fixes for S1267198
		if(overlay){
			return this.evalOverlay(overlay);
		}
		var dataKey=infoData.getDatakeyFromRowColumn(key);
		return this.project._values[dataKey]?this.project._values[dataKey]:this.data[rowIndex][colIndex];	// give preference to the project values
	},
	_evalCellByIndeces:function(rowIndex,colIndex){
		var infoData=this.sheetInfo.getData();
		var rowId=this.sheetInfo.getRowLong(rowIndex);
		var colId=this.sheetInfo.getColumnLong(colIndex);
		var overlay=this.ncFormulas["$"+colId+"$"+rowId]||this.cFormulas["$"+colId+"$"+rowId];	// fixes for S1203544.. look first in the non-committed
		if(overlay){
			return this.evalOverlay(overlay);
		}
		var key="$"+colId+"$"+rowId;
		var dataKey=infoData.getDatakeyFromRowColumn(key);
		return this.project._values[dataKey]?this.project._values[dataKey]:this.data[rowIndex][colIndex];	// give preference to the project values
	},
	printLongContext:function(incoming){
		var func=sas.stress.wkbench.CellEvaluationUtil.functions[incoming.toLowerCase()];
		if(func){
			return incoming;
		}
		var range=incoming.split(":");
		var trimmedstr=range[0].trim();
		if(this.RE_LONG_REFERENCE.test(trimmedstr)){//already in long format?
			return incoming;
		}
		var parsed=this.RE_SHORT_REFERENCE.exec(trimmedstr);
		var sheetName=parsed[1];
		if(sheetName&&sheetName!==this.sheetName){//range is from a different sheet, redirect to the other scenario
			var sheet=this.scenario.getSheetInfo().getSheet().getProject().getSheetByName(sheetName);
			var scenario=sheet.getInfo().getFilters();//sheet.getInfo().getScenarioByName(this.scenarioName);
			return "'"+sheetName+"'!"+scenario.getData().printLongContext(incoming);
		}
		var infoData=this.sheetInfo.getData();
		var rowIndex=infoData.getRowIndexByShort(parsed[4]);
		var colIndex=infoData.getColumnIndexByShort(parsed[3]);
        if (rowIndex===-1||colIndex===-1){
            return range[0];
        }
		var shortTxt="$"+infoData.getLongColumn(colIndex)+"$"+infoData.getLongRow(rowIndex);
		if(range.length===2){
			trimmedstr=range[1].trim();
			parsed=this.RE_SHORT_REFERENCE.exec(trimmedstr);
			var rowIndex2=infoData.getRowIndexByShort(parsed[4]);
			var colIndex2=infoData.getColumnIndexByShort(parsed[3]);
			shortTxt+=":$"+infoData.getLongColumn(colIndex2)+"$"+infoData.getLongRow(rowIndex2);
		}
		return shortTxt;
	},
	printShortContext:function(incoming){
		var func=sas.stress.wkbench.CellEvaluationUtil.functions[incoming.toLowerCase()];
		if(func){
			return incoming;
		}
		var range=incoming.split(":");
		var trimmedstr=range[0].trim();
		if(this.RE_SHORT_REFERENCE.test(trimmedstr)){//already in short format?
			return incoming;
		}
		var parsed=this.RE_LONG_REFERENCE.exec(trimmedstr);
		var sheetName=parsed[1];
		if(sheetName&&sheetName!==this.sheetName){//range is from a different sheet, redirect to the other scenario
			var sheet=this.scenario.getSheetInfo().getSheet().getProject().getSheetByName(sheetName);
			var scenario=sheet.getInfo().getFilters();/*sheet.getInfo().getScenarioByName(this.scenarioName);*/
			return "'"+sheetName+"'!"+scenario.getData().printShortContext(incoming);
		}
		var infoData=this.sheetInfo.getData();

		var colRowKey, datakey;
		colRowKey="$"+parsed[3]+"$"+parsed[4];
		datakey=infoData.getDatakeyFromRowColumn(colRowKey);
		if (!datakey){
			colRowKey="$"+parsed[4]+"$"+parsed[3];
			datakey=infoData.getDatakeyFromRowColumn(colRowKey);
		}
		var index;
		index=infoData.getColumnIndexByShort(parsed[3]);
		if (index<0){
			datakey=infoData.getDatakeyFromRowColumn(colRowKey);
			index=infoData.getColumnIndexByLongUsingDatakey(datakey);
		}
		var colIndex=index;
		index=infoData.getRowIndexByShort(parsed[4]);
		if (index<0){
			datakey=infoData.getDatakeyFromRowColumn(colRowKey);
			index=infoData.getRowIndexByLongUsingDatakey(datakey);
		}
		var rowIndex=index;

		var shortTxt=infoData.getShortColumn(colIndex)+infoData.getShortRow(rowIndex);
		if(range.length===2){
			trimmedstr=range[1].trim();
			parsed=this.RE_LONG_REFERENCE.exec(trimmedstr);
			var rowIndex2=infoData.getRowIndexByLong(parsed[4]);
			var colIndex2=infoData.getColumnIndexByLong(parsed[3]);
			shortTxt+=":"+infoData.getShortColumn(colIndex2)+infoData.getShortRow(rowIndex2);
		}
		return shortTxt;
	},
	destroy:function(){
		var that=this;
		var keys=[];
		keys= _.keys(this.ncFormulas);
		_.each(keys,function(key){
			that.ncFormulas[key]=undefined;
			delete that.ncFormulas[key];
		});
		keys=[];
		keys= _.keys(this.ncComments);
		_.each(keys,function(key){
			that.ncComments[key]=undefined;
			delete that.ncComments[key];
		});
		keys=[];
		keys= _.keys(this.cValidationErrors);
		_.each(keys,function(key){
			that.cValidationErrors[key]=undefined;
			delete that.cValidationErrors[key];
		});
		this.data=[];
		this.sheetInfo=undefined;
		delete this.sheetInfo;
		this.project=undefined;
		delete this.project;

		return this;
	}
};