jQuery.sap.declare("sas.stress.wkbench.model.SheetInfoData");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.parser.Lexer");
jQuery.sap.require("sas.stress.wkbench.parser.Parser");
jQuery.sap.require("sas.stress.wkbench.parser.ExpressionUtil");
jQuery.sap.require("sas.stress.wkbench.CellEvaluationUtil");
jQuery.sap.require("sas.stress.wkbench.model.Formatter");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.UserPrefs");
jQuery.sap.require("sas.stress.wkbench.model.TreeNode");

sas.stress.wkbench.model.SheetInfoData=function(json){
	this.columnShort={};
	this.columnLong={};
	this.columns=[];
	this.rowShort={};
	this.rowLong={};
	this.rows=[];
	this.rowsLocked=[];
	this.rowsRoots=[];
    this.columnHeadersCount=0;
    this.rowHierarchiesCount=0;
    this.json=json;
    this.mapRowHeaders={};
    this.mapColumnHeaders={};

    this.overlays=[];
    this.rowData={};
    this.datakeyToRowCol={};
    this.rowColumnToDatakey={};
    this.columnRowToDataKey={};

    this.rowsVisible=[];
};
sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER=new sas.stress.wkbench.model.Formatter("Numeric",2,1);
sas.stress.wkbench.model.SheetInfoData.prototype={
	loadColumns:function(columnsInfo,readOnlyColumns){
		var i,short,longvalue,longkey,col,readonly,colLength,rowLength;

        // the tree of the column hierarchy
        this.columnHeadersCount=0;
        this._generateColumnHierarchyTree(this.json.columnHierarchy);
        if (this._columnHiearchyRoot) {
            // the column headers to be displayed
            this.columnHeadersCount=_.size(this.mapColumnHeaders);
            // the number of columns to be displayed.. should be the last headerRow's length
            colLength=0;
            if (this.columnHeadersCount>0) {
                colLength = this.mapColumnHeaders["headerrow_" + (this.columnHeadersCount - 1)].length;
            } else {
                throw this.json.sheetName;
            }
        } else {
            colLength=0;/*columnsInfo.length;*/
        }

		for(i=0;i<colLength;i++){
			//short=sas.stress.wkbench.StressUtil.calculateColumnAlias(i+(this.rowHierarchiesCount+1));   // emulate Excel.. will start from charcode of A + charcode of (row headers +1)
            short=sas.stress.wkbench.StressUtil.calculateColumnAlias(i);   // do not emulate Excel.. start from "A"
            if (this.columnHeadersCount===0) {
                longvalue = columnsInfo[i];
                longkey=longvalue;
            } else{
                var column=this.mapColumnHeaders["headerrow_"+(this.columnHeadersCount-1)][i];
                longvalue=column?(column.getDataKey()?column.getDataKey():column.getLiKey()):"";
                // this should handle a lot of failure cases of "undefined" references.
                longkey=this.mapColumnHeaders["headerrow_"+(this.columnHeadersCount-1)][i].getDataKey()?this.mapColumnHeaders["headerrow_"+(this.columnHeadersCount-1)][i].getDataKey():this.mapColumnHeaders["headerrow_"+(this.columnHeadersCount-1)][i].getLiKey();
            };
			col={
				short:short,
				long:longvalue,
				index:column.getCellValueIndex(),
                dataKey:column.getDataKey(),
				readonly:column.getReadOnly(),
                node:column
			};
			this.columnShort[short]=col;
			this.columnLong[longkey]=col;
			this.columns.push(col);
		}
	},
    loadRowHierarchy:function(){
        // the tree of the row hierarchy
        this.rowHierarchiesCount = this.rowLength = 0;
        if (this.json.rowHierarchy/*&&(this.json.maxRowOrder&&this.json.maxRowOrder>1)*/) {
            this._generateRowHierarchyTree(this.json.rowHierarchy,this.json.maxRowOrder);

            if (this._rowHiearchyRoot) {
                this.rowHierarchiesCount = _.size(this.mapRowHeaders);
                //this.rowLength = this.mapRowHeaders["headerrow_"+(this.rowHierarchiesCount-1)]?this.mapRowHeaders["headerrow_"+(this.rowHierarchiesCount-1)].length:sas.stress.wkbench.model.TreeNode.getSpan(this._rowHiearchyRoot);
                this.rowLength=0;
                if (this.rowHierarchiesCount>0) {
                    this.rowLength = sas.stress.wkbench.model.TreeNode.getSpan(this._rowHiearchyRoot);
                }
            } else {
                this.rowLength = 0;
            }
        } else {
            this.loadRows(this.json.rowHierarchy);//used to be: rows
            return;
        }
        // the rowsinfo== the rows from the last column
        var rowsinfo=this.mapRowHeaders["headerrow_"+(this.rowHierarchiesCount-1)];
        if (rowsinfo) {
            this.loadRows(rowsinfo, this.rowHierarchiesCount);
        }
    },
    loadRows:function(rowsInfo,level){
        var i,index,short,long,row,rowInfo,rowLength,formatter;
        level=level||0;
        rowLength=rowsInfo.length;
        for(i=0;i<rowLength;i++){
            rowInfo=rowsInfo[i];
            index=this.rows.length;
            short=(index+1).toString();
            long=rowInfo.getDataKey()/*getLiKey()*/;
            if (!rowInfo.getSubtotalLikey()){
                formatter=rowInfo.format?new sas.stress.wkbench.model.Formatter(rowInfo.formatType,rowInfo.formatPrecision,rowInfo.formatScale):
                    sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER;
            }else{
                formatter=rowInfo.getParent().format?new sas.stress.wkbench.model.Formatter(rowInfo.getParent().formatType,rowInfo.getParent().formatPrecision,rowInfo.getParent().formatScale):
                    sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER;
            }
            // fixes for S1233528.. if its a subtotal node.. the locked = true, however for the UI it should be false..
            //  why because the subtotal row is a placeholder.. the root/parent is not LOCKED
            var locked=rowInfo.getSubtotalLikey()?rowInfo.getParent().getLocked():rowInfo.getLocked(); // if it's a subtotal node, ask the parent
            /*var isVisible=rowInfo.getVisible();*/
            row={
                short: short,
                long: long,
                index: index,
                level: rowInfo.getLevel(),
                name: rowInfo.name,
                locked: locked,
                formatter: formatter,
                formulaCode: rowInfo.getFormulaCode(),
                contra: rowInfo.getContra(),
                aggregation: rowInfo.getAggregation(),
                dataKey:long,
                node: rowInfo
            };

            if(long){
                this.rowShort[short]=row;
                this.rowLong[long]=row;
            }
            if(locked){
                this.rowsLocked.push(row);
            }
            /*if (!isVisible){
                this.rowsVisible.push(row);
            }*/
            if(!row.level){
                this.rowsRoots.push(row);
            }
            this.rows.push(row);
        }
    },
    setCellData:function(project,reload){
        var that=this;
        var data=this.json.data;
        var values/*,overlays=[]*/;
        var i, row, rowinfo, rowLength;
        var cellOverlay,cellValue;
        rowLength = data.length;
        if (reload){
            this.overlays=[];
            this.rowData={};
            this.datakeyToRowCol={};
            this.rowColumnToDatakey={};
            this.columnRowToDataKey={};
        }
        for (i = 0; i < rowLength; i++) {
            row = data[i];
            rowinfo=this.rowShort[i+1];
            values=[];
            _.each(row,function(datakey,key){
                if (datakey) {
                    cellOverlay=project.getCellOverlayByDataKey(datakey);
                    cellValue =project.getCellValueByDataKey(datakey)||null;
                    if (cellOverlay&&cellOverlay.value&&cellOverlay.value.timestamp) {  // should this be parsedFormula?
                        //that.overlays.push(cellOverlay);
                        values.push(cellValue);
                    }else{
                        values.push(null);
                    }
                }else{
                    values.push(null);
                }

                var colalias=sas.stress.wkbench.StressUtil.calculateColumnAlias(key);
                var colinfo=that.columnShort[colalias];
                that.datakeyToRowCol[datakey]={rowlong:rowinfo.long,rowshort:rowinfo.short,collong:colinfo.long,colshort:colinfo.short};

                var key="$"+rowinfo.long+"$"+colinfo.long;
                that.rowColumnToDatakey[key]=datakey;
                key="$"+colinfo.long+"$"+rowinfo.long;
                that.columnRowToDataKey[key]=datakey;
            });
            this.rowData[i]=values;
        }
    },
	isEditable:function(row,col,ignoreLockStatus){
        var colLong=this.getLongColumn(col-1);
        var rowLong=this.getLongRow(row);
        var key="$"+colLong+"$"+rowLong;
        var dataKey=this.getDatakeyFromRowColumn(key);

        var currentProject = sas.stress.wkbench.model.Project.getCurrentProject();
        var isCellReadOnly=currentProject.isCellByDatakeyReadonly(dataKey);
        var iterationNumber = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", currentProject.getId()) || currentProject.getCurrentIterationNo();

        if (isCellReadOnly===undefined||isCellReadOnly===null) {
            // project locking enforcement
            if (!!!ignoreLockStatus) {
                return (Number(iterationNumber) === currentProject.getCurrentIterationNo() && !currentProject.isLocked()) &&
                    !(this.rows[row].node.readOnly || this.columns[col - 1].readonly);
                /*&& !this.rows[row].locked && !this.rows[row].node.getVisible()*/
            }
            return (Number(iterationNumber) === currentProject.getCurrentIterationNo()) &&
                !(this.rows[row].node.readOnly || this.columns[col - 1].readonly);
        }else{
            if (!!!ignoreLockStatus) {
                return (Number(iterationNumber) === currentProject.getCurrentIterationNo() && !currentProject.isLocked()) && !isCellReadOnly;
            }else{
                return Number(iterationNumber) === currentProject.getCurrentIterationNo()&& !isCellReadOnly;
            }
        }
	},
	getShortRow:function(rowIndex){
		return this.rows[rowIndex].short;
	},
	getShortColumn:function(colIndex){
		return this.columns[colIndex].short;
	},
    getShortColumnByLong:function(long){
        var colIndex=this.getColumnIndexByLong(long);
        return this.getShortColumn(colIndex);
    },
	getLongRow:function(rowIndex){
		return this.rows[rowIndex].long;
	},
	getLongColumn:function(colIndex){
		return this.columns[colIndex].long;
	},
	getColumnIndexByLong:function(long){
        var exists=this.columnLong[long.toUpperCase()];
        if (exists){
            return exists.index;
        }
		return -1;
	},
	getRowIndexByLong:function(long){
        var exists=this.rowLong[long.toUpperCase()];
        if (exists){
            return exists.index;
        }
		return -1;
	},
	getColumnIndexByShort:function(short){
        var exists=this.columnShort[short.toUpperCase()];
        if (exists===undefined){
            return -1;
        }
        return exists.index;
	},
	getRowIndexByShort:function(short){
        var exists=this.rowShort[short];
        if (exists===undefined){
            return -1;
        }
		return exists.index;
	},
    getRowIndexByLongUsingDatakey:function(datakey){
        var rowlong=this.getRowLongFromDatakey(datakey);
        return this.getRowIndexByLong(rowlong);
    },
    getColumnIndexByLongUsingDatakey:function(datakey){
        var collong=this.getColLongFromDatakey(datakey);
        return this.getColumnIndexByLong(collong);
    },
    getRowLongFromDatakey:function(datakey){
        var info=this.datakeyToRowCol[datakey];
        return info.rowlong;
    },
    getRowShortFromDatakey:function(datakey){
        var info=this.datakeyToRowCol[datakey];
        return info.rowshort;
    },
    getColLongFromDatakey:function(datakey){
        var info=this.datakeyToRowCol[datakey];
        return info.collong;
    },
    getColShortFromDatakey:function(datakey){
        var info=this.datakeyToRowCol[datakey];
        return info.colshort;
    },
    getDatakeyFromRowColumn:function(rowColKey){
        var datakey=this.rowColumnToDatakey[rowColKey];
        if (datakey) {
            return datakey;
        }else{
            return this.columnRowToDataKey[rowColKey];
        }
    },
    _getColumnHeadersMap:function(){
        return this.mapColumnHeaders;
    },
    _getRowHeadersMap:function(){
        return this.mapRowHeaders;
    },
    _getHeaderTreeRoot:function(){
        return this._columnHiearchyRoot;
    },
    _generateColumnHierarchyTree:function(columnHierarchy){
        var that=this;
        if (columnHierarchy) {
            this._columnHiearchyRoot = sas.stress.wkbench.model.TreeNode.load(columnHierarchy,false);
            var headersSize =this._columnHiearchyRoot.getDepth();
            this._generateColumnHierarchyTreeRecurse(this._columnHiearchyRoot,(headersSize-1));

            // set up the previous and next nodes after the map of rowheaders has been fully populated..
            _.each(this.mapColumnHeaders,function(nodes,key){
                that._setupPrevAndNextNodes(nodes);
            });
        }
    },
    _generateColumnHierarchyTreeRecurse:function(node,maxLevel){
        var nodeIdx,idx;
        var allnodes = [], nodes = [], emptynode;
        var childrenLength=node.getChildren().length;
        for (nodeIdx=0;nodeIdx<childrenLength;++nodeIdx) {
            var child = node.getChildren()[nodeIdx];
            var currentlevel = child.getLevel();
            // get the existing set of nodes at this level
            if ("headerrow_" + (currentlevel - 1) in this.mapColumnHeaders) {
                allnodes = this.mapColumnHeaders["headerrow_" + (currentlevel - 1)];
            }
            // add the child to the existing set of nodes
            allnodes.push(child);
            // if this is a sub-total node and the current level is less than maxLevel
            if (!child.likey && currentlevel < maxLevel) {
                // this is a sub-total node
                if ("headerrow_" + (currentlevel) in this.mapColumnHeaders) {
                    // need to push from the currentlevel till the end level
                    for (idx = currentlevel; idx < maxLevel; idx++) {
                        this.mapColumnHeaders["headerrow_" + idx].push(child);
                    }
                } else {
                    // create a new level and push the child there too for each of the levels from the (current level+1)
                    for (idx = (currentlevel + 1); idx <= maxLevel; idx++) {
                        nodes = [];
                        nodes.push(child);
                        this.mapColumnHeaders["headerrow_" + (idx - 1)] = nodes;
                    }
                }
            }
            if (child.getChildren().length > 0) {
                this._generateColumnHierarchyTreeRecurse(child, maxLevel);
            } else if (child.likey && currentlevel < maxLevel) {
                // found a node that does not have children... ragged therefore, it needs to have
                //  subtotal nodes for the next levels
                if ("headerrow_" + (currentlevel) in this.mapColumnHeaders) {
                    // nneed to push from the currentlevel till the end level
                    for (idx = currentlevel; idx < maxLevel; idx++) {
                        emptynode=this._createEmptyNodesFromGivenNode(child);
                        this.mapColumnHeaders["headerrow_" + idx].push(emptynode);
                    }
                } else {
                    // create a new level and push the child there too for each of the levels from the (current level+1)
                    for (idx = (currentlevel + 1); idx <= maxLevel; idx++) {
                        nodes = [];
                        emptynode=this._createEmptyNodesFromGivenNode(child);
                        nodes.push(emptynode);
                        this.mapColumnHeaders["headerrow_" + (idx - 1)] = nodes;
                    }
                }
            }
            this.mapColumnHeaders["headerrow_" + (currentlevel - 1)] = allnodes;
        }
    },
    _generateRowHierarchyTree:function(rowHierarchy, maxRowOrder) {
        var that=this;
        if (rowHierarchy) {
            this._rowHiearchyRoot = sas.stress.wkbench.model.TreeNode.load(rowHierarchy,true,maxRowOrder*maxRowOrder);
            var headersSize =this._rowHiearchyRoot.getDepth()-1;
            this._generateRowHierarchyTreeRecurse(this._rowHiearchyRoot,headersSize);

            // set up the previous and next nodes after the map of rowheaders has been fully populated..
            _.each(this.mapRowHeaders,function(nodes,key){
                that._setupPrevAndNextNodes(nodes);
            });
        }
    },
    _generateRowHierarchyTreeRecurse:function(node,maxLevel){
        var nodeIdx,idx;
        var allnodes = [], nodes = [],emptynode;
        var mapHeaderRows={};
        var childrenLength=node.getChildren().length;
        for (nodeIdx=0;nodeIdx<childrenLength;++nodeIdx){
            var child=node.getChildren()[nodeIdx];
            var currentlevel=child.getLevel();
            // get the existing set of nodes at this level
            if ("headerrow_" + (currentlevel-1) in this.mapRowHeaders){
                allnodes=this.mapRowHeaders["headerrow_" + (currentlevel-1)];
            }
            // add the child to the existing set of nodes
            allnodes.push(child);
            // if this is a sub-total node and the current level is less than maxLevel
            if (!child.likey&&currentlevel<maxLevel){
                // this is a sub-total node
                if ("headerrow_" + (currentlevel) in this.mapRowHeaders){
                    // need to push from the currentlevel till the end level
                    for (idx=currentlevel;idx<maxLevel;idx++){
                        this.mapRowHeaders["headerrow_" + idx].push(child);
                    }
                } else {
                    // create a new level and push the child there too for each of the levels from the (current level+1)
                    for (idx=(currentlevel+1);idx<=maxLevel;idx++){
                        nodes=[];
                        nodes.push(child);
                        this.mapRowHeaders["headerrow_" + (idx-1)]=nodes;
                    }
                }
            }
            if (child.getChildren().length>0){
                this._generateRowHierarchyTreeRecurse(child,maxLevel);
            }else if (child.likey&&currentlevel<maxLevel) {
                // found a node that does not have children... ragged therefore, it needs to have
                //  empty nodes for the next levels which look like the child except no name and datakey
                 if ("headerrow_" + (currentlevel) in this.mapRowHeaders){
                     // need to push from the current-level till the end level
                     for (idx=currentlevel;idx<maxLevel;idx++){
                         emptynode=this._createEmptyNodesFromGivenNode(child);
                         this.mapRowHeaders["headerrow_" + idx].push(emptynode);
                     }
                 } else {
                     // create a new level and push the child there too for each of the levels from the (current level+1)
                     for (idx=(currentlevel+1);idx<=maxLevel;idx++){
                         nodes = [];
                         emptynode=this._createEmptyNodesFromGivenNode(child);
                         nodes.push(emptynode);
                         this.mapRowHeaders["headerrow_" + (idx-1)]=nodes;
                     }
                 }
            }
            this.mapRowHeaders["headerrow_" + (currentlevel-1)]=allnodes;
        }
    },
    _createEmptyNodesFromGivenNode:function(node){
        var emptynode = new sas.stress.wkbench.model.TreeNode("", "", node.order, node.cellValueIndex,
            node.dataKey, node.subtotallikey, node.readOnly, false, node);  // does not have children

        // the formats
        emptynode.setFormat(node.getFormat());
        emptynode.setFormatPrecision(node.getFormatPrecision());
        emptynode.setFormatScale(node.getFormatScale());
        emptynode.setFormatType(node.getFormatType());
        emptynode.setFormulaCode(node.getFormulaCode());
        emptynode.setLocked(node.getLocked());
        if (node.getDataKey()) {
            emptynode.likey = node.likey;
        }
        return emptynode;
    },
    _setupPrevAndNextNodes:function(allnodes){
        var idx=0,previdx= 0, nextidx=0;
        if (allnodes) {
            while (idx < allnodes.length) {
                // current
                var node = allnodes[idx];
                // the next
                node.setNext(allnodes[idx + 1]);
                if (previdx < idx) {
                    // the previous
                    node.setPrevious(allnodes[previdx]);
                }
                // increment
                previdx = idx++;
            }
        }
    },
    _generateColumnLongform:function(header,index){
        if (header){
            var node=header.labels[index];
            return node.getName();
        }
        return "";
    },
    getColumnHierarchyRoot:function(){
        return this._columnHiearchyRoot;
    },
    getRowHierarchyRoot:function(){
        return this._rowHiearchyRoot;
    },
    destroy:function(){
        var that=this;
        var keys=[];

        this.rows=[];
        this.rowsLocked=[];
        this.rowsRoots=[];
        this.columnHeadersCount=0;
        this.rowHierarchiesCount=0;
        this.json=undefined;
        this.columns=[];

        keys= _.keys(this.columnShort);
        _.each(keys,function(key){
            that.columnShort[key]=undefined;
            delete that.columnShort[key];
        });
        this.columnShort={};
        keys=[];
        keys= _.keys(this.columnLong);
        _.each(keys,function(key){
            that.columnLong[key]=undefined;
            delete that.columnLong[key];
        });
        this.columnLong={};
        keys=[];

        keys= _.keys(this.rowShort);
        _.each(keys,function(key){
            that.rowShort[key]=undefined;
            delete that.rowShort[key];
        });
        this.rowShort={};
        keys=[];
        keys= _.keys(this.rowLong);
        _.each(keys,function(key){
            that.rowLong[key]=undefined;
            delete that.rowLong[key];
        });
        this.rowLong={};
        keys=[];

        keys= _.keys(this.mapRowHeaders);
        _.each(keys,function(key){
            var nodes=that.mapRowHeaders[key];
            for (var i=nodes.length-1;i>=0;--i){
                var node=nodes[i];
                node.destroy();
                node =undefined;
                delete node;
            }
            nodes.splice(0);
            delete nodes;
            that.mapRowHeaders[key]=undefined;
            delete that.mapRowHeaders[key];
        });
        this.mapRowHeaders={};
        keys=[];
        keys= _.keys(this.mapColumnHeaders);
        _.each(keys,function(key){
            var nodes=that.mapColumnHeaders[key];
            for (var i=nodes.length-1;i>=0;--i){
                var node=nodes[i];
                node.destroy();
                node =undefined;
                delete node;
            }
            nodes.splice(0);
            delete nodes;
            that.mapColumnHeaders[key]=undefined;
            delete that.mapColumnHeaders[key];
        });
        this.mapColumnHeaders={};
        keys=[];

        this.overlays=[];
        keys= _.keys(this.rowData);
        _.each(keys,function(key){
            that.rowData[key]=undefined;
            delete that.rowData[key];
        });
        this.rowData={};
        keys=[];
        keys= _.keys(this.datakeyToRowCol);
        _.each(keys,function(key){
            that.datakeyToRowCol[key]=undefined;
            delete that.datakeyToRowCol[key];
        });
        this.datakeyToRowCol={};
        keys=[];
        keys= _.keys(this.rowColumnToDatakey);
        _.each(keys,function(key){
            that.rowColumnToDatakey[key]=undefined;
            delete that.rowColumnToDatakey[key];
        });
        keys=[];
        keys= _.keys(this.columnRowToDataKey);
        _.each(keys,function(key){
            that.columnRowToDataKey[key]=undefined;
            delete that.columnRowToDataKey[key];
        });
        this.columnRowToDataKey={};
        keys=[];

        this.rowsVisible=[];
        if (this._columnHiearchyRoot){
            this._columnHiearchyRoot.destroy();
        }
        this._columnHiearchyRoot=undefined;
        delete this._columnHiearchyRoot;

        if (this._rowHiearchyRoot){
            this._rowHiearchyRoot.destroy();
        }
        this._rowHiearchyRoot=undefined;
        delete this._rowHiearchyRoot;

        if (sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER){
            sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER=undefined;
            delete sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER;
        }
    }
    /* Not USed any more code*/
    ,
    _loadRow:function(node){
        var info=node;
        if (info.name&&info.name.length>0) {
            var index = this.rows.length;
            var short = (index + 1).toString();
            var long = info.cellValueIndex>-1 ? info.likey : "";
            var level = 0;
            var formatter = info ? new sas.stress.wkbench.model.Formatter(info.formatType, info.formatPrecision, info.formatScale) :
                sas.stress.wkbench.model.SheetInfoData.DEFAULT_FORMATTER;
            row = {
                short: short,
                long: long,
                index: index,
                level: level,
                name: info ? info.name : "",
                locked: info ? info.locked : true,
                formatter: formatter,
                node: info
            };
            if (long) {
                this.rowShort[short] = row;
                this.rowLong[long] = row;
            }
            if (info.locked) {
                this.rowsLocked.push(row);
            }
            if (!row.level) {
                this.rowsRoots.push(row);
            }
            this.rows.push(row);

            if (node.hasChildren) {
                var nodes=node.getChildren();
                for (var i=0;i<nodes.length;++i) {
                    var child=nodes[i];
                    this._loadRow(child);
                }
            }
        }
        return index+1;
    }
    /*_generateColumnHierarchyTree:function(columnHierarchy) {
     if (columnHierarchy) {
     this._columnHiearchyRoot = sas.stress.wkbench.model.TreeNode.load(columnHierarchy,false);
     // we will need to form a map of Key = headerrow_levelid, value= Node
     var mapHeaderColumns={};
     var headerrow=[];
     var headersSize =this._columnHiearchyRoot.getChildren().length;
     for (var idx=0; idx<(headersSize); idx++){
     headerrow[idx]="headerrow_"+idx;

     var nodes=[];
     if (idx===0){
     for (var i=0; i<headersSize; i++) {
     var node = this._columnHiearchyRoot.children[i];
     nodes.push(node);
     }
     mapHeaderColumns["headerrow_"+idx]=nodes;
     } else {
     for (var i=0; i<headersSize; i++) {
     var node = this._columnHiearchyRoot.children[i];
     var nodes = node.getNodesByLevel(idx);
     if (!("headerrow_"+idx in mapHeaderColumns)&&nodes.length>0) {
     mapHeaderColumns["headerrow_"+idx]=[]; // initialize
     }
     var exist=mapHeaderColumns["headerrow_"+idx];
     _.each(nodes, function(node){
     exist.push(node);
     });
     }
     }

     //this._setupPrevAndNextNodes( mapHeaderColumns["headerrow_"+idx]);
     }
     if (_.size(mapHeaderColumns)===1){
     var newmap={};
     var columnHeaderCount = this._columnHiearchyRoot.getDepth();
     for (var idx=1;idx<columnHeaderCount;idx++){
     var nodes = this._columnHiearchyRoot.getNodesByLevel(idx);
     if (nodes.length>0){
     mapHeaderColumns["headerrow_"+idx]=[]; // initialize
     var exist=mapHeaderColumns["headerrow_"+idx];
     _.each(nodes, function(node){
     exist.push(node);
     });
     }
     }
     }
     // save this for further usage
     this.mapColumnHeaders=mapHeaderColumns;
     _.each(mapHeaderColumns,function(header,key){
     var prevnode=undefined,nextnode=undefined, idx= 0,previdx=0;
     while (idx<header.length){
     var node=header[idx];

     // the next
     node.setNext(header[previdx+1]);
     if (previdx<idx){
     // the previous
     node.setPrevious(header[previdx]);
     }
     // increment
     previdx=idx++;
     }
     });
     }
     },*/
};