/**
 * Created by sinsrj on 10/29/2015.
 */

jQuery.sap.declare("sas.stress.wkbench.model.TreeNode");

sas.stress.wkbench.model.TreeNode=function(name,likey,order,cellValueIndex,dataKey,subtotallikey,readOnly,hasChildren,parent){
    this.children=[];
    this.name=name;
    this.likey=likey;
    this.order=order;
    this.parent=parent;
    this.hasChildren=hasChildren;
    this.next=undefined;
    this.prev=undefined;
    this.emptyNodeAdded=false;
    this.uniqueID=undefined;
    this.cellValueIndex=cellValueIndex;
    this.dataKey=dataKey;
    this.readOnly=readOnly;
    this.subtotallikey=subtotallikey;

    // the formatting items
    this.aggregation=0;
    this.contra=0;
    this.format="1.2";
    this.formatPrecision=2;
    this.formatScale=1;
    this.formatType="Numeric";
    this.formulaCode=0;
    this.locked=false;

    // visibility
    this.visible=true;
};


sas.stress.wkbench.model.TreeNode.prototype={
    getName:function(){
        return this.name;
    },
    getLiKey:function(){
        return this.likey;
    },
    getHasChildren:function(){
        return !!this.hasChildren;
    },
    getChildren:function(){
        return this.children;
    },
    getLevel:function(){
        if(this.likey===undefined){
            return 0;
        }//root
        return this.parent.getLevel()+1;
        /*return this.level;*/
    },
    getOrder:function(){
        return this.order;
    },
    getParent:function(){
        return this.parent;
    },
    addChild:function(treenode){
        this.hasChildren=true;
        this.children.push(treenode);
        var parent=this;
        //if (!this.dataKey) {
            var uniqueID;
            if (treenode.likey.length) {
                uniqueID = treenode.likey;
                while (parent) {
                    if (parent && parent.likey) {
                        uniqueID += " ";
                        uniqueID += parent.likey;
                    }
                    parent = parent.getParent();
                }
                // the uniqueID
                treenode.setUniqueID(uniqueID);
            }
        //}
    },
    findNodeByDataKey:function(dataKey){
        var i,node;
        for(i=0;i<this.children.length;i++){
            node=this.children[i];
            if(node.dataKey===dataKey){
                return node;
            }
            var result=node.findNode(dataKey);
            if(result){
                return result;
            }
        }
        return null;
    },
    findNode:function(likey){
        var i,node;
        for(i=0;i<this.children.length;i++){
            node=this.children[i];
            if(node.likey===likey){
                return node;
            }
            var result=node.findNode(likey);
            if(result){
                return result;
            }
        }
        return null;
    },
    getNodesByLevel:function(levelno){
        var node, nodes=[];
        for(var i=0;i<this.children.length;i++){
            node=this.children[i];
            var level=node.getLevel();
            if(level===(levelno+1)){
                nodes.push(node);
            } else {
                var result = node.getNodesByLevel(levelno);
                nodes = _.union(nodes, result);
            }
        }

        return nodes;
    },
    getDepth:function(){
        var depth=0;
        if (this.children.length>0) {
            _.each(this.children,function(child){
                var tmpdepth=child.getDepth();
                if (tmpdepth>depth){
                    depth=tmpdepth;
                }
            });
        }
        return 1+depth;
    },
    setNext:function(node){
        this.next=node;
    },
    getNext:function(){
        return this.next;
    },
    setPrevious:function(node){
        this.prev=node;
    },
    getPrevious:function(){
        return this.prev;
    },
    getIndex:function(likey){
        var index,node;
        for(index=0;index<this.children.length;index++){
            node=this.children[index];
            if(node.likey===likey){
                return index;
            }
            var result=node.findNode(likey);
            if(result){
                return result;
            }
        }
        return index;
    },
    getCellValueIndexByDatakey:function(dataKey){
        var node=this.findNodeByDataKey(dataKey);
        if (node) {
            return node.getCellValueIndex();
        }else{
            return -1;
        }
    },
    getUniqueID:function(){
        return this.uniqueID;
    },
    setUniqueID:function(uniqueID){
        this.uniqueID=uniqueID;
    },
    getCellValueIndex:function(){
        return this.cellValueIndex;
    },
    getDataKey:function(){
        return this.dataKey;
    },
    getReadOnly:function(){
        return this.readOnly;
    },
    getSubtotalLikey:function(){
        return this.subtotallikey;
    },
    setAggregation:function(aggregation){
        this.aggregation=aggregation;
    },
    getAggregation:function(){
        return this.aggregation;
    },
    setContra:function(contra){
        this.contra=contra;
    },
    getContra:function(){
        return this.contra;
    },
    setFormat:function(format){
        this.format=format;
    },
    getFormat:function(){
        return this.format;
    },
    setFormatPrecision:function(formatPrecision){
        this.formatPrecision=formatPrecision;
    },
    getFormatPrecision:function(){
        return this.formatPrecision;
    },
    setFormatScale:function(formatScale){
        this.formatScale=formatScale;
    },
    getFormatScale:function(){
        return this.formatScale;
    },
    setFormatType:function(formatType){
        this.formatType=formatType;
    },
    getFormatType:function(){
        return this.formatType;
    },
    setFormulaCode:function(formulaCode){
        this.formulaCode=formulaCode;
    },
    getFormulaCode:function(){
        return this.formulaCode;
    },
    setLocked:function(locked){
        this.locked=locked;
    },
    getLocked:function(){
        return this.locked;
    },
    getVisible:function(){
        return this.visible;
    },
    setVisible:function(isVisible){
        this.visible=isVisible;
    },
    isAnyChildMarkedContra:function(){
        var index,node;
        for(index=0;index<this.children.length;index++){
            node=this.children[index];
            if(node.contra){
                return true;
            }
            var result=node.isAnyChildMarkedContra();
            if(result){
                return result;
            }
        }
        return false;
    },

    _processChildren:function(hierarchies,addsubtotalnode,maxLevels){
        var i,hierarchy;
        for(i=0;i<hierarchies.length;i++){
            hierarchy=hierarchies[i];
            var node=new sas.stress.wkbench.model.TreeNode(hierarchy.name,hierarchy.liKey,hierarchy.order,hierarchy.cellValueIndex,hierarchy.dataKey,hierarchy.subtotalLiKey,hierarchy.readOnly,hierarchy.hasChildren,this);
            // the formats
            node.setAggregation(hierarchy.aggregation);
            node.setContra(hierarchy.contra);
            node.setFormat(hierarchy.format);
            node.setFormatPrecision(hierarchy.formatPrecision);
            node.setFormatScale(hierarchy.formatScale);
            node.setFormatType(hierarchy.formatType);
            node.setFormulaCode(hierarchy.formulaCode);
            node.setLocked(hierarchy.locked);

            if(hierarchy.children){
                node._processChildren(hierarchy.children,addsubtotalnode,maxLevels);
            }
            var found=_.find(this.children,function(child){
                return child.likey===node.likey;
            });
            if (!found) {
                this.addChild(node);
            }
        }
    },

    destroy:function(){
        var i, node;
        if (this.children&&this.children.length>0) {
            for (i = this.children.length - 1; i >= 0; --i) {
                node = this.children[i];
                node.destroy();
            }
        }
        this.parent=undefined;
        delete this.parent;
        if (this.children){
            this.children.splice(0);
        }
        delete this.children;

        this.next=undefined;
        delete this.next;
        this.prev=undefined;
        delete this.prev;
    }
};
//STATICS
sas.stress.wkbench.model.TreeNode.root=null;
sas.stress.wkbench.model.TreeNode.load=function(items,addsubtotalnode,maxLevels){
    sas.stress.wkbench.model.TreeNode.root=new sas.stress.wkbench.model.TreeNode();
    sas.stress.wkbench.model.TreeNode.root._processChildren(items,addsubtotalnode,maxLevels);

    return sas.stress.wkbench.model.TreeNode.root;
};

sas.stress.wkbench.model.TreeNode.getSpan=function(treenode){
    if (!treenode.getHasChildren()){
        return 1;
    }
    return _.reduce(treenode.getChildren(), function(c,node){
        return c + sas.stress.wkbench.model.TreeNode.getSpan(node);
    }, 0);
};

sas.stress.wkbench.model.TreeNode.getActualNodeBySubTotalKey=function(subtotalLiKey){
    return sas.stress.wkbench.model.TreeNode.root.findNodeByDataKey(subtotalLiKey);
};
