jQuery.sap.require("sas.stress.wkbench.CellPickerUtil");
jQuery.sap.require("sas.stress.wkbench.CellEvaluationUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.controller("sas.stress.wkbench.view.CellEditor",{
	gainedFocus:function(){
		var liveModel=this.getView().getModel("live");
		var selected=liveModel.getProperty("/selected");
		liveModel.setProperty("/editing",selected);
		this.checkEditMode();
	},
	formulaLiveChange:function(){
		var value=this.getView().formulaField.getLiveValue();
		// fixes for S1203451.. update the field with the uppercase always and use the same value
		//value=value.toLocaleUpperCase();
		this.getView().formulaField.setValue(value);
		var liveModel=this.getView().getModel("live");
		var slimsheet=liveModel.getProperty("/slimsheet");
		var inplaceEdit=document.getElementById(slimsheet.getId()+"-inplace-edit");
		if(inplaceEdit){
			inplaceEdit.value=value;
		}
		if(this.checkEditMode()){
			this.highlightFormulaRefs(value);
		}
	},
	checkEditMode:function(){
		switch(sas.stress.wkbench.CellPickerUtil.getOperationMode()){
			case sas.stress.wkbench.CellPickerUtil.EDIT_MODE:
				return true;
			default:
				if(this.getView().formulaField.getLiveValue().indexOf("=")===0){
					this.enterEditMode();
					return true;
				}
				break;
		}
		return false;
	},
	enterEditMode:function(){
		sas.stress.wkbench.CellPickerUtil.activateEditMode();
		//this.highlightFormulaRefs(this.getView().formulaField.getValue());
	},
	leaveEditMode:function(){
		var view=this.getView();
		var liveModel=view.getModel("live");
		var selected=liveModel.getProperty("/editing");
		if(!selected){
			return;
		}
		sas.stress.wkbench.CellPickerUtil.deactivateEditMode();
		this.clearFormulaRefs();
		liveModel.setProperty("/editing",null);
		var slimsheet=liveModel.getProperty("/slimsheet");
		slimsheet.focus();
		slimsheet.fireCellSelected({selected:true,row:selected.row,col:selected.column/*+1*/,filter:selected.getSheetFilters()/*scenario:selected.getScenario()*/});
	},
	saveCellEdits:function(e){
		var errored=false;
		var view=this.getView();
		var formula=view.formulaField.getLiveValue();
		if(formula===null || formula.trim()===""){
			return;
		}
		var liveModel=view.getModel("live");
		var selected=liveModel.getProperty("/selected");

		var key="$"+selected.colLong+"$"+selected.rowLong;
		var data=selected.getFilterData();
		var overlay=data.ncFormulas[key];
		var rb = sas.stress.wkbench.StressUtil.getBundle();
		var userName = sas.stress.wkbench.StressUtil.getUserName();
		var previousValue="";
		if (overlay) {
			var showEquals = (overlay.expr.type !== "NAME" && overlay.expr.type !== "PREFIX") || (overlay.expr.name && overlay.expr.name.indexOf("$") !== -1);
			try {
				var row = selected.row;
				var col = selected.column;
				previousValue = (showEquals ? "=" : "") + sas.stress.wkbench.parser.ExpressionUtil.print(overlay.expr, $.proxy(data.printShortContext, data));
				if (showEquals && !_.isNull(data.data[row][col]) &&
					(!_.isNumber(data.data[row][col]) && !_.isBoolean(data.data[row][col]) && !_.isString(data.data[row][col]))) {	//fixes for S1301860.. ignore numeric, text and boolean values
					previousValue = data.data[row][col] + " " + previousValue;
				}
			} catch (e) {
				console.error("Unable to evaluate expression " + sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr), overlay);
				previousValue = "#NAME? " + overlay.txt;
			}
		}

		var slimsheet = liveModel.getProperty("/slimsheet");
		try {
			sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(formula, selected, true);
			var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
			wkbenchView.getController().navChecker.setDirty(true);
			wkbenchView.getModel("live").setProperty("/toggleButtons", true);

			if (selected.getSheetFilters().getSheetInfo().getSheet().getProject().doesRequireComments()) {	// comments required if set
				selected.getSheetFilters().getData().addComment(key, selected.row, selected.column, userName,
					rb.getText("cell.override.default.comment.txt")+previousValue);
			}
		}catch(e) {
			errored=true;
		}finally{
			slimsheet.rerender();
			this.leaveEditMode();
			if(!errored&&selected.getSheetFilters().getSheetInfo().getSheet().getProject().doesRequireComments()){	// comments required if set
				wkbenchView.commentButton.firePress({});
			}
		}
	},
	enableAllocationActions:function(cellRef){
		var canBeEnabled=false;
		var actions=["even","equally","proportional"];
		var rb=sas.stress.wkbench.StressUtil.getBundle();
		if (cellRef) {
			var isEditable = !!cellRef && cellRef.getInfoData().isEditable(cellRef.row, cellRef.column + 1);
			// fixes for allocations on columns.. need to look at row and column.. if row is false then look at column
			var isSubtotalNode = !!cellRef && (cellRef.getRowNode().subtotallikey !== undefined||cellRef.getColumnNode().subtotallikey !== undefined);

			canBeEnabled=isEditable&&isSubtotalNode;
		}else{
			return canBeEnabled;
		}

		if (canBeEnabled) {
			var CellEvaluationUtil = sas.stress.wkbench.CellEvaluationUtil;
			var key = "$" + cellRef.colLong + "$" + cellRef.rowLong;
			var dataKey;
			if (cellRef.getSheetFilters().getData().ncFormulas.hasOwnProperty(key)) {
				dataKey = cellRef.getSheetFilters().getData().ncFormulas[key].dataKey;
			} else {
				dataKey = cellRef.getSheetFilters().getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var oldExpr = CellEvaluationUtil.getLatestExpr(cellRef);
			if (oldExpr) {	// make sure there is something... if null then we cannot allocate
				if (oldExpr.args) {
				} else {
					if (isFinite(oldExpr.name)) {
						return false;
					} else {
						try {
							sas.stress.wkbench.model.CellRef.getCellRefs(cellRef.getSheetFilters().getData().printShortContext(oldExpr.name), cellRef.getSheetFilters());
						} catch (e) {
							// the value cannot be parsed to a cell reference, we cannot proceed ahead... could be plain TEXT
							return false;
						}
					}
				}
			}else{
				return false;		// we should not enabling if the value is null.. we do not allocate to children that are null
			}
			var view = this.getView();
			view.actions.removeAllItems();
			_.each(actions, function (action) {
				var enabled=true;
				if (action==="equally"){	// if equally
					// check for contra children
					enabled=cellRef.getRowNode().getParent().isAnyChildMarkedContra()?false:true;	// and yes, enabled = false
				}
				var item = new sas.hc.ui.unified.MenuItem({
					text: rb.getText("cell.editor.allocation.rule." + action + ".menu.item.txt"),
					enabled:enabled
				});
				item.data("key", action);
				item.data("index", action);
				view.actions.addItem(item);
			});
		}

		return canBeEnabled;
	},
	performAllocations:function(e){
		var item = e.getParameter("item");
		var action = item.data("index");
		var view=this.getView();
		var formula=view.formulaField.getLiveValue();
		if(formula===null || formula.trim()===""){
			return;
		}
		var liveModel=view.getModel("live");
		var selected=liveModel.getProperty("/selected");
		var slimsheet = liveModel.getProperty("/slimsheet");
		try {
			sas.stress.wkbench.CellEvaluationUtil.performAllocations(formula, selected, action);
			selected.destroy();
			var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
			wkbenchView.getController().navChecker.setDirty(true);
			wkbenchView.getModel("live").setProperty("/toggleButtons", true);
		}catch(e) {
			if (e.stack){
				console.error("PERFORM ALLOCATIONS FAILURE: " + e.stack);
			}
			// need to show a message box about the invalidity of the input
			var message,rb=sas.stress.wkbench.StressUtil.getBundle();
			if (e===sas.stress.wkbench.CellEvaluationUtil.INVALID_ALLOCATION_VALUE) {
				message =rb .getText("cell.editor.allocation.invalid.value.fmt", formula);
			}else{
				message =rb.getText("cell.editor.allocation.failure.fmt", e);
			}
			sas.hc.m.MessageBox.error(message, rb.getText('error.window.title'));
		}finally{
			this.leaveEditMode();
			slimsheet.rerender();
		}

	},
	clearFormulaRefs:function(){
		var liveModel=this.getView().getModel("live");
		liveModel.setProperty("/informula",[]);
	},
	highlightFormulaRefs:function(formula){
		var liveModel=this.getView().getModel("live");
		var selected=liveModel.getProperty("/selected");
		var filters/*scenario*/=selected.getSheetFilters();
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		try{
			var expression=CellEvaluationUtil.parseFormula(formula);
			var refs=CellEvaluationUtil.getCellRefs(expression,filters/*scenario*/);
			console.info("Highlighting "+refs.length+" cells",new Date());
			liveModel.setProperty("/informula",refs);
		}catch(e){
			console.error(e);
		}
	}
});