jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.hc.m.Input");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.risk.common.widgets.HelpLabel");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");

sap.ui.jsfragment("sas.stress.wkbench.view.EditProjectDialog", {

    createContent: function (controller) {
        var NO_VERSION = "__NO__VERSION__";
        var rb=sas.stress.wkbench.StressUtil.getBundle();

        var nameInput = new sas.hc.m.Input({
            id: this.createId("EditProjectDialog.nameInput"),
            value:"{/name}",
            type:sap.m.InputType.Text,
            disableValidation:false,
            maxLength:64,
            liveChange: $.proxy(controller.editProjectDialogNameInputLiveChange,controller)
        });
        var nameInputLabel = new sas.hc.m.Label({
            text: rb.getText("editProjectDialog.name.control.label"),
            required:true,
            labelFor: nameInput
        });

        var descriptionInput = new sas.hc.m.Input({
            value:"{/description}",
            liveChange: $.proxy(controller.toggleEditProjectDialogSaveButton, controller),
            maxLength:256
        });

        var processNotification = new sas.hc.m.CheckBox({
            id: this.createId("EditProjectDialog.processNotification"),
            text: rb.getText("editProjectDialog.processNotification.control.label"),
            selected: "{/processNotificationType}",
            select: $.proxy(controller.toggleEditProjectDialogSaveButton, controller),
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return !isLocked;
                }
            }
        });

        var processStartDatePicker = new sas.hc.m.DatePicker({
            id: this.createId("EditProjectDialog.processStartDatePicker"),
            value: {
                path: "/processStartDate",
                type: new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}})
            },
            placeholder: "  ",
            change: $.proxy(controller.editProjectDialogProcessStartDateChange,controller),
            //liveChange: $.proxy(controller.editProjectDialogProcessStartDateLiveChange,controller),       //sas.hc.m.DatePicker does not support liveChange
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return false;
                    }
                    return !isLocked;
                }
            }
        });
        var processStartDatePickerLabel = new sas.risk.common.widgets.HelpLabel({
            id: this.createId("EditProjectDialog.processStartDatePickerLabel"),
            text: rb.getText("editProjectDialog.processStartDate.control.label"),
            labelFor: processStartDatePicker,
            library: "sas.stress",
            key: "newProjectDialog.processStartDate"
        });
        processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", true);

        var processDefnItemTemplate = new sas.hc.ui.core.Item({key: "{id}", text:"{name}"});
        var processDefnNameSelect = new sas.hc.m.Select({
            id: this.createId("EditProjectDialog.processDefnNameSelect"),
            items: {
                path: "/definitions",
                template: processDefnItemTemplate
            },
            change: function() {
                var processDefn = parseInt(this.getSelectedItem().getKey());
                processNotification.setEnabled(processDefn > 0);
                processNotification.setSelected(false);
                processStartDatePicker.setEnabled(processDefn > 0);
                processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", !processStartDatePicker.getEnabled());
                controller.toggleEditProjectDialogSaveButton();
            },
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;
                }
            }
        });
        var processDefnLabel = new sas.hc.m.Label({
            text: rb.getText("editProjectDialog.processDefinitionName.control.label"),
            labelFor: processDefnNameSelect
        });

        var asOfDatePicker = new sas.hc.m.DatePicker({
            id: this.createId("EditProjectDialog.asOfDatePicker"),
            value: {
                path: "/asOfDate",
                type: new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}})
            },
            placeholder: "  ",
            change: $.proxy(controller.editProjectDialogAsOfDateChange,controller),
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;
                }
            }
            //liveChange: $.proxy(controller.editProjectDialogAsOfDateLiveChange,controller)      //sas.hc.m.DatePicker does not support liveChange
        });
        var asOfDateLabel = new sas.risk.common.widgets.HelpLabel({
            text: rb.getText("editProjectDialog.asOfDate.control.label"),
            required:true,
            labelFor:asOfDatePicker,
            library: "sas.stress",
            key: "newProjectDialog.asOfDate"
        });

        this.workSheetList = new sas.risk.common.widgets.SelectTable({
            id: this.createId("EditProjectDialog.workSheetList"),
            nameAttribute: "nameVersion",
            listItems: "{/worksheets}",
            listName:  rb.getText("project.worksheets.name.txt"),
            visibleRowCount: 6,
            //columnHeaderVisible: true,
            selectionChange: $.proxy(controller.toggleEditProjectDialogSaveButton, controller)
        });
        var workSheetListLabel = new sas.hc.m.Label({
            text: rb.getText("project.worksheets.label"),
            labelFor: this.workSheetList
            //required: true           // worksheet is not required per Jeff
        });

        var worksheetVersion = new sas.hc.m.Text({wrapping: false}).bindProperty("text", {
            path: "version",
            formatter: function (version) {
                return (version === NO_VERSION) ? "" : version ;
            }
        });

        this.workSheetList.addColumn(new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({ text: rb.getText("project.worksheets.worksheetVersion.txt") }),
            template: worksheetVersion,
            sortProperty: "version"
        }));

        this.workSheetList.bindRows("/worksheets");

        var requireComments=new sas.hc.m.CheckBox({
            id: this.createId("EditProjectDialog.requireComments"),
            text: rb.getText("editProjectDialog.require.comments.control.label"),
            selected: "{/requireComments}",
            select: $.proxy(controller.toggleEditProjectDialogSaveButton, controller),
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;    // this is the opposite
                }
            }
        }).addStyleClass("stressEditProjectRequireCommentsCheckBox");

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 3,
                labelSpanL: 3
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: nameInputLabel,
                            fields: [nameInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: rb.getText("editProjectDialog.description.control.label"),
                            fields: [descriptionInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: processDefnLabel,
                            fields: [processDefnNameSelect, processNotification]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: processStartDatePickerLabel,
                            fields: [processStartDatePicker]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: asOfDateLabel,
                            fields: [asOfDatePicker]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: workSheetListLabel,
                            fields: [this.workSheetList]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields:[requireComments]
                        })
                    ]
                })
            ]
        });
        form.addStyleClass("stressLargeForm");

        var dialog = new sas.hc.m.Dialog({
            id: "sas.stress.wkbench.view.EditProjectDialog",
            title: "{/title}", //rb.getText("editProjectDialog.window.title"),
            initialFocus: nameInput,
            content: form,
            beginButton: new sas.hc.m.Button({
                id: this.createId("EditProjectDialog.saveProjectButton"),
                text: rb.getText("button.save.action.label"),
                enabled: false,
                press: $.proxy(controller.handleProjectDialogSaveButtonPress, controller)
            }),
            endButton: new sas.hc.m.Button({
                text: rb.getText("button.cancel.action.label"),
                press: function () {
                    dialog.close();
                }
            }),
            afterClose: function () {
                dialog.destroy();
            }
        });

        return dialog;
    }
});
