jQuery.sap.require({modName: "sas.stress.wkbench.view.HomePage", type: "controller"});
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToggleButton");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.stress.wkbench.widget.LifecycleTable");
jQuery.sap.require("sas.stress.wkbench.widget.CommentEntry");
jQuery.sap.require("sas.stress.wkbench.widget.CommentManager");
jQuery.sap.require("sas.stress.wkbench.widget.StressGrooveBar");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");

sap.ui.jsview("sas.stress.wkbench.view.HomePage", {

    rb: sap.ui.getCore().getLibraryResourceBundle("sas.stress.wkbench"),

    getControllerName: function () {
        return "sas.stress.wkbench.view.HomePage";
    },

    createContent: function (controller) {

        this.setHeight("100%");

        this.viewLayout = new sas.risk.common.widgets.RiskPage({
            iconBar: new sas.stress.wkbench.widget.StressGrooveBar({
                section: sas.stress.wkbench.widget.StressGrooveBar.PROJECTS
            }),
            mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                type: sas.risk.common.widgets.WidgetFactory.OBJECT,
                top: new sas.risk.common.widgets.RiskToolbar({
                    homeFunction: function () {
                        sas.stress.wkbench.ViewManager.loadProjectManager();
                    },
                    title: new sas.hc.m.Label({text: "{/project/name}"}),
                    headerContent: [
                        new sas.hc.m.Label({text: this.rb.getText("iteration.label.txt")}).addStyleClass("toolbarLabel"),
                        new sas.hc.m.Label({text: "{/project/currentIterationNo}"}).addStyleClass("toolbarValue"),
                        new sas.hc.m.Label({text: this.rb.getText("workbench.toolbar.asOfDate.label")}).addStyleClass("toolbarLabel"),
                        new sas.hc.m.Label({
                            text: {
                                path: "/project/asOfDate",
                                formatter: function (asOfDate) {
                                    var dateFormat = new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}}, {target: {pattern: "MMM dd, yyyy"}}, asOfDate);
                                    return dateFormat.formatValue(asOfDate, "string");
                                }
                            }
                        }).addStyleClass("toolbarValue")
                    ]
                }),
                content: new sas.risk.common.widgets.RiskPage({
                    mainContent: this._createMainContent(controller),
                    iconBar: new sas.stress.wkbench.widget.StressIconBar({
                        section: sas.stress.wkbench.widget.StressIconBar.HOME
                    })
                })
            })
        });

        return this.viewLayout;

    },

    _createMainContent: function (controller) {

        var taskPanel = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%"
        });
        var contentInnerContainerArea = new sas.hc.ui.commons.layout.BorderLayoutArea({content: this._createTaskTable(controller)});
        taskPanel.setCenter(contentInnerContainerArea);

        taskPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
            size: "60%"
        }));

        var splitterBetweenTasksAndDiagram = new sas.hc.ui.layout.Splitter({
            contentAreas: [
                taskPanel,
                this._createWorkflowPanel(controller)
            ],
            orientation: sap.ui.core.Orientation.Vertical,
            width: "100%",
            height: "100%"
        });

        var taskPane = sas.risk.common.widgets.WidgetFactory.buildPanel({
            content: splitterBetweenTasksAndDiagram,
            rightPadding: 0
        });

        var commentPane = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({
            content: this._createTaskDetailsPanel(controller).addStyleClass("riskFacetPanel")
        });

        commentPane.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
            size: "225px",
            minSize: 150
        }));

        var splitterBetweenTaskPaneAndCommentPane = new sas.hc.ui.layout.Splitter({
            contentAreas: [
                taskPane,
                commentPane
            ],
            orientation: sap.ui.core.Orientation.Horizontal,
            width: "100%",
            height: "100%"
        });

        // Attach the resize handler function to the two splitters so the diagram gets updated when the splitter resizes
        var resizeHandler = function () {
            controller.layoutDiagram();
        };
        splitterBetweenTasksAndDiagram.attachResize({}, resizeHandler);
        splitterBetweenTaskPaneAndCommentPane.attachResize({}, resizeHandler);

        return splitterBetweenTaskPaneAndCommentPane;
    },

    _createSearchField: function (controller) {
        this.searchField = new sas.hc.m.SearchField({
            id: "task-searchFld",
            placeholder: this.rb.getText("home.tasks.searchField.prompt.txt"),
            liveChange: $.proxy(controller.onFilterChanged, controller),
            search: $.proxy(controller.onFilterChanged, controller)
        });
        return this.searchField;
    },

    _createTaskTable: function (controller) {
        var that=this;
        this.taskTable = new sas.hc.ui.table.Table({
            visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto,
            rowHeight: 36,
            minAutoRowCount: 1,
            editable: false,
            enableColumnFreeze: false,
            showSelectionControls: true,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
            selectionMode: sap.ui.table.SelectionMode.Single,
            useRadioForSingleSelection: true,
            selectedIndex: "{/selectedTask}",
            showTableOptions: true,
            toolbar: this._createTableToolbar(controller),
            noData: this.rb.getText("task.tile.alltasks.noData.loading.txt"),
            afterColumnsDialogClose: $.proxy(controller.handleColumnVisibilityChange, controller)
        });

        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-id"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.taskid.header.title")}),
            template: new sas.hc.m.Text({text: "{id}", tooltip: "{id}", maxLines: 2}),
            sortProperty: "id",
            name: "id",
            width: "5rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-name"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.taskname.header.title")}),
            template: new sas.hc.m.Text({text: "{name}", tooltip: "{name}", maxLines: 2}),
            sortProperty: "name",
            name: "name",
            width: "17rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-users"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.owners.header.title")}),
            template: new sas.hc.m.Text({
                text: "{owner}", maxLines: 2, tooltip: {
                    path: "users",
                    formatter: function (users) {
                        return users && users.join(", ") || "";
                    }
                }
            }),
            sortProperty: "owner",
            name: "owner",
            width: "12rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-status"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.status.header.title")}),
            template: new sas.hc.ui.layout.HorizontalLayout({
                content: [
                    new sas.hc.m.Image({
                        src: {
                            path: "status",
                            formatter: function (status) {
                                if (status) {
                                    if (status.label === "task.status.reset.fmt.txt") {
                                        status.icon = "REVIEWSTATUS";
                                    }
                                    if (status.label === "task.status.inprogress.txt") {
                                        status.icon = "SUBMITTEDSTATUS";
                                    }
                                    var iconInfo = sap.ui.core.IconPool.getIconInfo(sas.icons.HC[status.icon]);
                                    return (!iconInfo) ? sas.icons.HC[status.icon] : iconInfo.uri;
                                }
                            }
                        }
                    }).addStyleClass("statusIcon"),
                    new sas.hc.m.Text({text: "{statusLabel}", tooltip: "{statusLabel}", maxLines: 2})
                ]
            }),
            sortProperty: "statusLabel",
            name: "statusLabel",
            width: "14rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-execution-status"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.SAS.Script.Execution.Status.header.title")}),
            template : new sas.hc.ui.layout.HorizontalLayout({
                content: [
                    new sas.hc.m.Image({
                        src: {
                            path: "script",
                            formatter:function (script){
                                if (_.isUndefined(script) || _.isNull(script)) {
                                    return null;
                                }
                                if (script.status===undefined||script.status===null){
                                    return null;
                                }
                                var status = script.status.toUpperCase();
                                var icon;
                                switch (status) {
                                    case "NOTRUN":
                                    case "NOT_STARTED":
                                        icon="NEUTRALSTATUS";
                                        break;
                                    case "READYTORUN":
                                        icon="SCHEDULEDSTATUS";
                                        break;
                                    case "COMPLETED":
                                    case "SUCCESS":
                                        icon="COMPLETESTATUS";
                                        break;
                                    case "ERROR":
                                        icon="ERRORSTATUS";
                                        break;
                                    case "RUNNING":
                                    case "EXECUTING":
                                        icon="PROGRESSSTATUS";
                                        break;
                                    case "TERMINATEDBYSHUTDOWN":
                                        icon="TERMINATEPROCESS";
                                        break;
                                    case "INCOMPLETE":
                                        icon="INCOMPLETESTATUS";
                                        break;
                                    default:
                                        icon="BLANK";
                                        break;
                                }
                                var iconInfo = sap.ui.core.IconPool.getIconInfo(sas.icons.HC[icon]);
                                return (!iconInfo) ? sas.icons.HC[icon] : iconInfo.uri;
                            }
                        },
                        tooltip:{
                            path: "script",
                            formatter: function(script){
                                if (_.isUndefined(script) || _.isNull(script)) {
                                    return "";
                                }
                                if (script.status===undefined||script.status===null){
                                    return "";
                                }
                                var status=script.status.toUpperCase();
                                var dttm= script.timestamp;
                                switch (status){
                                    case "NOTRUN":
                                    case "NOT_STARTED":
                                        return "";
                                    case "READYTORUN":
                                        return that.rb.getText("task.sas.script.scheduled.to.run.txt");
                                    case "COMPLETED":
                                    case "SUCCESS":
                                        if (!dttm){
                                            return that.rb.getText("task.sas.script.completed.success.txt");
                                        }else{
                                            var fmt = sap.ui.core.format.DateFormat.getDateTimeInstance();
                                            var time= fmt.format(new Date(dttm),false);
                                            return that.rb.getText("task.sas.script.completed.on.fmt", [time]);
                                        }
                                    case "ERROR":
                                        if (!dttm){
                                            return that.rb.getText("task.sas.script.completed.error.txt");
                                        }else{
                                            var fmt = sap.ui.core.format.DateFormat.getDateTimeInstance();
                                            var time= fmt.format(new Date(dttm),false);
                                            return that.rb.getText("task.sas.script.completed.error.on.fmt", [time]);
                                        }
                                    case "RUNNING":
                                    case "EXECUTING":
                                        return that.rb.getText("task.sas.script.executing.txt");
                                    case "TERMINATEDBYSHUTDOWN":
                                        return that.rb.getText("task.sas.script.terminated.by.workspace.server.shutdown.txt");
                                    case "INCOMPLETE":
                                        return that.rb.getText("task.sas.script.incomplete.txt");
                                    default:
                                        return "";
                                }
                            }
                        }
                    }).addStyleClass("taskSASExecutionStatusIcon"),
                    new sas.hc.m.Button({
                        icon: sas.icons.HC.LOG,
                        visible: {
                            path: "script",
                            formatter: function(script){
                                if (_.isUndefined(script) || _.isNull(script)) {
                                    return false;
                                }
                                if (!script.runKey){
                                    return false;
                                }
                                if (script.status===undefined||script.status===null){
                                    return false;
                                }
                                var status = script.status.toUpperCase();
                                switch(status){
                                    case "COMPLETED":
                                    case "SUCCESS":
                                    case "ERROR":   // fixes for S1300931
                                        return true;
                                    default:
                                        return false;
                                }
                                return true;
                            }
                        },
                        enabled: {
                            path: "script",
                            formatter:function(script){
                                if (_.isUndefined(script) || _.isNull(script)) {
                                    return false;
                                }
                                if (!script.runKey){
                                    return false;
                                }
                                return true;
                            }
                        },
                        customData: new sap.ui.core.CustomData({
                            key: "id",
                            value: "{id}"
                        }),
                        press: $.proxy(controller.onLogButton,controller)
                    })
                ]
            }),
            name: "scriptStatusLabel",
            width: "7rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-duration"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.duration.header.title")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({
                text: "{duration}", tooltip: {
                    path: "duration",
                    formatter: function (duration) {
                        return "" + duration; // It's an integer, but it needs to be a string (otherwise an error is thrown).
                    }
                },
                maxLines: 2
            }),
            sortProperty: "duration",
            name: "duration",
            hAlign: sap.ui.core.HorizontalAlign.Right,
            width: "5rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-planned-start"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.plannedstart.header.title")}),
            template: new sas.hc.m.Text({
                text: {
                    path: "plannedStart",
                    type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"}),
                    maxLines: 2
                },
                tooltip: {
                    path: "plannedStart",
                    type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"}),
                    maxLines: 2
                }
            }),
            sortProperty: "plannedStart",
            name: "plannedStart",
            width: "11rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-planned-end"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.plannedend.header.title")}),
            template: new sas.hc.m.Text({
                text: {
                    path: "plannedEnd",
                    type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"})
                },
                tooltip: {
                    path: "plannedEnd",
                    type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"})
                },
                maxLines: 2
            }),
            sortProperty: "plannedEnd",
            name: "plannedEnd",
            width: "11rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-iteration"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.iteration.header.title")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({
                text: "{iteration}", tooltip: {
                    path: "iteration",
                    formatter: function (iteration) {
                        return "" + iteration; // It's an integer, but it needs to be a string (otherwise an error is thrown).
                    }
                }, maxLines: 2
            }),
            sortProperty: "iteration",
            name: "iteration",
            hAlign: sap.ui.core.HorizontalAlign.Right,
            width: "5rem"
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-description"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.description.header.title")}),
            template: new sas.hc.m.Text({text: "{description}", tooltip: "{description}", maxLines: 2}),
            sortProperty: "description",
            name: "description",
            width: "8rem",
            visible: false
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-process"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.process.header.title")}),
            template: new sas.hc.m.Text({text: "{process}", tooltip: "{process}", maxLines: 2}),
            sortProperty: "process",
            name: "process",
            width: "13rem",
            visible: false
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-executed-user"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.script.executed.user.title")}),
            template :new sas.hc.m.Text({
                text: {
                    path: "script",
                    formatter:function(script){
                        if (_.isUndefined(script) || _.isNull(script)) {
                            return null;
                        }
                        if (script.user===undefined||script.user===null){
                            return null;
                        }
                        return script.user;
                    }
                }
            }),
            sortProperty: {
                path: "script",
                formatter:function(script){
                    return script?script.user:"";
                }
            },
            name:"script.user",
            width:"8rem",
            visible:false
        }));
        this.taskTable.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("tasktable-executed-on"),
            label: new sas.hc.m.Label({text: this.rb.getText("home.tasks.col.script.executed.on.title")}),
            template :new sas.hc.m.Text({
                text: {
                    path: "script",
                    formatter:function(script){
                        if (_.isUndefined(script) || _.isNull(script)) {
                            return null;
                        }
                        if (script.timestamp===undefined||script.timestamp===null){
                            return "";
                        }
                        var dttm=script.timestamp;
                        var fmt = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return fmt.format(new Date(dttm),false);
                    }
                }
            }),
            sortProperty: {
                path: "script",
                formatter:function(script){
                    return script?script.timestamp:"";
                }
            },
            name:"script.timestamp",
            width:"12rem",
            visible:false
        }));

        this.taskTable.bindRows("/myTasks");
        return this.taskTable;
    },

    _createTableToolbar: function (controller) {

        this.taskSelect = new sas.hc.m.Select({
            items: [
                new sas.hc.ui.core.Item({key: "myTasks", text: this.rb.getText("home.tasks.option.mine.txt")}),
                new sas.hc.ui.core.Item({key: "summaryTasks", text: this.rb.getText("home.tasks.option.summary.txt")}),
                new sas.hc.ui.core.Item({key: "historyTasks", text: this.rb.getText("home.tasks.option.log.txt")})
            ],
            change: $.proxy(controller._handleTaskSwitch, controller)
        }).addStyleClass("homepageTaskBarItem");

        this.actionButton = new sas.hc.ui.commons.MenuButton({
            id: "home-action-button",
            text: this.rb.getText("home.tasks.details.actions.txt"),
            tooltip: this.rb.getText("home.tasks.details.actions.txt"),
            enabled: {
                path: '/selectedTask',
                formatter: $.proxy(controller.enableTaskActions, controller)
            }
        }).addStyleClass("toolbarMenuButton");

        this.actionButton.attachItemSelected($.proxy(controller.performTaskAction, controller));
        this.actions = new sas.hc.ui.unified.Menu({});
        this.actionButton.setMenu(this.actions);

        return new sas.hc.m.Toolbar({
            content: [
                this._createSearchField(controller),
                this.taskSelect,
                new sas.hc.m.ToolbarSpacer(),
                this.actionButton.addStyleClass("homepageTaskBarItem")
            ]
        });

    },

    _createTaskDetailsPanel: function (controller) {
        var commentEntryTemplate = new sas.stress.wkbench.widget.CommentEntry({
            author: "{author}",
            date: "{date}",
            editable: "{editable}",
            text: "{text}",
            number: "{number}",
            attachmentName: "{attachmentName}",
            downloadAttachment: $.proxy(controller.downloadAttachment, controller),
            saveEntry: $.proxy(controller.saveComment, controller),
            deleteEntry: $.proxy(controller.deleteComment, controller)
        });
        return new sas.stress.wkbench.widget.CommentManager({
            editable: {
                path: '/selectedTask',
                formatter: $.proxy(controller.isCommentsEditable, controller)
            },
            entries: {
                path: "/comments",
                template: commentEntryTemplate
            },
            enabled: {
                path: '/selectedTask',
                formatter: $.proxy(controller.enableComments, controller)
            },
            notActiveText: '{i18N>home.tasks.details.notasks.txt}',
            noDataText: '{i18N>home.tasks.details.comments.nodata.txt}',
            post: $.proxy(controller.addComment, controller)
        });

    },

    _createWorkflowPanel: function (controller) {

        var that = this;
        if (this.workflowDiagram) {
            return this.workflowDiagram;
        }//cache it

        var templates = new sas.stress.wkbench.StressGoTemplates();

        this.workflowDiagram = new sas.stress.wkbench.widget.ProcessDiagram({
            content: '<div style="width:100%; height:100%; min-height:200px;"></div>',
            id: 'HomeProcessDiagram',
            preferDOM: false,
            options: {
                allowDelete: false,
                allowMove: false,
                allowLink: false,
                allowRelink: false,
                allowVerticalScroll: true,
                allowHorizontalScroll: true,
                hasHorizontalScrollbar: true,
                hasVerticalScrollbar: false,
                allowUndo: false,
                contentAlignment: go.Spot.Center,
                "toolManager.mouseWheelBehavior": go.ToolManager.WheelZoom,
                padding: 15
            },
            layout: templates.LAYOUT(),
            nodeTemplates: {
                "": templates.DEFAULT_TASK(),
                "_START_": templates.START_NODE(),
                "_END_": templates.END_NODE(),
                "_SYNCBAR_": templates.SYNC_BAR()
            },
            linkTemplates: {
                "": templates.LINK()
            },
            nodes: "{/process/nodes}",
            links: "{/process/links}"
        });

        zoomInButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMIN,
            tooltip: "{i18N>home.diagram.zoomInButton.txt}",
            press: function () {
                var commandHandler = that.workflowDiagram.getDiagram().commandHandler;
                if (commandHandler.canIncreaseZoom()) {
                    commandHandler.increaseZoom();
                }
            }
        });

        zoomOutButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMOUT,
            tooltip: "{i18N>home.diagram.zoomOutButton.txt}",
            press: function () {
                var commandHandler = that.workflowDiagram.getDiagram().commandHandler;
                if (commandHandler.canDecreaseZoom()) {
                    commandHandler.decreaseZoom();
                }
            }
        });

        zoomFitButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMTOFIT,
            tooltip: "{i18N>home.diagram.zoomFitButton.txt}",
            press: function () {
                var commandHandler = that.workflowDiagram.getDiagram().commandHandler;
                commandHandler.zoomToFit();
            }
        });

        var zoomToolbar = new sas.hc.m.Toolbar({
            content: [
                new sas.hc.m.ToolbarSpacer(),
                zoomInButton,
                zoomOutButton,
                zoomFitButton
            ]
        }).addStyleClass("hierarchyDiagramToolbar");

        var workflowLayout = new sas.hc.ui.commons.layout.BorderLayout({width: "100%", height: "100%"});
        var headerArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            size: "2.5rem",
            content: zoomToolbar
        }).addStyleClass("diagramToolbar");
        var diagramArea = new sas.hc.ui.commons.layout.BorderLayoutArea({content: this.workflowDiagram}).addStyleClass("diagramContainer");
        workflowLayout.setTop(headerArea);
        workflowLayout.setCenter(diagramArea);

        return workflowLayout;

    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    }
});