jQuery.sap.require({ modName: "sas.stress.wkbench.view.ProjectManager", type: "controller" });
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require('sas.hc.ui.commons.MenuButton');
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require('sas.hc.ui.unified.Menu');
jQuery.sap.require('sas.hc.ui.unified.MenuItem');
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.widget.StressGrooveBar");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");

sap.ui.jsview("sas.stress.wkbench.view.ProjectManager", {

    getControllerName: function () {
        return "sas.stress.wkbench.view.ProjectManager";
    },

    createContent: function (controller) {

        this.setHeight("100%");

        return new sas.risk.common.widgets.RiskPage({
            iconBar: new sas.stress.wkbench.widget.StressGrooveBar({
                section: sas.stress.wkbench.widget.StressGrooveBar.PROJECTS
            }),
            mainContent: new sas.risk.common.widgets.RiskPage({
                mainContent: this._createProjectPanel(controller),
                toolbar: new sas.risk.common.widgets.RiskToolbar({
                    title: new sas.hc.m.Label({text: "{i18N>home.shortcut.projects.txt}"})
                })
            })
        });

    },

    _createTableToolbar: function (controller) {

        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        this.newProjectButton = new sas.hc.m.Button({
            icon: sas.icons.HC.NEWSTARBURST,
            press: $.proxy(controller.newProjectButtonPress, controller),
            enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.CREATE),
            tooltip: "{i18N>project.createButton.tip}"
        });

        this.editProjectButton = new sas.hc.m.Button({
            icon: sas.icons.HC.EDIT,
            press: $.proxy(controller.editProjectButtonPress, controller),
            enabled: false,
            tooltip: "{i18N>project.editButton.tip}"
        });


        this.refreshButton = new sas.hc.m.Button({
            icon: sas.icons.HC.REFRESH,
            press: $.proxy(controller._loadProjects, controller),
            enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.READ),
            tooltip: "{i18N>refresh.button.tip}"
        });

        this.deleteProjectButton = new sas.hc.m.Button({
            icon: sas.icons.HC.DELETE,
            press: $.proxy(controller.deleteProjectButtonPress, controller),
            enabled: false,
            tooltip: "{i18N>project.deleteButton.tip}"
        });


        this.openProjectMenu = new sas.hc.ui.unified.MenuItem({
            select: $.proxy(controller.openProjectButtonPress, controller),
            enabled: false,
            text: "{i18N>project.openButton.tip}"
        });

        this.lockProjectMenu = new sas.hc.ui.unified.MenuItem({
            enabled: false,
            select: $.proxy(controller.lockProjectButtonPress, controller),
            text: "{i18N>project.lockButton.lock.tip}"
        });

        this.projectAccessControlsMenu = new sas.hc.ui.unified.MenuItem({
            select: $.proxy(controller.projectAccessControlsButtonPress, controller),
            enabled: false,
            text: "{i18N>project.workGroupButton.tip}"
        });

        var overflowMenu = new sas.hc.ui.unified.Menu({
            items: [
                this.openProjectMenu,
                this.projectAccessControlsMenu,
                this.lockProjectMenu
            ]
        });

        var overflowButton = new sas.hc.ui.commons.MenuButton({
            icon: sas.icons.HC.OVERFLOW,
            menu: overflowMenu
        });

        return new sas.hc.m.Toolbar({
            content: [
                this._createSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                this.newProjectButton,
                this.editProjectButton,
                this.refreshButton,
                this.deleteProjectButton,
                overflowButton
            ]
        });

    },

    _createProjectPanel: function (controller) {
        this.table = this._createProjectsTable(controller);
        this.taskTable = this._createTaskTable(controller);
        return sas.risk.common.widgets.WidgetFactory.buildPanel({
            content: new sas.hc.ui.layout.Splitter({
                contentAreas: [
                    this.table,
                    this.taskTable
                ],
                orientation: sap.ui.core.Orientation.Vertical,
                width: "100%",
                height: "100%"
            }),
            topPadding: ".5rem"
        });
    },

    _createTaskTable: function(controller) {

        var table = new sas.hc.ui.table.Table({
            visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto,
            selectionMode: sap.ui.table.SelectionMode.None,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: true,
            noData: "{i18N>home.mytasks.noData.loading.txt}",
            title: "{i18N>home.mytasks.header.title}",
            cellClick: $.proxy(controller.taskTableClick, controller)
        });

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectName"),
            width:"20%",
            label: new sas.hc.m.Label({text: "{i18N>home.mytasks.col.project.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{projectName}",
                tooltip: "{projectName}"
            }),
            sortProperty: "projectName"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("taskID"),
            width:"5%",
            label: new sas.hc.m.Label({text: "{i18N>home.mytasks.col.taskID.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{id}",
                tooltip: "{id}"
            }),
            sortProperty: "id"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("activity"),
            width:"20%",
            label: new sas.hc.m.Label({text: "{i18N>home.mytasks.col.activity.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{name}",
                tooltip: "{name}"
            }),
            sortProperty: "name"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("owners"),
            width:"20%",
            label: new sas.hc.m.Label({text: "{i18N>home.tasks.col.owners.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{owner}",
                tooltip: "{owner}"
            }),
            sortProperty: "owner"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("iteration"),
            width:"5%",
            label: new sas.hc.m.Label({text: "{i18N>home.tasks.col.iteration.header.title}"}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({
                text: "{iterationNum}", tooltip: {
                    path: "iterationNum",
                    formatter: function (iterationNum) {
                        return "" + iterationNum;
                    }
                }, maxLines: 2
            }),
            hAlign: sap.ui.core.HorizontalAlign.Right,
            sortProperty: "iterationNum"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("plannedend"),
            label: new sas.hc.m.Label({text: "{i18N>home.tasks.col.plannedend.header.title}"}),
            template: new sas.hc.m.Text({
                text: {
                    path: "plannedEndDate",
                    type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"})
                },
                tooltip: {
                    path: "plannedEndDate",
                    type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"})
                },
                maxLines: 2
            }),
            sortProperty: "plannedEndDate",
            name: "plannedEndDate",
            width: "11rem"
        }));

        table.bindRows("/");

        return table;

    },

    _createSearchField: function (controller) {
        return searchField = new sas.hc.m.SearchField({
            id: "projectsSearchFld",
            placeholder: "{i18N>projectsTable.searchField.prompt.txt}",
            liveChange: $.proxy(controller.filterProjects, controller),
            search: $.proxy(controller.filterProjects, controller)
        });
    },

    _createProjectsTable: function (controller) {

        var table = new sas.hc.ui.table.Table({
            id: "stressProjectTable",
            visibleRowCountMode: sap.ui.table.VisibleRowCountMode.Auto,
            selectionMode: sap.ui.table.SelectionMode.MultiToggle,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowSelector,
            toolbar: this._createTableToolbar(controller),
            noData: "{i18N>project.noData.loading.txt}",
            showTableOptions: true,
            cellClick: $.proxy(controller.projectsTableCellClick, controller),
            rowSelectionChange: $.proxy(controller.projectsTableRowSelectionChange, controller),
        });

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("lock"),
            label: new sas.hc.ui.core.Icon({
                size: "1rem",
                src: sas.icons.HC.LOCKCLOSED
            }),
            template: new sas.hc.ui.core.Icon({
                size: "1rem",
                src: {
                    path: 'locked',
                    formatter: function (locked) {
                        return (locked) ? sas.icons.HC.LOCKCLOSED : null;
                    }
                },
                tooltip: "{i18N>project.lockIndicator.tip}"
            }),
            sortProperty: 'readOnly',
            width: "2rem",
            resizable: false,
            hAlign: sap.ui.core.TextAlign.Center
        }).setStyleClass("managerLockCol"));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("name"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.name.header.title}" }),
            template: new sas.hc.m.Text({
                text: "{name}",
                tooltip: "{name}"
            }),
            sortProperty: "name",
            width: "17rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("asOfDate"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.asOfDate.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: {
                    path: "asOfDate",
                    formatter: function (dttm) {
                        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance();
                        return dateFormat.format(new Date(dttm), true);  // The true sets it to use UTC time zone;  without it, it subtracts a day
                    }
                },
                tooltip: {
                    path: "asOfDate",
                    formatter: function (dttm) {
                        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance();
                        return dateFormat.format(new Date(dttm), true);  // The true sets it to use UTC time zone;  without it, it subtracts a day
                    }
                }
            }),
            sortProperty: "asOfDate",
            width: "10rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("createdByUser"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.createdByUser.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{createdByUser}",
                tooltip: "{createdByUser}"
            }),
            sortProperty: "createdByUser",
            width: "11rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dateCreated"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.dateCreated.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "createdTime",
                    formatter: function (dttm) {
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "createdTime",
                    formatter: function (dttm) {
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "createdTime",
            width: "10rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("updatedByUser"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.updatedByUser.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{updatedByUser}",
                tooltip: "{updatedByUser}"
            }),
            sortProperty: "updatedByUser",
            width: "11rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dateModified"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.dateModified.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "lastModified",
                    formatter: function (dttm) {
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "lastModified",
                    formatter: function (dttm) {
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "lastModified",
            width: "10rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("description"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.description.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{description}",
                tooltip: "{description}"
            }),
            sortProperty: "description",
            width: "20rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectID"),
            label: new sas.hc.m.Label({ text: "{i18N>projectsTable.column.projectID.header.title}" }),
            template: new sas.hc.m.Text({
                text: "{id}"
            }),
            sortProperty: "id",
            width: "4rem",
            visible: false
        }));



        table.bindRows("/");
        return table;
    }

});