jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.SlimsheetSource");
jQuery.sap.require("sas.stress.wkbench.widget.SlimsheetCompareSource");
jQuery.sap.require("sas.stress.wkbench.CellPickerUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.model.Sheet");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.controller("sas.stress.wkbench.view.ScenarioEditor", {
    onInit: function () {
        var view = this.getView();
        var viewData = view.getViewData();
        var SourceClass = viewData.sourceClass;
        var source = new SourceClass(viewData.sheetId, viewData.scenarioId);
        var model = new sas.stress.wkbench.model.RiskJSONModel({source: source});
        view.setModel(model);

        // need to clear out the tokens displayed for the new sheet
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();
        wkbenchview.tokenizer.destroyTokens();
        this.displaySheetFilters(viewData.sheetId, viewData.filters, viewData.iterationId);
    },
    onAfterRendering: function () {
        var view = this.getView();
        var liveModel = view.getModel("live");
        if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.NORMAL_MODE) {
            liveModel.setProperty("/selected", null);
            liveModel.setProperty("/editing", null);
            liveModel.setProperty("/informula", []);
        }
        liveModel.setProperty("/slimsheet", view.slimsheet);
        view.slimsheet.removeCellWithSelectedClass();
        sas.stress.wkbench.ViewManager.getWorkbench().setBusy(false);
        sas.stress.wkbench.ViewManager.getWorkbench().getController().closeBzDialog();
    },
    onExit: function () {
        // free the key from the list of stored scenario editor views, this will free memory too
        var key = this.getView().getId();
        var tobeDestoryedviews = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
        tobeDestoryedviews[key] = undefined;
        delete tobeDestoryedviews[key];
    },
    cellSelectedAction: function (e) {
        var liveModel = this.getView().getModel("live");
        var selected = e.getParameter("selected");
        if (selected) {
            var cellRef = new sas.stress.wkbench.model.CellRef(e.getParameter("col")/*-1*/, e.getParameter("row"), e.getParameter("filter")/*.getParameter("scenario")*/);
            if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
                this._addSelectedCellToFormula(cellRef);
            } else {
                this.getView().slimsheet.selectCell(cellRef.row, cellRef.column/*+1*/, cellRef.filterId);
                liveModel.setProperty("/selected", null);//so that the next line triggers its listeners
                liveModel.setProperty("/selected", cellRef);
                liveModel.setProperty("/editing", null);
                liveModel.setProperty("/informula", []);
                //this.getView().slimsheet.focus();
            }
        }
    },
    _addSelectedCellToFormula: function (selected) {
        // Get the current cell info from the Cell Editor
        var cellEditor = sas.stress.wkbench.ViewManager.getCellEditor();
        var liveModel = cellEditor.getModel("live");
        var editing = liveModel.getProperty("/editing");
        var cellEditorScenario = editing.scenario;
        var cellEditorSheetId = editing.sheetId;
        var cellEditorScenarioId = editing.scenarioId;
        var cellEditorScenarioName = editing.scenarioName;

        // The selected cell info
        var selectedCellScenario = selected.scenario;
        var selectedCellScenarioId = selected.scenarioId;
        var selectedCellScenarioName = selected.scenarioName;
        var selectedCellSheetId = selected.sheetId;

        // Compare selected cell with the current cell to determine proper cell reference.
        // ie, if on the same sheet and scenario, then we do not include them in the cell ref,
        // just use row and column ref (Local Reference)
        var cellName;
        if ((selectedCellScenarioId === cellEditorScenarioId) && (selectedCellSheetId === cellEditorSheetId)) {
            cellName = selected.toString(sas.stress.wkbench.model.CellRef.FMT_CELL_SHORT_LOCAL);
        } else if (cellEditorScenarioName === selectedCellScenarioName) {
            cellName = selected.toString(sas.stress.wkbench.model.CellRef.FMT_CELL_SHORT_LOCAL_BY_SCENARIO);
        } else {
            cellName = selected.toString(sas.stress.wkbench.model.CellRef.FMT_CELL_SHORT);
        }
        var StressUtil = sas.stress.wkbench.StressUtil;
        // Find the caret in the formula
        var formulaField = sas.stress.wkbench.ViewManager.getCellEditor().formulaField;
        var pos = StressUtil.getCaretPosition(formulaField.getId());
        // The original formula
        var oldFormula = formulaField.getLiveValue();
        // Insert cell name into formula at caret position to create new formula
        var newFormula = oldFormula.substr(0, pos) + cellName + oldFormula.substr(pos);
        // Set the formula field value with the new formula
        formulaField.setValue(newFormula);
        // Set the caret to just after the inserted cell name in the formula
        StressUtil.setCaretPosition(formulaField.getId(), pos + cellName.length);
        formulaField.fireLiveChange();
    },
    _onPropertyPaneVisible: function (e) {
        var view = this.getView();
        view.slimsheet.rerender();
        sas.stress.wkbench.StressUtil._updateFiltersPanelSize(view.layout.getPropertyPaneVisible(), view.facetPanel.getFacets());
    },
    displaySheetFilters: function (sheetId, filterId, iterationId) {
        var that = this;
        var scview = that.getView();
        var facetPanel = scview.facetPanel;
        var facetModel = new sas.stress.wkbench.model.RiskJSONModel();

        facetPanel.removeAllFacets();
        var modelData = [];
        var parents = {};
        var parentNames = {};

        facetModel.setData({"data": modelData});
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var sheet = project.getSheet(sheetId);
        if (sheet) {
            sheet.loadSheetFilters(project.getId()).done(function (items) {
                _.each(items, function (item, idx) {
                    var name = item.hierarchy.name;
                    var nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(name, item.hierarchy.version);
                    var facet = sap.ui.getCore().byId(scview.createId(idx.toString()));
                    if (facet === undefined) {
                        facet = new sas.stress.wkbench.widget.DescriptionFacet({
                            id: scview.createId(idx.toString()),
                            text: nameVersion,
                            fieldName: name,
                            searchThreshold: 10,
                            key: item.hierarchyKey
                        });
                    }
                    facetPanel.addFacet(facet);

                    // If this is a scenario facet filter then sort alphanumeric, otherwise leave it as it came from midtier
                    var lineItems = (item.hierarchy.code === "SCENARIO") ? item.lineItems.sort(function (a, b) {
                        return a.memberName.localeCompare(b.memberName);
                    }) : item.lineItems;

                    _.each(lineItems, function (lineItem) {
                        // TODO: Add full parent info
                        parentNames[lineItem.liKey] = lineItem.memberName;
                        parents[lineItem.liKey] = {
                            key: lineItem.parentLiKey,
                            name: parentNames[lineItem.liKey]
                        };
                        var val = {};
                        val[name] = lineItem.memberName;
                        val.path = that._getFullParentInfo(parentNames, parents, lineItem.parentLiKey);
                        //val.path = (parents[lineItem.liKey].name) ? parents[lineItem.liKey].name + " > " : "";
                        val.liKey = lineItem.liKey;
                        val.hierarchyKey = lineItem.hierarchyKey;
                        modelData.push(val);
                    });
                });
                facetModel.setData({"data": modelData});
                var filterProvider = new sas.stress.wkbench.widget.DescriptionModelFilterProvider({
                    dataPath: "/data",
                    facetPanel: facetPanel
                });
                filterProvider.setModel(facetModel);
                facetPanel.setModel(facetModel);
                // select the defaults
                var defFilters = sheet.getPreferredSheetFilter();
                var facets = facetPanel.getFacets();
                var selectedFacets = [];
                var selectedFacet = {facetKey: 0, selectedFacetItem: undefined};
                var moveToNextFacet = false;
                _.each(defFilters, function (eachfilter, key) {
                    selectedFacet = {facetKey: 0, selectedFacetItem: undefined};
                    var facet, facetKey;
                    for (facetKey = 0; facetKey < facets.length; facetKey++) {
                        facet = facets[facetKey];

                        var facetItems, facetItem, facetItemIndex;
                        facetItems = facet.getFacetItems();
                        for (facetItemIndex = 0; facetItemIndex < facetItems.length; facetItemIndex++) {
                            var itemtoselect = _.find(facet.getFacetItems(), function (item) {
                                return item.liKey === eachfilter;
                            });
                            if (itemtoselect) {
                                // the item is found therefore set it as selected
                                itemtoselect.selected = true;
                                // store the key and item
                                selectedFacet.facetKey = facetKey;
                                selectedFacet.selectedFacetItem = itemtoselect;

                                selectedFacets.push(selectedFacet);
                                moveToNextFacet = true;
                                break;
                            }
                        }
                        if (moveToNextFacet) {
                            moveToNextFacet = false;
                            break;
                        }
                    }
                });
                // loop and fire the facet item as selected in the facets
                _.each(selectedFacets, function (selectedFacet) {
                    var facet = facets[selectedFacet.facetKey];
                    // fixes for: S1237759; somehow the set selected item is no longer working
                    try {
                        facet.list.setSelectedItem(selectedFacet.selectedFacetItem);
                    } catch (e) {
                    }
                    // fixes for: S1229696; fire the selection change on the facet's list control..
                    facet._fireSelectionChange({
                        listItem: selectedFacet.selectedFacetItem,
                        listItems: [selectedFacet.selectedFacetItem],
                        selected: true
                    });
                });
                // these are default filters therefore the apply button needs to be disabled
                scview.applyFilterButton.setEnabled(false);
            });
        }
    },
    pressFilterApplyButton: function (project) {
        var that = this;
        var view = this.getView();
        var viewData = view.getViewData();
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();
        var wkbenchController = wkbenchview.getController();
        project = wkbenchController.project;	// makes sure we have the latest and greatest
        var sheet = project.getSheet(viewData.sheetId);

        if (wkbenchController.navChecker.getDirty()) {	// fixes for S1236895
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            // show the warning dialog asking for refresh before if unsaved changes exist
            sas.hc.m.MessageBox.warning(rb.getText("project.sheet.filters.apply.question.txt"), rb.getText("project.sheet.filters.apply.title"),
                [sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL],
                null, function (oAction) {
                    if (oAction === sas.hc.m.MessageBox.Action.OK) {
                        // user decided to discard the changes... process the apply pressed button event

                        // fixes for S1265734... see if we are in the EDIT_MODE.. if yes, leave
                        if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
                            sas.stress.wkbench.ViewManager.getCellEditor().getController().leaveEditMode();
                        }
                        that._pressFilterApplyButton(project, sheet)
                            .done(function () {
                                // we want to discard the changes.. the navchecker is not dirty
                                wkbenchController.navChecker.setDirty(false);
                                // update the facet Panel
                                that._updateFacetPanel();
                        }).fail(function (jqxhr) {
                            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                        });

                    } else {
                        view.applyFilterButton.setEnabled(false);
                        that._updateFiltersDisplay();   // revert back to the original filters fixes for S1246706
                        view.setBusy(false);
                    }
                });
        } else {
            // fixes for S1265734... see if we are in the EDIT_MODE.. if yes, leave
            if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
                sas.stress.wkbench.ViewManager.getCellEditor().getController().leaveEditMode();
            }
            that._pressFilterApplyButton(project, sheet)
                .done(function () {
                    that._updateFacetPanel();
                })
                .fail(function (jqxhr) {
                    sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr.responseJSON, sas.config.appName);
                });
        }
    },
    _pressFilterApplyButton: function (project, sheet) {
        var deferred = $.Deferred();
        var that = this;
        var view = this.getView();
        var facetPanel = view.facetPanel;
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();
        var source=view.slimsheet.getSlimsheetSource();
        source.destroy();
        //source.info=undefined;

        var projectFilters=project.getFilters();
        _.each(facetPanel.getFacets(), function (facet) {
            _.each(facet.getSelected(), function (item) {
                var projectFilter=_.find(projectFilters,function(filter){
                    return filter.hierarchyKey.toString()===facet.getKey();
                });
                if (projectFilter){
                    projectFilter.liKey=item.liKey;
                }else{

                }
            });
        });

        // save the filters
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.project_filters",JSON.stringify(projectFilters),project.getId());
        //wkbenchview.setModel();//clear out view model mistake .. fixes for S1312558
        wkbenchview.getController().openIfRequiredAndUpdateBzDialog();
        wkbenchview.getController().refreshProject(false).done(function(project){
            deferred.resolve();
        }).fail(function(jqxhr){
            deferred.reject(jqxhr);
        });

        return deferred;
    },
    _updateFacetPanel: function () {
        var view = this.getView();
        // re-render the slimsheet.. needed when we apply the filter.. new data is now available
        // fixes for S1271169.. we need to clear the info so that it can re-loaded
/*
        var source=view.slimsheet.getSlimsheetSource();
        source.destroy();
*/
        //source.info=undefined;
        view.slimsheet.rerender();
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();

        wkbenchview.tokenizer.destroyTokens();
        var facets = view.facetPanel.getFacets();
        _.each(facets, function (facet) {
            facet._fireSelectionChange();
        });

        view.applyFilterButton.setEnabled(false);
        view.setBusy(false);
    },
    _updateFiltersDisplay: function () {
        var view = this.getView();
        var viewData = view.getViewData();
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();
        var project = wkbenchview.getController().project;	// makes sure we have the latest and greatest
        var sheet = project.getSheet(viewData.sheetId);

        if (wkbenchview.tokenizer.getTokens().length>0) {
            var selectedFacets = view.facetPanel.getFacets();
            _.each(selectedFacets, function (selectedFacet, index) {
                var token = wkbenchview.tokenizer.getTokens()[index];
                if (token) {
                    selectedFacet.resetToken(token);
                }
            });
        }else{
            view.facetPanel.removeAllFacets();
            //wkbenchview.tokenizer.destroyTokens();
        }
        this.displaySheetFilters(viewData.sheetId, viewData.filters, viewData.iterationId);
        view.applyFilterButton.setEnabled(false);
        view.setBusy(false);
    },
    /**
     * fixes for S1266122
     *
     * We need to look at the tokens on the workbench to see if the selecteditem in the facet
     * has the token, if it does, enabled =false else enabled =true
     * @private
     */
    _enableButton:function(){
        var view = this.getView();
        var enable=false;
        var selectedFacets = view.facetPanel.getFacets();
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();

        var selected;
        if (wkbenchview.tokenizer.getTokens().length>0) {
            _.each(selectedFacets, function (selectedFacet, index) {
                var items=selectedFacet.getFacetItems();
                selected=_.find(items,function(item){
                    return item.selected;
                });

                if (selected){
                    var found=_.find(wkbenchview.tokenizer.getTokens(),function(token){
                        return token.getKey()===selected.liKey; // S1303232.. it's all keys for the defect fixes
                    });
                    // there is a selected item and it was either found or not
                    // if not found, then we need to enable else same item disable
                    enable=!found||enable;
                }
            });
        }
        view.applyFilterButton.setEnabled(enable);
        delete selected;
    },
    _getFullParentInfo:function(parentNames,parents,currentParentLiKey){
        if (!currentParentLiKey){
            return "";
        }
        var parentLiKey=currentParentLiKey;
        var path=(parents[parentLiKey]&&parents[parentLiKey].name) ? parents[parentLiKey].name + " > " : "";
        parentLiKey=parents[parentLiKey]?parents[parentLiKey].key:undefined;
        path=this._getFullParentInfo(parentNames,parents,parentLiKey) + path;

        return path;
    }
});