jQuery.sap.require("sas.hc.m.BusyDialog");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.stress.scenarios.ScenarioAPI");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.CellHistoryItem");
jQuery.sap.require("sas.stress.wkbench.widget.MessageDetailsBox");
jQuery.sap.require("sas.stress.wkbench.widget.DescriptionFacet");
jQuery.sap.require("sas.stress.wkbench.widget.DescriptionModelFilterProvider");


sap.ui.controller("sas.stress.wkbench.view.Workbench", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("Workbench", this._init, this);
    },
    _init: function (event) {
        var projectId = event.getParameter("arguments").projectId;

        var that = this;
        that._processAllSheets=false;   // S1371893
        // open the busy dialog
        that.openIfRequiredAndUpdateBzDialog(sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt"));
        var rb = sas.stress.wkbench.StressUtil.getBundle();

        // S1273684: don't use the cached project since it might be stale or deleted by others
        sas.stress.wkbench.model.Project.loadProject(projectId, true).done(function (project) {
            //var project = sas.stress.wkbench.model.Project.getProject(projectId);
            that.refresh(project).done(function(){
                // fixes for CET defect S1266931
                that.closeBzDialog();
            }).fail(function(jqxhr){
                that.closeBzDialog();
                if (!jqxhr.status) {
                    // fixes for CET defect S1266931
                    sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr,sas.config.appName);
                    //sas.hc.m.MessageBox.error(jqxhr, rb.getText('error.window.title'));
                } else { //workbench.load.dataerror.txt
                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                }
                return sas.stress.wkbench.ViewManager.loadHomePage(project);
            });
        }).fail(function (jqxhr) {
            that.closeBzDialog();
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            return sas.stress.wkbench.ViewManager.loadHomePage(that.project);
        });

        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
    },
    /*
     * the optional argument 'bEnableSave' is for enabling the save
     * during 'refresh and merge' option select on the conflict resolution dialog
     */
    refresh: function (project, bEnableSave) {
        var def = $.Deferred();
        var that = this;
        this.project = project;
        var view = this.getView();
        var rb = sas.stress.wkbench.StressUtil.getBundle();

        if (view.getModel() === undefined && bEnableSave === undefined) { // initial
            /*view.setBusyIndicatorDelay(0);
             view.setBusy(true);*/
            sas.stress.wkbench.ViewManager.loadCellEditor();
            sas.stress.wkbench.CellPickerUtil.deactivateEditMode();//page refresh we reset the cell picker edit mode

            $.when(this.project.loadSheetsData(),this.project.getMyTasks()).done(function (newproject, myTasks) {
                that._processAllSheets=true;    // S1371893
                var json = {};
                json.name = that.project.getName();
                json.actions=myTasks.length?myTasks[0].getActions():[];

                // categories first
                var _categories = {};
                json.categories = [];
                // the all category will contain the total count and it goes first
                var filteredSheets = that.project.getSheets();	// fixes for S1231681... all the sheets should be used to figure out the categories
                _categories[rb.getText("filter.category.all.txt")] = {
                    count: filteredSheets.length,
                    key: "0",
                    name: rb.getText("filter.category.all.txt")
                };
                _.each(filteredSheets, function (sheet) {
                    var categoryKey = sheet.getSheetCategory().getId();
                    var categoryname = sheet.getSheetCategory().getName();

                    var sheetcategorycount = _categories[categoryname] || {count: 0, key: "0", name: ""};
                    // increment the count &&
                    sheetcategorycount.count++;
                    _categories[categoryname] = {count: sheetcategorycount.count, key: categoryKey, name: categoryname};
                });
                var idx = 0;
                _.each(_categories, function (categorydetails, category) {
                    json.categories.push({
                        key: categorydetails.key,
                        name: category,
                        count: categorydetails.count.toString()
                    });
                    idx++;
                });

                // handle process orchestration parameters, if they exist. Need to do this prior to selecting the category
                // or loading the filtered sheets.
                var currentSheet;
                var paramsModel = sap.ui.getCore().getModel("WorkbenchParams");
                if (paramsModel && paramsModel.getData()) {
                    if (paramsModel.getData().sheetName) {
                        currentSheet = paramsModel.getData().sheetName;

                        // S1251269: If a worksheet was specified in process orchestration, then we want to use the "ALL" category.
                        var prefs = {"sas.stress.wkbench.category_key": 0};
                        sas.stress.wkbench.UserPrefs.save(prefs, project.getId());
                    }
                    sap.ui.getCore().setModel(undefined, "WorkbenchParams");
                }

                var preferredCategory = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.category_key", that.project.getId());
                // use the preferred category key if it is present
                if (preferredCategory) {
                    // fixes for S1231681
                    view.sheetCategorySelect.setSelectedKey(Number(preferredCategory));
                }

                // the sheets next
                filteredSheets = that._getFilteredSheets();
                json.sheets=[];
                if (filteredSheets.length === 0) {
                    // fixes for CET defect S1266931
                    var messages=[];
                    messages.push(rb.getText("workbench.error.no.templates.txt"));
                    messages.push(rb.getText("workbench.error.no.permissions.txt"));
                    messages.push(rb.getText("workbench.error.no.data.txt"));

                    //message+=rb.getText("contact.sys.admin.txt");
                    var json={message:rb.getText("workbench.error.nosheets.main.fmt", project.getName()),details:messages};
                    def.reject(json);
                }
                view.sheetSelect.removeAllItems();
                var foundSheet = false;
                _.each(filteredSheets, function (sheet) {
                    var icon;
                    if(sheet.errored){ // S1266931
                        icon=sas.icons.HC.WARNINGSTATUS; // error sheets get an icon too
                    }
                    json.sheets.push({name: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
                    var item = new sas.hc.ui.core.Item({text: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
                    view.sheetSelect.addItem(item);
                    // Was a starting sheet specified? (Process Orchestration)
                    if (currentSheet && currentSheet.toLocaleLowerCase() === sheet.getName().toLocaleLowerCase()) {
                        // Save the key of the worksheet that was passed in so it is the default worksheet
                        sas.stress.wkbench.UserPrefs.save({"sas.stress.wkbench.sheet_key": sheet.getId()}, that.project.getId());
                        delete currentSheet;
                        foundSheet = true;
                    }
                });
                if (currentSheet && !foundSheet) {
                    // Invalid worksheet name specified.
                    sas.hc.m.MessageBox.error(rb.getText("workbench.processOrchestration.invalidSheetName.fmt.txt", [currentSheet]), rb.getText("error.window.title"));
                }
                // iteration last
                var projectIteration = newproject.getCurrentIterationNo();
                json.iterations = [];
                for (var i = projectIteration; i > 0; i--) {
                    json.iterations.push({
                        text: i,
                        key: i.toString()
                    });
                }
                var iteration=sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id",newproject.getId())||projectIteration;
                view.iterationSelect.setSelectedKey(iteration.toString());

                var model= new sas.stress.wkbench.model.RiskJSONModel(json);
                that.getView().setModel(model);
                var liveModel = that.getView().getModel("live") || new sas.stress.wkbench.model.RiskJSONModel({saveButtonEnabled: ((bEnableSave !== undefined) ? bEnableSave : false)});
                liveModel.setProperty("/iterationNumber", sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo());
                that.getView().setModel(liveModel, "live");//live selections,formula editing decors
                that._autoSelectFirstSheet();

                def.resolve(that);
            }).fail(function (jqxhr) {
                if (typeof(jqxhr)==="string"&&jqxhr===sas.stress.wkbench.model.Sheet.NO_PERMISSONS) {
                    // fixes for CET defect S1266931
                    var messages=[];
                    messages.push(rb.getText("workbench.error.no.templates.txt"));
                    messages.push(rb.getText("workbench.error.no.permissions.txt"));
                    messages.push(rb.getText("workbench.error.no.data.txt"));

                    //message+=rb.getText("contact.sys.admin.txt");
                    var json={message:rb.getText("workbench.error.nosheets.main.fmt", project.getName()),details:messages};
                    def.reject(json);
                }
                def.reject(jqxhr);
            });
        } else {
            view.sheetSelect.removeAllItems();
            var filteredSheets = that._getFilteredSheets();
            if (filteredSheets.length === 0) {
                // fixes for CET defect S1266931
                var messages=[];
                messages.push(rb.getText("workbench.error.no.templates.txt"));
                messages.push(rb.getText("workbench.error.no.permissions.txt"));
                messages.push(rb.getText("workbench.error.no.data.txt"));

                //message+=rb.getText("contact.sys.admin.txt");
                var json={message:rb.getText("workbench.error.nosheets.main.fmt", project.getName()),details:messages};
                def.reject(json);
            }
            _.each(filteredSheets, function (sheet) {
                var icon;
                if(sheet.errored){ // S1266931 and // S1297759
                    icon=sas.icons.HC.WARNINGSTATUS; // error sheets get an icon too
                }
                var item = new sas.hc.ui.core.Item({text: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
                view.sheetSelect.addItem(item);
            });
            var liveModel = that.getView().getModel("live");
            var defSheetId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", project.getId());
            // HF..S1352166
            if (that.project.getSheet(defSheetId)===undefined){
                defSheetId=liveModel.getProperty("/previousSheetId")||filteredSheets[0].getId();
            }
            var sheetId=defSheetId||liveModel.getProperty("/previousSheetId")||filteredSheets[0].getId();
            that.getView().selectSheet(sheetId);
            def.resolve(that);
        }
        //Note: flipping the busy flag is in ScenarioEditor.controller's onAfterRendering
        return def;
    },
    onBreadCrumbClick: function (e) {
        var item = e.getParameter("item");
        var that = this;
        switch (item.getProperty("key")) {
            case "ProjectManager":
                var keys= _.keys(sas.stress.wkbench.model.Project._projects);
                for (var i=keys.length-1;i>=0;--i){
                    sas.stress.wkbench.model.Project.clearProject(keys[i]);
                }
                keys=[];
                sas.stress.wkbench.ViewManager.loadProjectManager();
                break;
            default:
                sas.stress.wkbench.ViewManager.loadHomePage(that.project);
                break;
        }
    },

    _getFilteredSheets: function () {
        // not sure if this goes into HF.. ref: Tom Goossens email
        var sheets = this.project.getSheets();
        if (this._processAllSheets===false){    // S1371893
	        var preferredCategory = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.category_key", this.project.getId());
	        if (preferredCategory && preferredCategory !== "-1") {
	            // HF..S1352166
	            // general purpose guard.. make sure the preferred category exists
	            var categoryFound=_.find(this.project.getSheetCategories(),function(category){
	                return category.getId()===preferredCategory;
	            });
	            if (categoryFound) {
	                // HF..S1352166
	                sheets = _.filter(sheets, function (sheet) { // need to sert by name
	                    return (sheet.getSheetCategory().getId() === preferredCategory);
	                });

	            }else{
	                // remove the preference
	                sas.stress.wkbench.UserPrefs.delete({"sas.stress.wkbench.category_key":undefined}, this.project.getId());
	            }
	        }
        }
        return _.sortBy(sheets, function(sheet){ // need to sert by name
            return sheet.getName().toLocaleLowerCase();
        });
    },
    onExit: function () {
        // we would need to destory all the scenario editors that have been created
        var tobeDestroyedScenarioEditors = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
        var length = _.size(tobeDestroyedScenarioEditors);
        _.each(tobeDestroyedScenarioEditors, function (scenarioeditorView, key) {
            // destroy everything related to the scenario editor
            scenarioeditorView.destroy();
        });
        if (this.project) {
            this.project.destroy();
            delete this.project;
        }
        this.router.detachExactRouteMatched("Workbench", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.wkbench.view.Workbench", null);
        this.navChecker.stop();
    },
    openSheetNew: function (e) {
        var that=this;
        var view = this.getView();
        var liveModel = view.getModel("live");
        var sheetId = view.getSelectedSheetId();
        var sheet = this.project.getSheet(sheetId);
        // if the sheet is marked as "errored" during load
        if (sheet.errored){ // S1266931
            this.closeBzDialog();

            // show a dialog that mentions the sheet has an issue
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            var json={message:rb.getText("workbench.error.invalid.sheets.fmt", sheet.getName()),details:sheet.getNoVisibilityReasons()};
            sas.stress.wkbench.widget.MessageDetailsBox.error(json, sas.config.appName);
            // revert the drop down to the previous selected sheet
            sheetId=liveModel.getProperty("/previousSheetId");
            if (sheetId) {
                var olditem = view.sheetSelect.getItemByKey(sheetId);
                //var oldindex = view.sheetSelect.indexOfItem(olditem);
                view.sheetSelect.setSelectedItem(olditem);
                /*view.sheetSelect.setSelectedIndex(oldindex);
                view.sheetSelect.setSelectedItemId(olditem.getId());*/
            }else {
                // we will need reload the sheet
                var iterationId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                sas.stress.wkbench.ViewManager.loadScenarioEditor(sheetId, view.getDefaultFilterId(sheetId), iterationId);
            }
            return;
        }

        sas.stress.wkbench.UserPrefs.save({"sas.stress.wkbench.sheet_key": sheetId},this. project.getId());
        this._refreshSheets();
    },
    _autoSelectFirstSheet: function () {
        var workbenchView = this.getView();
        this.openIfRequiredAndUpdateBzDialog(sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt"));
        var sheets = this._getFilteredSheets();
        if (sheets.length) {
            var defaultSheetId = workbenchView.getDefaultSheetId(sheets);
            var item = workbenchView.sheetSelect.getItemByKey(defaultSheetId);
            workbenchView.sheetSelect.setSelectedItemId(item.getId());
            workbenchView.sheetSelect.fireChange({selectedItem: item});//for some odd reason i had to manually fire it.
        } else {
            //sas.stress.wkbench.ViewManager.getWorkbench().setBusy(false);
            // close the busy dialog
            // fixes for CET defect S1266931
            this.closeBzDialog();
        }
    },
    navigateToHomePage: function () {
        sas.stress.wkbench.ViewManager.loadHomePage(this.project);
    },
    changeIteration: function () {
        var that=this;
        var view = this.getView();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var selectedProjectIterationNo = view.getSelectedIterationId();

        if (this.navChecker.getDirty()){
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            // show the warning dialog asking for refresh before if unsaved changes exist
            sas.hc.m.MessageBox.warning(rb.getText("project.sheet.filters.apply.question.txt"), rb.getText("project.sheet.filters.apply.title"),
                [sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL],
                null, function (oAction) {
                    if (oAction === sas.hc.m.MessageBox.Action.OK) {
                        that._performProjectIterationChange(selectedProjectIterationNo);
                    }else{
                        var iterationId=view.getModel("live").getProperty("/iterationNumber")||sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id",that.project.getId());
                        view.iterationSelect.setSelectedKey(iterationId.toString());
                    }
                });
        } else {
            that._performProjectIterationChange(selectedProjectIterationNo);
        }
    },
    _performProjectIterationChange:function(selectedProjectIterationNo) {   // fixes for S1319225
        var view = this.getView();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();

        view.getModel("live").setProperty("/iterationNumber", selectedProjectIterationNo);
        project.setCurrentIterationNo(selectedProjectIterationNo);
        this._handleProjectIterationChange("sas.stress.wkbench.project_iteration_id", selectedProjectIterationNo).done(function () {
            // nothing to do
        }).fail(function (jqxhr) {
            sas.stress.wkbench.widget.MessageDetailsBox.error(jqxhr, sas.config.appName);
        });
    },
    _handleProjectIterationChange:function(preferenceKey, preferenceVal){
        var deferred= $.Deferred();
        var view = this.getView();
        //view.setModel();    // clear the existing model

        this.openIfRequiredAndUpdateBzDialog();
        if (preferenceKey&&preferenceVal) {
            sas.stress.wkbench.UserPrefs.save(preferenceKey, preferenceVal, this.project.getId());
        }

        this.refreshProject(false).done(function(project){  // S1324503 regression of S1319225
            deferred.resolve();
        }).fail(function(jqxhr){
            deferred.reject(jqxhr);
        });

        return deferred;
    },
    _handleSheetCategoryChange:function(preferenceKey, preferenceVal){
        var view = this.getView();
        var filteredSheets = this._getFilteredSheets();
        var modelData=view.getModel().getData();

        this.openIfRequiredAndUpdateBzDialog();
        if (preferenceKey&&preferenceVal) {
            sas.stress.wkbench.UserPrefs.save(preferenceKey, preferenceVal, this.project.getId());
        }
        modelData.sheets=[];
        if (filteredSheets.length === 0) {
            this.closeBzDialog();

            // fixes for CET defect S1266931
            var messages=[];
            messages.push(rb.getText("workbench.error.no.templates.txt"));
            messages.push(rb.getText("workbench.error.no.permissions.txt"));
            messages.push(rb.getText("workbench.error.no.data.txt"));

            var json={message:rb.getText("workbench.error.nosheets.main.fmt", this.project.getName()),details:messages};
            sas.stress.wkbench.widget.MessageDetailsBox.error(json,sas.config.appName);

            return;
        }
        _.each(filteredSheets, function (sheet) {
            var icon=null;
            if(sheet.errored){ // S1266931
                icon=sas.icons.HC.WARNINGSTATUS; // error sheets get an icon too
            }
            modelData.sheets.push({name: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
            var item = new sas.hc.ui.core.Item({text: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
            view.sheetSelect.addItem(item);
        });
        view.getModel().setData(modelData);
        var liveModel = view.getModel("live");
        var defSheetId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", this.project.getId());
        // HF..S1352166
        if (this.project.getSheet(defSheetId)===undefined){
            defSheetId=liveModel.getProperty("/previousSheetId")||filteredSheets[0].getId();
        }
        var sheetId=defSheetId||liveModel.getProperty("/previousSheetId")||filteredSheets[0].getId();
        view.selectSheet(sheetId);
    },
    changeSheetCategory: function (e) {
        var view = this.getView();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var sheetCategoryKey = view.getSelectedSheetCategoryKey();
        var liveModel = view.getModel("live");
        // save the new selected sheet category
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.category_key", sheetCategoryKey, this.project.getId());

        var previousSheetId=liveModel.getProperty("/previousSheetId");
        var foundprevioussheet=_.find(this._getFilteredSheets(),function(sheet){
            if (previousSheetId===undefined){
                return false;
            }
            return (sheet.getId().toString()===previousSheetId.toString());
        });
        if (foundprevioussheet===undefined) {
            sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.sheet_key",this._getFilteredSheets()[0].getId(), project.getId());
            liveModel.setProperty("/previousSheetId", undefined);
        }
        view.sheetSelect.removeAllItems();
        this._handleSheetCategoryChange();
    },
    openCellCommentPopup: function (e) {
        var view = this.getView();
        var liveModel = view.getModel("live");
        var selected = liveModel.getProperty("/selected");
        var slimsheet = liveModel.getProperty("/slimsheet");
        // the li_key column removed therefore, the actual column index is required.. scenarios do not exist therefore, use filter
        var cell = sas.stress.wkbench.widget.Slimsheet.findCell(slimsheet, selected.row, slimsheet.getSlimsheetSource().getCalculatedColumn(selected.column/*+1*/, selected.filterId/*.scenarioId*/), false);
        if (selected && slimsheet && cell) {
            var existingPopup=sap.ui.getCore().byId("sas.stress.wkbench.view.CellCommentPopup");
            if (existingPopup){
                existingPopup.destroy();
            }
            var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(selected,true,false);
            var cellCommentPopup = new sas.hc.m.ResponsivePopover({
                id: "sas.stress.wkbench.view.CellCommentPopup",
                title: sas.stress.wkbench.StressUtil.getBundle().getText("cell.comment.dialog.title.txt"),
                verticalScrolling: false,
                placement: sap.m.PlacementType.Top,
                modal: true,
                initialFocus: cellCommentEditor.commentTextArea,
                content: cellCommentEditor,
                offsetY:10,
                afterClose: function () {
                    this.destroy();
                },
            });
            cellCommentPopup.addStyleClass("stressCommentPopover");
            if (this.project.doesRequireComments()) {
                cellCommentPopup.addStyleClass("stressRequiredField");
            }
            cellCommentEditor.attachClosing(function () {
                this.close();
            }, cellCommentPopup);
            cellCommentPopup.openBy(cell);
        }
    },
    pressLogButton: function () {
        var that = this;
        var logDialogView = sas.stress.wkbench.ViewManager.loadLogDialogView();
        var logDialog = new sas.hc.m.Dialog({
            title: sas.stress.wkbench.StressUtil.getBundle().getText("cell.log.dialog.window.title"),
            /*width:"1600px",*/
            contentWidth: "1600px",
            content: logDialogView,
            beginButton: logDialogView.discardbutton,
            endButton: logDialogView.endbutton,
            afterClose: function () {
                logDialog.destroy();
            },
            afterOpen: function () {
                // once the dialog opens, lets sort... this should help in aligning right along with the css
                // disable default sort for now - the 'special' sort is done by the midtier
                // logDialogView.oTable.sort(logDialogView.oTable.getColumns()[5], sap.ui.table.SortOrder.Descending);
            },
            verticalScrolling: false,
            initialFocus: logDialogView.endbutton
        });
        logDialogView.attachClosing(function () {
            logDialog.close();
        }, logDialog);
        logDialog.open();

        // open the busy dialog on top of the log dialog
        that.openIfRequiredAndUpdateBzDialog(sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt"));
    },
    closeBzDialog: function () {
        if(this.bzdlg) {
            this.bzdlg.close();
            this.bzdlg = undefined;
        }
    },
    openIfRequiredAndUpdateBzDialog: function(title){
        var updatedtext="";
        if (!this.bzdlg){
            if (!title) {
                this.bzdlg = new sas.hc.m.BusyDialog({text: sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt")});
            }else{
                updatedtext=title;// no busy dialog and title given use the title directly
                this.bzdlg = new sas.hc.m.BusyDialog({text: updatedtext});
            }
            this.bzdlg.open();
        } else{
            updatedtext=sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.sheet.message.fmt",title);
            this.bzdlg.setText(updatedtext);
        }
    },
    pressImportButton: function () {
        var that = this;
        var view = sas.stress.wkbench.ViewManager.loadWorksheetImport();
        var dialog = new sas.hc.m.Dialog({
            title: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.import.title.txt"),
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                this.destroy();
            },
            initialFocus: view.cancelButton
        });
        view.attachClosing(function (e) {
            this.destroy();
        }, dialog);
        view.attachUploading(function (e) {
            if (e.getParameter("action") === "START") {
                view.setBusy(true);
            } else {
                var response = e.getParameter("response");
                if (response !== "Done") {
                    sas.stress.wkbench.widget.MessageDetailsBox.error(response, sas.config.appName);
                    view.setBusy(false);
                } else {
                    that.getView().setModel();//clear out view model
                    that.openIfRequiredAndUpdateBzDialog();
                    that._processAllSheets=true;    // S1371893
                    that.refreshProject(false).done(function(){     // no need to call loadsheet data twice PErformance S1304571
                        view.setBusy(false);
                        dialog.close();
                        that.closeBzDialog();
                        // need to clear the source, else after import workbench does not come up
                        //that.cleanUpScenarioViewsSource();
                    }).fail(function(jqxhr){
                        that.closeBzDialog();
                        view.setBusy(false);
                        dialog.close();
                        //sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
                    });
                }
            }
        }, dialog);
        dialog.open();
    },
    pressExportButton: function () {
        var that=this;
        var view = this.getView();
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        var controller = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        // make sure the project is upto date..
        sas.stress.wkbench.StressAPI.verifyProjectStatusWithServer(this.project).done(function (valid) {
            if (valid === true) {
                that.openIfRequiredAndUpdateBzDialog(rb.getText("export.excel.busy.indicator.message.txt"));
                // the iteration
                var iterationNum = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", controller.project.getId()) || controller.project.getCurrentIterationNo();
                // download
                sas.stress.wkbench.StressAPI.exportToExcel(controller.project.getId(), iterationNum, controller.project.getFilters())
                    .done(function () {
                        view.setBusy(false);
                        that.closeBzDialog();
                    })
                    .fail(function (error) {
                        that.closeBzDialog();
                        view.setBusy(false);
                        sas.stress.wkbench.StressUtil.handleServerFailure(error);
                    });
            } else {
                // show the warning dialog asking for refresh before export
                sas.hc.m.MessageBox.warning(rb.getText("exportToExcel.project.refresh.question.txt"), rb.getText("exportToExcel.project.refresh.title"),
                    //[sas.hc.m.MessageBox.Action.YES, sas.hc.m.MessageBox.Action.NO, sas.hc.m.MessageBox.Action.CANCEL],
                    [sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL],
                    undefined, $.proxy(controller._buttonpressed, controller));
            }
        }).fail(function (jqxhr) {
            view.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },
    /*
     * Internal function called during export to excel operation if the project is out-of-sync with server
     * this is a $.proxy() method, therefore, it has access to the controller and its methods and properties
     */
    _buttonpressed: function (oAction) {
        var controller = this;
        var view = this.getView();
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        // user answered YES
        if (oAction === undefined) {
            jQuery.sap.log.error("Internal method invoked!!!");
        }
        if (oAction === sas.hc.m.MessageBox.Action.OK/*sas.hc.m.MessageBox.Action.YES*/) {
            // we need to refresh
            controller.refreshProject(false).done(function (project) {
                // then export to excel
                var iterationNum = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", controller.project.getId()) || controller.project.getCurrentIterationNo();
                sas.stress.wkbench.StressAPI.exportToExcel(controller.project.getId(), iterationNum, controller.project.getFilters())
                    .done(function () {
                        view.setBusy(false);
                        controller.closeBzDialog();
                        var rb = sas.stress.wkbench.StressUtil.getBundle();
                        // show the message toast
                        sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                    })
                    .fail(function (error) {
                        view.setBusy(false);
                        controller.closeBzDialog();
                        sas.hc.m.MessageBox.error(rb.getText("excel.generate.failed.message.txt"), rb.getText("error.window.title"));
                    });
            });
        } else {
            view.setBusy(false);
            controller.closeBzDialog();
            // show the message toast
            sas.hc.m.MessageBox.warning(rb.getText("exportToExcel.project.refresh.needed.txt"), rb.getText("exportToExcel.cancel.project.refresh.title"));
        }
    },
    pressRefreshButton: function (e, bSave) {
        var controller = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var project = this.project;
        var that=this;

        // does this project need to be refreshed? the user's un-saved changes would be lost
        sas.stress.wkbench.StressAPI.verifyProjectStatusWithServer(project).done(function (valid) {
            if (!valid) {
                controller.openIfRequiredAndUpdateBzDialog();   // fixes for S1266018
                that._processAllSheets=true;    // S1371893
                controller._loadFreshProject(project,false).done(function (freshProject) {
                    var uncommittedPayload = project.getUncommittedPayload();
                    var conflicts = controller._calculateConflicts(freshProject, project, uncommittedPayload);
                    if (conflicts.length) {
                        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
                        if (existingDialog) {
                            existingDialog.destroy();
                        }
                        var cellconflictDialog = sap.ui.jsfragment("sas.stress.wkbench.view.CellConflictDialog", controller);
                        var conflictModel = new sas.stress.wkbench.model.RiskJSONModel({
                            modelData: conflicts,
                            saveProject: false,
                            freshProject: freshProject,
                            project: project
                        });
                        cellconflictDialog.setModel(conflictModel);
                        cellconflictDialog.open();
                    } else {
                        freshProject.commit();//make this project the real one
                        controller.refresh(freshProject).done(function () {
                            controller.mergeProject(freshProject, project);
                            //sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), !!uncommittedPayload.length);
                            controller.navChecker.setDirty(!!uncommittedPayload.length);
                            var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                            liveModel.setProperty("/toggleButtons", null);
                            liveModel.setProperty("/toggleButtons", !!uncommittedPayload.length);
                            liveModel.checkUpdate(true);//make that cell editor refresh
                            sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                        }).fail(function () {
                            sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
                        });
                    }
                }).fail(function (jqxhr) {
                    //sas.hc.m.MessageBox.error(rb.getText("error.message.txt"), rb.getText("error.window.title"));
                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    this.closeBzDialog();
                });
            } else {
                sas.hc.m.MessageBox.information(rb.getText("cell.conflictDialog.no.server.changes.txt", rb.getText("cell.conflictDialog.no.server.changes.title.txt")));
                this.closeBzDialog();
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            this.closeBzDialog();
        });
    },
    refreshProject: function (bEnableSave) {
        var deferred = $.Deferred();
        var projectId = this.project.getId();
        var view=this.getView();
        var that = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();

        // reload the project and wait for the completion
        this._loadFreshProject(this.project,true).done(function (freshProject) {
            // the old project has been deleted, delete the slimsheet sources that contain references to the dead objects
            that.cleanUpScenarioViewsSource();
            //Update property in model.. we should lose the changes!!! no saving!!
            var project = sas.stress.wkbench.model.Project.getProject(projectId);
            var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
            if (!bEnableSave) { //if not defined remove the items else leave them be
                liveModel.setProperty("/selected", null);
                //liveModel.setProperty("/previousSheetId", undefined);
            }
            liveModel.checkUpdate(true);//make that cell editor refresh
            that.refresh(project, bEnableSave).done(function () {
                deferred.resolve();
            });
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            deferred.reject(jqxhr);
        });

        return deferred;
    },
    _refreshSheets:function(){
        var that=this;
        var view = this.getView();
        var liveModel=view.getModel("live");
        var reloadDuetoDiscard=liveModel.getProperty("/reloadDueDiscard");
        //view.setBusy(true);
        var sheetId = view.getSelectedSheetId();
        var sheet = this.project.getSheet(sheetId);
        var isDirty=reloadDuetoDiscard||this.navChecker.getDirty();
        var defs=[];

        var info;
        var filteredSheets;
        if (that._processAllSheets===false){    // S1371893
            filteredSheets=this._getFilteredSheets();    // fixes for S1276689
        }else{
            filteredSheets=this.project.getSheets();
        }
        // open the busy dialog
        if (this.bzdlg) {
            this.openIfRequiredAndUpdateBzDialog(sheet.getName());
        }else{
            // open the busy dialog
            this.openIfRequiredAndUpdateBzDialog();
        }
        _.each(filteredSheets, function (s) {
            info = s.getInfo();
            if (info&& !s.errored) {
                defs.push(info.getFilters().loadCellData(isDirty));
            }
        });
        if (defs.length) {
            $.when.apply($, defs).done(function () {
                var i;
                for (i = 0; i < filteredSheets.length; i++) {
                    var info = filteredSheets[i].getInfo();//.getSheetScenario(scenarioId);
                    if (info&&!filteredSheets[i].errored) {
                        info.getFilters().getData().processOverlays();
                    }
                }
                liveModel.setProperty("/previousSheetId", sheetId);
                liveModel.setProperty("/reloadDueDiscard",false);
                var iterationId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                sas.stress.wkbench.ViewManager.loadScenarioEditor(sheetId, view.getDefaultFilterId(sheetId), iterationId);
            });
        }else{
            liveModel.setProperty("/previousSheetId", sheetId);
            liveModel.setProperty("/reloadDueDiscard",false);
            var iterationId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
            sas.stress.wkbench.ViewManager.loadScenarioEditor(sheetId, view.getDefaultFilterId(sheetId), iterationId);
        }

        that._processAllSheets=false;   // S1371893
    },
    cellconflictDialog_mergePressed: function (e) {
        var wkbenchController = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
        var model = dialog.getModel();
        var bSave = model.getProperty("/saveProject");
        var freshProject = model.getProperty("/freshProject");
        var project = model.getProperty("/project");
        dialog.close();
        freshProject.commit();//make this project the real one
        if (bSave) {
            wkbenchController.mergeProject(freshProject, project);
            wkbenchController.saveProject(freshProject).done(function () {
                wkbenchController.refresh(freshProject).done(function () {
                    wkbenchController.navChecker.setDirty(false);
                    var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                    liveModel.setProperty("/toggleButtons", null);
                    liveModel.setProperty("/toggleButtons", false);
                    liveModel.checkUpdate(true);//make that cell editor refresh
                }).fail(function () {
                    sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
                });
            }).fail(function (e) {
                sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
            });
        } else {
            sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
            wkbenchController.refresh(freshProject).done(function () {
                wkbenchController.mergeProject(freshProject, project);
                wkbenchController.navChecker.setDirty(true);
                wkbenchController.hasUnsavedValues = false;
                var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                liveModel.setProperty("/toggleButtons", null);
                liveModel.setProperty("/toggleButtons", true);
                liveModel.checkUpdate(true);//make that cell editor refresh
            }).fail(function () {
                sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
            });
        }
    },
    cellconflictDialog_discardSelectedPressed: function (e) {
        var wkbenchController = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
        var model = dialog.getModel();
        var bSave = model.getProperty("/saveProject");
        var freshProject = model.getProperty("/freshProject");
        var project = model.getProperty("/project");
        dialog.close();

        var table = dialog.getCellConflictsTable();
        var selectedIndices = table.getSelectedIndices();
        var i, conflict, data;
        for (i = 0; i < selectedIndices.length; i++) {
            conflict = table.getContextByIndex(selectedIndices[i]).getObject();
            var othersheet=project.getSheet(conflict.sheetId);
            if(!othersheet.errored) {
                data = othersheet.getInfo().getFilters()/*.getScenario(conflict.scenarioId)*/.getData();
                delete data.ncFormulas[conflict.key];
            }
        }
        var numberKept = table.getRows().length - selectedIndices.length;
        freshProject.commit();//make this project the real one
        if (bSave) {
            wkbenchController.mergeProject(freshProject, project);
            wkbenchController.saveProject(freshProject).done(function () {
                wkbenchController.refresh(freshProject).done(function () {
                    wkbenchController.navChecker.setDirty(false);
                    var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                    liveModel.setProperty("/toggleButtons", null);
                    liveModel.setProperty("/toggleButtons", false);
                    liveModel.checkUpdate(true);//make that cell editor refresh
                }).fail(function () {
                    sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
                });
            }).fail(function (e) {
                sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
            });
        } else {
            var viewDirty = (!!wkbenchController.hasUnsavedValues || numberKept > 0);
            wkbenchController.refresh(freshProject).done(function () {
                wkbenchController.mergeProject(freshProject, project);
                wkbenchController.navChecker.setDirty(viewDirty);
                var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                liveModel.setProperty("/toggleButtons", null);
                liveModel.setProperty("/toggleButtons", viewDirty);
                liveModel.checkUpdate(true);//make that cell editor refresh
                sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
            }).fail(function () {
                sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
            });
        }
        wkbenchController.hasUnsavedValues = false;
    },
    cellconflictDialog_cancelPressed: function (e) {
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
        if (dialog) {
            dialog.close();
        }
    },
    mergeProject: function (freshProject, previousProject) {
        freshProject.mergeUncommittedOverlays(previousProject);//merge unsaved data from old project into fresh project
        previousProject.destroy();//destroy old project
        this.cleanUpScenarioViewsSource(); // fixes for S1274034
        this.project = freshProject;
    },
    /*
     * FIXES for S1274034
     */
    cleanUpScenarioViewsSource:function(){
        var tobeDestroyedScenarioEditors = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
        _.each(tobeDestroyedScenarioEditors, function (scenarioeditorView, key) {
            // destroy everything related to the scenario editor
            var source=scenarioeditorView.slimsheet.getSlimsheetSource();
            if (source) {
                source.destroy();
            }
        });
    },
    _loadFreshProject: function (project,deleteOldProject) {
        var that=this;
        var def = $.Deferred();
        var defs=[],info;

        sas.stress.wkbench.model.Project.loadProject(project.getId(), deleteOldProject).done(function (freshProject) {
            //Preload the same scenarios on this fresh project as the ones loaded on the project we tried to save
            freshProject.loadSheetsData().done(function(){
                var sheets=freshProject.getSheets();
                _.each(sheets, function (s) {
                    if (!s.errored){    // fixes for S1297759
                        info = s.getInfo();
                        defs.push(info.getFilters().loadCellData());
                    }
                });
                $.when.apply($, defs).done(function() {
                    var i;
                    _.each(sheets, function (s) {
                        if (!s.errored){    // fixes for S1297759
                            info = s.getInfo();
                            info.getFilters().getData().processOverlays();
                        }
                    });
                    def.resolve(freshProject);
                });
            }).fail(function(jqxhr) {
                def.reject(jqxhr);
            });
        }).fail(function(jqxhr){
            def.reject(jqxhr);
        });
        return def;
    },
    _calculateConflicts: function (freshProject, project, uncommittedPayload) {
        //Identify conflicts between local project and fresh project
        var i, j, sheet, filters/*scenario*/, overlay, oldValue, entry, data, freshValue, key, filtertokens = "";
        var columnId,rowId,sheetInfoData;
        var timestamp = project.getLastModified();
        var conflicts = [];
        var view = this.getView();
        for (i = 0; i < uncommittedPayload.length; i++) {
            entry = uncommittedPayload[i];
            sheet = freshProject.getSheet(entry.sheetId);
            if (!sheet.errored) {
                //scenario=sheet.getInfo().getScenario(entry.scenarioId);
                filters = sheet.getInfo().getFilters();
                data = filters.getData();
                sheetInfoData = sheet.getInfo().getData();
                for (j = 0; j < entry.values.length; j++) {
                    overlay = entry.values[j];
                    oldValue = overlay.value.value;
                    columnId = sheetInfoData.getColLongFromDatakey(overlay.dataKey);
                    rowId = sheetInfoData.getRowLongFromDatakey(overlay.dataKey);
                    key = "$" + columnId + "$" + rowId;
                    freshValue = data.cFormulas[key];
                    if (freshValue && freshValue.timestamp > timestamp) {
                        filtertokens = "[";
                        var tokens = view.tokenizer.getTokens();
                        _.each(tokens, function (token, idx) {
                            if (idx > 0) {
                                filtertokens += ", ";
                            }
                            filtertokens += "\"" + token.getText() + "\"";
                        });
                        filtertokens += "]";

                        var rowIndex = sheet.getInfo().getData().getRowIndexByLongUsingDatakey(overlay.dataKey);
                        var columnIndex = sheet.getInfo().getData().getColShortFromDatakey(overlay.dataKey);
                        var cellref = columnIndex + (rowIndex + 1).toLocaleString();
                        /*column: columnIndex.toLocaleString(),
                         row: (rowIndex + 1).toLocaleString()*/
                        /*overlay.rowId*/
                        conflicts.push({
                            sheetId: sheet.getId(),
                            sheet: sheet.getName(),
                            /*scenarioId:scenario.getId(),
                             scenario:scenario.getName(),*/
                            tokens: filtertokens,
                            cellReference: cellref,
                            user: freshValue.user,
                            timestamp: freshValue.timestamp,
                            localValue: oldValue,
                            serverValue: freshValue.txt,
                            key: key
                        });
                    } else {
                        // there is a value that's changed
                        this.hasUnsavedValues = true;
                    }
                }
            }
        }
        return conflicts;
    },
    saveProject: function (project) {
        project = project || this.project;
        var view = this.getView();
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var that = this;
        // fixes for : no need to send the entity
        return sas.stress.wkbench.StressAPI.saveProject(project).done(function () {
            that.navChecker.setDirty(false);
            that.hasUnsavedValues = false;
            var liveModel = view.getModel("live");
            liveModel.setProperty("/toggleButtons", false);
            var slimsheet = liveModel.getProperty("/slimsheet");
            var selected=liveModel.getProperty("/selected");    // current selected

            slimsheet.rerender();   // need to rerender to remove the drilldownables if any reset
            liveModel.setProperty("/selected", null);//so that the next line triggers its listeners and disables the drill-down button
            liveModel.setProperty("/selected", selected);

            sap.m.MessageToast.show(rb.getText("save.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
        }).fail(function (uncommittedPayload, jqXHR) {
            if (jqXHR !== undefined && jqXHR.status === 412) { 	// if the status === conflict (412) display the resolution dialog
                view.getModel("live").setProperty("/toggleButtons", true);
                var updatedByUser = "unknown";
                if (jqXHR !== undefined) {
                    var error = JSON.parse(jqXHR.responseText);
                    updatedByUser = error.details;
                }
                that.openIfRequiredAndUpdateBzDialog();
                that._loadFreshProject(project,false).done(function (freshProject) {
                    var conflicts = that._calculateConflicts(freshProject, project, uncommittedPayload);
                    var conflictDialogView = sas.stress.wkbench.ViewManager.loadConflictDialogView(project, freshProject, conflicts, updatedByUser);
                    var conflictDialog = new sas.hc.m.Dialog({
                        id: "ConflictProjectDialog",
                        title: sas.stress.wkbench.StressUtil.getBundle().getText("projectConflictDialog.title.txt"),
                        content: conflictDialogView,
                        beginButton: conflictDialogView.okButton,
                        endButton: conflictDialogView.cancelButton,
                        afterClose: function () {
                            this.destroy();
                        },
                        initialFocus: conflictDialogView.okButton
                    });
                    conflictDialogView.attachClosing(function () {
                        conflictDialog.close();
                    }, conflictDialog);
                    that.closeBzDialog();
                    conflictDialog.open();
                }).fail(function () {
                    that.closeBzDialog();
                    sas.hc.m.MessageBox.error(rb.getText("error.message"), rb.getText("error.title"));
                });
            } else {
                if (jqXHR) {
                    var responseJSON = JSON.parse(jqXHR.responseText);
                    sas.stress.wkbench.widget.MessageDetailsBox.error(responseJSON, sas.config.appName);
                }
            }
        });
    },
    pressDrillDownButton:function(e){
        var view=this.getView();
        var that=this;
        view.setBusy(true);
        var iterationNumber=view.getModel("live").getProperty("/iterationNumber");

        var rb=sas.stress.wkbench.StressUtil.getBundle();
        var liveModel=view.getModel("live");
        var cellRef=liveModel.getProperty("/selected");
        this.openIfRequiredAndUpdateBzDialog(rb.getText("drilldown.generate.busy.indicator.message.fmt",[cellRef.colShort+cellRef.rowShort]));

        var selectedSheetId=view.getSelectedSheetId();
        var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
        var scenario=cellRef.getSheetFilters();
        var dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);

        var data={sheetKey:selectedSheetId,dataKeys:[dataKey]};
        sas.stress.reports.StressAPI.drillDown(this.project.getId(),data,iterationNumber||this.project.getCurrentIterationNo())// fixes for S1302840, S1304459
            .done(function () {
                view.setBusy(false);
                that.closeBzDialog();
            })
            .fail(function (error) {
                that.closeBzDialog();
                view.setBusy(false);
                //sas.hc.m.MessageBox.error(rb.getText("drilldown.generate.failed.message.txt"), rb.getText("error.window.title"));
                sas.stress.wkbench.StressUtil.handleServerFailure(error);
            });
    }
});