jQuery.sap.declare("sas.stress.wkbench.widget.CommentDialog");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.TextArea");
jQuery.sap.require("sas.hc.ui.commons.layout.VerticalLayout");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sas.stress.wkbench.widget.CommentDialog = {};

sas.stress.wkbench.widget.CommentDialog.show = function (title, callBack, cancelCallBack, changes) {
    var rb = sas.stress.wkbench.StressUtil.getBundle();
    var layout = new sas.hc.ui.commons.layout.VerticalLayout({
        width: "100%"
    });

    var logModel;
    if (changes&&changes.totalRecordCount>0){   // show the tables if there are records to show
        var summaryTable= new sas.hc.ui.table.Table({
            id: "SummaryChangeTable",
            editable: false,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: false,
            visibleRowCountMode:sap.ui.table.VisibleRowCountMode.Fixed,
            minAutoRowCount: 1,
            visibleRowCount:1,
            title:rb.getText("task.summary.log.title.txt")
        });
        summaryTable.addStyleClass("stressTableHeaderPadding");
        summaryTable.addColumn(new sas.hc.ui.table.Column({
            id: "summarychange.totalcount",
            label: new sas.hc.m.Label({text: rb.getText("changelog.total.header.txt")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "totalRecordCount"),
            hAlign: sap.ui.core.HorizontalAlign.Right
        }));
        summaryTable.addColumn(new sas.hc.ui.table.Column({
            id: "summarychange.globalDataLoadRecordCount",
            label: new sas.hc.m.Label({text: rb.getText("changelog.global.dataload.txt")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "globalDataLoadRecordCount"),
            hAlign: sap.ui.core.HorizontalAlign.Right
        }));
        summaryTable.addColumn(new sas.hc.ui.table.Column({
            id: "summarychange.projectDataLoadRecordCount",
            label: new sas.hc.m.Label({text: rb.getText("changelog.project.dataload.txt")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "projectDataLoadRecordCount"),
            hAlign: sap.ui.core.HorizontalAlign.Right
        }));
        summaryTable.addColumn(new sas.hc.ui.table.Column({
            id: "summarychange.modelGroupRunLoadRecordCount",
            label: new sas.hc.m.Label({text: rb.getText("changelog.modelgroup.run.load.txt")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "modelGroupRunLoadRecordCount"),
            hAlign: sap.ui.core.HorizontalAlign.Right
        }));
        summaryTable.addColumn(new sas.hc.ui.table.Column({
            id: "summarychange.worksheetRecordCount",
            label: new sas.hc.m.Label({text: rb.getText("changelog.worksheet.overrides.txt")}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "worksheetRecordCount"),
            hAlign: sap.ui.core.HorizontalAlign.Right
        }));
        summaryTable.bindRows("/summaryChange"); //new sap.ui.model.Sorter("timestamp", true)

        var table = new sas.hc.ui.table.Table({
            id: "ChangeLogTable",
            editable: false,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: false,
            visibleRowCountMode:sap.ui.table.VisibleRowCountMode.Fixed,
            minAutoRowCount: 5,
            visibleRowCount:{
                path:"/changeLog",
                formatter:function(changes){
                    return changes&&changes.length?10:5;
                }
            },
            title:rb.getText("task.change.log.title.txt"),
            noData: rb.getText("task.change.log.no.changes.txt")
        });
        table.addStyleClass("stressTableHeaderPadding");

        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.cellReference",
            label: new sas.hc.m.Label({text: rb.getText("historyDialog.cell.reference.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "sourceText"),
            sortProperty: "sheetChanges/sourceText",
            filterOperator: sap.ui.model.FilterOperator.EQ,
            filterProperty: "sheetChanges/sourceText"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.user",
            label: new sas.hc.m.Label({text: rb.getText("historyDialog.user.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", "createdUser"),
            sortProperty: "createdUser",
            filterOperator: sap.ui.model.FilterOperator.EQ,
            filterProperty: "createdUser"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.scope",
            label: new sas.hc.m.Label({text: rb.getText("historyDialog.scope.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", {
                path: "scopeCode",
                formatter: function(scope){
                    var value;
                    switch(scope) {
                        case "global":
                            value= rb.getText("historyDialog.scope.global.column.value.txt");
                            break;
                        case "project":
                            value= rb.getText("historyDialog.scope.project.column.value.txt");
                            break;
                    }
                    return value;
                }
            }),
            sortProperty: "scopeCode",
            filterOperator: sap.ui.model.FilterOperator.EQ,
            filterProperty: {
                path: "scopeCode",
                formatter: function(scope){
                    var value;
                    switch(scope) {
                        case "global":
                            value= rb.getText("historyDialog.scope.global.column.value.txt");
                            break;
                        case "project":
                            value= rb.getText("historyDialog.scope.project.column.value.txt");
                            break;
                    }
                    return value;
                }
            }
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.source",
            label: new sas.hc.m.Label({text: rb.getText("historyDialog.source.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", {
                parts:["sourceCode", "propertyId"],
                formatter:function(code,propertyId){
                    var value;
                    switch(code){
                        case "calculatedField":
                            value = rb.getText("historyDialog.source.calculatedfield.column.value.txt");
                            break;
                        case "sheetOverride":
                            value = rb.getText("historyDialog.source.sheetoverride.column.value.txt");
                            break;
                        case "dataLoad":
                            value = rb.getText("historyDialog.source.dataload.column.value.txt");
                            break;
                        case "modelExecution":
                            value = rb.getText("historyDialog.source.modelExecution.column.value.txt");
                            break;
                        case "rollup":
                            value = rb.getText("historyDialog.source.rollup.column.value.txt");
                            break;
                        case "excelImport":
                            value = rb.getText("historyDialog.source.excelimport.column.value.txt");
                            break;

                        case "allocationOverride":
                            value=rb.getText("historyDialog.source.allocationOverride.column.value.txt");
                            break;
                    }
                    if (propertyId){
                        value=value+" ("+update.propertyId+")";
                    }
                    return value;
                }
            }),
            sortProperty: "textSource",
            filterOperator: sap.ui.model.FilterOperator.EQ,
            filterProperty: "sourceCode"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.timestamp",
            label: new sas.hc.m.Label({text: rb.getText("historyDialog.date.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", {
                path: "createdDateTime",
                formatter: function(dttm){
                    if (!dttm) return "";
                    var fmt = sap.ui.core.format.DateFormat.getDateTimeInstance();
                    return fmt.format(new Date(dttm),false);
                }
            }),
            sortProperty: "createdDateTime",
            width:"13rem"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.dataValue",
            label: new sas.hc.m.Label({text: rb.getText("changelog.cell.changed.value.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", {
                parts:["dataValue","typeCode"],
                formatter:function(value,type){
                    var unformattedValue=("comment"!==type)?value:null;
                    var formatter=new sas.stress.wkbench.model.Formatter("Numeric",2,1);
                    return unformattedValue?formatter.format(unformattedValue):"";
                }
            }).bindProperty("tooltip","dataValue"),
            sortProperty: "dataValue"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id: "changeLog.comment",
            label: new sas.hc.m.Label({text: rb.getText("historyDialog.comment.header.txt")}),
            template: new sas.hc.m.Text({wrapping:false}).bindProperty("text", {
                parts:["dataValue","typeCode"],
                formatter:function(value,type){
                    return ("comment"===type)?value:"";
                }
            }).bindProperty("tooltip","dataValue"),
            sortProperty: "dataValue"
        }));
        logModel = new sas.stress.wkbench.model.RiskJSONModel({
            changeLog: changes.sheetChanges,
            summaryChange:[{
                "globalDataLoadRecordCount": changes.globalDataLoadRecordCount,
                "modelGroupRunLoadRecordCount": changes.modelGroupRunLoadRecordCount,
                "projectDataLoadRecordCount": changes.projectDataLoadRecordCount,
                "worksheetRecordCount": changes.worksheetRecordCount,
                "totalRecordCount": changes.totalRecordCount
            }]
        });
        table.bindRows("/changeLog"); //new sap.ui.model.Sorter("timestamp", true)

        layout.addContent(summaryTable);
        layout.addContent(table);
    }

    var label = new sas.hc.m.Label({
        text: rb.getText("comment.dialog.comment.txt")
    });
    var comment = new sas.hc.m.TextArea({
        rows: 6,
        width: "100%",
        maxLength: 2048
    });
    layout.addContent(label);
    layout.addContent(comment);
    var dialog = new sas.hc.m.Dialog({
        /*maxHeight: 600,
         maxWidth: 600,*/
        title: title,
        content: layout,
        afterClose: function () {
            dialog.destroy();
        }
    });
    if (changes&&changes.totalRecordCount>0){   // set up the dialog only if there are records to show
        comment.setRows(2);

        /*dialog.setContentHeight("40rem");*/
        dialog.setContentWidth("80%");

        dialog.setModel(logModel);
    }else{
        dialog.setMaxHeight(600);
        dialog.setMaxWidth(600);
    }
    var actionButton = new sas.hc.m.Button({
        text: rb.getText("comment.dialog.ok.txt"),
        press: function () {
            callBack(comment.getValue());
            dialog.close();
        }
    });
    dialog.setMiddleButton(actionButton);
    var cancelButton = new sas.hc.m.Button({
        text: rb.getText("comment.dialog.cancel.txt"),
        press: function () {
            cancelCallBack && cancelCallBack();
            dialog.close();
        }
    });
    dialog.setRightButton(cancelButton);
    dialog.addStyleClass("stressCommentDialog");
    dialog.setInitialFocus(cancelButton);
    if(!changes) {
        dialog.open();
    }else{
        setTimeout(function(){
            dialog.open();
        },600)
    }
};
