/***********************************************************************
 * Copyright (c) 2017 by SAS Institute Inc., Cary, NC, USA.            *
 *                                                                     *
 * NAME:          ssrinit                                              *
 *                                                                     *
 * PURPOSE:       Initializes macro paths for SAS Supervisory Risk     *
 *                Reporting , reads the macro path                     *
 *                from a metadata property and appends it to           *
 *                SASAUTO                                              *
 *                                                                     *
 * USAGE:         %ssrinit(VERSION=)                                   *
 *                                                                     *
 * PARAMETERS:    VERSION     The product version number               *
 *                            (defaults to 5.4 in this release)        *
 *                                                                     *
 **********************************************************************/
%macro ssrinit(VERSION=2.3);
*options mprint mlogic mprintnest mlogicnest symbolgen fullstimer;
   %global PRODUCT;
   %global RD_PRODNAME;
   %global RD_DVD;
   %global DVD;
   %global RD_SWC;


   %*------------------------------------------------------------------;
   %* Identify the product (note: values are case sensitive)           ;
   %*------------------------------------------------------------------;

   %let DVD=ssr;
   %let PRODUCT=SSRSVRC;
   %let RD_PRODNAME=SAS Regulatory Reporting;
   %let RD_SWC=Regulatory Rpt Cfg &VERSION;

   %*------------------------------------------------------------------;
   %* Check whether RD_DVD has already been set so that this logic is  ;
   %* not done twice.  To force the initialization to run again, set   ;
   %* RD_DVD back to empty.                                            ;
   %*------------------------------------------------------------------;

   %if (%bquote(&RD_DVD) eq &DVD) %then %do;
      %return;
   %end;

   %let RD_DVD = &DVD;

   /*set path for deployment root directory*/
   %global DEPLOYMENT_ROOT;
   %let DEPLOYMENT_ROOT=%sysget(SASROOT);

   /*get the ucmacros location from metadata property */
   %global outvar1 outvar2;

      %if ((&SYSSCP eq WIN) or (&SYSSCP eq DNTHOST)) %then %do;
      %let outvar1 = %sysget(SASROOT)/ssrsvr/ucmacros;
   %end;
   %else %do;
      %let outvar1 = %sysget(SASROOT)/ucmacros/ssrsvr;
   %end;

   /*get the Application location in Config location from metadata property */
   %global app_config_path;
   data _null_;
      n = 1;
      length uri asn_uri propertyValue$ 500;
      length propertyName $32;
      uri="";
      asn_uri="";
      propertyValue="";
      propertyName="";
      nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
      numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
      numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

      do while(1);
         rc = metadata_getattr(uri, "name", propertyName);
         if (propertyName = "ssrsvrc.appdata.dir") then do;
            rc = metadata_getattr(uri, "DefaultValue", propertyValue);
            call symput("app_config_path", trim(propertyValue));
            leave;
         end;
         else do;
            n = n + 1;
            if (n > numasn) then leave;
            rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
         end;
      end;

   run;

   /*get the SSR Install location from metadata property */
   %global ssr_install_path;
   %let ssr_install_path = %sysget(SASROOT);

   /*get the SSR Landing Area Libname metadata property */
   %global ssr_la_libref;
   data _null_;
      n = 1;
      length uri asn_uri propertyValue$ 500;
      length propertyName $32;
      uri="";
      asn_uri="";
      propertyValue="";
      propertyName="";
      nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
      numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
      numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

      do while(1);
         rc = metadata_getattr(uri, "name", propertyName);
         if (propertyName = "ssrsvrc.libref.la") then do;
            rc = metadata_getattr(uri, "DefaultValue", propertyValue);
            call symput("ssr_la_libref", trim(propertyValue));
            leave;
         end;
         else do;
            n = n + 1;
            if (n > numasn) then leave;
            rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
         end;
      end;

   run;

   /*get the SSR sasmisc location from metadata property */
   %global ssr_sasmisc_path;

   %if ((&SYSSCP eq WIN) or (&SYSSCP eq DNTHOST)) %then %do;
      %let ssr_sasmisc_path = %sysget(SASROOT)/ssrsvr/sasmisc;
   %end;
   %else %do;
      %let ssr_sasmisc_path = %sysget(SASROOT)/misc/ssrsvr;
   %end;

   %*------------------------------------------------------------------;
   %* Handling Lock down                                               ;
   %*------------------------------------------------------------------;
   filename tmp_file temp;
   proc printto log=tmp_file new;
   run;
   libname testlib "&app_config_path";
   data testlib.test;
      a=1;
   run;
   proc printto;
   run;
   %put &syscc *;
    %if %eval(&syscc > 4 ) %then %do;
        %let ssr_err_key = 7;
      %put ERROR: %sysfunc(sasmsg(sashelp.ssrmsg, &ssr_err_key., NOQUOTE)) &app_config_path..;
        %goto exit_postinit;
    %end;
    %else %do;
        proc sql noprint;
         drop table testlib.test;
        quit;
    %end;


   %*------------------------------------------------------------------;
   %* Set up HOST_UCMACRO_PATH according to the naming convention on   ;
   %* the current host machine                                         ;
   %*------------------------------------------------------------------;


   %let AUTOLIST = %sysfunc(compress(%sysfunc(getoption(sasautos)),%str(%(%))));
   %global HOST_UCMACRO_PATH;


      %let host_ucmacro_path =  "&outvar1." ;


   %*------------------------------------------------------------------;
   %* Prepend the host_ucmacro path and remove any nested parentheses  ;
   %* and commas                                                       ;
   %*------------------------------------------------------------------;

     options sasautos = (&host_ucmacro_path. , &AUTOLIST);


   %*------------------------------------------------------------------;
   %* Locate the SYSTEM_OPTIONS and USER_JOB_CONFIGURATION             ;
   %* Convert it to SAS macro variables for run time                   ;
   %*------------------------------------------------------------------;

   %let SSR_CONF_PATH=&app_config_path./data/config;

   libname TST_CONF "&SSR_CONF_PATH."; /*i18NOK:LINE */

   %if %sysfunc(exist(TST_CONF.SYSTEM_OPTIONS)) %then %do;
      %ssr_create_params(config_dset=TST_CONF.SYSTEM_OPTIONS);
   %end;
   %if %sysfunc(exist(TST_CONF.USER_JOB_CONFIGURATION)) %then %do;
      %ssr_create_params(config_dset=TST_CONF.USER_JOB_CONFIGURATION);
   %end;

   libname TST_CONF clear;

   %exit_postinit:
   %let syscc=0;

   ;

   /*get the SSR DPM Library Type Oracle/ PostgreSQL */
   %global ssrdpm_dbtype;
   data _null_;
      n = 1;
      length uri asn_uri propertyValue$ 500;
      length propertyName $32;
      uri="";
      asn_uri="";
      propertyValue="";
      propertyName="";
      nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
      numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
      numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

      do while(1);
         rc = metadata_getattr(uri, "name", propertyName);
         if (propertyName = "ssrsvrc.database.type") then do;
            rc = metadata_getattr(uri, "DefaultValue", propertyValue);
            call symput("ssrdpm_dbtype", trim(propertyValue));
            leave;
         end;
         else do;
            n = n + 1;
            if (n > numasn) then leave;
            rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
         end;
      end;

   run;

   %if &ssrdpm_dbtype. eq SASRiskCommonDataServer %then
   %do;
   /*get the PostgreSQL SSR DPM Schema name*/
      %global ssrdpm_schema_name;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.ssrdmp.schema.name") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_schema_name", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the PostgreSQL Port name*/
      %global ssrdpm_port;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.database.port") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_port", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the PostgreSQL Host machine*/
      %global ssrdpm_host_name;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.database.host") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_host_name", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the PostgreSQL Host machine*/
      %global ssrdpm_authdomain;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.ssrdpm.authdomain") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_authdomain", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the PostgreSQL database name*/
      %global ssrdpm_db_name;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.database.name") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_db_name", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

   %end;

   %if &ssrdpm_dbtype. eq Oracle %then %do;

   /*get the Oracle SSR DPM Schema name*/
      %global ssrdpm_schema_name;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.ssrdmp.schema.name") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_schema_name", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the Oracle Port name*/
      %global ssrdpm_port;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.database.port") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_port", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the Oracle Host machine*/
      %global ssrdpm_host_name;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.database.host") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_host_name", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the Oracle Host machine*/
      %global ssrdpm_authdomain;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.ssrdpm.authdomain") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_authdomain", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

      /*get the Oracle Service name */
      %global ssrdpm_service_name;
      data _null_;
         n = 1;
         length uri asn_uri propertyValue$ 500;
         length propertyName $32;
         uri="";
         asn_uri="";
         propertyValue="";
         propertyName="";
         nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
         numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
         numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

         do while(1);
            rc = metadata_getattr(uri, "name", propertyName);
            if (propertyName = "ssrsvrc.ssrdpm.ora.service") then do;
               rc = metadata_getattr(uri, "DefaultValue", propertyValue);
               call symput("ssrdpm_service_name", trim(propertyValue));
               leave;
            end;
            else do;
               n = n + 1;
               if (n > numasn) then leave;
               rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
            end;
         end;

      run;

   %end;



%mend ssrinit;
