/***********************************************************************
 * Copyright (c) 2017 by SAS Institute Inc., Cary, NC, USA.            *
 *                                                                     *
 * NAME:          ssr_assign_libs                                      *
 *                                                                     *
 * PURPOSE:       Initializes macro paths for SAS Supervisory Risk     *
 *                Reporting , reads the macro path                     *
 *                from a metadata property and appends it to           *
 *                SASAUTO                                              *
 *                                                                     *
 * USAGE:         %ssr_assign_libs                                     *
 *                                                                     *
 * PARAMETERS:    None.                                                *
 *                                                                     *
 *                                                                     *
 **********************************************************************/

%macro ssr_assign_libs;

/* Assign Various Libraries by Supervisory Reporting */

/* i18nOK:BEGIN */

%local rc fileref; 
%let rc = %sysfunc(filename(fileref,&app_config_path./data/config)) ; 

%if %sysfunc(fexist(&fileref))  %then %do;
   libname ssr_conf "&app_config_path./data/config";
%end;

%if &ssrdpm_dbtype. eq SASRiskCommonDataServer %then %do;

   /* Create Libname for PostgreSQL */
   LIBNAME ssr_dpm POSTGRES  DBMAX_TEXT=10000  DATABASE=&ssrdpm_db_name. SERVER="&ssrdpm_host_name."  PORT=&ssrdpm_port.  SCHEMA="&ssrdpm_schema_name."  AUTHDOMAIN="&ssrdpm_authdomain." ;

%end;

%else %if &ssrdpm_dbtype. eq Oracle %then %do;

   /* Create Libname for Oracle */
   LIBNAME ssr_dpm ORACLE DBMAX_TEXT=10000 AUTHDOMAIN="&ssrdpm_authdomain." PATH=&ssrdpm_service_name. SCHEMA=&ssrdpm_schema_name.;

%end;

%local rc fileref num_of_cr; 
%let rc = %sysfunc(filename(fileref,&app_config_path./data/config)) ; 

%if %sysfunc(fexist(&fileref))  %then %do;
   libname ssr_tmp "&app_config_path./data/config";
   %if %sysfunc(exist(ssr_tmp.CONTENT_META)) %then %do;
      data _null_;
      set ssr_tmp.CONTENT_META;
         call execute('libname ' || LA_LIBREF || compress("'" || LA_LIB_PATH || "';")) ;
         call execute('libname ' || REG_DPM_LIBREF || compress("'" || REG_DPM_LIB_PATH || "';")) ;
         call execute('libname ' || REG_TMP_LIBREF || compress("'" || REG_TMP_LIB_PATH || "';")) ;
      run;
   %end;
   libname ssr_tmp;
%end;

/* i18nOK:END */

%mend ssr_assign_libs;
