/* Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA */

/*!
   \file
   \brief   Script to delete SSR DPM tables.

   \par Details This script is used to drop/delete SSR DPM tables.

            NOTE: &LIBREF. refers to the library which stores SSR DPM data.

   \ingroup ucmacros
   \author  SAS Institute Inc.
   \date    2017
*/

%let LIBREF=ssr_dpm;
%let NLDTTMFMT=NLDATM21.;
%let NLDTFMT=NLDATEM.;

%ssrinit;
%ssr_postinit;

%MACRO ssr_dpm_drop_postgres_table(AUTHDOMAIN=, USER=, DATABASE=, SERVER=, PORT=, SCHEMA=, postgopt=);
%let connstr=;
%IF (&SCHEMA > "" AND %INDEX(&SCHEMA, ".") = 0) %then %LET SCHEMA=%str(&SCHEMA).;
PROC SQL NOERRORSTOP;
%IF &DATABASE > "" %then %do;
   %let connstr = &connstr DATABASE=&DATABASE;
%end;
%IF "&SERVER" > "" %then %do;
   %let connstr = &connstr SERVER="&SERVER";
%end;
%IF &PORT > "" %then %do;
   %let connstr = &connstr PORT=&PORT;
%end;
%IF (&AUTHDOMAIN > "") %then %do;
   %let connstr=AUTHDOMAIN=&AUTHDOMAIN &connstr;
%end;
%else %do;
     %let connstr=DBPROMPT=YES USER=&USER &connstr;
      %end;
CONNECT TO POSTGRES (&connstr);
%if %length(&postgopt) > 0 %then
%do;
&postgopt ;
%end;

EXECUTE (DROP TABLE IF EXISTS dpm_column) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_context) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_context_x_dim_x_member) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_currency) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_data_point) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_data_point_shaded) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_dimension) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_domain) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_framework) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_member) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_metric) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_module) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_open_sheet_member) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_owner) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_row) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_sheet) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_taxonomy) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_template) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_template_table) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS dpm_validation) BY POSTGRES;
EXECUTE (DROP TABLE IF EXISTS multipart_report_data) BY POSTGRES;

DISCONNECT FROM POSTGRES;
QUIT;
%mend ssr_dpm_drop_postgres_table;

%macro ssr_dpm_drop_oracle_table(AUTHDOMAIN=, USER=, PATH=, SCHEMA=);
%let connstr=;
%IF (&SCHEMA > "" AND %INDEX(&SCHEMA, ".") = 0) %then %LET SCHEMA=%str(&SCHEMA).;
PROC SQL NOERRORSTOP;
%if (&PATH > "") %then %do;
   %let connstr = &connstr PATH=&PATH;
%end;
%IF (&AUTHDOMAIN > "") %then %do;
   %let connstr=AUTHDOMAIN=&AUTHDOMAIN &connstr;
%end;
%else %do;
      %let connstr=DBPROMPT=YES USER=&USER &connstr;
      %end;
CONNECT TO ORACLE (&connstr);
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_column%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_context%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_context_x_dim_x_member%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_currency%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_data_point%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_data_point_shaded%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_dimension%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_domain%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_framework%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_member%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_metric%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_module%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_open_sheet_member%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_owner%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_row%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_sheet%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_taxonomy%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_template%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_template_table%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..dpm_validation%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
EXECUTE(
  BEGIN
      EXECUTE IMMEDIATE  %nrquote(%'DROP TABLE &ssrdpm_schema_name..multipart_report_data%');
  EXCEPTION
      WHEN OTHERS THEN NULL;
  END;
) BY ORACLE;
DISCONNECT FROM ORACLE;
QUIT;
%mend ssr_dpm_drop_oracle_table;

%macro ssr_dpm_drop_table;
%if &ssrdpm_dbtype. eq SASRiskCommonDataServer %then %do;
   %ssr_dpm_drop_postgres_table(AUTHDOMAIN=&ssrdpm_authdomain., DATABASE=&ssrdpm_db_name., SERVER=&ssrdpm_host_name., PORT=&ssrdpm_port., SCHEMA=&ssrdpm_schema_name., postgopt=);
%end;
%else %if &ssrdpm_dbtype. eq Oracle %then %do;
   %ssr_dpm_drop_oracle_table(AUTHDOMAIN=&ssrdpm_authdomain., PATH=&ssrdpm_service_name., SCHEMA=&ssrdpm_schema_name.);
%end;
%mend ssr_dpm_drop_table;
%ssr_dpm_drop_table;
