/***********************************************************************************************************************************************
*  NAME: ssr_edit_del_filing_pkg.sas
*
*  PURPOSE: Macro to edit/delete filing package as per input parameter provided
*
*  PARAMETERS: 
*     The following macro variables (filling package attributes) are passed by the UI:
*               ACTION_SELECTED: To select task to perform(Accepted values are "Delete"  or "Edit")
*               PKG_TOBE_DELTED: ID of package to be deleted, if ACTION_SELECTED="Delete Package"
*               PKG_TOBE_EDITED: ID of package to be edited, if ACTION_SELECTED="Edit Package"
*               NEW_PKG_NM     : New package name if package name to be edited
*               NEW_PKG_DESC   : New package description if package description to be edited
*
*  USAGE: 
* 1. To edit pacakge example PKG1:  %ssr_edit_del_filing_pkg (ACTION_SELECTED  =Edit
*                                          ,PKG_TOBE_EDITED  =PKG1
*                                          ,NEW_PKG_NM       =PKG6
*                                          ,NEW_PKG_DESC     =New description
*                                          ); 
* 2. To delete pacakge example PKG3:  %ssr_edit_del_filing_pkg (ACTION_SELECTED  =Delete
*                                                  ,PKG_TOBE_DELTED  = PKG3                                          
*                                                  );   
*
*  NOTES:                                                              
*
* Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA 
********************************************************************************************************************************************************/
%macro ssr_edit_del_filing_pkg (ACTION_SELECTED  =
                        ,PKG_TOBE_DELTED =
                        ,PKG_TOBE_EDITED =
                        ,NEW_PKG_NM      =
                        ,NEW_PKG_DESC    =
                        );
/**********************************************************************************************************************************************************/
/*Preprocessing                                                                                                                                           */
/**********************************************************************************************************************************************************/
%local NO_UPDATE_FLG 
   PKG_NM_UPDATE_FLG
   DEL_PKG_NM
   DEL_PKG_SK 
   DEL_PKG_VER_NO
   PKGNM_TOBE_EDITED
   EDIT_PKG_SK
   EDIT_PKG_VER_NO
   SAME_NM_PKG_CNT
   FILING_ENTITY_NM 
   FILING_RPTNG_DT 
   FILING_TAXNOMY
   FILING_MODULE
   FILING_CURRENCY
;

%let STP_NEW_PKG_NM=%nrquote(&NEW_PKG_NM);
%let STP_NEW_PKG_DESC=%nrquote(&NEW_PKG_DESC);
%let STP_PKG_TOBE_DELTED=%nrquote(&PKG_TOBE_DELTED);
%let STP_PKG_TOBE_EDITED=%nrquote(&PKG_TOBE_EDITED);

/************************************************************************************************************************************************************/
/*Package selected to mark as delete by user                                                                                                                */
/************************************************************************************************************************************************************/
%if "&ACTION_SELECTED."="Delete" %then /*i18NOK:LINE */
%do;
   /*Collecting attributes for package to be deleted*/
   data _null_;
      set &SSR_SDM_LIB..&FILING_PKG_DS_NM.(where=( kstrip(kupcase(FILING_PACKAGE_TXT))=kstrip(%kupcase("&STP_PKG_TOBE_DELTED.")) and
                                        kstrip(kupcase(DELETE_FLG))="N" /*i18NOK:LINE */
                                       )
                                 );
         /* i18nOK:BEGIN */
         call symputx('DEL_PKG_NM',FILING_PACKAGE_NM,L);
         call symputx('DEL_PKG_SK',FILING_PACKAGE_SK,L);
         call symputx('DEL_PKG_VER_NO',FILING_PACKAGE_VER_NO,L);
         call symputx('FILING_ENTITY_NM',LEGAL_ENTITY_ID,L);
         call symputx('FILING_RPTNG_DT',REPORT_AS_OF_DT,L);
         call symputx('FILING_TAXNOMY',TAXONOMY_CD,L);
         call symputx('FILING_MODULE',MODULE_CD,L);
         call symputx('FILING_CURRENCY',CURRENCY_CD,L);
       /* i18nOK:END */
   run;

   %let STP_DEL_PKG_NM=%nrquote(&DEL_PKG_NM);
   /*Marking package For delete*/
   proc sql;
   update SSR_SDM.&FILING_PKG_DS_NM.
     set DELETE_FLG="Y" /*i18NOK:LINE */
     where kstrip(kupcase(FILING_PACKAGE_TXT))=kstrip(%kupcase("&STP_PKG_TOBE_DELTED")) and kstrip(kupcase(DELETE_FLG))="N";/*i18NOK:LINE */
   quit;

   /*Updating package history*/
   data work.&FILING_PKG_HIST_DS_NM.;
    attrib
         FILING_PACKAGE_SK length=8.
         FILING_PACKAGE_VER_NO length=8.
         FILING_PACKAGE_TXT length=$500.
         FILING_PACKAGE_NM length=$40.
         UPDATE_TYPE_CD length=$10.
         UPDATE_DESC length=$100.
         PKG_UPDATE_DTTM format = NLDATM21.
         ;
       FILING_PACKAGE_SK=&DEL_PKG_SK.;
       FILING_PACKAGE_VER_NO=&DEL_PKG_VER_NO.;
       FILING_PACKAGE_NM=kstrip("&STP_DEL_PKG_NM");
       FILING_PACKAGE_TXT=kstrip("&STP_PKG_TOBE_DELTED");
       UPDATE_TYPE_CD=kstrip(%kupcase("&ACTION_SELECTED"));
       UPDATE_DESC="Marked for deletion";/*i18NOK:LINE */
       PKG_UPDATE_DTTM=int(%sysfunc(datetime()));
       output;
   run;

   /*Append details for &FILING_PKG_HIST_DS_NM into &ssr_sdm_lib.*/
   proc append base=&SSR_SDM_LIB..&FILING_PKG_HIST_DS_NM. data=work.&FILING_PKG_HIST_DS_NM. ;
   run;
%end;

/*****************************************************************************************************************************************************************/
/*Package selected to edit package name and description                                                                                                           */
/*****************************************************************************************************************************************************************/
%if "&ACTION_SELECTED."="Edit" %then /*i18NOK:LINE */
%do;
   /*Finding if user entered new name package is already exist for given package parameters */
   data _null_;
   set &SSR_SDM_LIB..&FILING_PKG_DS_NM.(where=( kstrip(kupcase(FILING_PACKAGE_TXT))=kstrip(%kupcase("&STP_PKG_TOBE_EDITED.")) and
                                       kstrip(kupcase(DELETE_FLG))="N" /*i18NOK:LINE */
                                     )
                                );
      /* i18nOK:BEGIN */
      call symputx('PKGNM_TOBE_EDITED',FILING_PACKAGE_NM,L);
      call symputx('EDIT_PKG_SK',FILING_PACKAGE_SK,L);
      call symputx('EDIT_PKG_VER_NO',FILING_PACKAGE_VER_NO,L);
      call symputx('FILING_ENTITY_NM',LEGAL_ENTITY_ID,L);
      call symputx('FILING_RPTNG_DT',REPORT_AS_OF_DT,L);
      call symputx('FILING_TAXNOMY',TAXONOMY_CD,L);
      call symputx('FILING_MODULE',MODULE_CD,L);
      call symputx('FILING_CURRENCY',CURRENCY_CD,L);
     /* i18nOK:BEGIN */
   run;

   proc sql noprint;
      select count(*) into: SAME_NM_PKG_CNT /*i18NOK:LINE */
      from 
         &SSR_SDM_LIB..&FILING_PKG_DS_NM.(where=(  kstrip(LEGAL_ENTITY_ID)=kstrip("&FILING_ENTITY_NM.") and 
                                 kstrip(TAXONOMY_CD)=kstrip("&FILING_TAXNOMY.") and 
                                 REPORT_AS_OF_DT=&FILING_RPTNG_DT.                        )
                               ) 
      where 
         kstrip(kupcase(FILING_PACKAGE_NM))=kstrip(%kupcase("&STP_NEW_PKG_NM.")) and kstrip(kupcase(DELETE_FLG))="N";
   quit;
   
   %let STP_PKGNM_TOBE_EDITED=%nrquote(&PKGNM_TOBE_EDITED);
   /*Editing package*/
   proc sql;
      update
         &SSR_SDM_LIB..&FILING_PKG_DS_NM.
         set %if &SAME_NM_PKG_CNT.=0 and "&STP_NEW_PKG_NM." ne "" %then 
              %do;
              FILING_PACKAGE_NM=kstrip("&STP_NEW_PKG_NM."),
              FILING_PACKAGE_TXT=tranwrd(FILING_PACKAGE_TXT, kstrip(%kupcase("&STP_PKGNM_TOBE_EDITED.")), kstrip(%kupcase("&STP_NEW_PKG_NM.")) ),
              %end;
            FILING_PACKAGE_DESC=COALESCEC(kstrip("&STP_NEW_PKG_DESC."),FILING_PACKAGE_DESC )
         where 
            kstrip(kupcase(FILING_PACKAGE_TXT))=kstrip(%kupcase("&STP_PKG_TOBE_EDITED")) and kstrip(kupcase(DELETE_FLG))="N" ;
   quit;

   %if ( &SAME_NM_PKG_CNT. ne 0 or "&STP_NEW_PKG_NM."="") and "&STP_NEW_PKG_DESC."="" %then 
      /*Creating variable when no updates done to use in webout*/
      %do;
         data _null_;
         call symputx('NO_UPDATE_FLG','Y',L);
         run;
      %end;

   %else 
      %do;
      %if ( &SAME_NM_PKG_CNT. ne 0 and "&STP_NEW_PKG_NM." ne "") %then
       /*Creating variable when package name not updated but description updated to use in webout*/
         %do; 
           data _null_;
           call symputx('PKG_NM_UPDATE_FLG','N',L);
           run;
         %end;

         /*Updating package history*/
         data work.&FILING_PKG_HIST_DS_NM.;
            attrib
               FILING_PACKAGE_SK length=8.
               FILING_PACKAGE_VER_NO length=8.
               FILING_PACKAGE_TXT length=$500.
               FILING_PACKAGE_NM length=$40.
               UPDATE_TYPE_CD length=$10.
               UPDATE_DESC length=$100.
               PKG_UPDATE_DTTM format = NLDATM21.
               ;
            FILING_PACKAGE_SK=&EDIT_PKG_SK.;
            FILING_PACKAGE_VER_NO=&EDIT_PKG_VER_NO.;
            FILING_PACKAGE_NM=kstrip("&STP_PKGNM_TOBE_EDITED");
            FILING_PACKAGE_TXT=kstrip("&STP_PKG_TOBE_EDITED");
            UPDATE_TYPE_CD=kstrip(%kupcase("&ACTION_SELECTED"));
            UPDATE_DESC=%if &SAME_NM_PKG_CNT.=0 and "&STP_NEW_PKG_NM." ne ""
                         %then %do; "Package name edited as '&STP_NEW_PKG_NM'. " || %end; 
                       %if "&STP_NEW_PKG_DESC." ne "" 
                          %then %do; "Package description updated to '&STP_NEW_PKG_DESC'." %end; %else "";;
            PKG_UPDATE_DTTM=int(%sysfunc(datetime()));
            output;
         run;

         /*Append details for &FILING_PKG_HIST_DS_NM into &ssr_sdm_lib.*/
         proc append base=&SSR_SDM_LIB..&FILING_PKG_HIST_DS_NM. data=work.&FILING_PKG_HIST_DS_NM. ;
         run;

      %end;

%end;

/*Changing format for date value for printing purpose*/
data _null_;
 call symputx('FILING_RPTNG_DT',put(&FILING_RPTNG_DT,NLDATE.),L);
run;

%if &WEBOUT_IND. = 1 %then
   %do;
     /*Message for user by webouts*/
     %if "&NO_UPDATE_FLG."="Y" %then
       %do;
         data _null_;
         file _webout;
       /* i18nOK:BEGIN */
         put "<!DOCTYPE html>";
         put "<html>";
         put "<body>";
         put "<br>";
         put '<font color="red">';
         put "<h1>%sysfunc(sasmsg(sashelp.ssrmsg, 33, NOQUOTE))</h1>";
         put "<br>";
         put "<br>";
            col1='window.open("http://'||"&_SRVNAME"||'/SASStoredProcess/do?_action=form,properties,execute,newwindow&_program='||"&spk_meta_loc_im"||'");';
               col2='window.open("http://'||"&_SRVNAME"||'/SASStoredProcess/do?_action=form,properties,execute,newwindow&_program='||"&spk_meta_loc_ct"||'");';
               put '<script type="text/javascript">';
               put "function DivertJS(){";
               put col1;
               put " }";
               put "function DivertJS1(){";
               put col2;
               put " }";
               put " </script>";
               put "<button onclick='DivertJS()'>%sysfunc(sasmsg(sashelp.ssrmsg, 16, NOQUOTE))</button>";
               put "<br>";
               put "<br>";
               put "<button onclick='DivertJS1()'>%sysfunc(sasmsg(sashelp.ssrmsg, 29, NOQUOTE))</button>";
               put "<br>";
         put "</body>";
         put "</html>";
         run; 
       %end;
     %else
       %do;
         data _null_;
         file _webout;
            put "<!DOCTYPE html>";
            put "<html>";
            put "<body>";
            put "<br>";
            put '<font color="blue">';
            %if "&PKG_NM_UPDATE_FLG."="N" %then
              %do;
                  put "<h1>%sysfunc(sasmsg(sashelp.ssrmsg, 34, NOQUOTE))</h1>";
              %end;
            %else 
              %do;
                  put "<br>";
                  put '<font color="black">';
                  %if "&ACTION_SELECTED."="Delete" %then
                     %do;
                        put "<p>%sysfunc(sasmsg(sashelp.ssrmsg, 35, NOQUOTE))<b> %trim(&STP_DEL_PKG_NM)</b> %sysfunc(sasmsg(sashelp.ssrmsg, 30, NOQUOTE))";
                     %end;
                  %if "&ACTION_SELECTED."="Edit" %then
                     %do;
                        put "<p>%sysfunc(sasmsg(sashelp.ssrmsg, 35, NOQUOTE)) :<b> %trim(&STP_PKGNM_TOBE_EDITED)</b> %sysfunc(sasmsg(sashelp.ssrmsg, 30, NOQUOTE))";
                     %end;
                  put "<br>";
                  put "%sysfunc(sasmsg(sashelp.ssrmsg, 9, NOQUOTE))= &FILING_ENTITY_NM<br>";
                  put "%sysfunc(sasmsg(sashelp.ssrmsg, 10, NOQUOTE))= &FILING_CURRENCY<br>";
                  put "%sysfunc(sasmsg(sashelp.ssrmsg, 11, NOQUOTE))=&FILING_RPTNG_DT<br>";
                  put "%sysfunc(sasmsg(sashelp.ssrmsg, 12, NOQUOTE)) = &FILING_TAXNOMY<br>";
                  put "%sysfunc(sasmsg(sashelp.ssrmsg, 13, NOQUOTE)) = &FILING_MODULE<br>";
                  put "<br>";
                  put '<font color="blue">';
                  %if "&ACTION_SELECTED."="Delete" %then
                     %do;
                       put "<b>%sysfunc(sasmsg(sashelp.ssrmsg, 36, NOQUOTE))</b> <br> </font>";
                     %end;
                  %if "&ACTION_SELECTED."="Edit" %then
                     %do;
                       put "<b>%sysfunc(sasmsg(sashelp.ssrmsg, 37, NOQUOTE))</b> <br> </font>";
                     %end;
                   put "<br>";
              %end;
            put "<br>";
            col1='window.open("http://'||"&_SRVNAME"||'/SASStoredProcess/do?_action=form,properties,execute,newwindow&_program='||"&spk_meta_loc_im"||'");';
               col2='window.open("http://'||"&_SRVNAME"||'/SASStoredProcess/do?_action=form,properties,execute,newwindow&_program='||"&spk_meta_loc_ct"||'");';
               put '<script type="text/javascript">';
               put "function DivertJS(){";
               put col1;
               put " }";
               put "function DivertJS1(){";
               put col2;
               put " }";
               put " </script>";
               put "<button onclick='DivertJS()'>%sysfunc(sasmsg(sashelp.ssrmsg, 16, NOQUOTE))</button>";
               put "<br>";
               put "<br>";
               put "<button onclick='DivertJS1()'>%sysfunc(sasmsg(sashelp.ssrmsg, 29, NOQUOTE))</button>";
               put "<br>";
            put "</body>";
            put "</html>";
         /* i18nOK:END */
         run;
         /*Print last record of package history to know user what information got updated*/
         %stpbegin;
            proc sql noprint;
              select count(*) into :nobs /*i18NOK:LINE */
              from &SSR_SDM_LIB..&FILING_PKG_HIST_DS_NM.;
            quit;

            title "%sysfunc(sasmsg(sashelp.ssrmsg, 38, NOQUOTE))"; /*i18NOK:LINE */
            proc print data=&SSR_SDM_LIB..&FILING_PKG_HIST_DS_NM.(firstobs=&nobs); 
              var FILING_PACKAGE_NM UPDATE_TYPE_CD UPDATE_DESC PKG_UPDATE_DTTM;
            run;
            title '';
         %stpend;
       %end;
   %end;

%else

   %do;
     proc print data=&SSR_SDM_LIB..&FILING_PKG_HIST_DS_NM.; 
       var FILING_PACKAGE_NM UPDATE_TYPE_CD UPDATE_DESC PKG_UPDATE_DTTM;
     run;
   %end;

%mend;
