/*****************************************************************************************************************************************
*  NAME: ssr_gen_datasufficncy_frm_la.sas
*
*  PURPOSE: Macro to import landing area data tagged with DPM attributes and generate data sufficiency reports 
*
*  PARAMETERS: 

*               MODULE_ID       : DPM module_id for which data suffiency report is to be generated
*                                 (one of module_id values from SSR_DPM.DPM_MODULE)
*               REPORTING_DT    : report_as_of_dt value from landing area data
*               CURRENCY_CD     : currency_cd value from landing area data
*               LEGAL_ENTITY_ID : legal_entity_id value from landing area data
*               DPM_LIBREF      : libname for dpm default value is ssr_dpm(optional)
*               LA_DS_PATH      : Landing area data location path
*               LA_DS_NM        : Landing area dataset name
*               TMPLT_LEV_GARPH_FLG:  Flag to indicate whether to generate template level graphical report of data availability
*               HTML_LOC_PATH:        Location path at which data sufficiency report to ge generated in HTML format(e.g.C:/report/misc )
*               HTML_FILE_NM:         File name of for html report to be generated (file name to be provided without extention, optional)
*
*  USAGE:  %ssr_gen_datasufficncy_frm_la(  MODULE_ID=31
*                                         ,REPORTING_DT=31DEC2015
*                                         ,CURRENCY_CD=EUR
*                                         ,LEGAL_ENTITY_ID=5493001XCEF2PSOAOK24
*                                         ,DPM_LIBREF=ssr_dpm
*                                         ,LA_DS_NM=Eba_finrep_gaap1
*                                         ,TMPLT_LEV_GARPH_FLG =NO
*                                         ,HTML_LOC_PATH= C:/report/misc
*                                         ,HTML_FILE_NM=DATA_SUFFICNCY
*                                          ); 
*  NOTES:RFW_PROJECT_ID,RFW_ENTITY will be made lowercsae while inserting in path to avoid issue with OS other than window                                                              
*
* Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA 
**********************************************************************************************************************************************/

%macro ssr_gen_datasufficncy_frm_la(MODULE_ID=
                                    ,REPORTING_DT=
                                    ,CURRENCY_CD=
                                    ,LEGAL_ENTITY_ID=
                                    ,DPM_LIBREF=SSR_DPM
                           ,LA_DS_PATH=
                                    ,LA_DS_NM=
                                    ,TMPLT_LEV_GARPH_FLG =NO
                                    ,HTML_LOC_PATH=
                                    ,HTML_FILE_NM=DATA_SUFFICNCY
                                    );
/**********************************************************************************************************************************************/
/*Pre-processing: compiling macros and assiging required libnames*/
/**********************************************************************************************************************************************/
%ssrinit;
%ssr_postinit;


%LOCAL rc file_ref dir_exist_flg;
   %let rc = %sysfunc(filename(file_ref,&LA_DS_PATH.));

   %if %sysfunc(fexist(&file_ref))  %then %let dir_exist_flg=1;    
      %else %let dir_exist_flg=0; /*i18NOK:LINE */  

%if &dir_exist_flg.=1 %then /*i18NOK:LINE */  
%do;
    LIBNAME TEMP_LA "&LA_DS_PATH.";
    %let LA_LIBREF_VAR=TEMP_LA;
    
   /**********************************************************************************************************************************************/
   /*Step1:Import data from la and dpm for given attributes*/
   /**********************************************************************************************************************************************/

   %ssr_prep_ds_for_datasufficny(
                                      MODULE_ID=&MODULE_ID.
                                      ,REPORTING_DT=&REPORTING_DT.
                                      ,CURRENCY_CD=&CURRENCY_CD.
                                      ,LEGAL_ENTITY_ID=&LEGAL_ENTITY_ID.
                                      ,DPM_LIBREF=&DPM_LIBREF.
                                      ,LA_LIBREF=&LA_LIBREF_VAR.
                                      ,LA_DS_NM=&LA_DS_NM.
                                      ,out_ds1=dsrep_ip_data
                                      ,out_ds2=dsrep_attr
                                      );

   /**********************************************************************************************************************************************/
   /*Step2:Generate data sufficiency html from data imported in step1*/
   /**********************************************************************************************************************************************/

   %ssr_prep_rpt_for_datasufficny(   
                                IN_PACKAGE_DATA=dsrep_ip_data
                                ,IN_REPTG_ATTRIB_DATA=dsrep_attr
                                ,TMPLT_LEV_GARPH_FLG =&TMPLT_LEV_GARPH_FLG.
                                ,HTML_LOC_PATH=&HTML_LOC_PATH.
                                ,HTML_FILE_NM=&HTML_FILE_NM..html
                                );

%end;
%else 
   %do;
      %put "%sysfunc(sasmsg(sashelp.ssrmsg, 39, NOQUOTE)), &LA_DS_PATH. %sysfunc(sasmsg(sashelp.ssrmsg, 41, NOQUOTE))";/*i18NOK:LINE */
   %end;

%mend ssr_gen_datasufficncy_frm_la;
