/*****************************************************************************************************************************************
*  NAME: ssr_gen_datasufficncy_rpt.sas
*
*  PURPOSE: Macro to generate data sufficiency reports for selected filing package
*
*  PARAMETERS: 
*     The following macro variables are passed by the UI:
*               PKG_SELECTED: Filing package for which data sufficiency report to be generated
*               TMPLT_LEV_GARPH_FLG: Flag to indicate whether to generate template level graphical report of data availability
*
*  USAGE: 
*  %ssr_gen_datasufficncy_rpt (PKG_SELECTED         =FILING_PACKAGE_TXT
*                             ,TMPLT_LEV_GARPH_FLG  =N
*                             );
*  NOTES:                                                              
*
* Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA 
**********************************************************************************************************************************************/

%macro ssr_gen_datasufficncy_rpt( PKG_SELECTED        =
                                 ,TMPLT_LEV_GARPH_FLG =
                                );
/********************************************************************************************************************************************/
/*Preprocessing                                                                                                                             */
/********************************************************************************************************************************************/
%local  COLR_CATGRY1
      COLR_CATGRY2
      COLR_CATGRY3
      COLR_CATGRY4
      FILING_PKG_NM
      FILING_ENTITY_NM
      FILING_RPTNG_DT
      FILING_TAXNOMY
      FILING_MODULE
      FILING_CURRENCY
      NO_OBS
      DATAVAIL_CATG_CNT
      CAT_CD_EXIST
      TOTAL_TMPLT
      PKG_AVAIL_DP_PCT
      DS_NM
      NUMOBS
      NO_OF_PARTS
      PANEL_COLMNS
      STPERR_PKGS
      STPERR_DATA
;

%let COLR_CATGRY1=Darkorange;
%let COLR_CATGRY2=Yellow;
%let COLR_CATGRY3=Dodgerblue; 
%let COLR_CATGRY4=Limegreen;
/*Mask special characters(if any) from user prompts(comes in form of global macro variable) which comes from stp */
%let STP_PKG_SELECTED=%nrquote(&PKG_SELECTED);

/*If input dataset &ssr_sdm_libname.filing_package is not present then show to user that input dataset is not present*/
%if  %sysfunc(exist(&SSR_SDM_LIB..&FILING_PKG_DS_NM., data)) eq 0 %then %let STPERR_PKGS=1;
%if  %sysfunc(exist(&SSR_SDM_LIB..&IMPORTD_PKG_DS_NM., data)) eq 0 %then %let STPERR_DATA=1;

%if ~(&STPERR_PKGS.=1 or &STPERR_DATA.=1) %then
%do;
proc sql noprint;
   create table WORK.pkg_info as 
       select a.*
      from 
         &SSR_SDM_LIB..&FILING_PKG_DS_NM.(where=(kstrip(kupcase(FILING_PACKAGE_TXT)) = kstrip(%kupcase("&STP_PKG_SELECTED.")) and kstrip(kupcase(DELETE_FLG))="N")) as a;/*i18NOK:LINE */
quit;

data _null_;
   set WORK.pkg_info;
       /* i18nOK:BEGIN */
       call symputx("FILING_PKG_NM",FILING_PACKAGE_NM,L);
      call symputx("FILING_ENTITY_NM",LEGAL_ENTITY_ID,L);
      call symputx("FILING_RPTNG_DT",put(REPORT_AS_OF_DT,NLDATE.),L);
      call symputx("FILING_TAXNOMY",TAXONOMY_CD,L);
      call symputx("FILING_MODULE",MODULE_CD,L);
      call symputx("FILING_CURRENCY",CURRENCY_CD,L);
      /* i18nOK:END */
run;

proc sql noprint;
   create table WORK.Req_pkg_data as
      select a.*
      from 
         &SSR_SDM_LIB..&IMPORTD_PKG_DS_NM. as a
      where a.filing_package_sk in (select b.filing_package_sk from WORK.pkg_info b) and DATA_POINT_CD ne "";
      select count(*) into :NO_OBS from WORK.Req_pkg_data;/*i18NOK:LINE */
quit;
%end;
%if ((&NO_OBS.=0) or (&STPERR_PKGS.=1) or (&STPERR_DATA.=1)) %then 
   %do;
     %if &WEBOUT_IND. %then 
        %do;
         data _null_;
         file _webout;
         /* i18nOK:BEGIN */
         put "<HTML>";
         put "<HEAD><TITLE>%sysfunc(sasmsg(sashelp.ssrmsg, 39, NOQUOTE))</TITLE></HEAD>";
         put;
         put "<BODY>";
         put "<H1>%sysfunc(sasmsg(sashelp.ssrmsg, 20, NOQUOTE))</H1>";
         put;
         %if (&STPERR_PKGS.=1) %then
         %do;
         put "%sysfunc(sasmsg(sashelp.ssrmsg, 40, NOQUOTE)) &SSR_SDM_LIB..&FILING_PKG_DS_NM. %sysfunc(sasmsg(sashelp.ssrmsg, 41, NOQUOTE)).";
         %end;

         %if (&STPERR_DATA.=1) %then
         %do;
         put "%sysfunc(sasmsg(sashelp.ssrmsg, 40, NOQUOTE)) &SSR_SDM_LIB..&IMPORTD_PKG_DS_NM. %sysfunc(sasmsg(sashelp.ssrmsg, 41, NOQUOTE)).";
         %end;

         %if (&NO_OBS.=0) %then

         %do;
         put "%sysfunc(sasmsg(sashelp.ssrmsg, 42, NOQUOTE))";
         %end;
         put "</BODY>";
         put "</HTML>";
          /* i18nOK:END */
         run;
        %end;
     %else
        %do;
          %if (&STPERR_PKGS.=1) %then
         %do;
         %put "%sysfunc(sasmsg(sashelp.ssrmsg, 40, NOQUOTE)) &SSR_SDM_LIB..&FILING_PKG_DS_NM. %sysfunc(sasmsg(sashelp.ssrmsg, 41, NOQUOTE))";/*i18NOK:LINE */
         %end;

         %if (&STPERR_DATA.=1) %then
         %do;
         %put "%sysfunc(sasmsg(sashelp.ssrmsg, 40, NOQUOTE)) &SSR_SDM_LIB..&IMPORTD_PKG_DS_NM. %sysfunc(sasmsg(sashelp.ssrmsg, 41, NOQUOTE))";/*i18NOK:LINE */
         %end;
         %if (&NO_OBS.=0) %then

         %do;
         %put "%sysfunc(sasmsg(sashelp.ssrmsg, 42, NOQUOTE))";/*i18NOK:LINE */
         %end;
        %end;
   %end;
%else
   %do;
        /****************************************************************************************************************************************/
      /*Data processing                                                                                                                       */
        /****************************************************************************************************************************************/
      proc sql noprint;
         create table WORK.pkg_dp_avaibality as
         select   a.FILING_PACKAGE_SK
               ,a.TEMPLATE_CD
                     /* i18nOK:BEGIN */
               ,case when a.CELL_VAL="" 
                      then "NO"
                    else "YES"
                 end as DATA_AVAIBALITY_CD                       
                ,count(*) as DATA_POINT_CNT
                      /* i18nOK:END */
         from 
            WORK.Req_pkg_data as a
         group by 
                  FILING_PACKAGE_SK,TEMPLATE_CD,DATA_AVAIBALITY_CD
         order by 
               FILING_PACKAGE_SK,TEMPLATE_CD,DATA_AVAIBALITY_CD;
      quit;

      /*Create format to categorise data availability percentages*/
      proc format;
         /* i18nOK:BEGIN */ 
      value pct_range low-<.25 ="0% - < 25% data"  
                  0.25-<0.5="25% - < 50% data"
                   0.5-<1   ="50% - < 100% data"   
                  1-high   ="100% data"; 

      value pct_catid low-<.25 ="A"  
                  0.25-<0.5="B"
                   0.5-<1   ="C"   
                  1-high   ="D"; 
         /* i18nOK:END */
      run;

        /****************************************************************************************************************************************/
      /*Data availability percentage calculations                                                                                             */
        /****************************************************************************************************************************************/
      /*Template level percentage calculations*/
      proc sql noprint;
         create table WORK.tmplt_datapoint_pct as
            select a.*
               ,sum(DATA_POINT_CNT) as TMPLT_DP_TOTAL
               ,(DATA_POINT_CNT/SUM(DATA_POINT_CNT)) AS TMPLT_DP_PCT 
               ,put(CALCULATED  TMPLT_DP_PCT,pct_range.) as TMPLT_DP_PCT_RANGE
                  ,put(CALCULATED  TMPLT_DP_PCT,pct_catid.) as CATEGORY_CD  
            from 
               WORK.pkg_dp_avaibality a
            group by
               FILING_PACKAGE_SK,TEMPLATE_CD;
      quit;

      /*Package level percentage calculation*/
      proc sql noprint;
         create table WORK.pkg_dpcnt_by_availcd as
            select a.FILING_PACKAGE_SK
                  ,a.DATA_AVAIBALITY_CD
                 ,sum(DATA_POINT_CNT) as PKG_DPCNT_BY_DAT_AVAILCD
            from 
               WORK.tmplt_datapoint_pct a
            group by
               FILING_PACKAGE_SK,DATA_AVAIBALITY_CD;

         create table WORK.pkg_datapoint_pct as
            select a.*
            ,sum(PKG_DPCNT_BY_DAT_AVAILCD) as PKG_DP_TOTAL
            ,(PKG_DPCNT_BY_DAT_AVAILCD/SUM(PKG_DPCNT_BY_DAT_AVAILCD)) AS PKG_DP_PCT
            from 
               WORK.pkg_dpcnt_by_availcd a
            group by
               FILING_PACKAGE_SK
            order by FILING_PACKAGE_SK,DATA_AVAIBALITY_CD;
      quit;

      proc sql noprint;
         select count(*) into: DATAVAIL_CATG_CNT from WORK.pkg_datapoint_pct;/*i18NOK:LINE */
      quit;

      %if &DATAVAIL_CATG_CNT.=1 %then 
         %do;
            proc sql noprint;
            select kstrip(kupcase(DATA_AVAIBALITY_CD)) into: CAT_CD_EXIST from WORK.pkg_datapoint_pct;
            quit;
             
             data Work.temp;
            set WORK.pkg_datapoint_pct;
            run;

            proc sql noprint;
               insert into pkg_datapoint_pct
               select FILING_PACKAGE_SK
                       /* i18nOK:BEGIN */
                  ,%if &CAT_CD_EXIST.="YES" %then
                     %do;"NO" %end; 
                  %else
                     %do;"YES" %end; as DATA_AVAIBALITY_CD 
                      /* i18nOK:END */
                  ,sum(PKG_DP_TOTAL,-PKG_DPCNT_BY_DAT_AVAILCD) as PKG_DPCNT_BY_DAT_AVAILCD
                  ,PKG_DP_TOTAL
                  ,sum(1,-PKG_DP_PCT) as PKG_DP_PCT
               from WORK.temp;
            quit;
      %end;

      /*Collecting tmplt and pkg data availability percentages and taking care of data in case if only one category present*/
         proc sql noprint;
         create table WORK.avail_dp_summry as
            select 
                a.FILING_PACKAGE_SK
               ,a.TEMPLATE_CD
               ,COALESCEC(b.DATA_AVAIBALITY_CD,"YES") as DATA_AVAIBALITY_CD /*i18NOK:LINE */

               ,COALESCE(b.DATA_POINT_CNT,0) AS TMPLT_AVAIL_DP_CNT
               ,COALESCE(b.TMPLT_DP_TOTAL,c.TMPLT_DP_TOTAL,0) AS TMPLT_DP_TOTAL
               ,COALESCE(b.TMPLT_DP_PCT,0.0) AS TMPLT_AVAIL_DP_PCT
               ,COALESCEC(b.TMPLT_DP_PCT_RANGE,"0% - < 25% data") as TMPLT_DP_PCT_RANGE length=20 /*i18NOK:LINE */
                  ,COALESCEC(b.CATEGORY_CD,"A") as CATEGORY_CD length=1 /*i18NOK:LINE */

            from (select distinct TEMPLATE_CD,FILING_PACKAGE_SK from WORK.pkg_dp_avaibality) as a
               left join WORK.tmplt_datapoint_pct(where=(DATA_AVAIBALITY_CD="YES")) b /*i18NOK:LINE */
                  on (a.TEMPLATE_CD=b.TEMPLATE_CD)
               left join WORK.tmplt_datapoint_pct(where=(DATA_AVAIBALITY_CD="NO")) c /*i18NOK:LINE */
                  on (a.TEMPLATE_CD=c.TEMPLATE_CD)

            order by FILING_PACKAGE_SK,CATEGORY_CD,TEMPLATE_CD,DATA_AVAIBALITY_CD,TMPLT_DP_PCT_RANGE;
         quit;
       
      /*Counting and collecting template code by data availability percentage ranges*/ 
      proc sort data=
               WORK.avail_dp_summry(where=(DATA_AVAIBALITY_CD="YES")) /*i18NOK:LINE */
                  out=
               WORK.avail_dp_summry_sort(keep=FILING_PACKAGE_SK TEMPLATE_CD TMPLT_DP_PCT_RANGE CATEGORY_CD);
         by TMPLT_DP_PCT_RANGE;
      run;

      data WORK.pkglev_availablty_summry;
      set WORK.avail_dp_summry_sort;
         length TMP_STACKS $1000; retain TMP_STACKS;
         by TMPLT_DP_PCT_RANGE ;

         * this resets the running total to 0 at the start of a family ;
         if first.TMPLT_DP_PCT_RANGE THEN
            do;
            TMP_CONT = 0;TMP_STACKS = kstrip(TEMPLATE_CD);
            end;
         else TMP_STACKS=kstrcat(kstrip(TMP_STACKS),",   ",kstrip(TEMPLATE_CD));

         tmp_cont + 1;

         if last.TMPLT_DP_PCT_RANGE then output;
         keep FILING_PACKAGE_SK TMPLT_DP_PCT_RANGE TMP_CONT TMP_STACKS CATEGORY_CD;
      run;

      proc sql noprint;
         select count(distinct TEMPLATE_CD) into: TOTAL_TMPLT from WORK.avail_dp_summry;/*i18NOK:LINE */
         select put(PKG_DP_PCT,nlpct7.2) into: PKG_AVAIL_DP_PCT from WORK.Pkg_datapoint_pct where DATA_AVAIBALITY_CD="YES";/*i18NOK:LINE */
      quit;

        /****************************************************************************************************************************************/
      /*Start printing the reports                                                                                                            */
        /****************************************************************************************************************************************/
      ods listing close; 
        %if &WEBOUT_IND. %then
         %do;
            data _null_;
               file _webout;
               /* i18nOK:BEGIN */
               put "<html>";
               put "<body>";
               put '<div id="pleasewait">';
               put "<h2><font color='green'>%sysfunc(sasmsg(sashelp.ssrmsg, 43, NOQUOTE))</font></h2>";  
               put '</div>';  
               do i = 1 to 10;
               put "                                       ";
               /* i18nOK:END */
               end;
            run;

            data _null_;
            x = sleep(3,1);
            run;
            %global  _odsoptions _odsdest; 
            %let _ODSDEST=tagsets.htmlpanel;
            %stpbegin;
         %end;
      %else 
          %do;
            filename odsout "&HTML_PATH";
            ods tagsets.htmlpanel path=odsout file = "&HTML_BODY";
         %end;

        /*Report title*/
      /* i18nOK:BEGIN */
      title1 h=15pt j=l color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 44, NOQUOTE)):" color=tomato "&FILING_PKG_NM" color=darkblue ", %sysfunc(sasmsg(sashelp.ssrmsg, 50, NOQUOTE))";
      title2 h=15pt j=l color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 9, NOQUOTE))  :" color=tomato "&FILING_ENTITY_NM";
      title3 h=15pt j=l color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 11, NOQUOTE)):" color=tomato "&FILING_RPTNG_DT";
      title4 h=15pt j=l color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 10, NOQUOTE)) :" color=tomato "&FILING_CURRENCY";
      title5 h=15pt j=l color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 12, NOQUOTE)) :" color=tomato "&FILING_TAXNOMY";
      title6 h=15pt j=l color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 13, NOQUOTE))   :" color=tomato "&FILING_MODULE";
      /* i18nOK:END */
        /****************************************************************************************************************************************/
        /*Package level summary:                                                                                                          */
        /****************************************************************************************************************************************/
        /*1.Package level templates distribution by avaiabality ranges*/
      ods tagsets.htmlpanel options(embedded_titles = "no" panelcolumns = "2" panelborder = "1");/*i18NOK:LINE */
      ods tagsets.htmlpanel event=panel(start);

      /*goptions xpixels=1500 ypixels=1500;*/
      goptions device  = GIF;
      /*Pie chart printing*/
      proc template;
      define statgraph piechart;
      begingraph;
         entrytitle "%sysfunc(sasmsg(sashelp.ssrmsg, 45, NOQUOTE))"/TEXTATTRS=(family="Arial" color=darkblue weight= bold size=12pt);/*i18NOK:LINE */
         discreteattrmap name="PCT_MAP_COL"/ ignorecase=true ;/*i18NOK:LINE */
             /* i18nOK:BEGIN */
            value "0% - < 25% data" / fillattrs=(color=&COLR_CATGRY1. );
            value "25% - < 50% data" / fillattrs=(color=&COLR_CATGRY2.    );
            value "50% - < 100% data" / fillattrs=(color=&COLR_CATGRY3. ); 
            value "100% data" / fillattrs=(color=&COLR_CATGRY4.);
                /* i18nOK:END */ 
         enddiscreteattrmap;
         discreteattrvar attrvar=DATA_AVAIL_RANGE var=TMPLT_DP_PCT_RANGE attrmap="PCT_MAP_COL";/*i18NOK:LINE */
         layout region;
            piechart category=DATA_AVAIL_RANGE response=TMP_CONT/datalabellocation=callout 
                                                   dataskin=GLOSS
                                                   datalabelattrs=(family="Arial" size=10pt Weight=Bold color=blue) /*i18NOK:LINE */
                                                   categorydirection=clockwise start=180
                                                   othersliceopts=(type=percent percent=2 label="Other");/*i18NOK:LINE */
         endlayout;
         entryfootnote "%sysfunc(sasmsg(sashelp.ssrmsg, 46, NOQUOTE)) &PKG_AVAIL_DP_PCT %sysfunc(sasmsg(sashelp.ssrmsg, 47, NOQUOTE))"/TEXTATTRS=(family="Arial"  color=white weight= bold size=12pt) /*i18NOK:LINE */
                                                                                        border=true 
                                                                  opaque=true backgroundcolor=darkblue;
      endgraph;
      end;

      proc sgrender data=WORK.pkglev_availablty_summry template=piechart;
      run;

      /*2.Templates by data availability ranges*/
      ods tagsets.htmlpanel options(embedded_titles = "yes");/*i18NOK:LINE */
      title h=12pt color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 48, NOQUOTE))";/*i18NOK:LINE */
      proc report data=WORK.pkglev_availablty_summry nowd split="\"
          style(column header)=[background=Dimgray  foreground=white];
         column CATEGORY_CD TMPLT_DP_PCT_RANGE TMP_STACKS; 
            define CATEGORY_CD / order noprint;
            define TMPLT_DP_PCT_RANGE /"Data availability\ranges"  STYLE=[FONT_WEIGHT=BOLD] ;/*i18NOK:LINE */  
      /*      define TMP_CONT / center 'Count of\Templates'; */
            define TMP_STACKS /"Templates Codes";/*i18NOK:LINE */

         compute CATEGORY_CD;
             /* i18nOK:BEGIN */ 
          if CATEGORY_CD="A" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY1 foreground=black]");
           if CATEGORY_CD="B" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY2 foreground=black]"); 
           if CATEGORY_CD="C" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY3 foreground=black]"); 
           if CATEGORY_CD="D" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY4 foreground=black]");
             /* i18nOK:END */ 
         endcomp;

         compute TMPLT_DP_PCT_RANGE; 
            call define("TMPLT_DP_PCT_RANGE","STYLE","STYLE=[background=Dimgray  foreground=white]");/*i18NOK:LINE */ 
         endcomp;

      run;

      ods tagsets.htmlpanel event=panel(finish);
        /****************************************************************************************************************************************/
        /*Report level data availability percentages                                                                                            */
        /****************************************************************************************************************************************/
      /*title1 h=15pt Box=1 BCOLOR=blue j=L color=white font='Arial/bo' 'Template Level Summary';*/

      title h=12pt color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 49, NOQUOTE))";/*i18NOK:LINE */

      /*Split data set in parts to print*/
      /*Create macro variable for data set to be used*/
      %local DS_NM;
      data _null_;
       call symputx("DS_NM","avail_dp_summry",L);/*i18NOK:LINE */
      run;
      data _null_;  
        if 0 then set &DS_NM. nobs=count; 
        call symput("NUMOBS",put(count,8.)); /*i18NOK:LINE */
      run;
      /*Finding maximum parts we can have for spliting*/
      %if &NUMOBS. le 15 %then  %do;%let NO_OF_PARTS=1;%end;

      %if (15 < &NUMOBS. and &NUMOBS. <= 30) %then %do;%let NO_OF_PARTS=2;%end;

      %if &NUMOBS. > 30 %then
          %do;
             data _null_;
             %do prts=6 %to 2 %by -1;
                  %let z=%sysevalf(&NUMOBS./&prts.,ceil);
                %if(15<=&z. and &z. <=25) %then
                     %do;
                         call symputx("NO_OF_PARTS",put(&prts.,8.));/*i18NOK:LINE */
                       stop;
                     %end; 
              %end;
             run;
         %end;
      /*For printing in panels purpose set 5 parts to 4*/
      %if &NO_OF_PARTS=5 %then %let NO_OF_PARTS=4;

      /*Start spliting of data set*/
      %let n=%sysevalf(&numobs/&NO_OF_PARTS,ceil); 
      data %do J=1 %to &NO_OF_PARTS ; &DS_NM._&J %end; ; 
      set &DS_NM.; 
      %do I=1 %to &NO_OF_PARTS; 
          if %eval(&n*(&i-1)) <_n_ <= %eval (&n*&I) then output &DS_NM._&I; 
       %end; 
      run; 
      
      /*Setting number of columns for paneling*/
      %if &NO_OF_PARTS. le 3 %then 
         %do;
            data _null_; 
            call symputx("PANEL_COLMNS",put(&NO_OF_PARTS.,8.));/*i18NOK:LINE */
            run; 
         %end;
      %if(4<=&NO_OF_PARTS. and &NO_OF_PARTS. <=6) %then 
         %do;
            data _null_; 
            call symputx("PANEL_COLMNS",%sysevalf(&NO_OF_PARTS./2,ceil));/*i18NOK:LINE */
            run; 
         %end;

      ods tagsets.htmlpanel options(embedded_titles = "no" panelcolumns = "&PANEL_COLMNS" panelborder = "1");/*i18NOK:LINE */
      ods tagsets.htmlpanel event=panel(start);

      %do part=1 %to &NO_OF_PARTS.;
         proc report data=&DS_NM._&part. nowd split="\"
            style(column header)=[background=Dimgray  foreground=white];

         column  CATEGORY_CD TEMPLATE_CD TMPLT_DP_TOTAL TMPLT_AVAIL_DP_PCT;
            define CATEGORY_CD / noprint;
            define TEMPLATE_CD /"Template\Code" STYLE=[FONT_WEIGHT=BOLD] center ;/*i18NOK:LINE */  
            define TMPLT_DP_TOTAL/  "Total Required\Data Points";/*i18NOK:LINE */ 
            define TMPLT_AVAIL_DP_PCT/ "Percentage of\Available Data" format=NLPCT7.2;/*i18NOK:LINE */

            compute CATEGORY_CD;
                /* i18nOK:BEGIN */ 
            if CATEGORY_CD="A" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY1 foreground=black]");
            if CATEGORY_CD="B" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY2 foreground=black]"); 
            if CATEGORY_CD="C" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY3 foreground=black]"); 
            if CATEGORY_CD="D" then CALL DEFINE(_ROW_, "STYLE", "STYLE=[BACKGROUND=&COLR_CATGRY4 foreground=black]");
                /* i18nOK:END */ 
            endcomp;

            compute TEMPLATE_CD; 
            call define("TEMPLATE_CD","STYLE","STYLE=[background=Dimgray  foreground=white]");/*i18NOK:LINE */ 
            endcomp;

         run;   
      %end;

      ods tagsets.htmlpanel event=panel(finish);

      %if %upcase("&TMPLT_LEV_GARPH_FLG.")="YES" %then/*i18NOK:LINE */
          
         %do; /*Start graph printing*/
            ods tagsets.htmlpanel event=panel(start);
            /*Creating data to select group colors*/
            data WORK.attrmap;
               length id $20. value $3. fillcolor $15. linecolor $8.;
               id="DATA_AVAIBALITY_CD"; value="YES";fillcolor="&COLR_CATGRY4";linecolor="black";output;/*i18NOK:LINE */
               id="DATA_AVAIBALITY_CD"; value="NO";fillcolor="&COLR_CATGRY1";linecolor="black";output;/*i18NOK:LINE */
            run;

            title h=12pt color=darkblue font="Arial/bo" "%sysfunc(sasmsg(sashelp.ssrmsg, 49, NOQUOTE))";/*i18NOK:LINE */
            %do part=1 %to &NO_OF_PARTS.;
               proc sql noprint;
                  create table WORK.data_part as
                  select a.TEMPLATE_CD
                        ,a.DATA_AVAIBALITY_CD
                       ,TMPLT_DP_PCT
                       ,CATEGORY_CD
                  from
                     WORK.tmplt_datapoint_pct a
                  where a.TEMPLATE_CD in (select b.TEMPLATE_CD from &DS_NM._&part. as b)
                      order by CATEGORY_CD,TEMPLATE_CD, DATA_AVAIBALITY_CD;
               quit;

               proc sgplot data=WORK.data_part dattrmap=WORK.attrmap;
                      format TMPLT_DP_PCT NLPCT7.2;
                  hbar TEMPLATE_CD /group=DATA_AVAIBALITY_CD 
                                   response=TMPLT_DP_PCT
                                 barwidth=0.5 
                                 dataskin=pressed 
                                 attrid=DATA_AVAIBALITY_CD;

               xaxis label="Percentage" LABELATTRS=(COLOR=Darkgreen  weight= bold) VALUEATTRS=(COLOR=blue weight= bold);/*i18NOK:LINE */
               yaxis label="Template Codes" LABELATTRS=(COLOR=Darkgreen  weight= bold) VALUEATTRS=(COLOR=blue weight= bold);/*i18NOK:LINE */
               keylegend / title="Data availability";/*i18NOK:LINE */
               run; 
            %end;
            ods tagsets.htmlpanel event=panel(finish);
         %end; /*end graph printing*/
         
      ods tagsets.htmlpanel close;

      %if &WEBOUT_IND. = 1 %then 
      %do;
        %stpend;
         /* Clear Please Wait message */
         data _null_;
           file _webout;
          /* i18nOK:BEGIN */
           put "<script>";
           put 'pleasewait.style.display="none";';
           put "</script>";
           put "</body></html>";
          /* i18nOK:END */
         run;
      %end;
   
      ods listing;
      
   %end;

%mend ssr_gen_datasufficncy_rpt;
