/***********************************************************************
 * Copyright (c) 2017 by SAS Institute Inc., Cary, NC, USA.            *
 *                                                                     *
 * NAME:          ssr_postinit                                         *
 *                                                                     *
 * PURPOSE:       Initializes macro paths for SAS Supervisory Risk     *
 *                Reporting , reads the macro path                     *
 *                from a metadata property and appends it to           *
 *                SASAUTO                                              *
 *                                                                     *
 * USAGE:         %ssrinit                                             *
 *                                                                     *
 * PARAMETERS:    None                                                 *
 *                                                                     *
 *                                                                     *
 **********************************************************************/
%macro ssr_postinit;
/* Please execute %ssrinit(VERSION=) prior to this */  
/* RD_SWC=Supervisory Rpt Cfg &VERSION is set in ssrinit*/

   %*------------------------------------------------------------------;
   %* Check whether RD_DVD has already been set so that this logic is  ;
   %* not done twice.  To force the initialization to run again, set   ;
   %* RD_DVD back to empty.                                            ;
   %*------------------------------------------------------------------;

   /*set path for deployment root directory*/
   %global DEPLOYMENT_ROOT;
   %let DEPLOYMENT_ROOT=%sysget(SASROOT);

   /*Get the solution data location from metadata property */
   %global ssr_data_location ssr_user_log_location;
   data _null_;
      n = 1;
      length uri asn_uri propertyValue$ 500;
      length propertyName $32;
      uri="";
      asn_uri="";
      propertyValue="";
      propertyName="";
      /* i18nOK:BEGIN */
      nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
      numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
      numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

      do while(1);
         rc = metadata_getattr(uri, "name", propertyName);
         if (propertyName = "ssrsvrc.data.location") then do;
            rc = metadata_getattr(uri, "DefaultValue", propertyValue);
            call symput("ssr_data_location", ktrim(propertyValue));
            leave;
         end;
         else do;
            n = n + 1;
            if (n > numasn) then leave;
            rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
         end;
      end;
      /* i18nOK:END */
   run;

   /*Get the solution log location from metadata property */
   data _null_;
      n = 1;
      length uri asn_uri propertyValue$ 500;
      length propertyName $32;
      uri="";
      asn_uri="";
      propertyValue="";
      propertyName="";
      /* i18nOK:BEGIN */
      nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&RD_SWC" || "'", n, uri);
      numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
      numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

      do while(1);
         rc = metadata_getattr(uri, "name", propertyName);
         if (propertyName = "ssrsvrc.logs.location") then do;
            rc = metadata_getattr(uri, "DefaultValue", propertyValue);
            call symput("ssr_user_log_location", ktrim(propertyValue));
            leave;
         end;
         else do;
            n = n + 1;
            if (n > numasn) then leave;
            rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
         end;
      end;
      /* i18nOK:END */
   run;

   %ssr_assign_libs;
   %let syscc=0;

   ;
%mend ssr_postinit;
