filename LUAPATH ( '!SASROOT/misc/stresssvr/lua/sas' '!SASROOT/stresssvr/sasmisc/lua/sas' );



proc lua restart;
submit;

local rfw = require 'risk.stress.rfw_api'
local JSON = require 'risk.stress.json'

rfw.rootdir_set()
local str = sas.symget("sst_url_base") -- take off SASRiskandfinanceworkbench
str = sas.substr(str,1, string.len(str)-26)
rfw.url_set(str)

local l = sas.symget("sst_selection")
local loadId = sas.symgetn("sst_property_id")
local projectId = sas.symgetn("sst_current_project_id")
local location = JSON:decode(l)
local pass, detailKind,hierarchyMapping, interval, path = rfw.detail_metadata_ldId_get(loadId, projectId)
local dest = sas.pathname("work") .. "/details.csv"

if not pass then return end

if path then  -- RegCap
    -- get where clause and path
    local where_cls, first_time = "", true
    local vname
    local entname = sas.symget("sst_entity_hierarchy_name")
    entname = entname or "Entity"
    
    for i, h in ipairs(location.cells) do       
       for j, lv in ipairs(h.levels) do
          if h.hierarchy ~= entname then
             local _, _, dim = string.find(h.hierarchy, ".*([XYZ])")

             local _, _, cd = string.find(lv.liKey, ".*_(%w+)$")
             
             if first_time then -- add the sheet where
                first_time = false
                where_cls = "template_cd = '"..string.gsub(sas.symget("sst_sheet"),"_", " ", 1).."'"
             end 
             if dim == "X" then 
                vname="column_cd" 
             elseif dim == "Y" then 
                vname="row_cd"
             else
                vname="sheet_cd"
             end
             if j == #h.levels   then      
               where_cls = where_cls .. " and " .. vname .. " = '" .. cd .."'"
             end
          end
       end    
    end

    local final_path, dsName = sas.io.split(path)
    dsName = string.gsub(dsName, ".sas7bdat","") 
    local pathname = sas.pathname("work")

   sas.submit([[ 
      %let dtl_loc=@pathname@;
      %let where_cls=1;
      data work.details; Bad="Can't find detail data matching where clause";where_cls="@where_cls@"; run;      
      libname sum "@final_path@";
      data _NULL_; set sum.@dsName@;
         where @where_cls@;
         call symput("where_cls", trim(where_clause));
         call symput("dtl_loc", trim(dtl_data_loc));
      run;
                                
      filename testfile "&dtl_loc.";
   ]])
                                
   if sas.fileexist(sas.symget("dtl_loc")) == 1 then
      sas.submit([[
                     libname det "&dtl_loc";
                 ]])
                     
      if sas.exists('det.details') then
         sas.submit([[
            proc export 
               data=det.details(where=(&where_cls) obs=100000) 
               dbms=csv 
               outfile="@dest@" 
               replace label;
            run;
                                     
         ]])
      else
         sas.submit([[                                     
            data error_file(keep=MESSAGE);
            format MESSAGE $200.;
               label MESSAGE = Drilldown Message;
               MESSAGE= compbl("Please check the physical data location '&dtl_loc.'") ;output ;
               MESSAGE= compbl("This location is available but it does not contain 'details.sas7bdat' dataset.") ;output ;
            run;

            proc export 
               data=error_file
               dbms=csv 
               outfile="@dest@" 
               replace label;
            run;
         ]])
      end
   else
      sas.submit([[
         data error_file(keep=MESSAGE);
         format MESSAGE $200.;
            label MESSAGE = Drilldown Message;
            MESSAGE= compbl("Please check the physical data location '&dtl_loc.'") ;output ;
            MESSAGE= compbl("This location is not available or has been deleted.") ;output ;
         run;

         proc export 
            data=error_file
            dbms=csv 
            outfile="@dest@" 
            replace label;
         run;
      ]])
   end
else
    hierarchyMapping=JSON:decode(hierarchyMapping)
    local pass = rfw.detail_data_to_csv(projectId, detailKind, location, hierarchyMapping, dest )
end
sas.print("OUTPUT FILE: " .. dest)
endsubmit;
run;
