/*--------------------------------------------------------------------------------
   Copyright (C), 2016 SAS Institute Inc.  All Rights Reserved
   sst_import_global_data:  Import global data into the Risk and Finance Workbench.

   inputs:  scheme     - http or https, defaults to http.  See https://en.wikipedia.org/wiki/Uniform_Resource_Locator 
                                        for the syntax (names of parts) of a URL
            hostname   - hostname of the server running the RFW, defaults to localhost
            port       - port number of the server running the RFW, defaults to 7980
            input_file - the server-tier file to upload containing the data. 
                         If your file is located in the public staging directory (that is, <SASConfig>/Lev1/AppData/SASRiskandFinanceWorkbench/staging)
                         input_file should be the following form:  public/<file name>
                         If your file is located in a work group's staging directory, 
                         input_file should be the following form:  <work group name>/<file name>.
            userid     - the userid to use for the HTTP call into the RFW mid-tier to perform the operation.
                         The userid/password combination is optional when this macro is executed within a SAS session created by
                         the mid-tier for executing an RFW Model Group or Script.
            password   - the password to use for the HTTP call into the RFW mid-tier to perform the operation.
                         The userid/password combination is optional when this macro is executed within a SAS session created by
                         the mid-tier for executing an RFW Model Group or Script.
            outvar_http_status_code - an optional *global* macro variable to place the HTTP response's status code from the HTTP call to the RFW mid-tier 
            outvar_http_body - an optional *global* macro variable to place the HTTP response body from the HTTP call to the RFW mid-tier

   Notes:     - Do not quote any of the input parameters to this macro.
              - This macro is meant to be used with a workspace server running SAS Host (basic) authentication.
                One can supply the userid and password via parameters to this macro or, if run within a Script or Model Group,
                the userid and password will be picked up automatically from the mid-tier execution.
------------------------------------------------------------------------------------*/


options NOQUOTELENMAX;
%macro sst_import_global_data(scheme=http, hostname=localhost, port=7980, input_file=, userid=, password=, outvar_http_status_code=, outvar_http_body=);

  %local encoded_input_file  url  request  source;
  %let encoded_input_file=%sysfunc(strip(%sysfunc(putc(%superq(input_file),$base64x32700.))));
  %let url=%superq(scheme)://%superq(hostname):&port./SASRiskandFinanceWorkbench/rest/;
  %let request=config/globalDataFromServer?fileId=&encoded_input_file.;
  %let source=sst_import_global_data;
  
  filename LUAPATH ( '!SASROOT/misc/stresssvr/lua/sas' '!SASROOT/stresssvr/sasmisc/lua/sas' );
  proc lua restart infile="risk.stress.macro_api_entry"; run;

%mend;
